package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.Command;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.intf.CommandService;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.model.RoleAuthParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 审计命令控制器
 *
 * @author 王宇
 */
@RestController
@RequestMapping("/v1/commands")
@Tag(name = "命令管理")
public class CommandController {

	@Autowired
	private CommandService commandService;
	@Autowired
	private TaskInstanceService taskInstanceService;

	/**
	 * 列表
	 *
	 * @param pager
	 * @return
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "命令列表")
	public GeneralResult<GridBean<Command>> list(Pager pager) {
		return commandService.list(pager);
	}

	/**
	 * 添加
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "命令创建")
	public GeneralResult<Void> create(@RequestBody Command params, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return commandService.create(params, context.getTarget());
	}

	/**
	 * 修改
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "命令修改")
	public GeneralResult<Void> modify(@RequestBody Command params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return commandService.modify(params, context.getTarget());
	}

	/**
	 * 删除
	 *
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "命令删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return commandService.remove(id, context.getTarget());
	}

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "命令详情")
	public GeneralResult<Command> detail(@PathVariable(value = Common.ID) Long id) {
		return commandService.detail(id);
	}

	/**
	 * 角色的命令列表
	 *
	 * @param roleId
	 * @return
	 */
	@GetMapping(value = "/authorized")
	@Operation(tags = { "CMC", "CSC" }, summary = "已授权命令")
	public GeneralResult<List<Command>> commands(@RequestParam(value = "roleId") Long roleId) {
		return commandService.listCommands(roleId);
	}

	/**
	 * 角色授权命令
	 *
	 * @param params
	 * @return
	 */
	@PatchMapping(value = "/accredit")
	@Operation(tags = { "CMC", "CSC" }, summary = "命令授权")
	public GeneralResult<Void> accCommand(@RequestBody RoleAuthParam params) {
		return commandService.accCommand(params.getRoleId(), params.getCommands());
	}

	/**
	 * 查询执行命令的任务历史
	 *
	 * @param pager
	 * @return
	 */
	@GetMapping(value = "/{id}/instances")
	@Operation(tags = { "CMC", "CSC" }, summary = "命令执行作业实例")
	public GeneralResult<GridBean<TaskInstance>> list(@PathVariable(value = Common.ID) Long id, Pager pager) {
		return taskInstanceService.list(pager, id);
	}

}
