package com.bocloud.cop.booter.controller;

import com.bocloud.cop.intf.inspection.InspectRecordService;
import com.bocloud.cop.intf.inspection.InspectRoutineService;
import com.bocloud.cop.intf.patch.PatchDeployHistoryService;
import com.bocloud.cop.intf.patch.PatchService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author zoucg
 * @Company: 博云 www.bocloud.com
 * @ClassName: TaskBoardController
 * @Description: 运维看板数据接口
 * @DATE: 2022/6/10
 */
@RestController
@RequestMapping("/v1/tasks/board")
@Tag(name = "运维看板")
public class CopBoardController {

	@Autowired
	private InspectRecordService inspectRecordService;
	@Autowired
	private PatchDeployHistoryService patchDeployHistoryService;
	@Autowired
	private InspectRoutineService inspectRoutineService;
	@Autowired
	private PatchService patchService;

	@GetMapping(value = "/stats/status")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务结果状态数量统计")
	public GeneralResult<?> statsStatus(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String code) {
		if ("INSPECTIONSTATUS".equalsIgnoreCase(code)) {
			return inspectRecordService.statsStatus(context, "INSPECTION");
		} else if ("COMPLIANCESTATUS".equalsIgnoreCase(code)) {
			return inspectRecordService.statsStatus(context, "COMPLIANCE");
		} else if ("INSPECTIONNUM".equalsIgnoreCase(code)) {
			return inspectRoutineService.statsNum(context, "INSPECTION");
		} else if ("COMPLIANCENUM".equalsIgnoreCase(code)) {
			return inspectRoutineService.statsNum(context, "COMPLIANCE");
		} else if ("PATCHDEPLOYSTATUS".equalsIgnoreCase(code)) {
			return patchDeployHistoryService.statsStatus(context);
		} else if ("PATCHS".equalsIgnoreCase(code)) {
			return patchService.statsStatus(context);
		}
		return new GeneralResult<>(false, "获取数据失败，参数错误");
	}

	@GetMapping(value = "/stats/trend")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务结果状态趋势统计")
	public GeneralResult<?> statsTrend(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String code) {
		if ("INSPECTION".equalsIgnoreCase(code)) {
			return inspectRecordService.statsTrend(context, code);
		} else if ("COMPLIANCE".equalsIgnoreCase(code)) {
			return inspectRecordService.statsTrend(context, code);
		} else if ("PATCHDEPLOY".equalsIgnoreCase(code)) {
			return patchDeployHistoryService.statsTrend(context);
		}
		return new GeneralResult<>(false, "获取数据失败，参数错误");
	}

}
