package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.NodeTemplate;
import com.bocloud.cop.intf.NodeTemplateService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 节点模板控制器
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1/tasknodes/templates")
@Tag(name = "节点模板管理")
public class NodeTemplateController {

    @Autowired
    private NodeTemplateService nodeTemplateService;

    /**
     * 获取节点模板列表
     *
     * @param pager
     * @return 数据结果集
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "节点模板列表")
    public GeneralResult<GridBean<NodeTemplate>> list(Pager pager) {
        return nodeTemplateService.list(pager);
    }

    /**
     * 获取节点模板列表
     *
     * @param pager
     * @return 数据结果集
     */
    @GetMapping(value = "/simple")
    @Operation(tags = {"CMC", "CSC"}, summary = "节点模板列表（简单）")
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        return nodeTemplateService.simple(pager);
    }

    /**
     * 添加节点模板
     *
     * @param params
     * @param context
     * @return 添加结果
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "节点模板创建")
    public GeneralResult<Void> create(@RequestBody NodeTemplate params,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setCreatorId(context.getTarget());
        params.setMenderId(context.getTarget());
        return nodeTemplateService.create(params);
    }

    /**
     * 修改节点模板
     *
     * @param params
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "节点模板修改")
    public GeneralResult<Void> modify(@RequestBody NodeTemplate params, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setMenderId(context.getTarget());
        return nodeTemplateService.modify(params);
    }

    /**
     * 移除节点模板
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "节点模板删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nodeTemplateService.remove(id, context.getTarget());
    }

    /**
     * 查看节点模板详细信息
     *
     * @param id
     * @return 操作结果
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "节点模板详情")
    public GeneralResult<NodeTemplate> detail(@PathVariable(value = Common.ID) Long id) {
        return nodeTemplateService.detail(id);
    }

}
