package com.bocloud.cop.booter.controller;

import com.bocloud.cop.intf.ServerUserService;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.params.*;
import com.megatron.common.enums.OSCategory;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 主机用户管理Controller(bmp-server)
 */
@RestController
@RequestMapping("/v1/servers/users")
@Tag(name = "主机用户管理")
public class ServerUserController {

	@Autowired
	private ServerUserService serverUserService;

	/**
	 * 增加主机用户
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "主机用户创建")
	public GeneralResult<String> create(@RequestBody ServerUserCreateParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return serverUserService.create(params, context);
	}

	/**
	 * 修改主机用户
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@PutMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "主机用户修改")
	public GeneralResult<String> modify(@RequestBody ServerUserModifyParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return serverUserService.modify(params, context);
	}

	/**
	 * 删除主机用户
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@DeleteMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "主机用户删除")
	public GeneralResult<String> remove(@RequestBody ServerUserRemoveParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return serverUserService.remove(params, context);
	}

	/**
	 * 保存主机用户的sudo权限
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@PostMapping(value = "/auths")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机用户sudo权限保存")
	public GeneralResult<String> saveAuth(@RequestBody ServerUserSaveAuthParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return serverUserService.saveAuth(params, context);
	}

	/**
	 * 删除主机用户的sudo权限
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@DeleteMapping(value = "/auths")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机用户sudo权限删除")
	public GeneralResult<String> removeAuth(@RequestBody ServerUserRemoveAuthParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return serverUserService.removeAuth(params, context);
	}

	/**
	 * 获取主机用户的sudo权限列表
	 *
	 * @param context
	 * @return
	 */
	@GetMapping(value = "/auths")
	@Operation(tags = { "CMC", "CSC" }, summary = "主机用户sudo权限列表")
	public GeneralResult<Map<String, String>> listAuth(ServerUserListAuthParams param,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		WorkerHost workerHost = new WorkerHost(param.getHost(), param.getUser(), param.getPasd(), param.getPort(),
				OSCategory.valueOf(param.getCategory()), param.getSegmentId());
		Boolean become = param.getBecome();
		String userName = param.getUserName();
		return serverUserService.listAuth(workerHost, userName, become, context);
	}

}
