package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.SoftwareGroup;
import com.bocloud.cop.intf.SoftwareGroupService;
import com.bocloud.cop.model.SoftwareGroupBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 文件分组控制器
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1/softwares/groups")
@Tag(name = "文件分组管理")
public class SoftwareGroupController {

    @Autowired
    private SoftwareGroupService softwareGroupService;

    /**
     * 获取文件分组列表
     *
     * @param parentId
     * @return 数据结果集
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "文件分组列表")
    public GeneralResult<List<SoftwareGroup>>
    list(@RequestParam(value = Common.PARENTID, required = false) Long parentId) {
        return softwareGroupService.list(parentId);
    }

    /**
     * 添加文件分组
     *
     * @param context
     * @return 添加结果
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "文件分组创建")
    public GeneralResult<Void> create(@RequestBody SoftwareGroup group,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        group.setCreatorId(context.getTarget());
        group.setMenderId(context.getTarget());
        return softwareGroupService.create(group);
    }

    /**
     * 修改文件分组
     *
     * @param bean
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "文件分组修改")
    public GeneralResult<Void> modify(@RequestBody SoftwareGroupBean bean, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return softwareGroupService.modify(bean, context.getTarget());
    }

    /**
     * 移除文件分组
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "文件分组删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return softwareGroupService.remove(id, context.getTarget());
    }

    /**
     * 查看文件分组详细信息
     *
     * @param id
     * @return 操作结果
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "文件分组详情")
    public GeneralResult<SoftwareGroup> detail(@PathVariable(value = Common.ID) Long id) {
        return softwareGroupService.detail(id);
    }

}
