package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cop.entity.TaskConfig;
import com.bocloud.cop.intf.TaskConfigService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/v1/tasks/configs")
@Tag(name = "运维配置管理")
public class TaskConfigController {

	@Autowired
	private TaskConfigService taskConfigService;

	/**
	 * 更新运维配置
	 *
	 * @param configs
	 * @param context
	 * @return
	 */
	@PutMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "运维配置更新")
	public Result modify(@RequestBody List<TaskConfig> configs, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskConfigService.batchUpdate(configs, context);
	}

	/**
	 * 查询运维配置
	 *
	 * @param category
	 * @param codes
	 * @return
	 */
	@GetMapping
	@Operation(summary = "运维配置查询")
	public GeneralResult<Map<String, String>> getByCodes(
			@RequestParam(value = "category", required = false) String category,
			@RequestParam(value = "codes", required = false) ArrayList<String> codes) {
		return taskConfigService.listConfigs(category, codes);
	}

	/**
	 * 查询运维配置类型树
	 *
	 * @param category
	 * @return
	 */
	@GetMapping("/tree")
	@Operation(summary = "运维配置类型树")
	public GeneralResult<JSONArray> listCategoryTree(@RequestParam(value = "category") String category) {
		return taskConfigService.listCategoryTree(category);
	}

}
