package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.enums.*;
import com.bocloud.cop.intf.ScriptService;
import com.bocloud.cop.intf.StatsTaskService;
import com.bocloud.cop.intf.TaskService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.ResultTrend;
import com.bocloud.cop.model.StatResult;
import com.bocloud.cop.model.TaskBean;
import com.bocloud.cop.model.params.StartModeParams;
import com.bocloud.cop.model.params.TaskTransferParams;
import com.bocloud.cop.service.utils.JobScheduler;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;

/**
 * 作业控制器
 *
 * @author weiwei
 */
@Slf4j
@RestController
@RequestMapping("/v1/tasks")
@Tag(name = "作业管理")
public class TaskController {

	@Autowired
	private TaskService taskService;
	@Autowired
	private StatsTaskService statsTaskService;
	@Autowired
	private JobScheduler jobScheduler;
	@Autowired
	private ScriptService scriptService;

	/**
	 * 作业统计信息
	 *
	 * @param context
	 * @param code
	 * @param dataType
	 * @return
	 */
	@GetMapping(value = "/stats")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业信息统计")
	public GeneralResult<?> statsDefault(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String code, @RequestParam(required = false) String dataType) {
		if (null == code) {
			return statsTaskService.statsDefault(dataType, context);
		}
		if ("TaskTrend".equals(code)) {
			return statsTaskService.statsTrend(dataType, context);
		} else if ("TaskDuration".equals(code)) {
			return statsTaskService.statsDuration(dataType, context);
		} else if ("TaskResult".equals(code)) {
			return statsTaskService.statsResult(dataType, context);
		} else if ("TaskInstance".equals(code)) {
			return statsTaskService.statsResult(dataType, context);
		} else if ("DATAVIEW".equals(code)) {
			return statsTaskService.statsDataview(dataType, context);
		} else {
			return statsTaskService.stats(code, dataType, context);
		}

	}

	/**
	 * 作业统计信息
	 *
	 * @param context
	 * @param dataType
	 * @return
	 */
	@GetMapping(value = "/stats/trend")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业趋势统计")
	public GeneralResult<ResultTrend> statsTrend(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String dataType) {
		return statsTaskService.statsTrend(dataType, context);
	}

	/**
	 * 作业统计信息
	 *
	 * @param context
	 * @param dataType
	 * @return
	 */
	@GetMapping(value = "/stats/duration")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业时长统计")
	public GeneralResult<List<StatResult>> statsDuration(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String dataType) {
		return statsTaskService.statsDuration(dataType, context);
	}

	/**
	 * 作业统计信息
	 *
	 * @param context
	 * @param dataType
	 * @return
	 */
	@GetMapping(value = "/stats/result")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业结果统计")
	public GeneralResult<List<StatResult>> statsResult(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String dataType) {
		return statsTaskService.statsResult(dataType, context);
	}

	/**
	 * 作业统计信息
	 *
	 * @param context
	 * @param dataType
	 * @return
	 */
	@GetMapping(value = "/stats/dataview")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业数量统计")
	public GeneralResult<List<StatResult>> statsDataview(@Value(Common.REQ_CONTEXT) RequestContext context,
			@RequestParam(required = false) String dataType) {
		return statsTaskService.statsDataview(dataType, context);
	}

	@PostMapping("/execute")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业执行")
	public GeneralResult<Void> operate(@RequestBody JSONObject object) {
		return taskService.execute(object);
	}

	/**
	 * 获取作业列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "作业列表")
	public GeneralResult<GridBean<Task>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskService.list(pager, context);
	}

	/**
	 * 添加作业
	 *
	 * @param task
	 * @param context
	 * @return 添加结果
	 */
	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "作业创建")
	public GeneralResult<Task> create(@RequestBody Task task, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskService.create(task, context);
	}

	@PostMapping("/onekey/release")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业创建发布")
	public GeneralResult<Task> onekeyRelease(@RequestBody JSONObject task,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskService.onekeyRelease(task, context);
	}

	/**
	 * 修改定时作业
	 *
	 * @param bean
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业修改")
	public GeneralResult<Task> modify(@RequestBody TaskBean bean, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "作业不存在");
		}

		return taskService.modify(bean, context);
	}

	/**
	 * 移除作业
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskService.remove(id, context.getTarget());
	}

	/**
	 * 查看作业详细信息
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")

	@Operation(tags = { "CMC", "CSC" }, summary = "作业详情")
	public GeneralResult<Task> detail(@PathVariable(value = Common.ID) Long id) {
		return taskService.detail(id);
	}

	/**
	 * 导出作业
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}/export")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业导出")
	public void export(@PathVariable(value = Common.ID) Long id, HttpServletRequest request,
			HttpServletResponse response) {
		GeneralResult<?> result = taskService.detail(id);
		if (result.isFailed()) {
			return;
		}
		String content = JSONObject.toJSONString(result.getData());
		response.setCharacterEncoding("utf-8");
		response.setContentType("multipart/form-data");
		try {
			String fileName = "task_" + System.currentTimeMillis() + ".txt";
			final String userAgent = request.getHeader("user-agent").toLowerCase();
			if (userAgent.contains("trident") || userAgent.contains("edge")) {
				fileName = URLEncoder.encode(fileName, "UTF-8");
			} else {
				fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
			}
			response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
			OutputStream os = response.getOutputStream();
			byte[] bytes = content.getBytes();
			os.write(bytes);
			os.close();
			os.flush();
		} catch (Exception e) {
			log.error("Download fail:", e);
		}
	}

	/**
	 * 操作作业
	 *
	 * @param id
	 * @return
	 */
	@PatchMapping(value = "/{id}/clone")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业克隆")
	public GeneralResult<Task> clone(@PathVariable(value = Common.ID) Long id) {
		return taskService.clone(id);
	}

	@PatchMapping(value = "/{id}/transfer")
	@Operation(tags = { "CMC", "CSC" }, summary = "定时作业")
	public GeneralResult<Void> transfer(@PathVariable(value = Common.ID) Long id,
			@RequestBody(required = false) TaskTransferParams params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskService.transferCron(id, params, context);
	}

	@PatchMapping(value = "/{id}/active")
	@Operation(tags = { "CMC", "CSC" }, summary = "定时作业启动")
	public GeneralResult<Void> active(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "作业不存在");
		}
		if (task.getStatus().equals(Status.Inst.WAITTING.name())
				|| task.getStatus().equals(Status.Inst.RUNNING.name())) {
			return new GeneralResult<>(false, "当前定时作业已经激活或正在运行中！");
		}
		GeneralResult<Void> result = taskService.isValid(task.getCron());
		if (result.isFailed()) {
			return new GeneralResult<>(false, "作业执行时间为过去时间，请重新修改执行时间");
		}
		result = taskService.active(id, context.getTarget());
		if (result.isFailed()) {
			return result;
		}
		task.setMenderId(context.getTarget());
		Job job = buildJob(task, task.getUuid(), task.getName(), Catalog.Task.SCHEDULED.getValue(), task.getCode(),
				task.getIsLoop(), task.getRunTime());
		job.setCronExpression(task.getCron());
		boolean flag = jobScheduler.insertTask(job);
		if (flag) {
			return new GeneralResult<>(true, "启动定时作业成功!");
		}
		return new GeneralResult<>(false, "启动定时作业失败");
	}

	@PatchMapping(value = "/{id}/suspend")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业挂起")
	public GeneralResult<Void> suspend(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "作业不存在");
		}
		if (task.getStatus().equals(Status.Inst.RUNNING.name())) {
			return new GeneralResult<>(false, "执行中作业不可挂起");
		}
		if (!task.getStatus().equals(Status.Inst.WAITTING.name())) {
			return new GeneralResult<>(false, "作业已经处于挂起状态或者未执行");
		}
		boolean result = jobScheduler.removeCronTask(task.getUuid());
		if (result) {
			return taskService.suspend(id, context.getTarget());
		}
		return new GeneralResult<>(false, "作业挂起失败：定时作业可能正在启动的过程中，请稍后再试！");
	}

	@PatchMapping(value = "/{id}/start")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业执行")
	public GeneralResult<Long> start(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "执行失败：作业不存在");
		}
		GeneralResult<Void> result = scriptService.checkCommand(task, context.getTarget());
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		TaskInstance instance = taskService.initInstance(id, ExecuteMode.WEB.name(), Catalog.Task.STANDARD.getValue(),
				context);
		if (null == instance) {
			return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
		}
		instance.setContext(context);
		Job job = buildJob(instance, Catalog.Task.STANDARD.getValue());
		boolean flag = jobScheduler.insertTask(job);
		return flag ? new GeneralResult<>(true, instance.getId(), "作业启动成功") : new GeneralResult<>(false, "作业启动失败");
	}

	@PatchMapping(value = "/{id}/applyActive")
	@Operation(tags = { "CMC", "CSC" }, summary = "定时作业激活")
	public GeneralResult<Void> active(@PathVariable(value = Common.ID) Long id,
			@RequestBody StartModeParams actionParams, @Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "作业不存在");
		}
		if (task.getStatus().equals(Status.Inst.WAITTING.name())
				|| task.getStatus().equals(Status.Inst.RUNNING.name())) {
			return new GeneralResult<>(false, "定时作业已经激活");
		}
		GeneralResult<Void> result = taskService.isValid(task.getCron());
		if (result.isFailed()) {
			return new GeneralResult<>(false, "作业执行时间为过去时间，请重新修改执行时间");
		}
		result = scriptService.checkCommand(task, context.getTarget());
		if (result.isFailed()) {
			return result;
		}
		taskService.updateStatus(id, ApplyStatus.APPROVED.name(), context.getTarget());
		if (StartMode.AUTO.name().equalsIgnoreCase(actionParams.getStartMode())) {
			result = taskService.active(id, context.getTarget());
			if (result.isFailed()) {
				return result;
			}
			task.setMenderId(context.getTarget());
			Job job = buildJob(task);
			boolean flag = jobScheduler.insertTask(job);
			if (flag) {
				return new GeneralResult<>(true, "激活定时作业成功!");
			}
			return new GeneralResult<>(false, "激活定时作业失败");
		}
		return new GeneralResult<>(true, "申请激活定时作业成功");
	}

	@PatchMapping(value = "/{id}/release")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业发布")
	public GeneralResult<Void> release(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "发布失败：作业不存在");
		}
		taskService.updateStatus(task.getId(), ApplyStatus.APPROVED.name(), context.getTarget());
		return new GeneralResult<>(true, "申请发布作业成功");
	}

	@PatchMapping(value = "/{id}/recall")
	@Operation(tags = { "CMC", "CSC" }, summary = "作业撤回")
	public GeneralResult<Void> recall(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		Task task = taskService.basic(id);
		if (null == task) {
			return new GeneralResult<>(false, "撤回失败：作业不存在");
		}
		GeneralResult<Void> result = taskService.updateStatus(id, ApplyStatus.DEVELOPING.name(), context.getTarget());
		if (result.isFailed()) {
			return new GeneralResult<>(false, "作业撤回失败");
		}
		return new GeneralResult<>(true, "作业撤回成功");
	}

	private Job buildJob(Object task, String taskId, String taskName, String type, String code, Boolean isLoop,
			Date startTime) {
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(task));
		job.setParam(TaskParam.TASKTYPE.name(), type);
		job.setTaskId(taskId);
		job.setParam(Common.NAME, taskName);
		job.setParam(TaskParam.TASKCODE.name(), code);
		job.setLoop(isLoop);
		job.setStartTime(startTime);
		return job;
	}

	private Job buildJob(TaskInstance instance, String type) {
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
		job.setParam(TaskParam.TASKTYPE.name(), type);
		job.setTaskId(instance.getUuid());
		job.setParam(Common.NAME, instance.getName());
		return job;
	}

	private Job buildJob(Task task) {
		Job job = new Job();
		job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(task));
		job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.SCHEDULED.getValue());
		job.setTaskId(task.getUuid());
		job.setParam(Common.NAME, task.getName());
		job.setCronExpression(task.getCron());
		return job;
	}

}
