package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.TaskPlatformAlert;
import com.bocloud.cop.intf.TaskPlatformAlertService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 任务实例平台告警控制器
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@RestController
@RequestMapping("/v1/tasks/platform/alert")
@Tag(name = "任务平台告警管理")
public class TaskIPlatformAlertController {

	@Autowired
	private TaskPlatformAlertService taskPlatformAlertService;

	/**
	 * 获取任务平台告警列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台告警列表")
	public GeneralResult<GridBean<TaskPlatformAlert>> list(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformAlertService.list(pager, context);
	}

	/**
	 * 移除任务平台告警
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台告警删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformAlertService.remove(id, context.getTarget());
	}

	/**
	 * 任务平台告警处理
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台告警处理")
	public GeneralResult<Void> modify(@RequestBody TaskPlatformAlert params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setId(id);
		return taskPlatformAlertService.modify(params, context.getTarget());
	}

	/**
	 * @param id
	 * @param pager
	 * @param type
	 * @Description: 任务平台任务列表查询
	 * @Author majiali
	 * @CreateTime 2022/9/19 17:00
	 */
	@GetMapping(value = "/{id}/instances")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台任务查询")
	public GeneralResult<?> listInstance(@PathVariable(value = Common.ID) Long id, Pager pager,
			@RequestParam(value = "type") String type) {
		return taskPlatformAlertService.instanceList(id, type, pager);
	}
}
