package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.TaskInstanceBak;
import com.bocloud.cop.entity.TaskNodeBak;
import com.bocloud.cop.entity.TaskNodeLogBak;
import com.bocloud.cop.intf.TaskInstanceBakService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 备份作业实例控制器
 *
 * @author majiali
 */
@RestController
@RequestMapping("/v1/tasks/instances/bak")
@Tag(name = "备份作业实例管理")
public class TaskInstanceBakController {

	@Autowired
	private TaskInstanceBakService taskInstanceBakService;

	/**
	 * 获取备份作业记录列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "备份作业实例列表")
	public GeneralResult<GridBean<TaskInstanceBak>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceBakService.list(pager, context);
	}

	/**
	 * 查看备份作业详情
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{name}/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "备份作业实例详情")
	public GeneralResult<TaskInstanceBak> detail(@PathVariable(value = Common.NAME) String name,
			@PathVariable(value = Common.ID) Long id) {
		return taskInstanceBakService.detail(id, name);
	}

	/**
	 * 查看备份作业详情·
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/nodes/{name}/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "备份作业实例节点详细")
	public GeneralResult<TaskNodeBak> nodeDetail(@PathVariable(value = Common.NAME) String name,
												 @PathVariable(value = Common.ID) Long id) {
		return taskInstanceBakService.detailNode(id, name);
	}

	@GetMapping(value = "/nodes/{name}/{id}/logs")
	@Operation(tags = { "CMC", "CSC" }, summary = "备份作业实例节点日志")
	public GeneralResult<TaskNodeLogBak> getNodeLog(@PathVariable(value = Common.ID) Long id,
													@PathVariable(value = Common.NAME) String name,
													@RequestParam(value = "priority", required = false) Integer priority,
													@RequestParam(value = "end", required = false) Boolean end) {
		return taskInstanceBakService.getLog(id, name, priority, end);
	}

}
