package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.TaskPlatformConfig;
import com.bocloud.cop.intf.TaskPlatformConfigService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

;

/**
 * 任务实例平台控制器
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@RestController
@RequestMapping("/v1/tasks/platform/config")
@Tag(name = "任务平台配置管理")
public class TaskPlatformConfigController {

	@Autowired
	private TaskPlatformConfigService taskPlatformConfigService;

	/**
	 * 获取任务平台列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台配置列表")
	public GeneralResult<GridBean<TaskPlatformConfig>> list(Pager pager,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformConfigService.list(pager, context);
	}

	/**
	 * 查看任务平台详情
	 *
	 * @param id
	 * @return 操作结果
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台配置详情")
	public GeneralResult<TaskPlatformConfig> detail(@PathVariable(value = Common.ID) Long id) {
		return taskPlatformConfigService.get(id);
	}

	/**
	 * 移除任务平台
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台配置删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformConfigService.remove(id, context.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台配置创建")
	public GeneralResult<Void> create(@RequestBody TaskPlatformConfig params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return taskPlatformConfigService.create(params, context.getTarget());
	}

	/**
	 * 任务平台修改
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台配置修改")
	public GeneralResult<Void> modify(@RequestBody TaskPlatformConfig params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setId(id);
		return taskPlatformConfigService.modify(params, context.getTarget());
	}

	@PutMapping(value = "/{id}/opened")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台告警是否开启")
	public GeneralResult<Void> updateOpened(@PathVariable(value = Common.ID) Long id,
			@RequestParam(value = "opened") Boolean opened, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformConfigService.updateOpened(id, opened, context.getTarget());
	}

}
