package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.TaskPlatform;
import com.bocloud.cop.intf.TaskPlatformService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 任务实例平台控制器
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@RestController
@RequestMapping("/v1/tasks/platform")
@Tag(name = "任务平台管理")
public class TaskPlatformController {

	@Autowired
	private TaskPlatformService taskPlatformService;

	/**
	 * 获取任务平台列表
	 *
	 * @param pager
	 * @param context
	 * @return 数据结果集
	 */
	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台列表")
	public GeneralResult<GridBean<TaskPlatform>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformService.list(pager, context);
	}

	/**
	 * 获取任务平台列表
	 *
	 * @return 数据结果集
	 */
	@GetMapping(value = "/simple")
	@Operation(tags = { "CMC", "CSC" }, summary = "全部任务平台")
	public GeneralResult<?> listAll() {
		return taskPlatformService.listAll();
	}

	/**
	 * 移除任务平台
	 *
	 * @param id
	 * @param context
	 * @return 操作结果
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskPlatformService.remove(id, context.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台创建")
	public GeneralResult<Void> create(@RequestBody TaskPlatform params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {

		return taskPlatformService.create(params, context.getTarget());
	}

	/**
	 * 任务平台修改
	 *
	 * @param params
	 * @param id
	 * @param context
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "任务平台修改")
	public GeneralResult<Void> modify(@RequestBody TaskPlatform params, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		params.setId(id);
		return taskPlatformService.modify(params, context.getTarget());
	}

}
