package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.TaskScene;
import com.bocloud.cop.intf.TaskSceneService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 作业场景控制器
 *
 * @author weiwei
 */
@RestController
@RequestMapping("/v1/tasks/scenes")
@Tag(name = "作业场景管理")
public class TaskSceneController {

    @Autowired
    private TaskSceneService taskSceneService;

    /**
     * 获取作业场景列表
     *
     * @param pager
     * @return 数据结果集
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "作业场景列表")
    public GeneralResult<GridBean<TaskScene>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return taskSceneService.list(pager, context);
    }

    /**
     * 获取作业场景列表
     *
     * @param pager
     * @return 数据结果集
     */
    @GetMapping(value = "/simple")
    @Operation(tags = {"CMC", "CSC"}, summary = "作业场景列表（简单）")
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        return taskSceneService.simple(pager);
    }

    /**
     * 添加作业场景
     *
     * @param params
     * @param context
     * @return 添加结果
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "作业场景创建")
    public GeneralResult<Void> create(@RequestBody TaskScene params,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setCreatorId(context.getTarget());
        params.setMenderId(context.getTarget());
        return taskSceneService.create(params);
    }

    /**
     * 修改作业场景
     *
     * @param params
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "作业场景修改")
    public GeneralResult<Void> modify(@RequestBody TaskScene params, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {

        params.setMenderId(context.getTarget());
        return taskSceneService.modify(params);

    }

    /**
     * 移除作业场景
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "作业场景删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return taskSceneService.remove(id, context.getTarget());
    }

    /**
     * 查看作业场景详细信息
     *
     * @param id
     * @return 操作结果
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "作业场景详情")
    public GeneralResult<TaskScene> detail(@PathVariable(value = Common.ID) Long id) {
        return taskSceneService.detail(id);
    }

}
