package com.bocloud.cop.booter.controller;

import com.bocloud.cop.entity.TaskTreatment;
import com.bocloud.cop.intf.TaskTreatmentService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author huliwei
 * @date 2023/4/23
 */
@RestController
@RequestMapping("/v1/tasks/treatment")
@Tag(name = "任务处理方案管理")
public class TaskTreatmentController {

	@Autowired
	private TaskTreatmentService taskTreatmentService;

	@GetMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "方案列表")
	public GeneralResult<GridBean<TaskTreatment>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTreatmentService.list(pager, context.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC", "CSC" }, summary = "方案创建")
	public GeneralResult<TaskTreatment> create(@RequestBody TaskTreatment params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTreatmentService.create(params, context);
	}

	@PutMapping()
	@Operation(tags = { "CMC", "CSC" }, summary = "方案修改")
	public GeneralResult<TaskTreatment> modify(@RequestBody TaskTreatment params,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTreatmentService.modify(params, context);
	}

	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC", "CSC" }, summary = "方案删除")
	public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskTreatmentService.remove(id, context.getTarget());
	}
}
