package com.bocloud.cop.booter.controller.compare;

import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.compare.StandardDocument;
import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparison;
import com.bocloud.cop.intf.compare.TaskInstanceDocumentComparisonItemService;
import com.bocloud.cop.intf.compare.TaskInstanceDocumentComparisonService;
import com.bocloud.cop.model.compare.ContrastFileModel;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 17:36:49
 * @Description:
 */
@RestController
@RequestMapping("/v1/tasks/instances/comparisons")
@Tag(name = "文件对比记录管理")
public class TaskInstanceDocumentComparisonController {

	@Autowired
	private TaskInstanceDocumentComparisonService taskInstanceDocumentComparisonService;
	@Autowired
	private TaskInstanceDocumentComparisonItemService taskInstanceDocumentComparisonItemService;

	/**
	 * 列表查询
	 *
	 * @param pager
	 * @return
	 */
	@Operation(tags = { "CMC" }, summary = "对比记录列表")
	@GetMapping
	public GeneralResult<GridBean<TaskInstanceDocumentComparison>> list(Pager pager) {
		return taskInstanceDocumentComparisonService.list(pager);
	}

	/**
	 * 文件对比差异明细
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}/{itemId}/contrast")
	@Operation(tags = { "CMC", "CSC" }, summary = "对比记录差异明细")
	public GeneralResult<List<ContrastFileModel>> contrast(@PathVariable(value = Common.ID) Long id,
			@PathVariable("itemId") Long itemId) {
		return taskInstanceDocumentComparisonService.contrast(id, itemId);
	}

	/**
	 * 对比实例主机信息列表
	 *
	 * @param id
	 * @param context 系统信息
	 * @return
	 */
	@GetMapping("{id}/hosts")
	@Operation(tags = { "CMC", "CSC" }, summary = "对比记录主机信息列表")
	public GeneralResult<List<TaskHost>> instanceHostList(@PathVariable(value = "id") Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceDocumentComparisonService.instanceHostList(id, context);
	}

	/**
	 * 对比路径
	 *
	 * @param id
	 * @param context 系统信息
	 * @return
	 */
	@GetMapping("{id}/file")
	@Operation(tags = { "CMC", "CSC" }, summary = "对比记录路径列表")
	public GeneralResult<List<StandardDocument>> instanceStandardFiles(@PathVariable(value = "id") Long id,
			@Value(Common.REQ_CONTEXT) RequestContext context) {
		return taskInstanceDocumentComparisonService.instanceStandardFiles(id, context);
	}

	@GetMapping("/item")
	@Operation(tags = { "CMC" }, summary = "对比记录结果列表")
	public GeneralResult<GridBean<TaskInstanceDocumentComparison>> instanceItems(Pager pager) {
		return taskInstanceDocumentComparisonItemService.instanceItems(pager);
	}

}