package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.intf.inspection.InspectItemService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author 焦洪涛
 */
@RestController
@RequestMapping("/v1/inspect/items")
@Tag(name = "巡检指标管理")
public class InspectItemController {

	@Autowired
	private InspectItemService inspectItemService;

	@GetMapping(value = "/params")
	@Operation(tags = { "CMC" }, summary = "巡检指标参数")
	public GeneralResult<List<String>> loadSameParams(@RequestParam(value = Common.PARAMS) String params) {
		return inspectItemService.loadSameParams(params);
	}

	@PostMapping()
	@Operation(tags = { "CMC" }, summary = "巡检指标创建")
	public GeneralResult<Void> create(@RequestBody InspectItem inspectItem,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectItemService.save(inspectItem, user.getTarget());
	}

	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检指标删除")
	public GeneralResult<Void> delete(@PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectItemService.delete(id, user.getTarget());
	}

	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检指标修改")
	public GeneralResult<Void> update(@RequestBody InspectItem inspectItem, @PathVariable(value = Common.ID) Long id,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectItemService.update(inspectItem, id, user.getTarget());
	}

	@GetMapping()
	@Operation(tags = { "CMC" }, summary = "巡检指标列表")
	public GeneralResult<GridBean<InspectItem>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectItemService.list(pager, user.getTarget());
	}

	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "巡检指标详情")
	public GeneralResult<InspectItem> get(@PathVariable(value = Common.ID) Long id) {
		return inspectItemService.get(id);
	}
}
