package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectTemplate;
import com.bocloud.cop.intf.inspection.InspectTemplateService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 巡检或者合规模板控制器
 */
@RestController
@RequestMapping("/v1/inspect/templates")
@Tag(name = "巡检或合规模板管理")
public class InspectTemplateController {

	@Autowired
	private InspectTemplateService inspectTemplateService;

	/**
	 * 新增模板
	 *
	 * @param param
	 * @param user
	 * @return
	 */
	@PostMapping
	@Operation(tags = { "CMC" }, summary = "模板创建")
	public GeneralResult<Void> create(@RequestBody InspectTemplateService.CreateParam param,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTemplateService.create(param, user.getTarget());
	}

	/**
	 * 更新模板
	 *
	 * @param id
	 * @param param
	 * @return
	 */
	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "模板修改")
	public GeneralResult<Void> modify(@PathVariable Long id, @RequestBody InspectTemplateService.UpdateParam param,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		param.setId(id);
		return inspectTemplateService.update(param, user.getTarget());
	}

	/**
	 * 删除模板
	 *
	 * @param id
	 * @param user
	 * @return
	 */
	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "模板删除")
	public GeneralResult<Void> delete(@PathVariable Long id, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTemplateService.delete(id, user.getTarget());
	}

	/**
	 * 查询模板
	 *
	 * @param pager
	 */
	@GetMapping
	@Operation(tags = { "CMC" }, summary = "模板列表")
	public GeneralResult<GridBean<InspectTemplate>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTemplateService.list(pager, user.getTarget());
	}

	/**
	 * 通过模板id查询巡检项
	 *
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "模板详情")
	public GeneralResult<List<InspectItem>> listItems(@PathVariable(value = Common.ID) Long id) {
		return inspectTemplateService.listItems(id);
	}
}
