package com.bocloud.cop.booter.controller.inspection;

import com.bocloud.cop.entity.inspection.InspectTiming;
import com.bocloud.cop.intf.inspection.InspectTimingService;
import com.bocloud.cop.model.OperationParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.web.bind.annotation.*;

;

@RestController
@RequestMapping("/v1/inspect/timings")
@Tag(name = "定时巡检管理")
public class InspectTimingController implements ApplicationListener<ApplicationReadyEvent> {

	@Autowired
	private InspectTimingService inspectTimingService;

	@GetMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "定时巡检详情")
	public GeneralResult<InspectTiming> get(@PathVariable(value = Common.ID) Long id) {
		return inspectTimingService.get(id);
	}

	@GetMapping
	@Operation(tags = { "CMC" }, summary = "定时巡检列表")
	public GeneralResult<GridBean<InspectTiming>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTimingService.list(pager, user.getTarget());
	}

	@PostMapping
	@Operation(tags = { "CMC" }, summary = "定时巡检创建")
	public GeneralResult<InspectTiming> create(@RequestBody InspectTiming timing,
			@Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTimingService.save(timing, user.getTarget());
	}

	@PutMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "定时巡检修改")
	public GeneralResult<Void> update(@PathVariable(value = Common.ID) Long id,
			@RequestBody InspectTiming timingInspect, @Value(Common.REQ_CONTEXT) RequestContext user) {
		timingInspect.setId(id);
		return inspectTimingService.update(timingInspect, user.getTarget());
	}

	@DeleteMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "定时巡检删除")
	public GeneralResult<Void> delete(@PathVariable("id") Long id, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTimingService.delete(id, user.getTarget());
	}

	@PatchMapping(value = "/{id}")
	@Operation(tags = { "CMC" }, summary = "定时巡检操作")
	public GeneralResult<Void> operate(@PathVariable("id") Long id,
			@RequestParam(value = Common.ACTION, required = false) String action,
			@RequestBody(required = false) OperationParam param, @Value(Common.REQ_CONTEXT) RequestContext user) {
		return inspectTimingService.operate(id, action, param, user);
	}

	@PostMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "定时巡检添加任务(主服务有效)")
	public GeneralResult<String> addScheduleIfLeader(@PathVariable(Common.ID) Long id) {
		return inspectTimingService.addScheduleIfLeader(id);
	}

	@DeleteMapping(value = "/{id}/schedule")
	@Operation(tags = { "CMC" }, summary = "定时巡检删除任务(主服务有效)")
	public GeneralResult<String> removeScheduleIfLeader(@PathVariable(Common.ID) Long id) {
		return inspectTimingService.removeScheduleIfLeader(id);
	}

	@Override
	public void onApplicationEvent(ApplicationReadyEvent event) {
		inspectTimingService.onApplicationReadyEvent();
	}
}
