package com.bocloud.cop.booter.controller.network;

import com.bocloud.cop.entity.NetworkGroup;
import com.bocloud.cop.intf.network.NetworkGroupService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/network/groups")
@Tag(name = "网段分组管理")
public class NetworkGroupController {

    @Autowired
    NetworkGroupService networkGroupService;

    /**
     * 获取网段分组列表
     *
     * @param parentId
     * @return 数据结果集
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "分组列表")
    public GeneralResult<List<NetworkGroup>> list(@RequestParam(required = false) Long parentId) {
        return networkGroupService.list();
    }

    /**
     * 添加网段分组
     *
     * @param group
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "网段创建")
    public GeneralResult<Void> create(@RequestBody NetworkGroup group,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        group.setCreatorId(context.getTarget());
        group.setMenderId(context.getTarget());
        return networkGroupService.create(group);
    }

    /**
     * 查看网段分组详细信息
     *
     * @param id
     * @return 操作结果
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "分组详情")
    public GeneralResult<NetworkGroup> detail(@PathVariable(value = Common.ID) Long id) {
        return networkGroupService.detail(id);
    }

    /**
     * 删除网段分组
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "网段删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return networkGroupService.remove(id, context.getTarget());
    }

    /**
     * 修改网段分组
     *
     * @param params
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "网段修改")
    public GeneralResult<Void> modify(@RequestBody NetworkGroup params, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setMenderId(context.getTarget());
        params.setId(id);
        return networkGroupService.modify(params);
    }
}
