package com.bocloud.cop.booter.listener;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.bocloud.cop.intf.inspection.DataAdaptorService;
import com.bocloud.cop.intf.inspection.InspectRecordService;
import com.bocloud.cop.repository.inspection.InspectRecordItemRepository;

import lombok.extern.slf4j.Slf4j;

/**
 * 巡检任务结果返回监听器 队列： task.job.inspection.result
 */
@Component
@Slf4j
public class InspectListener {

    @Autowired
    private InspectRecordItemRepository inspectRecordItemRepository;
    @Autowired
    private InspectRecordService recordService;
    @Autowired
    @Qualifier("inspect")
    private DataAdaptorService dataAdaptorService;

    @RabbitListener(queues = "task.job.inspection.result", admin = "rabbitAdmin", autoStartup = "true")
    public void taskJobPatchResult(Message message) {
        String body = new String(message.getBody(), StandardCharsets.UTF_8);
        log.info("task.job.inspection.result:::::" + body);
        TaskInstance instance = JSON.parseObject(body).getObject("instance", TaskInstance.class);
        List<InspectRecordItem> items = inspectRecordItemRepository.findAllByJobInstanceId(instance.getId());
        for (InspectRecordItem item : items) {
            recordService.synInspectRecord(item, dataAdaptorService);
        }
        Set<Long> inspectRecordIds = items.stream().map(InspectRecordItem::getRecordId).collect(Collectors.toSet());
        for (Long inspectRecordId : inspectRecordIds) {
            dataAdaptorService.synRecord(inspectRecordId);
        }
    }
}
