package com.bocloud.cop.booter.listener;

import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.patch.PatchDeployHistory;
import com.bocloud.cop.entity.patch.PatchDeployHistoryDetail;
import com.bocloud.cop.repository.patch.PatchDeployHistoryDetailRepository;
import com.bocloud.cop.repository.patch.PatchDeployHistoryRepository;
import com.bocloud.cop.service.internal.CopInternalService;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

/**
 * 补丁部署任务结果返回监听器 队列： task.job.inspection.result
 */
@Slf4j
@Component
public class PatchDeployListener {

    @Autowired
    private RabbitAdmin rabbitAdmin;
    // @Autowired
    // private PatchDeployRepository patchDeployRepository;
    @Autowired
    private PatchDeployHistoryRepository patchDeployHistoryRepository;
    @Autowired
    private PatchDeployHistoryDetailRepository patchDeployHistoryDetailRepository;
    @Autowired
    private CopInternalService copInternalService;

    @RabbitListener(queues = "task.job.patch.result", admin = "rabbitAdmin", autoStartup = "true")
    public void taskJobPatchResult(Message message) {
        String body = new String(message.getBody(), StandardCharsets.UTF_8);
        log.info("补丁部署结果消息内容：" + body);
        TaskInstance instance = JSON.parseObject(body).getObject("instance", TaskInstance.class);
        Map<String, Boolean> results = new HashMap<>();
        Map<String, String> messages = new HashMap<>();
        for (TaskNode node : instance.getNodes()) {
            // 过滤开始和结束节点
            if ("CTRL".equalsIgnoreCase(node.getCatalog())) {
                continue;
            }
            JSONObject output = JSON.parseObject(node.getOutput());
            if (null == output) {
                continue;
            }
            JSONArray datas = output.getJSONArray("data");
            if (CollectionUtils.isNotEmpty(datas)) {
                for (int i = 0; i < datas.size(); i++) {
                    JSONObject data = datas.getJSONObject(i);
                    String host = data.getString("host");
                    Boolean success = data.getBooleanValue("success");
                    String msg = data.getString("message");
                    if (!results.containsKey(host)) {
                        results.put(host, success);
                        messages.put(host, msg);
                        continue;
                    }
                    if (null != results.get(host) && results.get(host)) {
                        results.put(host, success);
                        messages.put(host, msg);
                    }
                }
            } else {
                JSONObject input = JSON.parseObject(node.getInput());
                JSONObject target = input.getJSONObject("target");
                JSONArray hosts = target.getJSONArray("data");
                if (!ListTool.isEmpty(hosts)) {
                    for (int i = 0; i < hosts.size(); i++) {
                        JSONObject host = hosts.getJSONObject(i);
                        results.put(host.getString("host"), false);
                        messages.put(host.getString("host"), output.getString("message"));
                    }
                }
            }
        }
        long allCnt = results.size();
        long successCnt = results.values().stream().filter(Boolean.TRUE::equals).count();
        long failedCnt = allCnt - successCnt;
        List<PatchDeployHistory> deployPatchHistories =
            patchDeployHistoryRepository.listByTaskInstanceId(instance.getId(), "RUNNING");
        for (PatchDeployHistory deployPatchHistory : deployPatchHistories) {
            deployPatchHistory.setStatus(instance.getStatus());
            deployPatchHistory.setFailSuccessTotal(String.format("%d / %d / %d", failedCnt, successCnt, allCnt));
            deployPatchHistory.setCompleteTime(new Date());
            patchDeployHistoryRepository.update(deployPatchHistory);
            List<PatchDeployHistoryDetail> deployPatchDetails =
                patchDeployHistoryDetailRepository.listByHistoryId(deployPatchHistory.getId());
            for (PatchDeployHistoryDetail detail : deployPatchDetails) {
                if (results.containsKey(detail.getHostIp()) && results.get(detail.getHostIp())) {
                    detail.setStatus("success");
                } else {
                    detail.setStatus("fail");
                }
                detail.setErrorInfo(messages.get(detail.getHostIp()));
                patchDeployHistoryDetailRepository.update(detail);
            }
        }
    }

    @RabbitListener(queues = "task.job.patch.delay", admin = "rabbitAdmin", autoStartup = "true")
    public void taskJobPatchDelay(Message message) {
        String body = new String(message.getBody(), StandardCharsets.UTF_8);
        JSONObject bodyObject = JSON.parseObject(body);
        Long id = bodyObject.getLong("id");
        String queue = bodyObject.getString("queue");
        rabbitAdmin.deleteQueue(queue);
        copInternalService.patchDeployPatchs(id);
    }
}
