package com.bocloud.cop.booter.scheduler;

import com.bocloud.cop.enums.TaskHostType;
import com.bocloud.cop.intf.TaskHostService;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class TaskHostScheduler {

    @Autowired
    private TaskHostService taskHostService;

    @Autowired
    private CurrentService currentService;

    @Scheduled(cron = "${taskhost.sync.corn:0 0 5 * * ?}")
    private void refreshCache() {
        if (!currentService.isLeader()) {
            log.info("当前服务非主服务，忽略主机资源同步任务");
            return ;
        }
        taskHostService.syncHostFromIMS(null, TaskHostType.Category.ALL.name());
        taskHostService.syncSegment();
    }

    /**
     * 同步Salt Minions相关信息
     */
    @Scheduled(cron = "${taskhost.salt.minion.sync.corn:0 0 6 * * ?}")
    private void syncSaltMinions() {
        if (!currentService.isLeader()) {
            log.info("当前服务非主服务，忽略Salt Minions同步任务");
            return ;
        }
        taskHostService.syncSaltMinions(null, null);
    }
}
