package com.bocloud.cop.entity;

import java.util.Date;

import org.springframework.format.annotation.DateTimeFormat;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

/**
 * 任务类
 * 
 * @author weiwei
 * @version 1.0
 * @since 2016.08.15
 * 
 */
@Data
@EqualsAndHashCode(callSuper = true)
@AllArgsConstructor
@NoArgsConstructor
@Table("task")
public class Task extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("uuid")
    private String uuid; // 任务ID
    @Column("code")
    private String code; // 任务编码
    @Column("graph")
    private Object graph;
    @Column("cron")
    private String cron; // cron表达式
    @Column("has_suspend")
    private Boolean hasSuspend;// 是否有暂停节点
    @Column("scene_id")
    private Long sceneId; // 场景ID
    @Column("template_id")
    private Long templateId; // 模板ID
    @Column("gmt_execute")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtExecute;
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("run_time")
    private Date runTime;
    @Column("is_loop")
    private Boolean isLoop;
    @Column("target")
    private String target; // 目标池 authenMode认证方式 sudo是否sudo executerType执行方式
    @Column("tenant_id")
    private Long tenantId; // 租户ID
    @Column("catalog")
    private Catalog catalog;// 账号类别
    @IgnoreAll
    private String creator;
    @IgnoreAll
    private String mender;
    @IgnoreAll
    private String templateName;
    @IgnoreAll
    private String sceneName;
    @IgnoreAll
    private String tenantName;
    @Column("email_enabled")
    private Boolean emailEnabled;

    // 所属平台
    @IgnoreAll
    private String platform;
    @IgnoreAll
    private Integer priority;
    @IgnoreAll
    private boolean untreate;
    @IgnoreAll
    private String type;
    /**
     * 执行方式
     */
    @IgnoreAll
    private String executeType;;

    public void preSave(Long creatorId, Long menderId, Long tenantId, Catalog catalog) {
        this.setCreatorId(creatorId);
        this.setMenderId(menderId);
        this.setTenantId(tenantId);
        this.setCatalog(catalog);
    }
}
