package com.bocloud.cop.entity;

import java.util.Date;
import java.util.List;

import org.springframework.format.annotation.DateTimeFormat;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

/**
 * 任务实例类
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("task_instance")
public class TaskInstance extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("uuid")
    private String uuid; // uuid
    @Column("type")
    private String type; // 任务类型：temporary-临时，standard-标准，inspection-巡检，compliance-合规，patch-补丁，compare-配置对比，backup-备份 recover-恢复
    @Column("code")
    private String code; // 任务种类：standard-标准，scheduled-定时
    @Column("graph")
    private String graph;
    @Column("cron")
    private String cron; // cron表达式
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtStart; // 开始时间
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish; // 结束时间
    @Column("task_id")
    private Long taskId; // 任务ID
    @Column("execute_mode")
    private String executeMode; // 启动方式
    @Column("execute_host")
    private String executeHost;// 执行主机
    @Column("tenant_id")
    private Long tenantId; // 租户ID
    @Column("catalog")
    private Catalog catalog;// 账号类别
    @Column("rollbacks")
    private Boolean rollbacks;// 超时回滚
    @Column("timeout")
    private Long timeout;// 超时时间
    @Column("is_backup")
    private Boolean isBackup; // 是否纯备份任务
    @IgnoreAll
    private Double cost; // 总耗时
    @IgnoreAll
    private String operator; // 启动人
    @IgnoreAll
    private List<JSONObject> results;
    @IgnoreAll
    private List<TaskNode> nodes;// 资源堆栈下的所有堆栈节点
    @IgnoreAll
    private String templateName;// 模板名称
    @IgnoreAll
    private String vendorName;// 平台名称
    @IgnoreAll
    private RequestContext context;
    @IgnoreAll
    private String tenantName;// 租户名称
    @IgnoreAll
    private Long count;
    @IgnoreAll
    private String extension;
    @Column("email_enabled")
    private Boolean emailEnabled;
    @IgnoreAll
    private List<TaskInstanceHost> taskInstanceHosts;

    @Column("priority")
    private Integer priority;

    /**
     * 所属平台
     */
    @Column("platform")
    private String platform;

    /**
     * 是否限制执行
     */
    @Column("is_limited")
    private Boolean isLimited;
    /**
     * 限制释放时间
     */
    @Column("limited_release_time")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date limitedReleaseTime;

    @Column("platform_alert_id")
    private Long platformAlertId;

    /**
     * 目标任务数
     */
    @IgnoreAll
    private Integer targetHostNum;

    /**
     * 备份场景名称
     */
    @IgnoreAll
    private String sceneName;

    @IgnoreAll
    private String tableName;

    @IgnoreAll
    private String executeType;

    public TaskInstance(Long creatorId, Long ownerId, Long menderId, String name, String status, String remark,
        String code, String graph, String cron, Long tenantId, Catalog catalog) {
        this.setName(name);
        this.setStatus(status);
        this.setRemark(remark);
        this.code = code;
        this.graph = graph;
        this.cron = cron;
        this.setCreatorId(creatorId);
        this.setOwnerId(ownerId);
        this.setMenderId(menderId);
        this.tenantId = tenantId;
        this.catalog = catalog;
    }

}
