package com.bocloud.cop.entity;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.annotation.ColumnType;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.IgnoreUpdate;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * @author huliwei
 * @date 2023/4/21
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Table("task_node_bak")
public class TaskNodeBak extends Generic {
    @PK(value = PKStrategy.SELF)
    private Long id; // ID
    @Column(value = "uuid",length = 64)
    private String uuid; // 任务ID
    @Column(value = "code",length = 64)
    private String code; // 节点编码
    @Column(value = "category",length = 32)
    private String category; // 节点类型
    @Column(value = "catalog",length = 32)
    private String catalog; // 节点分类
    @ColumnType(value = "text")
    @Column(value = "input",encryptable = true)
    private String input; // 参数
    @ColumnType(value = "longtext")
    @Column("output")
    private String output; // 输出
    @ColumnType(value = "text")
    @Column("content")
    private String content; // 默认参数
    @ColumnType(value = "text")
    @Column("rollbacks")
    private String rollbacks;// 回滚
    @ColumnType(value = "text")
    @Column("config")
    private String config; // 配置信息
    @ColumnType(value = "text")
    @Column("control")
    private String control;// 控制信息
    @ColumnType(value = "double")
    @Column("cost")
    private Double cost;// 任务耗时
    @Column(value = "node_id",length = 64)
    private String nodeId; // 节点ID
    @Column(value = "instance_id")
    private Long instanceId; // 实例ID
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtStart; // 开始时间
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtFinish; // 结束时间
    @Column(value = "module_type",length = 255)
    private String moduleType; // 节点类型
    @IgnoreUpdate
    @Column(
            value = "gmt_create",
            nullable = false
    )
    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtCreate;
    @Column("gmt_modify")
    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtModify;
    @IgnoreUpdate
    @Column(value = "creator_id",nullable = false)
    private Long creatorId;
    @Column("owner_id")
    private Long ownerId;
    @Column("mender_id")
    private Long menderId;
    @Column(value = "name",length = 256)
    private String name;
    @Column(value = "status",length = 16)
    private String status;
    @Column(value = "is_deleted",length = 1,nullable = false)
    private Boolean deleted;
    @Column(value = "props",length = 1024)
    private String props;
    @Column(value = "remark",length = 512)
    private String remark;

    @IgnoreAll
    private List<Object> successes; // 成功的
    @IgnoreAll
    private List<Object> failures; // 失败的
    @IgnoreAll
    private JSONObject outputs;
    @IgnoreAll
    private RequestContext context;
    @IgnoreAll
    private Boolean dicision;
    @IgnoreAll
    private String params;
    @IgnoreAll
    private Long createrId;
    @IgnoreAll
    private String instanceName;
}
