package com.bocloud.cop.entity.inspection;

import com.alibaba.fastjson.JSONObject;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

// 巡检指标项
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("inspect_item")
public class InspectItem extends GenericEntity {
    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column
    private Long scriptId;// 脚本id
    @Column
    private String scriptContent;// 脚本内容
    @Column
    private String scriptType;// 脚本内容
    @Column
    private String description;// 描述
    @Column
    private String itemParam;// 参数
    @Column
    private Boolean sensitiveParam;// 敏感参数

    @IgnoreAll
    private Boolean paramChanged; // 参数是否被改变 用于编辑字段 前端反馈
    @IgnoreAll
    private String mender;
    @IgnoreAll
    private String creator;

    @Column
    private Integer timeout; // 超时时间
    @Column
    private String itemType; // 指标分类
    @Column
    private String scriptName;// 脚本名称
    @Column
    private Object targetRule;// 指标规则 json格式
    @Column
    private Object scriptParam;// 输入参数 json格式
    @Column
    private Boolean multiCompare; // 单值匹配还是多值匹配
    @Column
    private Object scriptGroupId; // 脚本分组信息
    @Column
    private String type;

    public enum Type {
        COMPLIANCE, INSPECTION
    }

    public enum Status {
        CREATED, APPROVAL, REJECT, PUSHED
    }

    public String getScriptGroupId() {
        if (null == scriptGroupId) {
            return null;
        }
        if (scriptGroupId instanceof String) {
            return scriptGroupId.toString();
        }
        return JSONObject.toJSONString(scriptGroupId);
    }

    public String getScriptParam() {
        if (null == scriptParam) {
            return null;
        }
        if (scriptParam instanceof String) {
            return scriptParam.toString();
        }
        return JSONObject.toJSONString(scriptParam);
    }

    public String getTargetRule() {
        if (null == targetRule) {
            return null;
        }
        if (targetRule instanceof String) {
            return targetRule.toString();
        }
        return JSONObject.toJSONString(targetRule);
    }

}
