package com.bocloud.cop.entity.inspection;

import com.alibaba.fastjson.JSONObject;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 巡检报表类
 *
 * @author jiaohongtao
 * @since 2020-10-28 14:19:05
 */
@EqualsAndHashCode(callSuper = true)
@Data
@Table("inspect_report")
public class InspectReport extends GenericEntity {
    @PK
    private Long id;
    /**
     * 巡检ID
     */
    @Column("inspect_id")
    private Long inspectId;
    /**
     * 报表周期
     */
    @Column("period")
    private String period;
    /**
     * 今日/次日 本周/下周 本月/次月 current/next
     */
    @Column("choose")
    private String choose;
    /**
     * 周/星期
     */
    @Column("week")
    private Integer week;
    /**
     * 月/号
     */
    @Column("day")
    private Integer day;
    /**
     * 生成时间 mm:ss，配合周期使用
     */
    @Column(value = "generate_time", nullable = false)
    private String generateTime;
    /**
     * 报表类型 模板/自定义 template/custom
     */
    @Column("type")
    private String type;
    /**
     * 报表模板
     */
    @Column("template")
    private String template;
    /**
     * 发送人信息[{"id":1,"email":"123@123.com"},...]
     */
    @Column("send_msg")
    private Object sendMsg;
    /**
     * 报表标题
     */
    @Column("title")
    private String title;
    /**
     * 基础信息,[{"name":"name/title","remark":"巡检名称/标题"},...]
     */
    @Column("base_msg")
    private Object baseMsg;
    /**
     * 列表信息, // [{"name":"ip","remark":"10.20.10.","rule":"","value":""},...]
     * [{"name":"hostIp","rule":"","value":""},...] hostIp, 1.LIKE包含，模糊匹配 2.ALL/“空”全部 3.NOTLIKE 不包含，4.EQ 等于 5.NEQ 不等于
     * item,ruleName,threshold, 指定 itemValue, 模糊匹配 LIKE，所有 ALL warning, 告警层级查询 一级查所有 LT,GT,EQ
     */
    @Column("list_msg")
    private Object listMsg;

    @Column("template_id")
    private String templateId;

    @IgnoreAll
    private String creator;
    @IgnoreAll
    private String mender;
    /**
     * 巡检分类
     */
    /*@IgnoreAll
    private String inspectType;*/
    /**
     * 巡检名称
     */
    @IgnoreAll
    private String inspectName;

    /**
     * 巡检名称
     */
    @IgnoreAll
    private String inspectType;

    @IgnoreAll()
    private String detailBaseMsg;

    public void setCreatorMender(Long userId) {
        this.setCreatorId(userId);
        this.setMenderId(userId);
    }

    public enum Status {
        /**
         * 启用
         */
        On,
        /**
         * 禁用
         */
        Off
    }

    /**
     * 报表周期
     */
    public enum Period {
        /**
         * 日
         */
        Day,
        /**
         * 周
         */
        Week,
        /**
         * 月
         */
        Month;
    }

    public enum Week {
        /**
         * 星期一
         */
        Monday("星期一"),
        /**
         * 星期二
         */
        Tuesday("星期二"),
        /**
         * 星期三
         */
        Wednesday("星期三"),
        /**
         * 星期四
         */
        Thursday("星期四"),
        /**
         * 星期五
         */
        Friday("星期五"),
        /**
         * 星期六
         */
        Saturday("星期六"),
        /**
         * 星期日
         */
        Sunday("星期日");

        private final String name;

        Week(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }
    }

    public enum ListMsg {
        ip("主机ip"), itemName("指标项"), rule("规则"), threshold("阈值"), inspectResult("指标值"), warning("警告");

        private final String name;

        ListMsg(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }
    }

    public String getListMsg(){
        if(null == listMsg ){
            return null;
        }
        if (listMsg instanceof String){
            return listMsg.toString();
        }
        return JSONObject.toJSONString(listMsg);
    }
    public String getBaseMsg(){
        if(null == baseMsg ){
            return null;
        }
        if (baseMsg instanceof String){
            return baseMsg.toString();
        }
        return JSONObject.toJSONString(baseMsg);
    }
    public String getSendMsg(){
        if(null == sendMsg ){
            return null;
        }
        if (sendMsg instanceof String){
            return sendMsg.toString();
        }
        return JSONObject.toJSONString(sendMsg);
    }
}
