package com.bocloud.cop.enums;

import org.apache.commons.lang3.StringUtils;

import com.megatron.common.model.GeneralResult;

/**
 * 分类枚举类，用来提供各种资源的分类枚举
 * 
 * @author dmw
 *
 */
public class Catalog {

    public static enum Task {
        /**
         * 定时任务
         */
        SCHEDULED("scheduled"),
        /**
         * 标准任务
         */
        STANDARD("standard"),
        /**
         * 临时任务
         */
        TEMPORARY("temporary"),
        /**
         * 任务模板
         */
        TEMPLATE("template");

        public static GeneralResult<Task> hasValue(String value) {
            if (StringUtils.isEmpty(value)) {
                return new GeneralResult<>(false, "参数为空！");
            }
            Task[] tasks = Task.values();
            for (Task task : tasks) {
                if (task.name().equalsIgnoreCase(value)) {
                    return new GeneralResult<Task>(true, task, "success");
                }
            }
            return new GeneralResult<>(false, "数值不存在！");
        }

        private String value;

        Task(String value) {
            this.value = value;
        }

        public String getValue() {
            return this.value;
        }
    }

    /**
     * 任务节点分类
     * 
     * @author dmw
     *
     */
    public static enum Node {
        /**
         * 任务节点
         */
        TASK("task"),
        /**
         * 控制节点
         */
        CTRL("ctrl");

        public static GeneralResult<Node> hasValue(String value) {
            if (StringUtils.isEmpty(value)) {
                return new GeneralResult<>(false, "参数为空！");
            }
            Node[] nodes = Node.values();
            for (Node node : nodes) {
                if (node.name().equalsIgnoreCase(value)) {
                    return new GeneralResult<Node>(true, node, "success");
                }
            }
            return new GeneralResult<>(false, "数值不存在！");
        }

        private String value;

        Node(String value) {
            this.value = value;
        }

        public String getValue() {
            return this.value;
        }
    }
}
