package com.bocloud.cop.repository;

import com.bocloud.cop.entity.NetworkSegment;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("networkSegmentRepository")
public class NetworkSegmentRepository extends JdbcGenericDao<NetworkSegment, Long> {

    public NetworkSegmentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from network_segment where is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<NetworkSegment> list(Pager pager) throws Exception {
        String sql =
            "select a.*,b.name group_name from network_segment a left join network_group b on a.group_id = b.id where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(NetworkSegment.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public List<NetworkSegment> list() throws Exception {
        String sql = "select * from network_segment where is_deleted = 0";
        return this.list(NetworkSegment.class, sql);
    }

    public List<NetworkSegment> list(Long groupId) throws Exception {
        String sql = "select * from network_segment where is_deleted = 0 and group_id = :groupId";
        return this.list(NetworkSegment.class, sql, MapTools.simpleMap("groupId", groupId));
    }

    public NetworkSegment query(Long id) {
        String sql =
            "select a.*,b.name group_name from network_segment a left join network_group b on a.group_id = b.id where a.is_deleted = 0 and a.id = :id";
        List<NetworkSegment> list = this.list(NetworkSegment.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql =
            "update network_segment set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String name) throws Exception {
        String sql = "select id from network_segment a where a.is_deleted = 0 and a.name = :name";
        List<NetworkSegment> list = this.list(NetworkSegment.class, sql, MapTools.simpleMap("name", name));
        return !list.isEmpty();
    }

    /**
     * 根据分组id获取到网卡相关信息
     * @param groupIds 分组id列表
     * @return 网卡列表
     */
    public List<NetworkSegment> listByGroupId(List<Long> groupIds)  {
        String sql = "select * from network_segment where is_deleted = 0 and group_id in (:groupIds) ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupIds",groupIds);
        return this.list(NetworkSegment.class, sql,paramMap);
    }
}
