package com.bocloud.cop.repository;

import com.bocloud.cop.entity.ResourcePermissionRelation;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 主机资源与权限的关系接口实现
 *
 * @author hongqingyu
 * @version 1.0
 * @since 2022.06.09
 */
@Repository("resourcePermissionRelationRepository")
public class ResourcePermissionRelationRepository extends BasicGenericDao<ResourcePermissionRelation, Long> {

    public ResourcePermissionRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据主机资源id删除
     * 
     * @param resourceId
     */
    public void removeByTaskHostId(Long resourceId, String resourceType) {
        String sql = "delete from resource_permission_relation "
            + " where resource_id = :resourceId and resource_type =:resourceType ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("resourceType", resourceType);
        this.execute(sql, params);
    }

    /**
     * 根据主机资源id获取权限信息
     * @param resourceId
     * @param permissionType
     * @param resourceType
     * @return
     */
    public List<Long> listPermissionIdsByTaskHostId(Long resourceId, String permissionType, String resourceType) {
        String sql = "select permission_id from resource_permission_relation where resource_id = :resourceId "
            + " and permission_type =:permissionType and resource_type =:resourceType ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("permissionType", permissionType);
        params.put("resourceType", resourceType);
        List<ResourcePermissionRelation> list = this.list(ResourcePermissionRelation.class, sql, params);
        return list.stream().map(ResourcePermissionRelation::getPermissionId).collect(Collectors.toList());
    }

    /**
     * 根据主机资源id获取权限信息
     * @param resourceId
     * @param permissionType
     * @param resourceType
     * @return
     */
    public List<ResourcePermissionRelation> listRelationByTaskHostId(Long resourceId, String permissionType,
        String resourceType) {
        String sql = "select * from resource_permission_relation where resource_id = :resourceId "
            + " and permission_type =:permissionType and resource_type =:resourceType ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("permissionType", permissionType);
        params.put("resourceType", resourceType);
        return this.list(ResourcePermissionRelation.class, sql, params);
    }
}
