package com.bocloud.cop.repository;

import com.bocloud.cop.entity.ScriptContent;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 脚本提交记录Dao实现类
 */
@Repository("scriptContentRepository")
public class ScriptContentRepository extends JdbcGenericDao<ScriptContent, Long> {

    public ScriptContentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long scriptId, Long userId) {
        String sql =
            "update script_content set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where script_id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", scriptId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from script_content a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from script_content a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<ScriptContent> contents = this.list(ScriptContent.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (ScriptContent content : contents) {
            beans.add(new SimpleBean(content.getId(), content.getName()));
        }
        return beans;
    }

    public List<ScriptContent> list(Pager pager) {
        String sql = "select a.* from script_content a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ScriptContent.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public ScriptContent query(Long id) {
        String sql = "select a.* from script_content a where a.id = :id and a.is_deleted = 0";
        List<ScriptContent> list = this.list(ScriptContent.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean removeByVersion(Long versionId, Long userId) {
        String sql =
            "update script_content set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where version_id = :versionId";
        Map<String, Object> params = MapTools.simpleMap("versionId", versionId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean removeByScript(Long scriptId, Long userId) {
        String sql =
            "update script_content set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where script_id = :scriptId";
        Map<String, Object> params = MapTools.simpleMap("scriptId", scriptId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

}
