package com.bocloud.cop.repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.cop.entity.ScriptGroup;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;

/**
 * 脚本分组DAO接口实现类
 */
@Repository("scriptGroupRepository")
public class ScriptGroupRepository extends JdbcGenericDao<ScriptGroup, Long> {

    public ScriptGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<ScriptGroup> list(Long parentId) {
        if (null == parentId) {
            parentId = 0L;
        }
        String sql = "select * from script_group a where a.is_deleted = 0 and a.parent_id = :parentId";
        return this.list(ScriptGroup.class, sql, MapTools.simpleMap("parentId", parentId));
    }

    public boolean delete(Long id, Long userId) {
        String sql =
            "update script_group set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public ScriptGroup query(Long id) {
        String sql = "select * from script_group where is_deleted = 0 and id = :id";
        List<ScriptGroup> list = this.list(ScriptGroup.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, Long parentId) {
        String sql = "select id from script_group where is_deleted = 0 and name = :name and parent_id = :parentId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("parentId", parentId);
        List<ScriptGroup> list = this.list(ScriptGroup.class, sql, params);
        return !list.isEmpty();
    }

}
