package com.bocloud.cop.repository;

import com.bocloud.cop.entity.ScriptVersion;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 脚本版本类Dao实现类
 * 
 * @author wangshuai
 * @since 2011.01.11
 */
@Repository("scriptVersionRepository")
public class ScriptVersionRepository extends JdbcGenericDao<ScriptVersion, Long> {

    public ScriptVersionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<ScriptVersion> list(Pager pager) {
        String sql = "select a.* from script_version a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ScriptVersion.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.version from script_version a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<ScriptVersion> versions = this.list(ScriptVersion.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (ScriptVersion version : versions) {
            beans.add(new SimpleBean(version.getId(), version.getVersion()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from script_version a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public ScriptVersion query(Long id) {
        String sql = "select * from script_version where id = :id and is_deleted = 0";
        List<ScriptVersion> list = this.list(ScriptVersion.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) {
        String sql =
            "update script_version set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<ScriptVersion> listByScript(Long scriptId) {
        String sql = "select * from script_version where is_deleted = 0 and script_id = :scriptId";
        return this.list(ScriptVersion.class, sql, MapTools.simpleMap("scriptId", scriptId));
    }

    public List<ScriptVersion> noUseList(Long scriptId) {
        String sql = "select a.* from script_version a,script b where a.id != b.version_id"
            + " and a.is_deleted = 0 and a.script_id = b.id and b.id = :scriptId";
        return this.list(ScriptVersion.class, sql, MapTools.simpleMap("scriptId", scriptId));
    }

    public boolean removeByScript(Long scriptId, Long userId) {
        String sql =
            "update script_version set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where script_id = :scriptId";
        Map<String, Object> params = MapTools.simpleMap("scriptId", scriptId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

}
