package com.bocloud.cop.repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import com.bocloud.cop.entity.SoftwareGroup;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;

/**
 * 脚本分组DAO接口实现类
 */
@Repository("softwareGroupRepository")
public class SoftwareGroupRepository extends JdbcGenericDao<SoftwareGroup, Long> {

    public SoftwareGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<SoftwareGroup> list(Long parentId) {
        if (null == parentId) {
            parentId = 0L;
        }
        String sql = "select * from software_group a where a.is_deleted = 0 and a.parent_id = :parentId";
        return this.list(SoftwareGroup.class, sql, MapTools.simpleMap("parentId", parentId));
    }

    public boolean delete(Long id, Long menderId) {
        String sql =
            "update software_group set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    public SoftwareGroup query(Long id) {
        String sql = "select * from software_group where is_deleted = 0 and id = :id";
        List<SoftwareGroup> list = this.list(SoftwareGroup.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name) {
        String sql = "select id from software_group where is_deleted = 0 and name = :name";
        List<SoftwareGroup> list = this.list(SoftwareGroup.class, sql, MapTools.simpleMap(Common.NAME, name));
        return !list.isEmpty();
    }

}
