package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskInstanceHost;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 * @date 2023年04月03日 17:50
 */
@Repository("taskInstanceHostRepository")
public class TaskInstanceHostRepository extends BasicGenericDao<TaskInstanceHost, Long> {
    public TaskInstanceHostRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean deleteByInstanceId(Long instanceId) throws Exception {
        String sql = "delete from task_instance_host where instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteByInstanceIds(List<Long> instanceIds) {
        String sql = "delete from task_instance_host where instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.execute(sql, params) > 0;
    }

    public boolean updateByInstanceIds(Long instanceId, String host, String executeHost, String executeWorker, Long nodeId) throws Exception {
        String sql = "update task_instance_host set execute_host = :executeHost, execute_worker = :executeWorker where instance_id = :instanceId and host = :host and node_id = :nodeId ";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("host", host);
        params.put("executeHost", executeHost);
        params.put("executeWorker", executeWorker);
        params.put("nodeId", nodeId);
        return this.execute(sql, params) > 0;
    }

    public List<TaskInstanceHost> listByInstances(List<Long> instanceIds) {
        String sql = "select * from task_instance_host a where a.instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.list(TaskInstanceHost.class, sql, params);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_instance_host a where 1=1 ";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<TaskInstanceHost> list(Pager pager) throws Exception {
        String sql = " select * from task_instance_host a where 1=1 ";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskInstanceHost.class, sql);
    }

    public List<TaskInstanceHost> listByInstance(Long instanceId) throws Exception {
        String sql = "select * from task_instance_host a where a.instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.list(TaskInstanceHost.class, sql, params);
    }

}
