package com.bocloud.cop.repository;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskNodeBak;
import com.bocloud.cop.utils.DynamicClassUtils;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author huliwei
 * @date 2023/4/21
 */
@Repository("taskNodeBakRepository")
public class TaskNodeBakRepository extends BasicGenericDao<TaskNodeBak, Long> {
    public TaskNodeBakRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<TaskNodeBak> listByInstance(Long instanceId, String taskNodeBakName) throws Exception {
        String sql = "select * from " + taskNodeBakName
                + " a where a.is_deleted = 0 and a.instance_id = :instanceId order by a.gmt_start asc";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.list(TaskNodeBak.class, sql, params);
    }

    public TaskNodeBak query(Long id, String taskNodeBakName) throws Exception {
        String sql = "select * from " + taskNodeBakName + " a where a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<TaskNodeBak> list = this.list(TaskNodeBak.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    public int countTable(String tableName,String databaseDriver) throws Exception {
        String sql;
        if("dm".equalsIgnoreCase(databaseDriver)){
            sql = "SELECT COUNT(*)  FROM all_tables WHERE OWNER = 'COP' AND TABLE_NAME = :tableName ";
        }else{
            sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND  TABLE_NAME = :tableName ";
        }
        Map<String, Object> params = MapTools.simpleMap("tableName", tableName);
        return this.countQuery(sql, params).intValue();
    }

    public void createTable(String tableName, String databaseDriver , TaskNodeBak taskNodeBak) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + tableName + " (\n");
        Map<String, JSONObject> fieldMap = DynamicClassUtils.getDeclaredFieldsInfo(taskNodeBak);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL ,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = DynamicClassUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if(null != fieldDBType){
                        sqlBuilder.append(" " + fieldDBType +   " NOT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        sqlBuilder.append(" " + "SMALLINT" + " NOT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                    }
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if(null != fieldDBType){
                        if("dm".equalsIgnoreCase(databaseDriver)){
                            if("longtext".equals(fieldDBType)){
                                sqlBuilder.append(" " + "LONG" + " DEFAULT NULL,\n");
                            }else{
                                sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                            }
                        }else{
                            sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                        }
                    }else{
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        sqlBuilder.append(" " + "SMALLINT" + " DEFAULT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                    }
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id) \n");
        if("dm".equalsIgnoreCase(databaseDriver)){
            sqlBuilder.append(") ;");
        }else{
            sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        }
        this.execute(sqlBuilder.toString(), null);
    }
}
