package com.bocloud.cop.repository;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskNodeLogBak;
import com.bocloud.cop.utils.DynamicClassUtils;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 任务节点日志DAO接口实现
 *
 * @author weiwei
 */
@Repository("taskNodeLogBakRepository")
public class TaskNodeLogBakRepository extends BasicGenericDao<TaskNodeLogBak, Long> {

    public TaskNodeLogBakRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public TaskNodeLogBak get(Long nodeId, Integer priority, String tableName) {
        String sql = "select * from " + tableName + " where node_id = :nodeId and priority = :priority";
        Map<String, Object> params = MapTools.simpleMap("nodeId", nodeId);
        params.put("priority", priority);
        List<TaskNodeLogBak> list = this.list(TaskNodeLogBak.class, sql, params);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public TaskNodeLogBak getLast(Long nodeId, String tableName) {
        String sql = "select * from  " + tableName + "  where node_id = :nodeId order by id desc";
        List<TaskNodeLogBak> list = this.list(TaskNodeLogBak.class, sql, MapTools.simpleMap("nodeId", nodeId));
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public int countTable(String tableName,String databaseDriver) throws Exception {
        String sql;
        if("dm".equalsIgnoreCase(databaseDriver)){
            sql = "SELECT COUNT(*)  FROM all_tables WHERE OWNER = 'COP' AND TABLE_NAME = :tableName ";
        }else{
            sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :tableName ";
        }
        Map<String, Object> params = MapTools.simpleMap("tableName", tableName);
        return this.countQuery(sql, params).intValue();
    }

    public void createTable(String tableName, String databaseDriver , TaskNodeLogBak taskNodeLogBak) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + tableName + " (\n");
        Map<String, JSONObject> fieldMap = DynamicClassUtils.getDeclaredFieldsInfo(taskNodeLogBak);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL ,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = DynamicClassUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if (null != fieldDBType) {
                        sqlBuilder.append(" " + fieldDBType + " NOT NULL,\n");
                    } else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        sqlBuilder.append(" " + "SMALLINT" + " NOT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                    }
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    String fieldDBType = fieldJSON.getString("fieldDBType");
                    if (null != fieldDBType) {
                        if("dm".equalsIgnoreCase(databaseDriver)){
                            if("longtext".equals(fieldDBType)){
                                sqlBuilder.append(" " + "LONG" + " DEFAULT NULL,\n");
                            }else{
                                sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                            }
                        }else{
                            sqlBuilder.append(" " + fieldDBType + " DEFAULT NULL,\n");
                        }
                    } else {
                        sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                    }
                } else if ("tinyint".equals(fieldType)) {
                    if("dm".equalsIgnoreCase(databaseDriver)){
                        sqlBuilder.append(" " + "SMALLINT" + " DEFAULT NULL,\n");
                    }else{
                        sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                    }
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id) \n");
        if("dm".equalsIgnoreCase(databaseDriver)){
            sqlBuilder.append(") ;");
        }else{
            sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        }
        this.execute(sqlBuilder.toString(), null);
    }
}
