package com.bocloud.cop.repository;

import com.bocloud.cop.entity.Task;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务DAO接口实现类
 * 
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
@Repository("taskRepository")
public class TaskRepository extends JdbcGenericDao<Task, Long> {

    public TaskRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Task> list(Pager pager) {
        String sql = "select a.*,t.name template_name,s.name scene_name from task a"
            + " left join task_template t on a.template_id = t.id left join task_scene s on a.scene_id = s.id where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(Task.class, sql);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name,a.status from task a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<Task> tasks = this.list(Task.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (Task task : tasks) {
            beans.add(new SimpleBean(task.getId(), task.getName(), task.getStatus()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from task a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean delete(Long id, Long userId) {
        String sql = "update task set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public Task query(Long id) {
        String sql = "select a.* from task a where a.is_deleted = 0 and a.id = :id";
        List<Task> list = this.list(Task.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, String code, Long tenantId) {
        String sql = "select id from task a where a.is_deleted = 0 and a.name = :name and a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        if (!StringUtils.isEmpty(code)) {
            sql += " and a.code = :code";
            params.put("code", code);
        }
        params.put("tenantId", tenantId);
        List<Task> list = this.list(Task.class, sql, params);
        return !list.isEmpty();
    }

    public boolean exists(String name, String code) throws Exception {
        String sql = "select * from task a where a.is_deleted = 0 and code != 'temporary' and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        if (!StringUtils.isEmpty(code)) {
            sql += " and a.code = :code";
            params.put("code", code);
        }
        List<Task> list = this.list(Task.class, sql, params);
        return !list.isEmpty();
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql = "update task set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<Task> listByCode(String code) {
        String sql = "select a.* from task a where a.is_deleted = 0 and a.code like '%" + code + "%'";
        return this.list(Task.class, sql);
    }

    public List<Task> listByScene(Long sceneId) {
        String sql = "select a.* from task a where a.is_deleted = 0 and a.scene_id = :sceneId";
        return this.list(Task.class, sql, MapTools.simpleMap("sceneId", sceneId));
    }

    public Task get(String uuid) {
        String sql = "select a.* from task a where a.is_deleted = 0 and a.uuid = :uuid";
        List<Task> list = this.list(Task.class, sql, MapTools.simpleMap("uuid", uuid));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean updateExecute(Long id, Date date) {
        String sql = "update task set gmt_execute = :gmtExecute where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtExecute", date);
        return this.execute(sql, params) > 0;
    }

    public List<Task> listSchedule() {
        String sql =
            "select a.* from task a where a.is_deleted = 0 and a.code = 'scheduled' and (a.status = 'RUNNING' or a.status = 'WAITTING')";
        return this.list(Task.class, sql);
    }

}
