package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskScene;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务场景DAO接口实现类
 * 
 * @author wei wei
 * @version 1.0
 * @since 2019.01.10
 */
@Repository("taskSceneRepository")
public class TaskSceneRepository extends JdbcGenericDao<TaskScene, Long> {

    public TaskSceneRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskScene> list(Pager pager) {
        String sql = "select a.* from task_scene a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskScene.class, sql);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name from task_scene a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<TaskScene> tasks = this.list(TaskScene.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (TaskScene task : tasks) {
            beans.add(new SimpleBean(task.getId(), task.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from task_scene a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update task_scene set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public TaskScene query(Long id) {
        String sql = "select a.* from task_scene a where a.is_deleted = 0 and a.id = :id";
        List<TaskScene> list = this.list(TaskScene.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name) {
        String sql = "select id from task_scene a where a.is_deleted = 0 and a.name = :name";
        List<TaskScene> list = this.list(TaskScene.class, sql, MapTools.simpleMap("name", name));
        return !list.isEmpty();
    }

}
