package com.bocloud.cop.repository;

import com.bocloud.cop.entity.TaskTemplate;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 任务模板DAO接口实现类
 * 
 * @author wei wei
 * @version 1.0
 * @since 2019.01.11
 */
@Repository("taskTemplateRepository")
public class TaskTemplateRepository extends JdbcGenericDao<TaskTemplate, Long> {

    public TaskTemplateRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskTemplate> list(Pager pager) {
        String sql =
            "select a.*,b.name scene_name from task_template a left join task_scene b on a.scene_id = b.id where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskTemplate.class, sql);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name from task_template a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<TaskTemplate> tasks = this.list(TaskTemplate.class, sql, this.getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (TaskTemplate task : tasks) {
            beans.add(new SimpleBean(task.getId(), task.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from task_template a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<TaskTemplate> listByTenant(Pager pager, Long tenantId) {
        String sql =
            "select a.*,b.name scene_name from task_template a left join task_scene b on a.scene_id = b.id,task_template_tenant c where a.is_deleted = 0 and a.id = c.template_id and c.tenant_id = "
                + tenantId;
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskTemplate.class, sql);
    }

    public int countByTenant(List<Param> params, Long tenantId) {
        String sql =
            "select count(1) from task_template a,task_template_tenant c where a.is_deleted = 0 and a.id = c.template_id and c.tenant_id = "
                + tenantId;
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update task_template set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public TaskTemplate query(Long id) {
        String sql = "select a.* from task_template a where a.is_deleted = 0 and a.id = :id";
        List<TaskTemplate> list = this.list(TaskTemplate.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, Long tenantId) {
        String sql =
            "select id from task_template a where a.is_deleted = 0 and a.name = :name and a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("tenantId", tenantId);
        List<TaskTemplate> list = this.list(TaskTemplate.class, sql, params);
        return !list.isEmpty();
    }

    public List<TaskTemplate> listByScene(Long sceneId) {
        String sql = "select a.* from task_template a where a.is_deleted = 0 and a.scene_id = :sceneId";
        return this.list(TaskTemplate.class, sql, MapTools.simpleMap("sceneId", sceneId));
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update task_template set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public Boolean hasSaltStack(List<Long> ids) {
        String sql =
            "select a.* from task_template a where a.is_deleted = 0 and execute_type = 'SaltStack' and a.id in (:ids) ";
        List<TaskTemplate> list = this.list(TaskTemplate.class, sql, MapTools.simpleMap("ids", ids));
        return list.isEmpty() ? false : true;
    }

}
