package com.bocloud.cop.repository.backup;

import com.bocloud.cop.entity.backup.TaskBackup;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author huliwei
 * @date 2022/10/19
 */
@Repository("taskBackupRepository")
public class TaskBackupRepository extends BasicGenericDao<TaskBackup, Long> {
    public TaskBackupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskBackup> listBySenceIds(List<Long> sceneIds) throws Exception {
        String sql = "select * from task_backup a where a.scene_id in(:sceneIds)";
        Map<String, Object> params = MapTools.simpleMap("sceneIds", sceneIds);
        return this.list(TaskBackup.class, sql, params);
    }

    public boolean exists(String name, String code) throws Exception {
        String sql = "select * from task_backup a where a.is_deleted = 0 and code != 'temporary' and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        if (!StringUtils.isEmpty(code)) {
            sql += " and a.code = :code";
            params.put("code", code);
        }
        List<TaskBackup> list = this.list(TaskBackup.class, sql, params);
        return !list.isEmpty();
    }
    public TaskBackup query(Long id) throws Exception {
        String sql = "select a.* from task_backup a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<TaskBackup> list = this.list(TaskBackup.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }
    public List<TaskBackup> queryList(Long id) throws Exception {
        String sql = "select a.* from task_backup a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = new HashMap<>();
        if(null != id){
            sql += " and a.id = :id ";
            paramMap = MapTools.simpleMap("id", id);
        }
        List<TaskBackup> list = this.list(TaskBackup.class, sql, paramMap);
        return list;
    }
    public List<TaskBackup> queryListByTaskId(Long taskId) throws Exception {
        String sql = "select a.* from task_backup a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = new HashMap<>();
        if(null != taskId){
            sql += " and a.task_id = :taskId ";
            paramMap = MapTools.simpleMap("taskId", taskId);
        }
        List<TaskBackup> list = this.list(TaskBackup.class, sql, paramMap);
        return list;
    }

    public List<TaskBackup> queryListByTaskId(List<Long> taskId) throws Exception {
        String sql = "select a.* from task_backup a where a.is_deleted = 0 and a.task_id in (:taskId)";
        Map<String, Object> paramMap = MapTools.simpleMap("taskId", taskId);
        List<TaskBackup> list = this.list(TaskBackup.class, sql, paramMap);
        return list;
    }
    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update task_backup set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_backup a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }
    public List<TaskBackup> list(Pager pager) throws Exception {
        String sql = "select a.*  from task_backup a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(TaskBackup.class, sql);
    }

    public boolean updateStatus(Long id, String status, Long userId) throws Exception {
        String sql = "update task_backup set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<TaskBackup> findAllByStatus (String status) {
        String sql = "select a.* from task_backup a where a.status =:status  and a.is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap("status", status);
        return this.list(TaskBackup.class, sql, param);
    }
}
