package com.bocloud.cop.repository.compare;

import com.bocloud.cop.entity.compare.StandardDocumentVersion;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 10:55:25
 * @Description:
 */
@Repository
public class StandardDocumentVersionRepository extends BasicGenericDao<StandardDocumentVersion, Long> {

    public StandardDocumentVersionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<StandardDocumentVersion> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from standard_document_version a where 1=1 and is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.list(StandardDocumentVersion.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from standard_document_version a where 1=1 and a.is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        List<StandardDocumentVersion> list = this.list(StandardDocumentVersion.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (StandardDocumentVersion standardDocumentVersion : list) {
            beans.add(new SimpleBean(standardDocumentVersion.getId(), standardDocumentVersion.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from standard_document_version a where 1=1 and a.is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public StandardDocumentVersion query(Long id) throws Exception {
        String sql = "select * from standard_document_version where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<StandardDocumentVersion> list = this.list(StandardDocumentVersion.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean existsByName(String name) throws Exception {
        String sql = "select * from standard_document_version where is_deleted = 0 and name = :name ";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<StandardDocumentVersion> list = this.list(StandardDocumentVersion.class, sql, params);
        return !list.isEmpty();
    }

    public boolean existsByCode(String code) throws Exception {
        String sql = "select * from standard_document_version where is_deleted = 0 and code = :code ";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<StandardDocumentVersion> list = this.list(StandardDocumentVersion.class, sql, params);
        return !list.isEmpty();
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update standard_document_version set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<StandardDocumentVersion> queryByIds(List<Long> ids) throws Exception {
        String sql = "select * from standard_document_version where id in (:id) and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, ids);
        List<StandardDocumentVersion> list = this.list(StandardDocumentVersion.class, sql, params);
        return list;
    }

}