package com.bocloud.cop.repository.compare;

import com.bocloud.cop.entity.compare.TaskDocumentComparison;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 16:04:48
 * @Description:
 */
@Repository
public class TaskDocumentComparisonRepository extends BasicGenericDao<TaskDocumentComparison, Long> {

    public TaskDocumentComparisonRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskDocumentComparison> list(int page, int rows, List<Param> params, Map<String, String> sorter)
        throws Exception {
        String sql = "select a.* from task_document_comparison a where 1=1 and is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.list(TaskDocumentComparison.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from task_document_comparison a where 1=1 and a.is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        List<TaskDocumentComparison> list = this.list(TaskDocumentComparison.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (TaskDocumentComparison documentComparison : list) {
            beans.add(new SimpleBean(documentComparison.getId(), documentComparison.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_document_comparison a where 1=1 and a.is_deleted=0 ";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public TaskDocumentComparison query(Long id) throws Exception {
        String sql = "select * from task_document_comparison where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<TaskDocumentComparison> list = this.list(TaskDocumentComparison.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql =
            "update task_document_comparison set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String name) throws Exception {
        String sql = "select * from task_document_comparison where is_deleted = 0 and name = :name ";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<TaskDocumentComparison> list = this.list(TaskDocumentComparison.class, sql, params);
        return !list.isEmpty();
    }

    public boolean updateStatus(Long id, String status, Long userId) throws Exception {
        String sql =
            "update task_document_comparison set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<TaskDocumentComparison> findAllByStatus(String status) {
        String sql = "select a.* from task_document_comparison a where a.status =:status and a.is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap("status", status);
        return this.list(TaskDocumentComparison.class, sql, param);
    }

    public List<TaskDocumentComparison> queryByIds(List<Long> ids) throws Exception {
        String sql = "select * from task_document_comparison where id in (:id) and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, ids);
        List<TaskDocumentComparison> list = this.list(TaskDocumentComparison.class, sql, params);
        return list;
    }

}