package com.bocloud.cop.repository.compare;

import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparisonItem;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-293 15:42:00
 * @Description:
 */
@Repository
public class TaskInstanceDocumentComparisonItemRepository extends BasicGenericDao<TaskInstanceDocumentComparisonItem, Long> {

    public TaskInstanceDocumentComparisonItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<TaskInstanceDocumentComparisonItem> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from task_instance_document_comparison_item a where 1=1 and a.is_deleted=0 " ;
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        return this.list(TaskInstanceDocumentComparisonItem.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from task_instance_document_comparison_item a where 1=1 and a.is_deleted=0 " ;
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = this.getQueryBuilder().getParam(params);
        List<TaskInstanceDocumentComparisonItem> list = this.list(TaskInstanceDocumentComparisonItem.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (TaskInstanceDocumentComparisonItem taskInstanceDocumentComparisonItem : list) {
            beans.add(new SimpleBean(taskInstanceDocumentComparisonItem.getId(), taskInstanceDocumentComparisonItem.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from task_instance_document_comparison_item a where 1=1 and a.is_deleted=0 " ;
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public TaskInstanceDocumentComparisonItem query(Long id) throws Exception {
        String sql = "select * from task_instance_document_comparison_item where id = :id and is_deleted = 0" ;
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<TaskInstanceDocumentComparisonItem> list = this.list(TaskInstanceDocumentComparisonItem.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<TaskInstanceDocumentComparisonItem> queryByIds(List<Long> ids) throws Exception {
        String sql = "select * from task_instance_document_comparison_item where id in (:ids) and is_deleted = 0" ;
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        return this.list(TaskInstanceDocumentComparisonItem.class, sql, params);
    }

    public List<TaskInstanceDocumentComparisonItem> queryByInstanceId(Long instanceId) throws Exception {
        String sql = "select * from task_instance_document_comparison_item where document_comparison_instance_id = :instanceId and is_deleted = 0" ;
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        return this.list(TaskInstanceDocumentComparisonItem.class, sql, params);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update task_instance_document_comparison_item set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id" ;
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String name) throws Exception {
        String sql = "select * from task_instance_document_comparison_item where is_deleted = 0 and name = :name " ;
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<TaskInstanceDocumentComparisonItem> list = this.list(TaskInstanceDocumentComparisonItem.class, sql, params);
        return !list.isEmpty();
    }

    public TaskInstanceDocumentComparisonItem queryByInstanceAndIp(Long instanceId , String targetHost , String hostFilePath) throws Exception {
        String sql = "select * from task_instance_document_comparison_item where document_comparison_instance_id = :instanceId and target_host = :targetHost and host_file_path = :hostFilePath and is_deleted = 0" ;
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("targetHost",targetHost);
        params.put("hostFilePath",hostFilePath);
        List<TaskInstanceDocumentComparisonItem> list = this.list(TaskInstanceDocumentComparisonItem.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

}