package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectItem;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Repository("inspectItemRepository")
public class InspectItemRepository extends JdbcGenericDao<InspectItem, Long> {

    public InspectItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InspectItem> findAllByIdIn(Set<Long> idSet) {
        String ids = StringUtils.join(idSet, ",");
        String sql = "select a.* from inspect_item a where a.id in (" + ids + ")   and a.is_deleted = 0";
        return this.list(InspectItem.class, sql, new HashMap<>());
    }

    public Integer delete(Long id, Long menderId) {
        String sql = "update inspect_item set is_deleted = 1 , mender_id = :menderId ,gmt_modify = now() where id =:id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", menderId);
        return this.execute(sql, params);
    }

    public boolean exists(String name, String type) {
        String sql = "select a.id from inspect_item a where a.is_deleted = 0 and a.name = :name and a.type = :type";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("type", type);
        List<InspectItem> list = this.list(InspectItem.class, sql, params);
        return !list.isEmpty();
    }

    public InspectItem query(Long id) {
        String sql = "select a.* from inspect_item a where a.id = :id and a.is_deleted = 0";
        List<InspectItem> list = this.list(InspectItem.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InspectItem> list(Pager pager) {
        String sql = "select a.*  from inspect_item a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InspectItem.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from inspect_item a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<InspectItem> listItems(Long templateId) {
        String sql = "select a.* from inspect_item a,inspect_template_item b,inspect_template c where a.is_deleted = 0 "
            + "and a.id = b.item_id and b.template_id = c.id and c.id = :templateId";
        return this.list(InspectItem.class, sql, MapTools.simpleMap("templateId", templateId));
    }

    public List<InspectItem> findByLevel(Long levelId) {
        String sql = "select a.* from inspect_item a,inspect_item_warn b,inspect_warn_level c where a.is_deleted = 0 "
            + "and a.id = b.item_id and b.warn_id = c.id and c.id = :levelId limit 3";
        return this.list(InspectItem.class, sql, MapTools.simpleMap("levelId", levelId));
    }

    public InspectItem findByScriptId(Long scriptId) {
        String sql = "select a.* from inspect_item a where a.script_id = :scriptId and a.is_deleted = 0";
        List<InspectItem> list = this.list(InspectItem.class, sql, MapTools.simpleMap("scriptId", scriptId));
        return list.isEmpty() ? null : list.get(0);
    }

}
