package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository("inspectRecordItemRepository")
public class InspectRecordItemRepository extends JdbcGenericDao<InspectRecordItem, Long> {

    public InspectRecordItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InspectRecordItem> findAllByJobInstanceId(Long instanceId) {
        String sql = "select a.* from inspect_record_item a where a.job_instance_id = :instanceId and a.is_deleted = 0";
        return this.list(InspectRecordItem.class, sql, MapTools.simpleMap("instanceId", instanceId));
    }

    public List<InspectRecordItem> findAllByRecordId(Long recordId) {
        String sql = "select a.* from inspect_record_item a where a.record_id = :recordId and a.is_deleted = 0";
        return this.list(InspectRecordItem.class, sql, MapTools.simpleMap("recordId", recordId));
    }

    public Long countByStatusAndRecordId(String status, Long recordId) {
        String sql = "select count(id) from inspect_record_item where status = :status and record_id = :recordId";
        Map<String, Object> param = MapTools.simpleMap("recordId", recordId);
        param.put("status", status);
        return this.countQuery(sql, param);
    }

    public Long countByWarningAndRecordId(Boolean warning, Long recordId) {
        String sql = "select count(id) from inspect_record_item where warning = :warning and record_id = :recordId";
        Map<String, Object> param = MapTools.simpleMap("recordId", recordId);
        param.put("warning", warning);
        return this.countQuery(sql, param);
    }

    public InspectRecordItem findOne(Long id) {
        String sql = "select a.* from inspect_record_item a where a.id = :id and a.is_deleted = 0";
        return this.list(InspectRecordItem.class, sql, MapTools.simpleMap("id", id)).stream().findFirst().orElse(null);
    }

    public List<InspectRecordItem> listByRule(Long recordId, StringBuilder appendSql, Map<String, Object> paramMap) {
        String sql = "select a.* from inspect_record_item a " + "where a.is_deleted = 0 and a.record_id = :recordId "
            + appendSql;
        paramMap.put("recordId", recordId);
        return this.list(InspectRecordItem.class, sql, paramMap);
    }

    public int count(Long recordId, List<Param> params, boolean onlyError) {
        String sql = "select count(1) from inspect_record_item a where a.is_deleted = 0 and record_id =:recordId";
        if (onlyError) {
            sql = sql + " and status = '" + InspectRecordItem.Status.ERROR.name() + "'";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        param.put("recordId", recordId);
        return this.countQuery(sql, param).intValue();
    }

    public List<InspectRecordItem> list(Long recordId, Pager pager, boolean onlyError) {
        String sql = "select a.*  from inspect_record_item a  where a.is_deleted = 0 and record_id =:recordId";
        if (onlyError) {
            sql = sql + " and status = '" + InspectRecordItem.Status.ERROR.name() + "'";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        param.put("recordId", recordId);
        return this.list(InspectRecordItem.class, sql, param);
    }

    public List<InspectRecordItem> listByRecordIdAndParams(Long recordId, Pager pager) {
        String sql = "select a.* from inspect_record_item a where a.record_id = :recordId and a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager.getParams(), pager.getSorter(), "a");
        return this.list(InspectRecordItem.class, sql, MapTools.simpleMap("recordId", recordId));
    }

}
