package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectRecord;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Repository("inspectRecordRepository")
public class InspectRecordRepository extends JdbcGenericDao<InspectRecord, Long> {

    public InspectRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public Integer updateStatus(Long id, String status, Boolean warning, Double totalTime) {
        String sql =
            "update inspect_record  set status =:status  ,gmt_modify = now(),total_time=:totalTime ,warning=:warning where  id=:id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status);
        params.put("warning", warning);
        params.put("totalTime", totalTime);
        return this.execute(sql, params);
    }

    public List listCountByDateBase(Date limitDay, String category, String status, Boolean warning, String type) {
        StringBuilder extraSql = new StringBuilder();
        if (StringUtils.isNotEmpty(category)) {
            extraSql.append(" and a.category = :category ");
        }
        if (StringUtils.isNotEmpty(status)) {
            extraSql.append(" and a.status = :status ");
        }
        if (warning != null) {
            extraSql.append(" and  a.warning = :warning ");
        }
        String sql =
            "select count(a.id) as count ,DATE_FORMAT( a.gmt_create,'%Y-%m-%d') as create_date from inspect_record as a where a.gmt_create > :limitDay and type = :type "
                + extraSql.toString() + " group by DATE_FORMAT( a.gmt_create,'%Y-%m-%d')";
        Map<String, Object> param = MapTools.simpleMap("limitDay", limitDay);
        param.put("category", category);
        param.put("status", status);
        param.put("warning", warning);
        param.put("type", type);
        return this.list(sql, param);
    }

    public Long countByStatus(Date limitDay, String status, String category, String type) {
        String sql =
            "select count(a.id) as count  from inspect_record as a where a.gmt_create > :limitDay   and a.status=:status and type = :type ";
        if (StringUtils.isNotEmpty(category)) {
            sql += "and a.category = :category ";
        }
        Map<String, Object> param = MapTools.simpleMap("limitDay", limitDay);
        param.put("status", status);
        param.put("category", category);
        param.put("type", type);
        return this.countQuery(sql, param);
    }

    public InspectRecord findOne(Long id) {
        String sql = "select a.* from inspect_record a where a.id = :id and  a.is_deleted = 0";
        List<InspectRecord> list = this.list(InspectRecord.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InspectRecord> list(Pager pager) {
        String sql = "select a.*  from inspect_record a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InspectRecord.class, sql);
    }

    public Long count(Integer limitDay, Integer executeType, String status, Boolean warning, Long userId, String type) {
        String sql = "select count(*)  from inspect_record a where a.is_deleted = 0 and type = '" + type + "' ";
        Map<String, Object> param = new HashMap<>();
        if (limitDay != null && limitDay >= 0) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.add(Calendar.DATE, -limitDay);
            Date createStartTime = calendar.getTime();// 今日零时零分零秒
            Date createEndTime = new Date();
            param.put("createStartTime", createStartTime);
            param.put("createEndTime", createEndTime);
            sql += " and gmt_create > :createStartTime and gmt_create < :createEndTime ";
        }
        if (userId != null && userId != 0) {
            sql += " and creator_id = :userId ";
            param.put("userId", userId);
        }
        if (executeType != null && executeType >= 0) {
            sql += " and execute_type = :executeType ";
            param.put("executeType", executeType);
        }

        if (StringUtils.isNotEmpty(status)) {
            sql += " and status = :status ";
            param.put("status", status);
        }

        if (warning != null) {
            sql += " and warning = :warning ";
            param.put("warning", warning);
        }
        return this.countQuery(sql, param);
    }

    public Map<String, Object> countGroupByCategory(Integer limitDay, String type) {
        String sql = "select category, count(*) as count from inspect_record a where a.is_deleted = 0 and type = '"
            + type + "' ";
        Map<String, Object> param = new HashMap<>();
        if (limitDay != null && limitDay >= 0) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.add(Calendar.DATE, -limitDay);
            Date createStartTime = calendar.getTime();// 今日零时零分零秒
            Date createEndTime = new Date();
            param.put("createStartTime", createStartTime);
            param.put("createEndTime", createEndTime);
            sql += " and gmt_create > :createStartTime and gmt_create < :createEndTime ";
        }
        sql += " group by category";
        List<Map<String, Object>> list = this.list(sql, param);
        Map<String, Object> result = new HashMap<>();
        for (Map<String, Object> line : list) {
            result.put(line.get("category").toString(), line.get("count"));
        }
        return result;

    }

    public int count(List<Param> params) {
        String sql = "select count(1) from inspect_record a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public InspectRecord queryTimingLast(Long timingId) {
        String sql =
            "select * from inspect_record where is_deleted = 0 and status != 'STARTING' and status != 'RUNNING' and timing_id =:timingId"
                + " order by gmt_create desc limit 1";
        List<InspectRecord> list = this.list(InspectRecord.class, sql, MapTools.simpleMap("timingId", timingId));
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public InspectRecord queryTimingLastCompleted(Long timingId) {
        String status = Stream.of(InspectRecord.Status.SUCCESS, InspectRecord.Status.ERROR).map(Enum::name)
            .map(s -> "'" + s + "'").collect(Collectors.joining(","));
        String sql = "select * from inspect_record " + "where is_deleted = 0 and timing_id = :timingId and status in ("
            + status + ") " + "order by gmt_create desc limit 1";
        List<InspectRecord> list = this.list(InspectRecord.class, sql, MapTools.simpleMap("timingId", timingId));
        return ListTool.isEmpty(list) ? null : list.get(0);
    }
}
