package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectReportDetail;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 巡检报表详情Dao实现
 */
@Repository("inspectReportDetailRepository")
public class InspectReportDetailRepository extends JdbcGenericDao<InspectReportDetail, Long> {

    public InspectReportDetailRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InspectReportDetail> list(Pager pager) {
        String sql = "select a.*, tir.name report_name from inspect_report_detail a"
            + " left join inspect_report tir on tir.id=a.report_id " + " where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InspectReportDetail.class, sql);
    }

    public InspectReportDetail query(Long id) {
        String sql = "select * from inspect_report_detail where is_deleted = 0 and id = :id";
        List<InspectReportDetail> list = this.list(InspectReportDetail.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public int count(List<Param> params) {
        String sql = "select count(*) from inspect_report_detail a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public InspectReportDetail lastOfqQuery(Long reportId) {
        String sql =
            "select * from inspect_report_detail where report_id = :id and is_deleted = 0 order by gmt_create desc";
        return this.list(InspectReportDetail.class, sql, MapTools.simpleMap(Common.ID, reportId)).get(0);
    }

}
