package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectReport;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 巡检报表Dao实现
 *
 * @author jiaohongtao
 * @version v1.0
 * @since 2020-10-30 16:50:33
 */
@Repository("inspectReportRepository")
public class InspectReportRepository extends JdbcGenericDao<InspectReport, Long> {

    public InspectReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InspectReport> list(Pager pager, Map<String, Object> anotherParams) {
        StringBuilder sql = new StringBuilder("select a.*, ti.name inspect_name, ti.category inspect_type "
            + " from inspect_report a left join inspect_timing ti on a.inspect_id = ti.id "
            + " where a.is_deleted = 0");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        for (Map.Entry<String, Object> entry : anotherParams.entrySet()) {
            String key = entry.getKey();
            if ("inspectName".equals(key)) {
                sql.append(" and ti.name like '%").append(entry.getValue()).append("%'");
            } else if ("category".equals(key)) {
                sql.append(" and ti.category = :category");
            }
            param.put(key, entry.getValue());
        }
        sql = new StringBuilder(this.getQueryBuilder().buildRaw(sql.toString(), pager, "a"));
        return this.list(InspectReport.class, sql.toString(), param);
    }

    public InspectReport query(Long id) {
        String sql = "select a.*, ti.name inspect_name,ti.category inspect_type from inspect_report a"
            + " left join inspect_timing ti on a.inspect_id = ti.id where a.is_deleted = 0 and a.id = :id";
        List<InspectReport> list = this.list(InspectReport.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) {
        String sql =
            "update inspect_report set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put(Common.MENDER_ID, userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params, Map<String, Object> anotherParams) {
        StringBuilder sql = new StringBuilder("select count(*) from inspect_report a"
            + " left join inspect_timing ti on a.inspect_id = ti.id" + " where a.is_deleted = 0");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        for (Map.Entry<String, Object> entry : anotherParams.entrySet()) {
            String key = entry.getKey();
            if ("inspectName".equals(key)) {
                sql.append(" and ti.name like '%").append(entry.getValue()).append("%'");
            } else if ("category".equals(key)) {
                sql.append(" and ti.category = :category");
            }
            param.put(key, entry.getValue());
        }

        sql = new StringBuilder(this.getQueryBuilder().buildRaw(sql.toString(), params, null, "a"));
        return this.countQuery(sql.toString(), param).intValue();
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update inspect_report set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("status", status);
        params.put(Common.MENDER_ID, userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public InspectReport get(Long id) {
        String sql = "select * from inspect_report where is_deleted = 0 and id = :id";
        List<InspectReport> list = this.list(InspectReport.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public int countName(String name) {
        String sql = "select count(1) from  inspect_report where name = :name and is_deleted = 0";
        return this.countQuery(sql, MapTools.simpleMap("name", name)).intValue();
    }

    public List<InspectReport> queryStatus(String status) {
        String sql = "select * from inspect_report where is_deleted = 0 and status = :status";
        return this.list(InspectReport.class, sql, MapTools.simpleMap("status", status));
    }

    public Long countAllByInspectId(Long inspectId) {
        String sql = "select count(a.id) from inspect_report a where a.inspect_id= :inspectId and a.is_deleted = 0";
        return this.countQuery(sql, MapTools.simpleMap("inspectId", inspectId));
    }

}
