package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.Inspect;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("inspectRepository")
public class InspectRepository extends JdbcGenericDao<Inspect, Long> {

    public InspectRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public Integer delete(Long id, Long menderId) {
        String sql = "update inspect set is_deleted = 1 , mender_id = :menderId ,gmt_modify = now() where id =:id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", menderId);
        return this.execute(sql, params);
    }

    public Inspect findOne(Long id) {
        String sql = "select a.* from inspect a where a.id = :id and  a.is_deleted = 0";
        List<Inspect> list = this.list(Inspect.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, String type) {
        String sql = "select id from inspect a where a.is_deleted = 0 and a.name = :name and a.type = :type";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("type", type);
        List<Inspect> list = this.list(Inspect.class, sql, params);
        return !list.isEmpty();
    }

    public List<Inspect> list(Pager pager) {
        String sql = "select a.* from inspect a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(Inspect.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from inspect a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update inspect set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

}
