package com.bocloud.cop.repository.inspection;

import com.bocloud.cop.entity.inspection.InspectWarnLevel;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository("inspectWarnRepository")
public class InspectWarnRepository extends JdbcGenericDao<InspectWarnLevel, Long> {

    public InspectWarnRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InspectWarnLevel> findAll() {
        String sql = "select a.* from inspect_warn_level a where a.is_deleted = 0";
        return this.list(InspectWarnLevel.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from inspect_warn_level a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<InspectWarnLevel> list(Pager pager) {
        String sql = "select a.* from inspect_warn_level a  where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InspectWarnLevel.class, sql);
    }

    public Integer delete(Long id, Long menderId) {
        String sql = "update inspect_warn_level " + "set is_deleted = 1, mender_id = :menderId, gmt_modify = now() "
            + "where id =:id ";
        Map<String, Object> params = MapTools.simpleMap("menderId", menderId);
        params.put("id", id);
        return this.execute(sql, params);
    }

    public boolean exists(String level) {
        String sql = "select id from inspect_warn_level a where a.is_deleted = 0 and a.warn_level = :level";
        List<InspectWarnLevel> list = this.list(InspectWarnLevel.class, sql, MapTools.simpleMap("level", level));
        return !list.isEmpty();
    }

    public InspectWarnLevel findById(Long id) {
        String sql = "select a.* from inspect_warn_level a where a.id = :id and  a.is_deleted = 0";
        return this.list(InspectWarnLevel.class, sql, MapTools.simpleMap("id", id)).stream().findFirst().orElse(null);
    }

}
