package com.bocloud.cop.repository.module;

import com.bocloud.cop.entity.module.NodeModule;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 编排节点模型DAO实现
 *
 * @author weiwei
 * @version 1.0
 * @since 2018年12月19日
 */
@Repository("nodeModuleRepository")
public class NodeModuleRepository extends JdbcGenericDao<NodeModule, Long> {

    public NodeModuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<NodeModule> list(Pager pager, Long groupId) {
        String sql = "select a.* from task_node_module a where a.is_deleted = 0";
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%[" + groupId
                + ",%' or a.group_ids like '%, " + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(pager.getParams());
        return this.list(NodeModule.class, sql, param);
    }

    public List<NodeModule> list() {
        String sql = "select a.* from task_node_module a where a.is_deleted = 0";
        return this.list(NodeModule.class, sql);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from task_node_module a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public boolean remove(Long id, Long userId) {
        String sql =
            "update task_node_module set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public NodeModule query(Long id) {
        String sql = "select * from task_node_module a where a.is_deleted = 0 and a.id = :id";
        List<NodeModule> list = this.list(NodeModule.class, sql, MapTools.simpleMap("id", id));
        return list.isEmpty() ? null : list.get(0);
    }

    public NodeModule getByCode(String moduleCode) {
        String sql = "select * from task_node_module where code = :moduleCode";
        List<NodeModule> list = this.list(NodeModule.class, sql, MapTools.simpleMap("moduleCode", moduleCode));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean exists(String name, Long groupId) {
        String sql =
            "select id from task_node_module a where a.name = :name and a.group_id = :groupId and a.is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("groupId", groupId);
        List<NodeModule> list = this.list(NodeModule.class, sql, paramMap);
        return !list.isEmpty();
    }

    public boolean existsCode(String code) {
        String sql = "select * from task_node_module a where a.code = :code and a.is_deleted = 0";
        List<NodeModule> list = this.list(NodeModule.class, sql, MapTools.simpleMap("code", code));
        return !list.isEmpty();
    }

    public int count(List<Param> params, Long groupId) {
        String sql = "select count(1) from task_node_module a where a.is_deleted = 0";
        if (null != groupId && 0L != groupId) {
            sql += " and (a.group_id = " + groupId + " or a.group_ids like '%[" + groupId
                + ",%' or a.group_ids like '%, " + groupId + ",%')";
        }
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = this.getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<NodeModule> list(Long groupId, String executeType, Boolean available) {
        String sql =
            "select a.* from task_node_module a where a.is_deleted = 0 and a.group_id = :groupId and a.status = 'APPROVED'";
        if (!StringUtils.isEmpty(executeType)) {
            sql += " and a.execute_type like '%" + executeType + "%'";
        }
        if (null != available) {
            sql += " and a.available = " + available;
        }
        Map<String, Object> params = MapTools.simpleMap("groupId", groupId);
        return this.list(NodeModule.class, sql, params);
    }

    public boolean updateStatus(Long id, String status, Long userId) {
        String sql =
            "update task_node_module set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

}
