package com.bocloud.cop.repository.patch;

import com.bocloud.cop.entity.patch.PatchDeployHistoryDetail;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository("patchDeployHistoryDetailRepository")
public class PatchDeployHistoryDetailRepository extends JdbcGenericDao<PatchDeployHistoryDetail, Long> {

    public PatchDeployHistoryDetailRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<PatchDeployHistoryDetail> query(Long id) {
        String sql = "select * from patch_deploy_history_detail where node_id = :id";
        return this.list(PatchDeployHistoryDetail.class, sql, MapTools.simpleMap(Common.ID, id));
    }

    public boolean update(Long id, Long nodeId) {
        String sql = "update patch_deploy_history_detail set history_id = :id where node_id = :nodeId ";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        param.put("nodeId", nodeId);
        return this.execute(sql, param) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from patch_deploy_history_detail a where a.is_deleted = 0 ";
        sql = this.getQueryBuilder().build(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<PatchDeployHistoryDetail> list(Pager pager) {
        String sql = "select a.* from patch_deploy_history_detail a  where a.is_deleted = 0 ";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(PatchDeployHistoryDetail.class, sql);
    }

    public List<PatchDeployHistoryDetail> listByHistoryId(Long historyId) {
        String sql = "select * from patch_deploy_history_detail where is_deleted = 0 and history_id = :historyId ";
        return this.list(PatchDeployHistoryDetail.class, sql, MapTools.simpleMap("historyId", historyId));
    }

    public List<PatchDeployHistoryDetail> listByPatchIdAndStatus(Long patchId, String status) {
        String sql =
            "select * from patch_deploy_history_detail where is_deleted = 0 and patch_id = :patchId and status = :status";
        Map<String, Object> param = MapTools.simpleMap("patchId", patchId);
        param.put("status", status);
        return this.list(PatchDeployHistoryDetail.class, sql, param);
    }

    public List<PatchDeployHistoryDetail> listByDeployIdAndStatus(Long deployId, String status) {
        String sql =
            "select * from patch_deploy_history_detail where is_deleted = 0 and deploy_id = :deployId and status = :status";
        Map<String, Object> param = MapTools.simpleMap("deployId", deployId);
        param.put("status", status);
        return this.list(PatchDeployHistoryDetail.class, sql, param);
    }

}
