package com.bocloud.cop.repository.patch;

import com.bocloud.cop.entity.patch.PatchDeployHistory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository("patchDeployHistoryRepository")
public class PatchDeployHistoryRepository extends JdbcGenericDao<PatchDeployHistory, Long> {

    public PatchDeployHistoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from patch_deploy_history a where a.is_deleted = 0";
        sql = this.getQueryBuilder().build(sql, params, null, "a");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public List<PatchDeployHistory> list(Pager pager) {
        String sql = "select a.* from patch_deploy_history a where a.is_deleted = 0 ";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(PatchDeployHistory.class, sql);
    }

    public List<PatchDeployHistory> listByPatchId(Long patchId, String status) {
        String sql = "select * from patch_deploy_history where is_deleted = 0 and patch_id = :patchId ";
        Map<String, Object> params = new HashMap<>();
        params.put("patchId", patchId);
        if (StringUtils.isNotEmpty(status)) {
            sql += " and status = :status";
            params.put("status", status);
        }
        return this.list(PatchDeployHistory.class, sql, params);
    }

    public List<PatchDeployHistory> listByPatchDeployId(Long deployId, String status) {
        String sql = "select * from patch_deploy_history where is_deleted = 0 and deploy_id = :deployId ";
        Map<String, Object> params = new HashMap<>();
        params.put("deployId", deployId);
        if (StringUtils.isNotEmpty(status)) {
            sql += " and status = :status";
            params.put("status", status);
        }
        return this.list(PatchDeployHistory.class, sql, params);
    }

    public PatchDeployHistory query(Long id) {
        String sql = "select a.* from patch_deploy_history a where a.is_deleted = 0 and a.id = :id ";
        List<PatchDeployHistory> list = this.list(PatchDeployHistory.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<PatchDeployHistory> listByTaskInstanceId(Long taskInstanceId, String status) {
        String sql = "select * from patch_deploy_history where is_deleted = 0 and task_instance_id = :taskInstanceId ";
        Map<String, Object> params = new HashMap<>();
        params.put("taskInstanceId", taskInstanceId);
        if (StringUtils.isNotEmpty(status)) {
            sql += " and status = :status";
            params.put("status", status);
        }
        return this.list(PatchDeployHistory.class, sql, params);
    }

}
