package com.bocloud.cop.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.annotation.ColumnType;
import com.megatron.common.model.Param;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.Table;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.EnumUtils;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @ClassName: CmdbUtils
 * @Description
 * @Author yangzhenrong
 * @Date 2022/5/11 14:56
 * @Version 1.0
 **/
@Slf4j
public class DynamicClassUtils {

    public static final String TYPE_STRING = "java.lang.String";
    public static final String TYPE_LONG = "java.lang.Long";
    public static final String TYPE_BOOLEAN = "java.lang.Boolean";
    public static final String TYPE_DATE = "java.util.Date";
    public static final String TYPE_FLOAT = "java.lang.Float";
    public static final String TYPE_DOUBLE = "java.lang.Double";
    public static final String TYPE_BIGDECIMAL = "java.math.BigDecimal";

    public static final String TYPE_INTEGER = "java.lang.Integer";

    public static final String INSTANCE_RELATION_TYPE_SOURCE = "SOURCE";
    public static final String INSTANCE_RELATION_TYPE_TARGET = "TARGET";

    // 转换数据类型
    public static String judgeDataType(String type) {
        switch (type) {
            case TYPE_STRING:
                return "varchar";
            case TYPE_LONG:
                return "bigint";
            case TYPE_BOOLEAN:
                return "tinyint";
            case TYPE_DATE:
                return "datetime";
            case TYPE_FLOAT:
                return "float";
            case TYPE_DOUBLE:
                return "double";
            case TYPE_BIGDECIMAL:
                return "decimal";
            case TYPE_INTEGER:
                return "int";
            default:
                return "varchar";
        }
    }

    /***
     * 获取对象指定注解Column的相关字段及类型
     * 
     * @param object
     * @return
     */
    public static Map<String, JSONObject> getDeclaredFieldsInfo(Object object) {
        Map<String, JSONObject> map = new LinkedHashMap<>();
        // 父类属性
        Field[] genericFields = object.getClass().getSuperclass().getDeclaredFields();
        // 属性
        Field[] fields = object.getClass().getDeclaredFields();
        for (Field field : fields) {
            boolean annotationColumn = field.isAnnotationPresent(Column.class);
            if (annotationColumn) {
                String annotationVal = field.getAnnotation(Column.class).value();
                String fieldType = field.getType().getName();
                boolean nullable = field.getAnnotation(Column.class).nullable();
                long length = field.getAnnotation(Column.class).length();
                boolean columnType = field.isAnnotationPresent(ColumnType.class);
                JSONObject fieldJSON = new JSONObject();
                fieldJSON.put("fieldType", fieldType);
                fieldJSON.put("length", length);
                fieldJSON.put("nullable", nullable);
                if(columnType){
                    String columnTypeValue = field.getAnnotation(ColumnType.class).value();
                    fieldJSON.put("fieldDBType", columnTypeValue);
                }
                map.put(annotationVal, fieldJSON);
            }
        }
        for (Field field : genericFields) {
            boolean annotationColumn = field.isAnnotationPresent(Column.class);
            if (annotationColumn) {
                String annotationVal = field.getAnnotation(Column.class).value();
                String fieldType = field.getType().getName();
                boolean nullable = field.getAnnotation(Column.class).nullable();
                long length = field.getAnnotation(Column.class).length();
                //// 实例表名称字段长度和实例值表name长度保持一致
                //if ("name".equalsIgnoreCase(annotationVal)) {
                //    length = 100;
                //}
                boolean columnType = field.isAnnotationPresent(ColumnType.class);
                JSONObject fieldJSON = new JSONObject();
                fieldJSON.put("fieldType", fieldType);
                fieldJSON.put("length", length);
                fieldJSON.put("nullable", nullable);
                if(columnType){
                    String columnTypeValue = field.getAnnotation(ColumnType.class).value();
                    fieldJSON.put("fieldDBType", columnTypeValue);
                }
                map.put(annotationVal, fieldJSON);
            }
        }
        return map;
    }
    // public static Map<String, String> getDeclaredFieldsInfo(Object object) {
    // Map<String, String> map = new LinkedHashMap<>();
    // //父类属性
    // Field[] genericFields = object.getClass().getSuperclass().getDeclaredFields();
    // //属性
    // Field[] fields = object.getClass().getDeclaredFields();
    // for (Field field : fields) {
    // boolean annotationColumn = field.isAnnotationPresent(Column.class);
    // if (annotationColumn) {
    // String annotationVal = field.getAnnotation(Column.class).value();
    // String fieldType = field.getType().getName();
    // map.put(annotationVal, fieldType);
    // }
    // }
    // for (Field field : genericFields) {
    // boolean annotationColumn = field.isAnnotationPresent(Column.class);
    // if (annotationColumn) {
    // String annotationVal = field.getAnnotation(Column.class).value();
    // String fieldType = field.getType().getName();
    // map.put(annotationVal, fieldType);
    // }
    // }
    // return map;
    // }

    /***
     * 更改Table注解值
     *
     * @param clazz
     * @param newTableValue
     */
    public static void changeTableAnnotationValue(Class clazz, String newTableValue) {
        try {
            Table tableAnnotation = (Table)clazz.getAnnotation(Table.class);
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(tableAnnotation);
            // 获取私有 memberValues 属性
            Field memberValuesField = invocationHandler.getClass().getDeclaredField("memberValues");
            memberValuesField.setAccessible(true);
            Map<String, Object> memberValuesValue = (Map<String, Object>)memberValuesField.get(invocationHandler);
            // 修改属性值
            memberValuesValue.put("value", newTableValue);
        } catch (Exception e) {
            log.error("changeTableAnnotationValue fail", e);
        }
    }

    public static Class changeTableAnnotation(Class clazz, String newTableValue) {
        try {
            Table tableAnnotation = (Table)clazz.getAnnotation(Table.class);
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(tableAnnotation);
            // 获取私有 memberValues 属性
            Field memberValuesField = invocationHandler.getClass().getDeclaredField("memberValues");
            memberValuesField.setAccessible(true);
            Map<String, Object> memberValuesValue = (Map<String, Object>)memberValuesField.get(invocationHandler);
            // 修改属性值
            memberValuesValue.put("value", newTableValue);
        } catch (Exception e) {
            log.error("changeTableAnnotation fail", e);
        }
        return clazz;
    }

    /***
     * 判断字符串是否满足正则表达式
     * 
     * @param regex
     * @param source
     * @return
     */
    public static boolean contains(String regex, String source) {
        Pattern pattern = Pattern.compile(regex);
        return pattern.matcher(source).find();
    }

    public static boolean isJSON(String str) {
        boolean result = false;
        try {
            Object obj = JSON.parse(str);
            result = true;
        } catch (Exception e) {
            result = false;
        }
        return result;
    }

    public static boolean isDate(String str) {
        // 日期格式校验
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            sdf.parse(str);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

    public static boolean isTime(String str) {
        // 时间格式校验
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        try {
            sdf.parse(str);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

    /***
     * 判断是否为布尔型
     * 
     * @param str
     * @return
     */
    public static boolean isBool(String str) {
        try {
            if ("true".equalsIgnoreCase(str) || "0".equals(str) || "false".equalsIgnoreCase(str) || "1".equals(str)) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            return false;
        }
    }

    /***
     * 判断是否为数值
     * 
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        try {
            Pattern pattern = Pattern.compile("^(\\-|\\+)?\\d+(\\.\\d+)?$");
            Matcher isNum = pattern.matcher(str);
            if (!isNum.matches()) {
                return false;
            }
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /***
     * 从查询参数内获取指定字段的值
     * 
     * @param params
     * @param field
     * @return
     */
    public static String getField(List<Param> params, String field) {
        if ((null != params) && (!params.isEmpty())) {
            for (Param param : params) {
                for (String key : param.getParam().keySet()) {
                    if (field.equals(key)) {
                        return param.getParam().get(key).toString();
                    }
                }
            }
        }
        return null;
    }

    /***
     * 带下划线字段转驼峰
     * 
     * @param name
     * @return
     */
    public static String toCamelCase(String name) {
        if (null == name) {
            return null;
        }
        if (name.contains("_")) {
            name = name.toLowerCase();
            StringBuilder sb = new StringBuilder(name.length());
            boolean upperCase = false;
            for (int i = 0; i < name.length(); i++) {
                char c = name.charAt(i);
                if (c == '_') {
                    upperCase = true;
                } else if (upperCase) {
                    sb.append(Character.toUpperCase(c));
                    upperCase = false;
                } else {
                    sb.append(c);
                }
            }
            return sb.toString();
        } else {
            return name;
        }
    }

    // 判断某个值是否在枚举类中
    public static boolean isInclude(Class enumClass, String value) {
        List enumList = EnumUtils.getEnumList(enumClass);
        for (int i = 0; i < enumList.size(); i++) {
            Object en = enumList.get(i);
            Class<?> enClass = en.getClass();
            try {
                Method method = enClass.getMethod("getValue"); // 需要与枚举类方法对应
                Object invoke = method.invoke(en, null);
                if (invoke.toString().equalsIgnoreCase(value)) {
                    return true;
                }
            } catch (Exception e) {
                return false;
            }
        }
        return false;
    }
}
