package com.bocloud.cop.core;

import java.util.List;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.bocloud.cop.entity.TaskConfig;
import com.bocloud.cop.repository.TaskConfigRepository;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class TaskConfigCache implements InitializingBean {

    private final StringRedisTemplate redisTemplate;
    private final TaskConfigRepository taskConfigRepository;
    private final ThreadPoolTaskExecutor taskExecutor;

    public TaskConfigCache(StringRedisTemplate redisTemplate, TaskConfigRepository taskConfigRepository,
        ThreadPoolTaskExecutor taskExecutor) {
        this.redisTemplate = redisTemplate;
        this.taskConfigRepository = taskConfigRepository;
        this.taskExecutor = taskExecutor;
    }

    public void putConfigCache(String code, String value) {
        this.redisTemplate.opsForValue().set(code, value);
    }

    public String getConfigValue(String code) {
        return redisTemplate.opsForValue().get(code);
    }

    @Override
    public void afterPropertiesSet() {
        taskExecutor.execute(this::addConfigCache);
    }

    public void addConfigCache() {
        try {
            List<TaskConfig> configs = taskConfigRepository.listAll();
            for (TaskConfig config : configs) {
                if (null == config.getValue()) {
                    continue;
                }
                if (!StringUtils.hasText(getConfigValue(config.getCode()))) {
                    putConfigCache(config.getCode(), config.getValue());
                }
            }
        } catch (Exception e) {
            log.error("put task config cache error,", e);
        }
    }

}
