package com.bocloud.cop.core;

import java.util.List;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.bocloud.cop.entity.TaskTreatment;
import com.bocloud.cop.repository.TaskTreatmentRepository;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class TaskTreatmentCache implements InitializingBean {

    private final StringRedisTemplate redisTemplate;
    private final TaskTreatmentRepository taskTreatmentRepository;
    private final ThreadPoolTaskExecutor taskExecutor;

    public TaskTreatmentCache(StringRedisTemplate redisTemplate, TaskTreatmentRepository taskTreatmentRepository,
        ThreadPoolTaskExecutor taskExecutor) {
        this.redisTemplate = redisTemplate;
        this.taskTreatmentRepository = taskTreatmentRepository;
        this.taskExecutor = taskExecutor;
    }

    public void putTreatmentCache(String name, String value) {
        this.redisTemplate.opsForValue().set(name, value);
    }

    public String getTreatmentValue(String name) {
        return redisTemplate.opsForValue().get(name);
    }

    @Override
    public void afterPropertiesSet() {
        taskExecutor.execute(this::addConfigCache);
    }

    public void addConfigCache() {
        try {
            List<TaskTreatment> configs = taskTreatmentRepository.listAll();
            for (TaskTreatment config : configs) {
                if (null == config.getValue()) {
                    continue;
                }
                if (!StringUtils.hasText(getTreatmentValue("Treatment_" + config.getName()))) {
                    putTreatmentCache("Treatment_" + config.getName(), config.getValue());
                }
            }
        } catch (Exception e) {
            log.error("put task treatment cache error,", e);
        }
    }

}
