package com.bocloud.cop.core;

import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.entity.TaskWorker.Category;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.registry.ZooKeeperClient;

import lombok.extern.slf4j.Slf4j;

/**
 * Worker数据维护，对数据库中的worker是否真是存在和可用进行交验
 *
 * @author dmw
 */
@Slf4j
public class WorkerDaemon implements Runnable {

    private ZooKeeperClient client;
    private String workerPath;
    private TaskWorkerRepository workerRepository;
    private WorkerCache workerCache;

    public WorkerDaemon(ZooKeeperClient client, String workerPath, TaskWorkerRepository workerRepository,
        WorkerCache workerCache) {
        super();
        this.client = client;
        this.workerPath = workerPath;
        this.workerRepository = workerRepository;
        this.workerCache = workerCache;
    }

    @Override
    public void run() {
        try {
            List<String> instanceIds = this.client.getChildren(workerPath);
            if (ListTool.isEmpty(instanceIds)) {
                return;
            }
            List<String> instances = new ArrayList<>();
            for (String instanceId : instanceIds) {
                String instancePath = this.workerPath + "/" + instanceId;
                String instanceData = this.client.getData(instancePath);
                String workerAddr = JSONObject.parseObject(instanceData).getString("address");
                String workerPort = JSONObject.parseObject(instanceData).getString("port");
                String workerInstance = workerAddr + ":" + workerPort;
                instances.add(workerInstance);
            }
            List<TaskWorker> workers = workerRepository.listWorkers(Category.Ansible.name());
            if (ListTool.isEmpty(workers)) {
                return;
            }
            for (TaskWorker worker : workers) {
                if (instances.contains(worker.getInstance())) {
                    continue;
                }
                // 更新缓存
                if (worker.getIsSuper()) {
                    workerCache.delCache(CacheConstant.SUPER_WORKERS, worker.getInstance());
                } else {
                    if (null != worker.getGroupId()) {
                        workerCache.delCache(CacheConstant.WORKERGROUP_ + worker.getGroupId(), worker.getInstance());
                    }
                }
                // 从workers缓存删除
                // workerCache.delCache(CacheConstant.WORKERS, worker.getInstance());
                workerRepository.delete(worker.getInstance(), Category.Ansible.name());
            }
        } catch (Exception e) {
            log.error("定时更新worker缓存失败：", e);
        }
    }

}
