package com.bocloud.cop.intf;

import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.model.FastTask;
import com.bocloud.cop.model.ScriptBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;

/**
 * 脚本抽象Service接口
 * 
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
public interface ScriptService {

    /**
     * 查询脚本列表
     * 
     * @param pager
     * @return
     */
    public GeneralResult<GridBean<Script>> list(Pager pager, Long useId);

    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager, Long useId);

    /**
     * 创建脚本
     * 
     * @param script
     * @param userid
     * @return
     */
    public GeneralResult<Void> create(Script script, Long userid);

    /**
     * 脚本基本信息
     * 
     * @param bean
     * @param userId
     * @return
     */
    public GeneralResult<Void> modify(ScriptBean bean, Long userId);

    /**
     * 删除脚本
     * 
     * @param id
     * @param userId
     * @return
     */
    public GeneralResult<Void> remove(Long id, Long userId);

    /**
     * 查看脚本详情
     * 
     * @param id
     * @return
     */
    public GeneralResult<Script> detail(Long id);

    public Script get(Long id);

    public GeneralResult<Void> updateStatus(Long id, String status, Long userId);

    /**
     * 脚本版本切换
     * 
     * @param id
     * @param userId
     * @return
     */
    public GeneralResult<Void> modifyVersion(Long id, Long userId);

    /**
     * 快速执行脚本
     * 
     * @param fastTask
     * @return
     */
    public TaskInstance fastExecute(FastTask fastTask, RequestContext context);

    /**
     * 判断是否存在高危命令
     * 
     * @param task
     * @param userId
     * @return
     */
    public GeneralResult<Void> checkCommand(Task task, Long userId);

}
