package com.bocloud.cop.intf;

import java.util.Map;

import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.params.ServerUserCreateParams;
import com.bocloud.cop.model.params.ServerUserModifyParams;
import com.bocloud.cop.model.params.ServerUserRemoveAuthParams;
import com.bocloud.cop.model.params.ServerUserRemoveParams;
import com.bocloud.cop.model.params.ServerUserSaveAuthParams;

/**
 * 主机用户管理服务接口
 * 
 * @author sxp
 * @since Dec 27, 2017
 */
public interface ServerUserService {

    /**
     * 增加主机用户
     * 
     * @author sxp
     * @since Dec 27, 2017
     * @param params
     *            包含主机用户信息的json对象
     * @param context
     *            当前登录用户的唯一标识
     * @return 返回处理结果信息
     */
    GeneralResult<String> create(ServerUserCreateParams params, RequestContext context);

    /**
     * 修改主机用户
     * 
     * @author sxp
     * @since Dec 27, 2017
     * @param params
     *            包含主机用户信息的json对象
     * @param context
     *            当前登录用户的唯一标识
     * @return 返回处理结果信息
     */
    GeneralResult<String> modify(ServerUserModifyParams params, RequestContext context);

    /**
     * 删除主机用户
     * 
     * @author sxp
     * @since Dec 27, 2017
     * @param params
     *            包含主机用户信息的json对象
     * @param context
     *            当前登录用户的唯一标识
     * @return 返回处理结果信息
     */
    GeneralResult<String> remove(ServerUserRemoveParams params, RequestContext context);

    /**
     * 保存主机用户的sudo权限
     * 
     * @author sxp
     * @since Dec 28, 2017
     * @param params
     *            包含主机用户sudo权信息的json对象
     * @param context
     *            当前登录用户的唯一标识
     * @return 返回处理结果信息
     */
    GeneralResult<String> saveAuth(ServerUserSaveAuthParams params, RequestContext context);

    /**
     * 删除主机用户的sudo权限
     * 
     * @author sxp
     * @since Dec 28, 2017
     * @param params
     *            包含主机用户sudo权信息的json对象
     * @param context
     *            当前登录用户的唯一标识
     * @return 返回处理结果信息
     */
    GeneralResult<String> removeAuth(ServerUserRemoveAuthParams params, RequestContext context);

    /**
     * 获取主机用户的sudo权限列表
     * 
     * @author sxp
     * @since Dec 28, 2017
     * @param workerHost
     *            包含主机用户sudo权信息的json对象
     * @param context
     *            当前登录用户的唯一标识
     * @return 返回处理结果信息
     */
    GeneralResult<Map<String, String>> listAuth(WorkerHost workerHost, String userName, Boolean become,
        RequestContext context);
}
