package com.bocloud.cop.intf;

import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskFailRecord;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.enums.Status.Inst;
import com.bocloud.cop.enums.TaskAction;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.params.DeployGaiaParams;
import com.bocloud.cop.model.result.TaskInstanceExecResult;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

import java.util.List;

/**
 * 任务记录Service接口
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.17
 */
public interface TaskInstanceService {

    /**
     * 作业实例执行链路列表
     *
     * @param pager
     * @param context
     * @return
     */
    GeneralResult<GridBean<TaskInstance>> links(Pager pager, RequestContext context);

    /**
     * 查询任务记录列表
     *
     * @param pager
     * @param context
     * @return
     */
    public GeneralResult<GridBean<TaskInstance>> list(Pager pager, RequestContext context);

    /**
     * 查询任务记录详细
     *
     * @param id
     * @return
     */
    public GeneralResult<TaskInstance> detail(Long id);

    /**
     * 查询任务记录详细
     *
     * @param id
     * @return
     */
    public GeneralResult<TaskInstance> log(Long id);

    /**
     * 更新任务记录
     *
     * @param instance
     * @return
     */
    public GeneralResult<Void> modify(TaskInstance instance);

    /**
     * 通过ID得到任务记录
     *
     * @param id
     * @return
     */
    public TaskInstance get(Long id);

    public GeneralResult<Void> remove(Long id, Long userId);

    public GeneralResult<Void> updateStatus(Long id, String status, Long userId);

    /**
     * 创建恢复任务
     *
     * @param id
     * @param context
     * @return
     */
    public Task recover(Long id, RequestContext context);

    /**
     * 根据执行机器获取任务实例
     *
     * @param status
     *            任务实例的状态
     * @param executeHost
     *            任务执行所在的机器实例：IP_PORT
     * @return
     */
    public GeneralResult<List<TaskInstance>> listByExecHost(Inst status, String executeHost);

    /**
     * 获取异常任务实例
     *
     * @param executeHost
     *            执行机器
     * @param excludeHost
     *            需要排除机器
     * @return
     */
    public GeneralResult<List<TaskInstance>> listExceptionInstance(String executeHost, String excludeHost);

    /**
     * 操作检查
     *
     * @param id
     * @param action
     * @return
     */
    public GeneralResult<TaskInstance> handlerCheck(Long id, TaskAction action);

    /**
     * 查询未执行任务记录列表
     *
     * @param pager
     * @return
     */
    public GeneralResult<GridBean<TaskInstance>> listAwaits(Pager pager, RequestContext context);

    /**
     * 结果
     *
     * @param id
     * @return
     */
    public GeneralResult<List<TaskInstanceExecResult>> result(Long id);

    public List<TaskInstance> list(Long taskId);

    /**
     * 查询命令执行的任务记录列表
     *
     * @param pager
     * @param
     * @return
     */
    public GeneralResult<GridBean<TaskInstance>> list(Pager pager, Long commandId);

    public GeneralResult<Void> cancel(Long id);

    public GeneralResult<Void> cancel(List<Long> ids);

    public GeneralResult<?> putJob(Job job);

    public List<Job> getJobs();

    public List<Job> getAllJobs();

    GeneralResult<GridBean<TaskFailRecord>> listFailRecord(Pager pager);

    GeneralResult<?> updateLimited(Long id, Long userId, String dealMethod, Long platformAlertId);

    /**
     * 任务实例化（创建任务实例）
     *
     * @param task
     *            任务信息
     * @param mode
     *            调用模式
     * @param context
     *            用户信息
     * @return
     */
    TaskInstance initInstance(Task task, String mode, RequestContext context);

    /**
     * 作业实例失败后重新执行，初始化状态
     *
     * @param id
     * @return
     */
    public GeneralResult<TaskInstance> initFailedStatus(Long id, RequestContext context);

    public GeneralResult<TaskInstance> executeCheck(Long id, RequestContext context);

    /**
     * 构建gaia部署任务
     *
     * @param params
     *            任务参数
     * @return
     */
    GeneralResult<Task> taskDeployGaia(DeployGaiaParams params);

}
