package com.bocloud.cop.schedule.inspection.report;

import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.TimeZone;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.enums.ComplianceCategory;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * @author zoucg
 * @Company: 博云 www.bocloud.com
 * @ClassName: ComplianceRecordPdfUtil
 * @Description:
 * @DATE: 2022/4/6
 */

public class ComplianceRecordPdfUtil {

    public static void covertPdf(InspectRecord record, OutputStream outputStream)
        throws IOException, DocumentException {

        // 第一步，实例化一个document对象
        Document document = new Document();
        // 清空response，否则可能会对程序有影响，因为在此之前有可能也用到了response，若不reset，后面的setCntentoType()有可能失效
        // 第二步，设置要到出的路径
        // 如果是浏览器通过request请求需要在浏览器中输出则使用下面方式
        // ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // OutputStream out = response.getOutputStream();
        // 第三步,设置字符,宋体
        BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        Font titleFont = new Font(bfChinese, 16.0F, Font.NORMAL);
        Font contentFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        Font tableFont = new Font(bfChinese, 8.0F, Font.NORMAL);
        // 第四步，将pdf文件输出到磁盘
        PdfWriter.getInstance(document, outputStream);
        // 第五步，打开生成的pdf文件
        document.open();

        Font greenFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        greenFont.setColor(BaseColor.GREEN);
        Font redFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        redFont.setColor(BaseColor.RED);
        Font orangeFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        orangeFont.setColor(BaseColor.ORANGE);

        String title = "合规信息";
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Element.ALIGN_CENTER);
        paragraph.add(new Chunk(title, titleFont).setLocalDestination(title));
        // paragraph.add(new Chunk(title, new Font(bfChinese, 20F, Font.BOLD)).setLocalDestination(title));
        document.add(paragraph);
        document.add(new Paragraph("\n"));

        PdfPTable table = new PdfPTable(2);
        table.setHorizontalAlignment(Element.ALIGN_CENTER);
        float[] widths = {44f, 100f};
        table.setWidths(widths);

        PdfPCell cell;

        cell = new PdfPCell(new Paragraph("合规名称", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getName(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("合规分类", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(ComplianceCategory.valueOf(record.getCategory()).getName(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("合规类型", contentFont));
        setAndAddCellStyle(table, cell);

        Integer recordType = record.getExecuteType();
        String typeText = "-";
        if (recordType == 0) {
            typeText = "常规检查";
        } else if (recordType == 1) {
            typeText = "定时检查";
        }
        cell = new PdfPCell(new Paragraph(typeText, contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("执行人员", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getCreator(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("合规检查状态", contentFont));
        setAndAddCellStyle(table, cell);

        String recordStatus = record.getStatus();
        if (recordStatus.equals(InspectRecord.Status.STARTING.name())) {
            cell = new PdfPCell(new Paragraph("启动中", contentFont));
        } else if (recordStatus.equals(InspectRecord.Status.RUNNING.name())) {
            cell = new PdfPCell(new Paragraph("执行中", contentFont));
        } else if (recordStatus.equals(InspectRecord.Status.SUCCESS.name())) {
            cell = new PdfPCell(new Paragraph("成功", greenFont));
        } else if (recordStatus.equals(InspectRecord.Status.ERROR.name())) {
            cell = new PdfPCell(new Paragraph("错误", redFont));
        } else {
            cell = new PdfPCell(new Paragraph("-", contentFont));
        }
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph("检查时间", contentFont));
        setAndAddCellStyle(table, cell);

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        String createTime = simpleDateFormat.format(record.getGmtCreate());
        cell = new PdfPCell(new Paragraph(createTime, contentFont));
        setAndAddCellStyle(table, cell);
        // --------------------------------
        cell = new PdfPCell(new Paragraph("检查服务器数量", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getHostNum() + "", contentFont));
        setAndAddCellStyle(table, cell);
        // --------------------------------
        cell = new PdfPCell(new Paragraph("基线检查项", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getItemNum() + "", contentFont));
        setAndAddCellStyle(table, cell);
        // --------------------------------
        cell = new PdfPCell(new Paragraph("不合规服务器数", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getNoPassHostNum() + "", contentFont));
        setAndAddCellStyle(table, cell);
        // --------------------------------
        cell = new PdfPCell(new Paragraph("不合规基线项数", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getNoPassItemNum() + "", contentFont));
        setAndAddCellStyle(table, cell);
        // --------------------------------
        cell = new PdfPCell(new Paragraph("合规通过率", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getPassRates() + "%", contentFont));
        setAndAddCellStyle(table, cell);

        // ---------详细表格 ------- 基线详情 ----------
        cell = new PdfPCell(new Paragraph("", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("基线详情", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);
        PdfPTable detailTable = new PdfPTable(4);
        float[] floats = {16f, 16f, 16f, 16f};
        detailTable.setWidths(floats);
        cell = new PdfPCell(getItemDetailInfo(record.getItemDetail(), detailTable, tableFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        // ---------详细表格 ------- 设备详情 ----------
        cell = new PdfPCell(new Paragraph("", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("设备详情", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);
        PdfPTable hostDetailTable = new PdfPTable(6);
        float[] hostFloats = {16f, 16f, 16f, 16f, 16f, 16f};
        hostDetailTable.setWidths(hostFloats);
        cell = new PdfPCell(getHostDetailInfo(record.getHostDetail(), hostDetailTable, tableFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        // ---------详细表格 ------- 合规详情 ----------
        cell = new PdfPCell(new Paragraph("", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("合规详情", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);
        PdfPTable complianceDetailTable = new PdfPTable(7);
        float[] complianceFloats = {16f, 16f, 16f, 16f, 16f, 16f, 16f};
        complianceDetailTable.setWidths(complianceFloats);
        cell = new PdfPCell(getComplianceDetailInfo(record.getComplianceDetail(), complianceDetailTable, tableFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        // ---------详细表格 ------- 错误详情 ----------
        cell = new PdfPCell(new Paragraph("", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("错误详情", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);
        PdfPTable errorDetailTable = new PdfPTable(5);
        float[] errorFloats = {16f, 16f, 16f, 16f, 16f};
        errorDetailTable.setWidths(errorFloats);
        cell = new PdfPCell(getErrorDetailInfo(record.getErrorDetail(), errorDetailTable, tableFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);

        // 添加大表格
        document.add(table);
        // 第七步，关闭document
        document.close();
    }

    /**
     * 基线详情列表
     *
     * @param array
     * @param detailTable
     * @param fontZH
     * @return
     * @throws IOException
     * @throws DocumentException
     */
    private static PdfPTable getItemDetailInfo(List<JSONObject> array, PdfPTable detailTable, Font fontZH) {
        PdfPCell cell;

        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("基线项名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("合规设备数", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("不合规设备数", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("错误设备数", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.get(i);
            cell = new PdfPCell(new Paragraph(obj.getString("itemName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("passHost"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("noPassHost"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("error"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);
        }
        return detailTable;
    }

    /**
     * 设备详情
     *
     * @param array
     * @param detailTable
     * @param fontZH
     * @return
     * @throws IOException
     * @throws DocumentException
     */
    private static PdfPTable getHostDetailInfo(List<JSONObject> array, PdfPTable detailTable, Font fontZH) {
        PdfPCell cell;

        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机IP", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("合规数", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("不合规数", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("错误数", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.get(i);
            cell = new PdfPCell(new Paragraph(obj.getString("groupName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("hostName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("ip"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("passHost"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("noPassHost"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("error"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

        }
        return detailTable;
    }

    /**
     * 合规详情
     *
     * @param array
     * @param detailTable
     * @param fontZH
     * @return
     * @throws IOException
     * @throws DocumentException
     */
    private static PdfPTable getComplianceDetailInfo(List<JSONObject> array, PdfPTable detailTable, Font fontZH) {
        PdfPCell cell;

        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机IP", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("合规基线项", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("当前值", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("基线标准", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("合规状态", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.get(i);
            cell = new PdfPCell(new Paragraph(obj.getString("groupName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("hostName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("ip"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("itemName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("result"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("rule"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("pass"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

        }
        return detailTable;
    }

    /**
     * 错误详情
     *
     * @param array
     * @param detailTable
     * @param fontZH
     * @return
     * @throws IOException
     * @throws DocumentException
     */
    private static PdfPTable getErrorDetailInfo(List<JSONObject> array, PdfPTable detailTable, Font fontZH) {
        PdfPCell cell;
        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机IP", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("合规基线项", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("错误信息", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.get(i);
            cell = new PdfPCell(new Paragraph(obj.getString("groupName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("hostName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("ip"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("itemName"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(obj.getString("errorInfo"), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

        }
        return detailTable;
    }

    private static void setAndAddCellStyle(PdfPTable table, PdfPCell cell) {
        cell.setMinimumHeight(30.0F);
        cell.setPaddingTop(10f);
        cell.setPaddingBottom(10f);
        setCellCenter(cell);
        cell.setBorder(Rectangle.NO_BORDER);
        table.addCell(cell);
    }

    private static void setCellCenter(PdfPCell cell) {
        cell.setMinimumHeight(40);
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
    }

    private static final short BOLD_FONT = 20;
    private static final short NORMAL_FONT = 12;

    public static void covertExcel(InspectRecord record, OutputStream outputStream) throws IOException {

        HSSFWorkbook workbook = new HSSFWorkbook();

        HSSFSheet sheet = workbook.createSheet();
        sheet.setColumnWidth(0, 16 * 256);
        sheet.setColumnWidth(1, 20 * 256);
        sheet.setColumnWidth(2, 20 * 256);
        sheet.setColumnWidth(3, 40 * 256);
        sheet.setColumnWidth(4, 40 * 256);
        sheet.setColumnWidth(5, 40 * 256);
        sheet.setColumnWidth(6, 20 * 256);
        sheet.setColumnWidth(7, 20 * 256);
        sheet.setColumnWidth(8, 20 * 256);
        sheet.setColumnWidth(9, 40 * 256);

        sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, 3));

        for (int i = 2; i <= 12; i++) {
            sheet.addMergedRegion(new CellRangeAddress(i, i, 1, 3));
        }

        HSSFRow row = sheet.createRow(0);
        HSSFCell cell = row.createCell(0);
        cell.setCellStyle(getStyle(workbook, BOLD_FONT, true));
        cell.setCellValue("合规信息");

        HSSFRow row2 = sheet.createRow(2);
        HSSFCell cell2 = row2.createCell(0);
        cell2.setCellValue("合规名称");
        cell2.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell2 = row2.createCell(1);
        cell2.setCellValue(record.getCreator());
        cell2.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row20 = sheet.createRow(3);
        HSSFCell cell20 = row20.createCell(0);
        cell20.setCellValue("合规分类");
        cell20.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell20 = row20.createCell(1);
        cell20.setCellValue(ComplianceCategory.valueOf(record.getCategory()).getName());
        cell20.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row3 = sheet.createRow(4);
        HSSFCell cell3 = row3.createCell(0);
        cell3.setCellValue("合规类型");
        cell3.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell3 = row3.createCell(1);

        Integer recordType = record.getExecuteType();
        if (recordType == 0) {
            cell3.setCellValue("常规检查");
        } else if (recordType == 1) {
            cell3.setCellValue("定时检查");
        } else {
            cell3.setCellValue("-");
        }
        cell3.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row4 = sheet.createRow(5);
        HSSFCell cell4 = row4.createCell(0);
        cell4.setCellValue("执行人员");
        cell4.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell4 = row4.createCell(1);
        cell4.setCellValue(record.getCreator());
        cell4.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row5 = sheet.createRow(6);
        HSSFCell cell5 = row5.createCell(0);
        cell5.setCellValue("执行状态");
        cell5.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell5 = row5.createCell(1);

        String status = record.getStatus();
        if (status.equals(InspectRecord.Status.STARTING.name())) {
            cell5.setCellValue("启动中");
            cell5.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        } else if (status.equals(InspectRecord.Status.RUNNING.name())) {
            cell5.setCellValue("执行中");
            cell5.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        } else if (status.equals(InspectRecord.Status.SUCCESS.name())) {
            setStatus(workbook, cell5, "成功", HSSFColor.GREEN.index);
        } else if (status.equals(InspectRecord.Status.ERROR.name())) {
            setStatus(workbook, cell5, "错误", HSSFColor.RED.index);
        } else {
            cell5.setCellValue("-");
            cell5.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        }

        HSSFRow row7 = sheet.createRow(7);
        HSSFCell cell7 = row7.createCell(0);
        cell7.setCellValue("检查时间");
        cell7.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell7 = row7.createCell(1);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        sdf.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        cell7.setCellValue(sdf.format(record.getGmtCreate()));
        cell7.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row8 = sheet.createRow(8);
        HSSFCell cell8 = row8.createCell(0);
        cell8.setCellValue("检查服务器数量");
        cell8.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell8 = row8.createCell(1);
        cell8.setCellValue(record.getHostNum());
        cell8.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row9 = sheet.createRow(9);
        HSSFCell cell9 = row9.createCell(0);
        cell9.setCellValue("基线检查项");
        cell9.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell9 = row9.createCell(1);
        cell9.setCellValue(record.getItemNum());
        cell9.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row10 = sheet.createRow(10);
        HSSFCell cell10 = row10.createCell(0);
        cell10.setCellValue("不合规服务器数量");
        cell10.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell10 = row10.createCell(1);
        cell10.setCellValue(record.getNoPassHostNum());
        cell10.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        HSSFRow row11 = sheet.createRow(11);
        HSSFCell cell11 = row11.createCell(0);
        cell11.setCellValue("合规通过率");
        cell11.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell11 = row11.createCell(1);
        cell11.setCellValue(record.getPassRates() + "%");
        cell11.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        // 详情必须显示
        // ------------------------- 基线详情 ---------------------- -------------
        HSSFRow row13 = sheet.createRow(14);
        HSSFCell cell14 = row13.createCell(0);
        cell14.setCellValue("基线详情");
        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell14 = row13.createCell(1);
        cell14.setCellValue("基线项名称");
        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell14 = row13.createCell(2);
        cell14.setCellValue("合规设备数");
        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell14 = row13.createCell(3);
        cell14.setCellValue("不合规设备数");
        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell14 = row13.createCell(4);
        cell14.setCellValue("错误设备数");
        cell14.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        List<JSONObject> array = record.getItemDetail();
        int count = 0;
        for (int i = 0; i < array.size(); i++) {
            JSONObject obj = array.get(i);
            count += 1;
            HSSFRow row14 = sheet.createRow(14 + count);
            HSSFCell cell15 = row14.createCell(1);

            cell15.setCellValue(obj.getString("itemName"));
            cell15.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell15 = row14.createCell(2);
            cell15.setCellValue(obj.getString("passHost"));
            cell15.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell15 = row14.createCell(3);
            cell15.setCellValue(obj.getString("noPassHost"));
            cell15.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell15 = row14.createCell(4);
            cell15.setCellValue(obj.getString("error"));
            cell15.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        }
        // ------------------------- 设备详情 ---------------------- -------------
        int itemSize = array.size();
        HSSFRow row15 = sheet.createRow(16 + itemSize + count);
        HSSFCell cell16 = row15.createCell(0);
        cell16.setCellValue("设备详情");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell16 = row15.createCell(1);
        cell16.setCellValue("资源组名称");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
        cell16 = row15.createCell(2);
        cell16.setCellValue("主机名称");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell16 = row15.createCell(3);
        cell16.setCellValue("主机IP");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell16 = row15.createCell(4);
        cell16.setCellValue("合规数");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell16 = row15.createCell(5);
        cell16.setCellValue("不合规数");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell16 = row15.createCell(6);
        cell16.setCellValue("错误数");
        cell16.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        java.util.List<JSONObject> hostDetailArray = record.getHostDetail();
        for (int i = 0; i < hostDetailArray.size(); i++) {
            JSONObject obj = hostDetailArray.get(i);
            count += 1;
            HSSFRow row16 = sheet.createRow(16 + count + itemSize);

            HSSFCell cell17 = row16.createCell(1);
            cell17.setCellValue(obj.getString("groupName"));
            cell17.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell17 = row16.createCell(2);
            cell17.setCellValue(obj.getString("hostName"));
            cell17.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell17 = row16.createCell(3);
            cell17.setCellValue(obj.getString("ip"));
            cell17.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell17 = row16.createCell(4);
            cell17.setCellValue(obj.getString("passHost"));
            cell17.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell17 = row16.createCell(5);
            cell17.setCellValue(obj.getString("noPassHost"));
            cell17.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell17 = row16.createCell(6);
            cell17.setCellValue(obj.getString("noPassHost"));
            cell17.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        }

        // ------------------------- 合规详情 ---------------------- -------------
        int hostSize = hostDetailArray.size();
        HSSFRow row17 = sheet.createRow(18 + itemSize + hostSize + count);
        HSSFCell cell18 = row17.createCell(0);
        cell18.setCellValue("合规详情");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(1);
        cell18.setCellValue("资源组名称");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(2);
        cell18.setCellValue("主机名称");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(3);
        cell18.setCellValue("基线项名称");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(4);
        cell18.setCellValue("主机IP");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(5);
        cell18.setCellValue("当前值");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(6);
        cell18.setCellValue("基线标准");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell18 = row17.createCell(7);
        cell18.setCellValue("合规状态");
        cell18.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        java.util.List<JSONObject> complianceDetailArray = record.getComplianceDetail();
        for (int i = 0; i < complianceDetailArray.size(); i++) {
            JSONObject obj = complianceDetailArray.get(i);
            count += 1;
            HSSFRow row18 = sheet.createRow(18 + count + itemSize + hostSize);
            HSSFCell cell19 = row18.createCell(1);

            cell19.setCellValue(obj.getString("groupName"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row18.createCell(2);
            cell19.setCellValue(obj.getString("hostName"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row18.createCell(3);
            cell19.setCellValue(obj.getString("itemName"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row18.createCell(4);
            cell19.setCellValue(obj.getString("ip"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row18.createCell(5);
            cell19.setCellValue(obj.getString("result"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row18.createCell(6);
            cell19.setCellValue(obj.getString("rule"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row18.createCell(7);
            cell19.setCellValue(obj.getString("pass"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
        }

        // ------------------------- 错误详情 ---------------------- -------------
        int complianceSize = complianceDetailArray.size();
        HSSFRow row22 = sheet.createRow(20 + itemSize + hostSize + complianceSize + count);
        HSSFCell cell22 = row22.createCell(0);
        cell22.setCellValue("错误详情");
        cell22.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell22 = row22.createCell(1);
        cell22.setCellValue("资源组名称");
        cell22.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell22 = row22.createCell(2);
        cell22.setCellValue("主机名称");
        cell22.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell22 = row22.createCell(3);
        cell22.setCellValue("基线项名称");
        cell22.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell22 = row22.createCell(4);
        cell22.setCellValue("主机IP");
        cell22.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        cell22 = row22.createCell(5);
        cell22.setCellValue("错误信息");
        cell22.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

        List<JSONObject> errorDetailArray = record.getErrorDetail();

        for (int i = 0; i < errorDetailArray.size(); i++) {
            JSONObject obj = errorDetailArray.get(i);
            count += 1;
            HSSFRow row23 = sheet.createRow(20 + count + itemSize + hostSize + complianceSize);
            HSSFCell cell19 = row23.createCell(1);

            cell19.setCellValue(obj.getString("groupName"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row23.createCell(2);
            cell19.setCellValue(obj.getString("hostName"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row23.createCell(3);
            cell19.setCellValue(obj.getString("itemName"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row23.createCell(4);
            cell19.setCellValue(obj.getString("ip"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

            cell19 = row23.createCell(5);
            cell19.setCellValue(obj.getString("errorInfo"));
            cell19.setCellStyle(getStyle(workbook, NORMAL_FONT, false));

        }

        /*      sheet.addMergedRegion(new CellRangeAddress(14 + itemSize + hostSize + complianceSize, 14 + itemSize + hostSize + complianceSize + count, 0,
                0));*/

        workbook.write(outputStream);
    }

    /**
     * 标题或者第一列数据信息单元格样式
     *
     * @param fontHeight
     *            标题20， 第一列12，第二列12
     */
    private static HSSFCellStyle getStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold) {

        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }

    /**
     * @param cellValue
     *            单元格填充值
     * @param workbook
     *            工作表
     * @param cell
     *            单元格
     * @param colorIndex
     *            单元格字体颜色
     */
    private static void setStatus(HSSFWorkbook workbook, HSSFCell cell, String cellValue, short colorIndex) {
        cell.setCellValue(cellValue);
        HSSFFont font = workbook.createFont();
        font.setColor(colorIndex);
        HSSFCellStyle style = getStyle(workbook, NORMAL_FONT, false);
        style.setFont(font);
        cell.setCellStyle(style);
    }
}
