package com.bocloud.cop.schedule.inspection.report;

import java.awt.Color;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFPalette;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.bocloud.cop.entity.inspection.InspectRecordWarning;
import com.bocloud.cop.enums.InspectCategory;
import com.bocloud.cop.model.inspection.WarningRule;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * 巡检历史Pdf工具
 */
public class InspectRecordPdfUtil {

    public static void covertPdf(InspectRecord record, OutputStream outputStream)
        throws IOException, DocumentException {
        // 第一步，实例化一个document对象
        Document document = new Document();
        // 第三步,设置字符,宋体
        BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        Font titleFont = new Font(bfChinese, 16.0F, Font.NORMAL);
        Font contentFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        Font tableFont = new Font(bfChinese, 8.0F, Font.NORMAL);
        // 第四步，将pdf文件输出到磁盘
        PdfWriter.getInstance(document, outputStream);
        // 第五步，打开生成的pdf文件
        document.open();

        Font greenFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        greenFont.setColor(BaseColor.GREEN);
        Font redFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        redFont.setColor(BaseColor.RED);
        Font orangeFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        orangeFont.setColor(BaseColor.ORANGE);

        String title = "巡检信息";
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Element.ALIGN_CENTER);
        paragraph.add(new Chunk(title, titleFont).setLocalDestination(title));
        document.add(paragraph);
        document.add(new Paragraph("\n"));

        PdfPTable table = new PdfPTable(2);
        table.setHorizontalAlignment(Element.ALIGN_CENTER);
        float[] widths = {44f, 100f};
        table.setWidths(widths);

        PdfPCell cell;

        cell = new PdfPCell(new Paragraph("巡检名称", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getName(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("巡检分类", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(InspectCategory.valueOf(record.getCategory()).getName(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("巡检类型", contentFont));
        setAndAddCellStyle(table, cell);

        Integer recordType = record.getExecuteType();
        String typeText = "-";
        if (recordType == 0) {
            typeText = "快速巡检-脚本";
        } else if (recordType == 1) {
            typeText = "常规巡检";
        } else if (recordType == 2) {
            typeText = "定时巡检";
        }
        cell = new PdfPCell(new Paragraph(typeText, contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("执行人员", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph(record.getCreator(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("巡检状态", contentFont));
        setAndAddCellStyle(table, cell);

        String recordStatus = record.getStatus();
        if (recordStatus.equals(InspectRecord.Status.STARTING.name())) {
            cell = new PdfPCell(new Paragraph("启动中", contentFont));
        } else if (recordStatus.equals(InspectRecord.Status.RUNNING.name())) {
            cell = new PdfPCell(new Paragraph("执行中", contentFont));
        } else if (recordStatus.equals(InspectRecord.Status.SUCCESS.name())) {
            cell = new PdfPCell(new Paragraph("成功", greenFont));
        } else if (recordStatus.equals(InspectRecord.Status.ERROR.name())) {
            cell = new PdfPCell(new Paragraph("错误", redFont));
        } else {
            cell = new PdfPCell(new Paragraph("-", contentFont));
        }
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("告警状态", contentFont));
        setAndAddCellStyle(table, cell);

        Boolean recordWarning = record.getWarning();
        if (null == recordWarning) {
            cell = new PdfPCell(new Paragraph("--", greenFont));
        } else if (!recordWarning) {
            cell = new PdfPCell(new Paragraph("正常", greenFont));
        } else {
            cell = new PdfPCell(new Paragraph("告警", redFont));
        }
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("巡检时间", contentFont));
        setAndAddCellStyle(table, cell);

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        String createTime = simpleDateFormat.format(record.getGmtCreate());
        cell = new PdfPCell(new Paragraph(createTime, contentFont));
        setAndAddCellStyle(table, cell);

        // 错误开始
        List<InspectRecordItem> list = record.getRecordItems();

        cell = new PdfPCell(new Paragraph("", contentFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);
        if (null != list && !list.isEmpty()) {
            cell = new PdfPCell(new Paragraph("执行详情", contentFont));
            cell.setColspan(2);
            setAndAddCellStyle(table, cell);
            PdfPTable detailTable = new PdfPTable(9);
            float[] floats = {16f, 16f, 16f, 16f, 16f, 16f, 16f, 16f, 16f};
            detailTable.setWidths(floats);
            cell = new PdfPCell(getDetailInfo(list, detailTable, tableFont));
            cell.setColspan(2);
            setAndAddCellStyle(table, cell);
        }
        if (null != record.getWarning() && record.getWarning()) {
            cell = new PdfPCell(new Paragraph("告警信息", contentFont));
            cell.setColspan(2);
            setAndAddCellStyle(table, cell);
            PdfPTable warningTable = new PdfPTable(8);
            float[] warningWidth = {18f, 18f, 18f, 18f, 18f, 18f, 18f, 18f};
            warningTable.setWidths(warningWidth);
            cell = new PdfPCell(getWarningInfo(record.getRecordWarnings(), warningTable, tableFont));
            cell.setColspan(2);
            setAndAddCellStyle(table, cell);
        }

        if (record.getStatus().equals("ERROR")) {
            cell = new PdfPCell(new Paragraph("错误信息", contentFont));
            cell.setColspan(2);
            setAndAddCellStyle(table, cell);
            PdfPTable errorTable = new PdfPTable(5);
            float[] errorWidth = {28f, 28f, 28f, 28f, 28f};
            errorTable.setWidths(errorWidth);
            cell = new PdfPCell(getErrorInfo(list, errorTable, tableFont));
            cell.setColspan(2);
            setAndAddCellStyle(table, cell);
        }

        // 添加大表格
        document.add(table);
        // 第七步，关闭document
        document.close();
    }

    private static PdfPTable getWarningInfo(List<InspectRecordWarning> list, PdfPTable warningTable, Font fontZH)
        throws IOException, DocumentException {
        PdfPCell cell;
        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机ip", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("指标项", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("指标值", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("告警", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("规则", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("命中规则", fontZH));
        setCellCenter(cell);
        warningTable.addCell(cell);

        for (InspectRecordWarning item : list) {
            cell = new PdfPCell(new Paragraph(item.getGroupName(), fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);
            cell = new PdfPCell(new Paragraph(
                StringUtils.isEmpty(item.getResourceName()) ? item.getIp() : item.getResourceName(), fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(item.getIp(), fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            String itemName = StringUtils.isEmpty(item.getInspectKey()) ? item.getItemName()
                : item.getItemName() + "_" + item.getInspectKey();
            cell = new PdfPCell(new Paragraph(itemName, fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            String inspectValue = StringUtils.isEmpty(item.getValueName()) ? item.getValue()
                : item.getValueName() + ":" + item.getValue();
            cell = new PdfPCell(new Paragraph(inspectValue, fontZH));
            setCellCenter(cell);
            warningTable.addCell(cell);

            String rule = item.getTargetRule();
            if (StringUtils.isEmpty(rule)) {
                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);
            } else {
                BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                Font colorfulFont = new Font(bfChinese, 8.0F, Font.NORMAL);

                // {"color":"#E50F5D","description":"二级告警","rule":"GT","threshold":"55%","warnLevel":2}

                // 告警标记
                JSONObject ruleJson = JSONObject.parseObject(rule);
                String color16 = ruleJson.getString("color").replace("#", "");
                Integer color = Integer.parseInt(color16, 16) + (255 << 24);
                colorfulFont.setColor(new BaseColor(color));
                String description = ruleJson.getString("description");
                cell = new PdfPCell(new Paragraph(description, colorfulFont));
                setCellCenter(cell);
                warningTable.addCell(cell);
                // 规则符号转文字
                String ruleStr = getRuleStr(ruleJson);
                cell = new PdfPCell(new Paragraph(StringUtils.isBlank(ruleStr) ? "-" : ruleStr, fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);

                // 命中规则
                cell = new PdfPCell(
                    new Paragraph(StringUtils.isBlank(item.getContent()) ? "-" : item.getContent(), fontZH));
                setCellCenter(cell);
                warningTable.addCell(cell);
            }

        }
        return warningTable;
    }

    private static PdfPTable getErrorInfo(List<InspectRecordItem> list, PdfPTable detailTable, Font fontZH) {

        PdfPCell cell;

        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机ip", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("指标项", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("错误信息", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        for (InspectRecordItem item : list) {
            if (!item.getStatus().equals("ERROR")) {
                continue;
            }
            cell = new PdfPCell(new Paragraph(item.getGroupName(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);
            cell = new PdfPCell(
                new Paragraph(StringUtils.isEmpty(item.getHostName()) ? item.getIp() : item.getHostName(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(item.getIp(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);
            cell = new PdfPCell(new Paragraph(item.getItemName(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            String scriptErrorInfo = item.getScriptErrorInfo();
            cell = new PdfPCell(new Paragraph(scriptErrorInfo, fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);
        }
        return detailTable;
    }

    private static PdfPTable getDetailInfo(List<InspectRecordItem> list, PdfPTable detailTable, Font fontZH)
        throws IOException, DocumentException {
        PdfPCell cell;
        // 2.0版本添加
        cell = new PdfPCell(new Paragraph("资源组名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("主机名称", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机ip", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("指标项", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("指标值", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("告警", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("规则", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);
        cell = new PdfPCell(new Paragraph("命中规则", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        cell = new PdfPCell(new Paragraph("错误信息", fontZH));
        setCellCenter(cell);
        detailTable.addCell(cell);

        for (InspectRecordItem item : list) {
            cell = new PdfPCell(new Paragraph(item.getGroupName(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);
            cell = new PdfPCell(
                new Paragraph(StringUtils.isEmpty(item.getHostName()) ? item.getIp() : item.getHostName(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            cell = new PdfPCell(new Paragraph(item.getIp(), fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

            String inspectKey = item.getItemName();
            String allWarnResult = item.getResult();
            List<JSONObject> jsonArrayResult = JSONArray.parseArray(allWarnResult, JSONObject.class);
            if (jsonArrayResult == null) {
                cell = new PdfPCell(new Paragraph(inspectKey, fontZH));
                setCellCenter(cell);
                detailTable.addCell(cell);
                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                detailTable.addCell(cell);
                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                detailTable.addCell(cell);
                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                detailTable.addCell(cell);
                cell = new PdfPCell(new Paragraph("-", fontZH));
                setCellCenter(cell);
                detailTable.addCell(cell);
            } else {
                PdfPCell iCell;
                PdfPTable inspectResultTable = new PdfPTable(2);
                float[] iwidth = {16f, 64f};
                inspectResultTable.setWidths(iwidth);
                for (int i = 0; i < jsonArrayResult.size(); i++) {
                    JSONObject jsonResult = jsonArrayResult.get(i);
                    String key = jsonResult.getString("key");
                    JSONArray entities = jsonResult.getJSONArray("entities");
                    List<JSONObject> ruleList = new ArrayList<>();
                    String ruleArr = jsonResult.getString("rules");
                    if (StringUtils.isNotBlank(ruleArr)) {
                        ruleList = JSONArray.parseArray(ruleArr, JSONObject.class);
                    }
                    // 添加告警，规则，阈值表格
                    iCell = new PdfPCell(
                        new Paragraph(StringUtils.isEmpty(key) ? inspectKey : inspectKey + "_" + key, fontZH));
                    setCellCenter(iCell);
                    // 插入指标项
                    inspectResultTable.addCell(iCell);

                    // 插入指标值 告警 规则 阈值
                    PdfPCell jCell;
                    PdfPTable entityTable = new PdfPTable(4);
                    float[] jwidth = {16f, 16f, 16f, 16f};
                    entityTable.setWidths(jwidth);
                    for (int j = 0; j < entities.size(); j++) {
                        JSONObject entity = entities.getJSONObject(j);
                        String value = entity.getString("value");
                        String valueName = entity.getString("valueName");// 可能为null
                        JSONObject warningRuleResult = entity.getJSONObject("warningRuleResult");
                        String warning = "-";
                        String rule = "-";
                        String content = "-";
                        Font colorfulFont = fontZH;

                        if (warningRuleResult != null) {
                            JSONObject targetRule = warningRuleResult.getJSONObject("targetRule");
                            if (targetRule != null) {
                                // color
                                BaseFont bfChinese =
                                    BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                                colorfulFont = new Font(bfChinese, 8.0F, Font.NORMAL);
                                String color16 = targetRule.getString("color").replace("#", "");
                                Integer color = Integer.parseInt(color16, 16) + (255 << 24);
                                colorfulFont.setColor(new BaseColor(color));
                                // warning
                                warning = targetRule.getString("description");
                                // 规则
                                rule = getRuleStr(ruleList);
                                // 获取命中规则信息
                                content = warningRuleResult.getString("warningInfo");
                            }
                        } else {
                            rule = getRuleStr(ruleList);
                            if (StringUtils.isNotBlank(rule)) {
                                content = "未命中规则";
                            }
                        }
                        // 插入指标值
                        jCell = new PdfPCell(
                            new Paragraph(StringUtils.isEmpty(valueName) ? value : valueName + ":" + value, fontZH));
                        setCellCenter(jCell);
                        entityTable.addCell(jCell);

                        // 告警
                        jCell = new PdfPCell(new Paragraph(warning, colorfulFont));
                        setCellCenter(jCell);
                        entityTable.addCell(jCell);

                        // 规则
                        jCell = new PdfPCell(new Paragraph(StringUtils.isBlank(rule) ? "-" : rule, fontZH));
                        setCellCenter(jCell);
                        entityTable.addCell(jCell);

                        // 命中规则
                        jCell = new PdfPCell(new Paragraph(StringUtils.isBlank(content) ? "-" : content, fontZH));
                        setCellCenter(jCell);
                        entityTable.addCell(jCell);

                    }
                    iCell = new PdfPCell(entityTable);
                    iCell.setColspan(4);
                    inspectResultTable.addCell(iCell);
                }
                cell = new PdfPCell(inspectResultTable);
                cell.setColspan(5);
                detailTable.addCell(cell);
            }

            String scriptErrorInfo = item.getScriptErrorInfo();
            cell = new PdfPCell(new Paragraph(scriptErrorInfo, fontZH));
            setCellCenter(cell);
            detailTable.addCell(cell);

        }
        return detailTable;
    }

    private static void setAndAddCellStyle(PdfPTable table, PdfPCell cell) {
        cell.setMinimumHeight(30.0F);
        cell.setPaddingTop(10f);
        cell.setPaddingBottom(10f);
        setCellCenter(cell);
        cell.setBorder(Rectangle.NO_BORDER);
        table.addCell(cell);
    }

    private static void setCellCenter(PdfPCell cell) {
        cell.setMinimumHeight(40);
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
    }

    private static final short BOLD_FONT = 20;
    private static final short NORMAL_FONT = 12;

    public static void covertExcel(InspectRecord record, OutputStream outputStream) throws IOException {

        HSSFWorkbook workbook = new HSSFWorkbook();

        HSSFSheet sheet = workbook.createSheet();
        sheet.setColumnWidth(0, 16 * 256);
        sheet.setColumnWidth(1, 20 * 256);
        sheet.setColumnWidth(2, 20 * 256);
        sheet.setColumnWidth(3, 40 * 256);
        sheet.setColumnWidth(4, 40 * 256);
        sheet.setColumnWidth(5, 40 * 256);
        sheet.setColumnWidth(6, 20 * 256);
        sheet.setColumnWidth(7, 20 * 256);
        sheet.setColumnWidth(8, 20 * 256);
        sheet.setColumnWidth(9, 40 * 256);

        HSSFCellStyle normalStyle = getStyle(workbook, NORMAL_FONT, false);
        HSSFCellStyle normalBoldStyle = getStyle(workbook, NORMAL_FONT, true);

        HSSFCellStyle normalStyleGreen = getStyle(workbook, NORMAL_FONT, false, HSSFColor.GREEN.index);
        HSSFCellStyle normalStyleRed = getStyle(workbook, NORMAL_FONT, false, HSSFColor.RED.index);
        HSSFCellStyle normalStyleOrange = getStyle(workbook, NORMAL_FONT, false, HSSFColor.ORANGE.index);

        Map<String, HSSFCellStyle> colorStyleMap = new HashMap<>();

        sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, 3));

        for (int i = 2; i <= 9; i++) {
            sheet.addMergedRegion(new CellRangeAddress(i, i, 1, 3));
        }

        HSSFRow row = sheet.createRow(0);
        HSSFCell cell = row.createCell(0);
        cell.setCellStyle(getStyle(workbook, BOLD_FONT, true));
        cell.setCellValue("巡检信息");

        HSSFRow row2 = sheet.createRow(2);
        HSSFCell cell2 = row2.createCell(0);
        cell2.setCellValue("巡检名称");
        cell2.setCellStyle(normalBoldStyle);
        cell2 = row2.createCell(1);
        cell2.setCellValue(record.getName());
        cell2.setCellStyle(normalStyle);

        HSSFRow row20 = sheet.createRow(3);
        HSSFCell cell20 = row20.createCell(0);
        cell20.setCellValue("巡检分类");
        cell20.setCellStyle(normalBoldStyle);
        cell20 = row20.createCell(1);
        cell20.setCellValue(InspectCategory.valueOf(record.getCategory()).getName());
        cell20.setCellStyle(normalStyle);

        HSSFRow row3 = sheet.createRow(4);
        HSSFCell cell3 = row3.createCell(0);
        cell3.setCellValue("巡检类型");
        cell3.setCellStyle(normalBoldStyle);
        cell3 = row3.createCell(1);

        Integer recordType = record.getExecuteType();
        if (recordType == 0) {
            cell3.setCellValue("快速巡检-脚本");
        } else if (recordType == 1) {
            cell3.setCellValue("常规巡检");
        } else if (recordType == 2) {
            cell3.setCellValue("定时巡检");
        } else {
            cell3.setCellValue("-");
        }
        cell3.setCellStyle(normalStyle);

        HSSFRow row4 = sheet.createRow(5);
        HSSFCell cell4 = row4.createCell(0);
        cell4.setCellValue("执行人员");
        cell4.setCellStyle(normalBoldStyle);
        cell4 = row4.createCell(1);
        cell4.setCellValue(record.getCreator());
        cell4.setCellStyle(normalStyle);

        HSSFRow row5 = sheet.createRow(6);
        HSSFCell cell5 = row5.createCell(0);
        cell5.setCellValue("巡检状态");
        cell5.setCellStyle(normalBoldStyle);
        cell5 = row5.createCell(1);

        String status = record.getStatus();
        if (status.equals(InspectRecord.Status.STARTING.name())) {
            cell5.setCellValue("启动中");
            cell5.setCellStyle(normalStyle);
        } else if (status.equals(InspectRecord.Status.RUNNING.name())) {
            cell5.setCellValue("执行中");
            cell5.setCellStyle(normalStyle);
        } else if (status.equals(InspectRecord.Status.SUCCESS.name())) {
            cell5.setCellStyle(normalStyleGreen);
            cell5.setCellValue("成功");
        } else if (status.equals(InspectRecord.Status.ERROR.name())) {
            cell5.setCellStyle(normalStyleRed);
            cell5.setCellValue("错误");
        } else {
            cell5.setCellValue("-");
            cell5.setCellStyle(normalStyle);
        }

        HSSFRow row6 = sheet.createRow(7);
        HSSFCell cell6 = row6.createCell(0);
        cell6.setCellValue("告警状态");
        cell6.setCellStyle(normalBoldStyle);
        cell6 = row6.createCell(1);
        if (null == record.getWarning()) {
            cell6.setCellStyle(normalStyleGreen);
            cell6.setCellValue("--");
        } else if (!record.getWarning()) {
            cell6.setCellStyle(normalStyleGreen);
            cell6.setCellValue("正常");
        } else {
            cell6.setCellStyle(normalStyleOrange);
            cell6.setCellValue("告警");
        }

        HSSFRow row7 = sheet.createRow(8);
        HSSFCell cell7 = row7.createCell(0);
        cell7.setCellValue("巡检时间");
        cell7.setCellStyle(normalBoldStyle);
        cell7 = row7.createCell(1);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        sdf.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
        cell7.setCellValue(sdf.format(record.getGmtCreate()));
        cell7.setCellStyle(normalStyle);

        // 嵌套
        List<InspectRecordWarning> warnings = record.getRecordWarnings();
        int warnSize = warnings.size();

        if (warnSize > 0) { // 如果有告警信息才显示对象

            sheet.addMergedRegion(new CellRangeAddress(12, 12 + warnSize, 0, 0));

            HSSFRow row10 = sheet.createRow(12);
            HSSFCell cell10 = row10.createCell(0);
            cell10.setCellValue("告警信息");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(1);
            cell10.setCellValue("资源组名称");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(2);
            cell10.setCellValue("主机名称");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(3);
            cell10.setCellValue("主机ip");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(4);
            cell10.setCellValue("指标项");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(5);
            cell10.setCellValue("指标值");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(6);
            cell10.setCellValue("告警");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(7);
            cell10.setCellValue("规则");
            cell10.setCellStyle(normalBoldStyle);
            cell10 = row10.createCell(8);
            cell10.setCellValue("命中规则");
            cell10.setCellStyle(normalBoldStyle);

            for (int i = 0; i < warnSize;) {
                InspectRecordWarning warning = warnings.get(i);
                i += 1;
                HSSFRow row11 = sheet.createRow(12 + i);
                HSSFCell cell11 = row11.createCell(1);
                cell11.setCellValue(warning.getGroupName());
                cell11.setCellStyle(normalStyle);

                cell11 = row11.createCell(2);
                cell11.setCellValue(
                    StringUtils.isEmpty(warning.getResourceName()) ? warning.getIp() : warning.getResourceName());
                cell11.setCellStyle(normalStyle);
                cell11 = row11.createCell(3);
                cell11.setCellValue(warning.getIp());
                cell11.setCellStyle(normalStyle);
                cell11 = row11.createCell(4);
                cell11.setCellValue(StringUtils.isEmpty(warning.getInspectKey()) ? warning.getItemName()
                    : warning.getItemName() + "_" + warning.getInspectKey());
                cell11.setCellStyle(normalStyle);
                cell11 = row11.createCell(5);
                cell11.setCellValue(StringUtils.isEmpty(warning.getValueName()) ? warning.getValue()
                    : warning.getValueName() + ":" + warning.getValue());
                cell11.setCellStyle(normalStyle);
                cell11 = row11.createCell(6);

                String rule = warning.getTargetRule();
                String description = "-";
                String ruleName = "-";
                String content = "-";
                String fontColor = null;
                Integer level = null;
                if (rule != null) {
                    JSONObject targetRule = JSONObject.parseObject(rule);

                    // color
                    fontColor = targetRule.getString("color").replace("#", "");
                    // warning
                    description = targetRule.getString("description");
                    // rule
                    String ruleStr = getRuleStr(targetRule);
                    ruleName = StringUtils.isBlank(ruleStr) ? "-" : ruleStr;
                    content = StringUtils.isBlank(warning.getContent()) ? "-" : warning.getContent();
                    level = targetRule.getInteger("warnLevel");
                }

                cell11.setCellValue(description);

                String styleKey = fontColor + "-" + level;
                HSSFCellStyle colorStyle = colorStyleMap.get(styleKey);
                if (colorStyle == null) {
                    colorStyle = getWarningStyle(workbook, NORMAL_FONT, false, fontColor, level);
                }

                cell11.setCellStyle(colorStyle);
                cell11 = row11.createCell(7);
                cell11.setCellValue(ruleName);
                cell11.setCellStyle(normalStyle);
                cell11 = row11.createCell(8);
                cell11.setCellValue(content);
                cell11.setCellStyle(normalStyle);
            }
        }

        // 错误开始
        List<InspectRecordItem> list = record.getRecordItems();
        if (null == list) {
            list = new ArrayList<>();
        }
        int listSize = list.size();
        int errorSize = 0;
        if (record.getStatus().equals("ERROR")) {
            HSSFRow row12 = sheet.createRow(13 + warnSize);
            HSSFCell cell12 = row12.createCell(0);
            cell12.setCellValue("错误信息");
            cell12.setCellStyle(normalBoldStyle);
            cell12 = row12.createCell(1);
            cell12.setCellValue("资源组名称");
            cell12.setCellStyle(normalBoldStyle);
            cell12 = row12.createCell(2);
            cell12.setCellValue("主机名称");
            cell12.setCellStyle(normalBoldStyle);
            cell12 = row12.createCell(3);
            cell12.setCellValue("主机ip");
            cell12.setCellStyle(normalBoldStyle);
            cell12 = row12.createCell(4);
            cell12.setCellValue("指标项");
            cell12.setCellStyle(normalBoldStyle);
            cell12 = row12.createCell(5);
            cell12.setCellValue("错误信息");
            cell12.setCellStyle(normalBoldStyle);

            for (int i = 0; i < listSize;) {
                InspectRecordItem item = list.get(i);
                i += 1;
                if (!item.getStatus().equals("ERROR")) {
                    continue;
                }
                errorSize++;

                HSSFRow row13 = sheet.createRow(13 + errorSize + warnSize);
                HSSFCell cell13 = row13.createCell(1);
                cell13.setCellValue(item.getGroupName());
                cell13.setCellStyle(normalStyle);

                cell13 = row13.createCell(2);
                cell13.setCellValue(StringUtils.isEmpty(item.getHostName()) ? item.getIp() : item.getHostName());
                cell13.setCellStyle(normalStyle);

                cell13 = row13.createCell(3);
                cell13.setCellValue(item.getIp());
                cell13.setCellStyle(normalStyle);

                cell13 = row13.createCell(4);
                cell13.setCellValue(item.getItemName());
                cell13.setCellStyle(normalStyle);

                cell13 = row13.createCell(5);
                String errorInfo = item.getScriptErrorInfo();
                cell13.setCellValue(errorInfo == null ? "-" : errorInfo);
                cell13.setCellStyle(normalStyle);
            }
            sheet.addMergedRegion(new CellRangeAddress(13 + warnSize, 13 + warnSize + errorSize, 0, 0));
        }
        // 详情不为空的时候显示
        if (null != list && !list.isEmpty()) {
            HSSFRow row13 = sheet.createRow(14 + warnSize + errorSize);
            HSSFCell cell14 = row13.createCell(0);
            cell14.setCellValue("详细信息");
            cell14.setCellStyle(normalBoldStyle);

            cell14 = row13.createCell(1);
            cell14.setCellValue("资源组名称");
            cell14.setCellStyle(normalBoldStyle);
            cell14 = row13.createCell(2);
            cell14.setCellValue("主机名称");
            cell14.setCellStyle(normalBoldStyle);

            cell14 = row13.createCell(3);
            cell14.setCellValue("主机ip");
            cell14.setCellStyle(normalBoldStyle);
            cell14 = row13.createCell(4);
            cell14.setCellValue("指标项");
            cell14.setCellStyle(normalBoldStyle);

            cell14 = row13.createCell(5);
            cell14.setCellValue("指标值");
            cell14.setCellStyle(normalBoldStyle);
            cell14 = row13.createCell(6);
            cell14.setCellValue("告警");
            cell14.setCellStyle(normalBoldStyle);
            cell14 = row13.createCell(7);
            cell14.setCellValue("规则");
            cell14.setCellStyle(normalBoldStyle);
            cell14 = row13.createCell(8);
            cell14.setCellValue("命中规则");
            cell14.setCellStyle(normalBoldStyle);

            cell14 = row13.createCell(9);
            cell14.setCellValue("错误信息");
            cell14.setCellStyle(normalBoldStyle);

            int count = 0;
            for (int i = 0; i < listSize; i++) {
                InspectRecordItem item = list.get(i);
                String inspectKey = item.getItemName();
                // 包含 告警，规则，阈值
                String allWarnResult = item.getResult();
                List<JSONObject> jsonArrayResult = JSONArray.parseArray(allWarnResult, JSONObject.class);
                if (jsonArrayResult == null) {
                    count += 1;
                    HSSFRow row14 = sheet.createRow(14 + count + warnSize + errorSize);
                    HSSFCell cell15 = row14.createCell(1);

                    cell15.setCellValue(item.getGroupName());
                    cell15.setCellStyle(normalStyle);

                    cell15 = row14.createCell(2);
                    cell15.setCellValue(StringUtils.isEmpty(item.getHostName()) ? item.getIp() : item.getHostName());
                    cell15.setCellStyle(normalStyle);

                    cell15 = row14.createCell(3);
                    cell15.setCellValue(item.getIp());
                    cell15.setCellStyle(normalStyle);
                    cell15 = row14.createCell(4);
                    cell15.setCellValue(item.getItemName());
                    cell15.setCellStyle(normalStyle);
                    cell15 = row14.createCell(5);
                    cell15.setCellValue(inspectKey);
                    cell15.setCellStyle(normalStyle);
                    cell15 = row14.createCell(6);
                    cell15.setCellValue("-");
                    cell15.setCellStyle(normalStyle);
                    cell15 = row14.createCell(7);
                    cell15.setCellValue("-");
                    cell15.setCellStyle(normalStyle);
                    cell15 = row14.createCell(8);
                    cell15.setCellValue("-");
                    cell15.setCellStyle(normalStyle);

                    cell15 = row14.createCell(9);
                    cell15.setCellValue(item.getScriptErrorInfo());
                    cell15.setCellStyle(normalStyle);
                } else {
                    int countBeforeJ = count + 1;
                    for (int j = 0; j < jsonArrayResult.size(); j++) {
                        JSONObject jsonResult = jsonArrayResult.get(j);
                        String key = jsonResult.getString("key");
                        List<JSONObject> ruleList = new ArrayList<>();
                        String ruleArr = jsonResult.getString("rules");
                        if (StringUtils.isNotBlank(ruleArr)) {
                            ruleList = JSONArray.parseArray(ruleArr, JSONObject.class);
                        }
                        JSONArray entities = jsonResult.getJSONArray("entities");
                        // 添加告警，规则，阈值表格
                        int countBeforeK = count + 1;
                        // 插入指标值 告警 规则 命中规则
                        for (int k = 0; k < entities.size(); k++) {
                            JSONObject entity = entities.getJSONObject(k);
                            String value = entity.getString("value");
                            String valueName = entity.getString("valueName");// 可能为null
                            JSONObject warningRuleResult = entity.getJSONObject("warningRuleResult");
                            String warning = "-";
                            String rule = "-";
                            String content = "-";
                            String fontColor = "";
                            Integer level = 0;
                            if (warningRuleResult != null) {
                                JSONObject targetRule = warningRuleResult.getJSONObject("targetRule");
                                if (targetRule != null) {
                                    // color
                                    fontColor = targetRule.getString("color").replace("#", "");
                                    // warning
                                    warning = targetRule.getString("description");
                                    // rule
                                    rule = getRuleStr(ruleList);
                                    content = warningRuleResult.getString("warningInfo");
                                    level = targetRule.getInteger("warnLevel");
                                }

                            } else {
                                rule = getRuleStr(ruleList);
                                if (StringUtils.isNotBlank(rule)) {
                                    content = "未命中规则";
                                }
                            }
                            count += 1;
                            HSSFRow row14 = sheet.createRow(14 + count + warnSize + errorSize);
                            HSSFCell cell15 = row14.createCell(1);

                            cell15.setCellValue(item.getGroupName());
                            cell15.setCellStyle(normalStyle);

                            cell15 = row14.createCell(2);
                            cell15.setCellValue(
                                StringUtils.isEmpty(item.getHostName()) ? item.getIp() : item.getHostName());
                            cell15.setCellStyle(normalStyle);

                            cell15 = row14.createCell(3);
                            cell15.setCellValue(item.getIp());
                            cell15.setCellStyle(normalStyle);
                            cell15 = row14.createCell(4);
                            cell15.setCellValue(
                                StringUtils.isEmpty(key) ? item.getItemName() : item.getItemName() + "_" + key);
                            cell15.setCellStyle(normalStyle);
                            cell15 = row14.createCell(5);
                            cell15.setCellValue(valueName == null ? value : valueName + ":" + value);
                            cell15.setCellStyle(normalStyle);
                            cell15 = row14.createCell(6);
                            cell15.setCellValue(warning);

                            String styleKey = fontColor + "-" + level;
                            HSSFCellStyle colorStyle = colorStyleMap.get(styleKey);
                            if (colorStyle == null) {
                                colorStyle = getWarningStyle(workbook, NORMAL_FONT, false, fontColor, level);
                            }

                            cell15.setCellStyle(colorStyle);
                            cell15 = row14.createCell(7);
                            cell15.setCellValue(rule);
                            cell15.setCellStyle(normalStyle);
                            cell15 = row14.createCell(8);
                            cell15.setCellValue(content);
                            cell15.setCellStyle(normalStyle);

                            cell15 = row14.createCell(9);
                            cell15.setCellValue(item.getScriptErrorInfo());
                            cell15.setCellStyle(normalStyle);
                            // 插入指标值

                        }
                        if (count > countBeforeK) {
                            sheet.addMergedRegion(new CellRangeAddress(14 + warnSize + errorSize + countBeforeK,
                                14 + warnSize + errorSize + count, 4, 4));
                        }

                    }
                    if (count > countBeforeJ) {
                        sheet.addMergedRegion(new CellRangeAddress(14 + warnSize + errorSize + countBeforeJ,
                            14 + warnSize + errorSize + count, 1, 1));
                        sheet.addMergedRegion(new CellRangeAddress(14 + warnSize + errorSize + countBeforeJ,
                            14 + warnSize + errorSize + count, 2, 2));
                        sheet.addMergedRegion(new CellRangeAddress(14 + warnSize + errorSize + countBeforeJ,
                            14 + warnSize + errorSize + count, 3, 3));
                        sheet.addMergedRegion(new CellRangeAddress(14 + warnSize + errorSize + countBeforeJ,
                            14 + warnSize + errorSize + count, 9, 9));
                    }

                }
            }
            sheet.addMergedRegion(
                new CellRangeAddress(14 + warnSize + errorSize, 14 + warnSize + errorSize + count, 0, 0));
        }
        workbook.write(outputStream);
    }

    /**
     * 标题或者第一列数据信息单元格样式
     *
     * @param fontHeight
     *            标题20， 第一列12，第二列12
     */
    private static HSSFCellStyle getStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold, short colorIndex) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");
        font.setColor(colorIndex);
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setFont(font);
        return style;
    }

    /**
     * 标题或者第一列数据信息单元格样式
     *
     * @param fontHeight
     *            标题20， 第一列12，第二列12
     */
    private static HSSFCellStyle getStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }

    /**
     * 从库里选几个颜色进行覆盖
     */
    private static short[] warningColors =
        {HSSFColor.GREY_25_PERCENT.index, HSSFColor.GREY_40_PERCENT.index, HSSFColor.GREY_50_PERCENT.index,
            HSSFColor.GREY_80_PERCENT.index, HSSFColor.LIGHT_CORNFLOWER_BLUE.index, HSSFColor.LEMON_CHIFFON.index,
            HSSFColor.ROYAL_BLUE.index, HSSFColor.AQUA.index, HSSFColor.TAN.index, HSSFColor.PALE_BLUE.index};

    private static HSSFCellStyle getWarningStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold,
        String colorString, Integer level) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");

        if ((!StringUtils.isEmpty(colorString)) && level != null && level > 0 && level <= 10) {
            String color16 = colorString.replace("#", "");
            Integer colorInt = Integer.parseInt(color16, 16);
            Color color = new Color(colorInt);
            // 获取工作簿的调色面板

            HSSFPalette palette = workbook.getCustomPalette();
            // 将调色面板中蓝色索引替换为红色
            short index = warningColors[level];
            palette.setColorAtIndex(index, (byte)color.getRed(), (byte)color.getGreen(), (byte)color.getBlue());
            // 会出现红色字体
            font.setColor(index);
        }
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }

    /**
     * 获取规则信息
     * 
     * @param list
     *            规则对象
     * @return 规则信息
     */
    private static String getRuleStr(List<JSONObject> list) {
        StringBuffer buffer = new StringBuffer();
        for (JSONObject obj : list) {
            buffer.append(getRuleStr(obj));
        }
        return buffer.toString();
    }

    /**
     * 规则转换规则名称
     *
     * @param ruleSign
     *            规则符号
     * @return 规则名称
     */
    private static String getRuleSignStr(String ruleSign) {
        return WarningRule.RULE.valueOf(ruleSign).getName();
    }

    /**
     * 获取规则信息
     *
     * @param obj
     *            规则对象
     * @return 规则信息
     */
    private static String getRuleStr(JSONObject obj) {
        StringBuffer buffer = new StringBuffer().append(getRuleSignStr(obj.getString("rule"))).append(" ")
            .append(obj.getString("threshold")).append(" ").append(obj.getString("description")).append(" ");
        return buffer.toString();
    }

}
