package com.bocloud.cop.schedule.inspection.report;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectReportDetail;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * 巡检报表Pdf工具
 *
 * @author jiaohongtao
 * @version 1.0
 * @since 2020年11月10日
 */
public class InspectReportPdfUtil {

    public static byte[] covertPdf(InspectReportDetail reportDetail) throws IOException, DocumentException {
        // 第一步，实例化一个document对象
        Document document = new Document();
        // 清空response，否则可能会对程序有影响，因为在此之前有可能也用到了response，若不reset，后面的setCntentoType()有可能失效
        // 第二步，设置要到出的路径
        // 如果是浏览器通过request请求需要在浏览器中输出则使用下面方式
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // OutputStream out = response.getOutputStream();
        // 第三步,设置字符,宋体
        BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        Font fontChinese = new Font(bfChinese, 12.0F, Font.NORMAL);
        // 第四步，将pdf文件输出到磁盘
        PdfWriter.getInstance(document, baos);
        // 第五步，打开生成的pdf文件
        document.open();

        Font greenFont = new Font(bfChinese);
        greenFont.setColor(BaseColor.GREEN);
        Font redFont = new Font(bfChinese);
        redFont.setColor(BaseColor.RED);
        Font orangeFont = new Font(bfChinese);
        orangeFont.setColor(BaseColor.ORANGE);

        // 设置总标题
        String title = reportDetail.getTitle();
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Element.ALIGN_CENTER);
        paragraph.add(new Chunk(title, fontChinese).setLocalDestination(title));
        document.add(paragraph);
        document.add(new Paragraph("\n"));

        PdfPTable table = new PdfPTable(2);
        table.setHorizontalAlignment(Element.ALIGN_CENTER);
        float[] widths = {20f, 90f};
        table.setWidths(widths);

        PdfPCell cell;

        String baseMsg = reportDetail.getBaseMsg();
        List<JSONObject> baseList = JSONArray.parseArray(baseMsg, JSONObject.class);

        // 设置基础信息
        for (JSONObject baseJson : baseList) {
            cell = new PdfPCell(new Paragraph(baseJson.getString("remark"), fontChinese));
            setAndAddCellStyle(table, cell);

            String value = baseJson.get("value").toString();
            cell = new PdfPCell(new Paragraph(value, fontChinese));
            setAndAddCellStyle(table, cell);
        }

        // 设置列表表头信息
        String listMsg = reportDetail.getListMsg();
        List<JSONObject> listTitleList = JSONArray.parseArray(listMsg, JSONObject.class);

        PdfPTable listTable = new PdfPTable(listTitleList.size());
        float[] floats = new float[listTitleList.size()];
        for (int i = 0; i < listTitleList.size(); i++) {
            floats[i] = 22f;
        }
        listTable.setWidths(floats);
        // 添加大表格
        document.add(table);
        document.add(getListInfo(reportDetail, listTable, fontChinese));
        // 第七步，关闭document
        document.close();
        return baos.toByteArray();
    }

    private static PdfPTable getListInfo(InspectReportDetail reportDetail, PdfPTable listTable, Font fontChinese) {

        PdfPCell cell;

        // 设置列表表头信息
        String listMsg = reportDetail.getListMsg();
        List<JSONObject> listTitleList = JSONArray.parseArray(listMsg, JSONObject.class);

        PdfPCell cellIp = null;
        PdfPCell cellItemName = null;
        PdfPCell cellRule = null;
        PdfPCell cellThreshold = null;
        PdfPCell cellInspectResult = null;
        PdfPCell cellWarning = null;

        List<String> titleCodes = new ArrayList<>();
        for (JSONObject list : listTitleList) {
            String code = list.getString("code");
            titleCodes.add(code);
            if ("ip".equals(code)) {
                cellIp = new PdfPCell(new Paragraph(list.get("name") + "", fontChinese));
                setCellCenter(cellIp);

            }
            if ("itemName".equals(code)) {
                cellItemName = new PdfPCell(new Paragraph(list.get("name") + "", fontChinese));
                setCellCenter(cellItemName);
            }
            if ("inspectResult".equals(code)) {
                cellInspectResult = new PdfPCell(new Paragraph(list.get("name") + "", fontChinese));
                setCellCenter(cellInspectResult);
            }
            if ("rule".equals(code)) {
                cellRule = new PdfPCell(new Paragraph(list.get("name") + "", fontChinese));
                setCellCenter(cellRule);
            }
            if ("threshold".equals(code)) {
                cellThreshold = new PdfPCell(new Paragraph(list.get("name") + "", fontChinese));
                setCellCenter(cellThreshold);
            }
            if ("warning".equals(code)) {
                cellWarning = new PdfPCell(new Paragraph(list.get("name") + "", fontChinese));
                setCellCenter(cellWarning);
            }
        }
        addCellExcludeNull(listTable, cellIp);
        addCellExcludeNull(listTable, cellItemName);
        addCellExcludeNull(listTable, cellRule);
        addCellExcludeNull(listTable, cellThreshold);
        addCellExcludeNull(listTable, cellInspectResult);
        addCellExcludeNull(listTable, cellWarning);

        // 设置列表内容信息
        List<JSONObject> listObjects = reportDetail.getListObjects();

        for (JSONObject listObj : listObjects) {
            // Set<String> listKeys = listObj.keySet();
            if (titleCodes.contains("ip")) {
                cell = new PdfPCell(new Paragraph(listObj.get("ip") + "", fontChinese));
                setCellCenter(cell);
                listTable.addCell(cell);
            }
            if (titleCodes.contains("itemName")) {
                cell = new PdfPCell(new Paragraph(listObj.get("itemName") + "", fontChinese));
                setCellCenter(cell);
                listTable.addCell(cell);
            }
            if (titleCodes.contains("inspectResult")) {
                String inspectResult = listObj.getString("inspectResult");
                cell = new PdfPCell(
                    new Paragraph(StringUtils.isNotBlank(inspectResult) ? inspectResult : " ", fontChinese));
                setCellCenter(cell);
                listTable.addCell(cell);
            }
            if (titleCodes.contains("rule")) {
                // String ruleValue = listObj.get("rule") + "";
                cell = new PdfPCell(new Paragraph(listObj.getString("rule"), fontChinese));
                setCellCenter(cell);
                listTable.addCell(cell);
            }
            if (titleCodes.contains("threshold")) {
                cell = new PdfPCell(new Paragraph(listObj.get("threshold") + "", fontChinese));
                setCellCenter(cell);
                listTable.addCell(cell);
            }
            if (titleCodes.contains("warning")) {
                String warning = listObj.getString("warning");
                cell = new PdfPCell(new Paragraph(StringUtils.isNotBlank(warning) ? warning : " ", fontChinese));
                setCellCenter(cell);
                listTable.addCell(cell);
            }
        }
        return listTable;
    }

    private static void addCellExcludeNull(PdfPTable table, PdfPCell cell) {
        if (cell != null) {
            table.addCell(cell);
        }
    }

    private static void setAndAddCellStyle(PdfPTable table, PdfPCell cell) {
        cell.setMinimumHeight(30.0F);
        cell.setPaddingTop(10f);
        cell.setPaddingBottom(10f);
        setCellCenter(cell);
        cell.setBorder(Rectangle.NO_BORDER);
        table.addCell(cell);
    }

    private static void setCellCenter(PdfPCell cell) {
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
    }

}
