package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.module.NodeModule;
import com.bocloud.cop.entity.module.TaskNodeModuleParam;
import com.bocloud.cop.entity.module.TaskNodeModuleParamItem;
import com.bocloud.cop.entity.module.TaskNodeModuleScript;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.intf.NodeModuleService;
import com.bocloud.cop.repository.module.NodeModuleRepository;
import com.bocloud.cop.repository.module.TaskNodeModuleParamItemRepository;
import com.bocloud.cop.repository.module.TaskNodeModuleParamRepository;
import com.bocloud.cop.repository.module.TaskNodeModuleScriptRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 编排模型service实现
 *
 * @author weiwei
 * @version 1.0
 * @since 2018年12月19日
 */
@Slf4j
@Service("nodeModuleService")
public class NodeModuleServiceImpl implements NodeModuleService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private NodeModuleRepository nodeModuleRepository;
    @Autowired
    private TaskNodeModuleScriptRepository taskNodeModuleScriptRepository;
    @Autowired
    private TaskNodeModuleParamItemRepository taskNodeModuleParamItemRepository;
    @Autowired
    private TaskNodeModuleParamRepository taskNodeModuleParamRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<NodeModule>> list(Pager pager) {
        List<NodeModule> list = null;
        int total = 0;
        GridBean<NodeModule> gridBean = null;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ZERO);
            Long groupId = null;
            for (com.megatron.common.model.Param pm : pager.getParams()) {
                Map<String, Object> param = pm.getParam();
                Object obj = param.get("groupId");
                if (null != obj) {
                    groupId = Long.valueOf(obj.toString());
                    param.remove("groupId");
                    break;
                }
            }
            total = nodeModuleRepository.count(pager.getParams(), groupId);
            list = nodeModuleRepository.list(pager, groupId);
            for (NodeModule module : list) {
                module.setCreator(redisTemplate.opsForValue().get("User_Name_" + module.getCreatorId()));
                if (null != module.getMenderId()) {
                    module.setMender(redisTemplate.opsForValue().get("User_Name_" + module.getMenderId()));
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list ros module failure:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<NodeModule> detail(Long id) {
        try {
            NodeModule module = nodeModuleRepository.query(id);
            // groups
            List<TaskNodeModuleScript> taskNodeModuleScripts = taskNodeModuleScriptRepository.queryByModuleId(id);
            module.setScripts(taskNodeModuleScripts);
            // taskNodeModuleParams
            List<TaskNodeModuleParam> taskNodeModuleParams = taskNodeModuleParamRepository.queryByModuleId(id);
            // taskNodeModuleParamItems
            Set<Long> paramIds =
                taskNodeModuleParams.stream().map(TaskNodeModuleParam::getId).collect(Collectors.toSet());
            if (!paramIds.isEmpty()) {
                List<TaskNodeModuleParamItem> taskNodeModuleParamItems =
                    taskNodeModuleParamItemRepository.queryByParamIds(paramIds);
                Map<Long, List<TaskNodeModuleParamItem>> itemsMap = taskNodeModuleParamItems.stream()
                    .collect(Collectors.groupingBy(TaskNodeModuleParamItem::getParamId));
                for (TaskNodeModuleParam taskNodeModuleParam : taskNodeModuleParams) {
                    taskNodeModuleParam.setParamItems(itemsMap.get(taskNodeModuleParam.getId()));
                }
            }
            module.setTaskNodeModuleParams(taskNodeModuleParams);
            return new GeneralResult<>(true, module, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<NodeModule> detailByCode(String code) {
        try {
            NodeModule module = nodeModuleRepository.getByCode(code);
            if (null == module) {
                log.warn("module data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            Long id = module.getId();
            // groups
            List<TaskNodeModuleScript> taskNodeModuleScripts = taskNodeModuleScriptRepository.queryByModuleId(id);
            module.setScripts(taskNodeModuleScripts);
            // taskNodeModuleParams
            List<TaskNodeModuleParam> taskNodeModuleParams = taskNodeModuleParamRepository.queryByModuleId(id);
            // taskNodeModuleParamItems
            Set<Long> paramIds =
                taskNodeModuleParams.stream().map(TaskNodeModuleParam::getId).collect(Collectors.toSet());
            if (!paramIds.isEmpty()) {
                List<TaskNodeModuleParamItem> taskNodeModuleParamItems =
                    taskNodeModuleParamItemRepository.queryByParamIds(paramIds);
                Map<Long, List<TaskNodeModuleParamItem>> itemsMap = taskNodeModuleParamItems.stream()
                    .collect(Collectors.groupingBy(TaskNodeModuleParamItem::getParamId));
                for (TaskNodeModuleParam taskNodeModuleParam : taskNodeModuleParams) {
                    taskNodeModuleParam.setParamItems(itemsMap.get(taskNodeModuleParam.getId()));
                }
            }
            module.setTaskNodeModuleParams(taskNodeModuleParams);
            return new GeneralResult<>(true, module, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> create(NodeModule module) {
        try {
            boolean result = nodeModuleRepository.exists(module.getName(), module.getGroupId());
            if (result) {
                return new GeneralResult<>(false, "此名称已存在!");
            }
            boolean hasCode = nodeModuleRepository.existsCode(module.getCode());
            if (hasCode) {
                return new GeneralResult<>(false, "此编码已存在!");
            }
            verify(module); // 校验参数
            module.setStatus(ApplyStatus.DEVELOPING.name());
            module.setId(null);
            nodeModuleRepository.save(module);
            // 处理自定义参数
            saveParams(module);
            return new GeneralResult<>(true, "添加模型成功");
        } catch (Exception e) {
            log.error("Create script fail:", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new GeneralResult<>(false, "添加模型失败," + e.getMessage());
        }

    }

    @Transactional
    @Override
    public GeneralResult<Void> modify(NodeModule module, Long userId) {
        String path = NodeModule.class.getSimpleName() + "_" + module.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NodeModule taskModule = nodeModuleRepository.query(module.getId());
            if (null == taskModule) {
                log.warn("Script data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            if (taskModule.getInternal()) {
                return new GeneralResult<>(false, "内置组件不允许修改!");
            }
            if (!taskModule.getName().equals(module.getName())) {
                boolean result = nodeModuleRepository.exists(module.getName(), module.getGroupId());
                if (result) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            if (!taskModule.getCode().equals(module.getCode())) {
                boolean result = nodeModuleRepository.existsCode(module.getCode());
                if (result) {
                    return new GeneralResult<>(false, "此编码已存在!");
                }
            }
            verify(module); // 校验参数
            taskModule.setMenderId(userId);
            taskModule.setCode(module.getCode());
            taskModule.setPreConstraints(module.getPreConstraints());
            taskModule.setPostConstraints(module.getPostConstraints());
            taskModule.setIcon(module.getIcon());
            taskModule.setName(module.getName());
            taskModule.setModuleType(module.getModuleType());
            taskModule.setStatus(ApplyStatus.DEVELOPING.name());
            taskModule.setScripts(module.getScripts());
            taskModule.setDefaultSetting(module.getDefaultSetting());
            taskModule.setTaskNodeModuleParams(module.getTaskNodeModuleParams());
            taskModule.setOutput(module.getOutput());
            Boolean result = nodeModuleRepository.update(taskModule);
            // 处理自定义参数
            saveParams(taskModule);
            return result ? new GeneralResult<>(true, "修改模型成功") : new GeneralResult<>(false, "修改模型失败");
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("Modify script fail:", e);
            return new GeneralResult<>(false, "修改模型失败," + e.getMessage());
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = NodeModule.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 移除数据
            // removePropertyGroup(id);
            result = nodeModuleRepository.remove(id, userId);
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("Remove script fail:", e);
            return new GeneralResult<>(false, "删除节点模型失败：" + e.getMessage());
        }
        return result ? new GeneralResult<>(true, "删除节点模型成功") : new GeneralResult<>(false, "删除节点模型失败");
    }

    @Override
    public List<NodeModule> listAll() {
        try {
            return nodeModuleRepository.list();
        } catch (Exception e) {
            return Lists.newArrayList();
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> settingDefaultParams(Long moduleId, List<TaskNodeModuleParam> taskNodeModuleParams) {
        String path = NodeModule.class.getSimpleName() + "_" + moduleId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NodeModule taskModule = nodeModuleRepository.query(moduleId);
            if (null == taskModule) {
                log.warn("data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            // 移除旧数据
            List<TaskNodeModuleParam> paramsList = taskNodeModuleParamRepository.queryByModuleId(moduleId);
            Set<Long> paramIds = paramsList.stream().map(TaskNodeModuleParam::getId).collect(Collectors.toSet());
            if (!paramIds.isEmpty()) {
                taskNodeModuleParamRepository.deleteByIds(paramIds);
                List<TaskNodeModuleParamItem> paramItemList =
                    taskNodeModuleParamItemRepository.queryByParamIds(paramIds);
                Set<Long> itemIds =
                    paramItemList.stream().map(TaskNodeModuleParamItem::getId).collect(Collectors.toSet());
                if (!itemIds.isEmpty()) {
                    taskNodeModuleParamItemRepository.deleteByIds(itemIds);
                }
            }
            if (!ListTool.isEmpty(taskNodeModuleParams)) {
                taskNodeModuleParams.stream().forEach(t -> {
                    t.setModuleId(moduleId);
                    t.setIsDefault(true);
                });
                // 插入新的param数据
                List<TaskNodeModuleParamItem> items = new ArrayList<>();
                for (TaskNodeModuleParam taskNodeModuleParam : taskNodeModuleParams) {
                    taskNodeModuleParamRepository.save(taskNodeModuleParam);
                    if (null != taskNodeModuleParam.getParamItems()) {
                        taskNodeModuleParam.getParamItems().stream()
                            .forEach(t -> t.setParamId(taskNodeModuleParam.getId()));
                        items.addAll(taskNodeModuleParam.getParamItems());
                    }
                }
                if (!ListTool.isEmpty(items)) {
                    // 插入item数据
                    for (TaskNodeModuleParamItem item : items) {
                        taskNodeModuleParamItemRepository.save(item);
                    }
                }
            }
            return new GeneralResult<>(true, "设置默认参数成功");
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("settingDefaultParams  fail:", e);
            return new GeneralResult<>(false, "设置默认参数失败：" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<TaskNodeModuleParam>> defaultInfo(String code) {
        NodeModule taskModule = nodeModuleRepository.getByCode(code);
        if (null == taskModule) {
            log.warn("module data not exist");
            return new GeneralResult<>(false, "数据不存在");
        }
        List<TaskNodeModuleParam> taskNodeModuleParams =
            taskNodeModuleParamRepository.queryByModuleId(taskModule.getId());
        Set<Long> paramIds = taskNodeModuleParams.stream().map(TaskNodeModuleParam::getId).collect(Collectors.toSet());
        // 处理integer类型数据返回
        for (TaskNodeModuleParam param : taskNodeModuleParams) {
            if ("INTEGER".equalsIgnoreCase(param.getType())) {
                param.setDefaultValue(
                    Integer.parseInt(param.getDefaultValue() == null ? "0" : param.getDefaultValue().toString()));
            }
        }
        if (!paramIds.isEmpty()) {
            List<TaskNodeModuleParamItem> items = taskNodeModuleParamItemRepository.queryByParamIds(paramIds);
            Map<Long, List<TaskNodeModuleParamItem>> itemMap =
                items.stream().collect(Collectors.groupingBy(TaskNodeModuleParamItem::getParamId));
            for (TaskNodeModuleParam taskNodeModuleParam : taskNodeModuleParams) {
                taskNodeModuleParam.setParamItems(itemMap.get(taskNodeModuleParam.getId()));
            }
        }
        return new GeneralResult<List<TaskNodeModuleParam>>(true, taskNodeModuleParams, "查询成功");
    }

    @Override
    public GeneralResult<Void> updateStatus(Long moduleId, String status, RequestContext context) {
        String path = NodeModule.class.getSimpleName() + "_" + moduleId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            nodeModuleRepository.updateStatus(moduleId, status, context.getTarget());
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("update module status fail:", e);
            return new GeneralResult<>(false, "操作失败");
        }
    }

    private void saveParams(NodeModule module) {
        log.info("saveParamsGroup module ->【{}】", JSONObject.toJSONString(module));
        Long moduleId = module.getId();
        List<TaskNodeModuleScript> taskNodeModuleScripts = module.getScripts();
        // 移除数据
        removeParamGroup(module.getId());
        // 插入group数据
        taskNodeModuleScripts.stream().forEach(t -> {
            t.setModuleId(moduleId);
            t.setId(null);
            taskNodeModuleScriptRepository.save(t);
        });
        // 处理参数
        List<TaskNodeModuleParam> taskNodeModuleParamList = module.getTaskNodeModuleParams();
        if (!ListTool.isEmpty(taskNodeModuleParamList)) {
            // 插入param数据
            List<TaskNodeModuleParamItem> items = new ArrayList<>();
            for (TaskNodeModuleParam taskNodeModuleParam : taskNodeModuleParamList) {
                taskNodeModuleParam.setModuleId(moduleId);
                taskNodeModuleParam.setId(null);
                taskNodeModuleParamRepository.save(taskNodeModuleParam);

                if (null != taskNodeModuleParam.getParamItems()) {
                    taskNodeModuleParam.getParamItems().stream()
                        .forEach(t -> {
                            t.setParamId(taskNodeModuleParam.getId());
                            t.setId(null);
                        });
                    items.addAll(taskNodeModuleParam.getParamItems());
                }
            }
            if (!ListTool.isEmpty(items)) {
                // 插入item数据
                taskNodeModuleParamItemRepository.batchSave(items, 1000);
            }

        }
    }

    /**
     * 删除分组，组件参数，参数的配置
     *
     * @param moduleId
     */
    private void removeParamGroup(Long moduleId) {
        // 分组
        List<TaskNodeModuleScript> groups = taskNodeModuleScriptRepository.queryByModuleId(moduleId);
        Set<Long> groupIds = groups.stream().map(TaskNodeModuleScript::getId).collect(Collectors.toSet());
        if (!groupIds.isEmpty()) {
            taskNodeModuleScriptRepository.deleteByIds(groupIds);
        }
        // 参数
        List<TaskNodeModuleParam> taskNodeModuleParams = taskNodeModuleParamRepository.queryByModuleId(moduleId);
        Set<Long> paramIds = taskNodeModuleParams.stream().map(TaskNodeModuleParam::getId).collect(Collectors.toSet());
        if (paramIds.isEmpty()) {
            return;
        }
        taskNodeModuleParamRepository.deleteByIds(paramIds);
        // 参数配置
        List<TaskNodeModuleParamItem> items = taskNodeModuleParamItemRepository.queryByParamIds(paramIds);
        Set<Long> itemIds = items.stream().map(TaskNodeModuleParamItem::getId).collect(Collectors.toSet());
        if (itemIds.isEmpty()) {
            return;
        }
        taskNodeModuleParamItemRepository.deleteByIds(itemIds);
    }

    private void verify(NodeModule module) {
        Assert.notNull(module, "参数不能为空");
        Assert.notNull(module.getName(), "组件名称不能为空");
        Assert.notNull(module.getGroupId(), "组件分类不能为空");
        Assert.notNull(module.getCode(), "组件编码不能为空");
        Assert.notNull(module.getModuleType(), "组件脚本配置不能为空");
        Assert.notNull(module.getIcon(), "组件图标不能为空");
        Assert.notNull(module.getScripts(), "组件参数组不能为空");
        List<TaskNodeModuleScript> taskNodeModuleScripts = module.getScripts();
        for (TaskNodeModuleScript taskNodeModuleScript : taskNodeModuleScripts) {
            Assert.notNull(taskNodeModuleScript.getContent(), "组件参数组[" + taskNodeModuleScript.getName() + "]脚本内容不能为空");
        }
    }

}
