package com.bocloud.cop.service;

import com.bocloud.cop.entity.ScriptContent;
import com.bocloud.cop.intf.ScriptContentService;
import com.bocloud.cop.repository.ScriptContentRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

/**
 * 脚本提交记录Service实现类
 *
 * @author wangshuai
 * @version 4.1.0-SNAPSHOT
 * @since 2018/12/25
 */
@Slf4j
@Service("scriptContentService")
public class ScriptContentServiceImpl implements ScriptContentService {

    @Autowired
    private ScriptContentRepository scriptContentRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<ScriptContent>> list(Long id , Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ZERO);
            pager.getParams().add(new Param(MapTools.simpleMap("scriptId", id),Sign.EQ));
            int total = scriptContentRepository.count(pager.getParams());
            List<ScriptContent> list = scriptContentRepository.list(pager);
            for (ScriptContent content : list) {
                content.setCreator(redisTemplate.opsForValue().get("User_Name_" + content.getCreatorId()));
            }
            GridBean<ScriptContent> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询脚本提交记录成功");
        } catch (Exception e) {
            log.error("List script content fail:", e);
            return new GeneralResult<>(false, "查询脚本提交记录失败");
        }
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ZERO);
            int total = scriptContentRepository.count(pager.getParams());
            List<SimpleBean> list = scriptContentRepository.list(pager.getParams(), pager.getSorter());
            GridBean<SimpleBean> bean = new GridBean<>(1, 1, total, list);
            return new GeneralResult<>(true, bean, "查询脚本提交记录成功");
        } catch (Exception e) {
            log.error("List script content fail:", e);
            return new GeneralResult<>(false, "查询脚本提交记录失败");
        }
    }

}
