package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Command;
import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.ScriptContent;
import com.bocloud.cop.entity.ScriptVersion;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.TaskTemplate;
import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.intf.CommandService;
import com.bocloud.cop.intf.ScriptService;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.model.FastTask;
import com.bocloud.cop.model.FastTaskItemScript;
import com.bocloud.cop.model.Input;
import com.bocloud.cop.model.Item;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.Role;
import com.bocloud.cop.model.ScriptBean;
import com.bocloud.cop.model.ScriptFront;
import com.bocloud.cop.model.ScriptInput;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.TaskGraph;
import com.bocloud.cop.repository.CommandRepository;
import com.bocloud.cop.repository.ScriptContentRepository;
import com.bocloud.cop.repository.ScriptRepository;
import com.bocloud.cop.repository.ScriptVersionRepository;
import com.bocloud.cop.repository.TaskTemplateRepository;
import com.bocloud.cop.repository.inspection.InspectItemRepository;
import com.bocloud.cop.service.internal.SmsInternalService;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 脚本抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
@Slf4j
@Service("scriptService")
public class ScriptServiceImpl implements ScriptService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ScriptRepository scriptRepository;
    @Autowired
    private ScriptContentRepository scriptContentRepository;
    @Autowired
    private ScriptVersionRepository scriptVersionRepository;
    @Autowired
    private TaskTemplateRepository taskTemplateRepository;
    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    private CommandRepository commandRepository;
    @Autowired
    private SmsInternalService smsInternalService;
    @Autowired
    private CommandService commandService;
    @Autowired
    private TaskInstanceService taskInstanceService;

    @Override
    public GeneralResult<GridBean<Script>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (pager.getSimple()) {
                pager.setRows(Integer.MAX_VALUE);
            }
            Long groupId = null;
            for (Param pm : pager.getParams()) {
                Map<String, Object> param = pm.getParam();
                Object obj = param.get("groupId");
                if (null != obj) {
                    groupId = Long.valueOf(obj.toString());
                    param.remove("groupId");
                    break;
                }
            }
            int total = scriptRepository.count(pager.getParams(), userId, groupId);
            List<Script> list = scriptRepository.list(pager, userId, groupId);
            GridBean<Script> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询脚本列表成功");
        } catch (Exception e) {
            log.error("List script fail:", e);
            return new GeneralResult<>(false, "查询脚本列表失败");
        }
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (pager.getSimple()) {
                pager.setRows(Integer.MAX_VALUE);
            }
            Long groupId = null;
            for (Param pm : pager.getParams()) {
                Map<String, Object> param = pm.getParam();
                Object obj = param.get("groupId");
                if (null != obj) {
                    groupId = Long.valueOf(obj.toString());
                    param.remove("groupId");
                    break;
                }
            }
            int total = scriptRepository.count(pager.getParams(), userId, groupId);
            List<SimpleBean> beans = scriptRepository.list(pager.getParams(), pager.getSorter(), userId, groupId);
            GridBean<SimpleBean> gridBean = new GridBean<>(1, 1, total, beans);
            return new GeneralResult<>(true, gridBean, "查询脚本列表成功");
        } catch (Exception e) {
            log.error("List script fail:", e);
            return new GeneralResult<>(false, "查询脚本列表失败");
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> create(Script script, Long userId) {
        try {
            boolean flag = scriptRepository.exists(script.getName(), script.getGroupId());
            if (flag) {
                return new GeneralResult<>(false, "该名称已存在");
            }
            // 先往script中写入数据
            script.setVersion("master");
            script.setStatus(ApplyStatus.DEVELOPING.name());
            script.setCreatorId(userId);
            script.setMenderId(userId);
            script.setGroupIds(JSONArray.toJSONString(script.getGroupIds()));
            scriptRepository.save(script);
            // 往script_version中写入数据
            ScriptVersion version = new ScriptVersion();
            version.setCreatorId(userId);
            version.setScriptId(script.getId());
            version.setVersion(script.getVersion());
            version.setMenderId(userId);
            scriptVersionRepository.save(version);
            // 往script_conmtent表中写入数据
            ScriptContent content = new ScriptContent();
            content.setContent(script.getContent());
            content.setCreatorId(userId);
            content.setMenderId(userId);
            content.setScriptId(script.getId());
            content.setVersionId(version.getId());
            content.setVersion(version.getVersion());
            scriptContentRepository.save(content);
            // 存放关联键数据
            script.setVersionId(version.getId());
            scriptRepository.update(script);
            version.setContentId(content.getId());
            scriptVersionRepository.update(version);
            return new GeneralResult<>(true, "脚本创建成功");
        } catch (Exception e) {
            log.error("Create script fail:", e);
            return new GeneralResult<>(false, "脚本创建失败");
        }

    }

    @Transactional
    @Override
    public GeneralResult<Void> modify(ScriptBean bean, Long userId) {
        String path = Script.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 这里是根据版本号获取的ID
            Script script = scriptRepository.query(bean.getId());
            if (null == script) {
                log.warn("Script data not exist");
                return new GeneralResult<>(false, "脚本信息不存在");
            }
            if (ApplyStatus.APPROVED.name().equalsIgnoreCase(script.getStatus())) {
                return new GeneralResult<>(false, "选择的脚本已发布，请先下架后再操作！");
            }
            if (!bean.getContent().equals(script.getContent())) {
                ScriptContent content = new ScriptContent();
                content.setContent(bean.getContent());
                content.setCreatorId(userId);
                content.setScriptId(script.getId());
                content.setVersionId(script.getVersionId());
                content.setMenderId(userId);
                content.setVersion(script.getVersion());
                scriptContentRepository.save(content);
                ScriptVersion version = scriptVersionRepository.query(script.getVersionId());
                if (null == version) {
                    version = new ScriptVersion();
                    version.setCreatorId(userId);
                    version.setScriptId(script.getId());
                    version.setVersion(script.getVersion());
                    version.setMenderId(userId);
                    version.setContentId(content.getId());
                    scriptVersionRepository.save(version);
                    content.setVersionId(version.getId());
                    scriptContentRepository.update(content);
                    script.setVersionId(version.getId());
                } else {
                    version.setContentId(content.getId());
                    version.setMenderId(userId);
                    version.setGmtModify(new Date());
                    scriptVersionRepository.update(version);
                }
            }
            bean.setVersionId(script.getVersionId());
            bean.setVersion(script.getVersion());
            BeanUtils.copyProperties(bean, script);
            script.setMenderId(userId);
            script.setStatus(ApplyStatus.DEVELOPING.name());
            script.setMenderId(userId);
            script.setGmtModify(new Date());
            script.setGroupIds(JSONArray.toJSONString(script.getGroupIds()));
            Boolean result = scriptRepository.update(script);
            return result ? new GeneralResult<>(true, "修改脚本成功") : new GeneralResult<>(false, "修改脚本失败");
        } catch (Exception e) {
            log.error("Modify script fail:", e);
            return new GeneralResult<>(false, "修改脚本异常");
        }
    }

    /**
     * @since 2018-05-24 modify by sxp 在作业中被引用的脚本，不允许删除
     */
    @Transactional
    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Script.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Script script = scriptRepository.query(id);
            if (ApplyStatus.APPROVED.name().equalsIgnoreCase(script.getStatus())) {
                return new GeneralResult<>(false, "选择的脚本已发布，请先下架后再操作！");
            }
            // 检查脚本是否被使用
            checkUsed(id);
            // 删除脚本
            this.scriptContentRepository.removeByScript(id, userId);
            this.scriptVersionRepository.removeByScript(id, userId);
            scriptRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除脚本成功");
        } catch (Exception e) {
            log.error("Remove script fail:", e);
            return new GeneralResult<>(false, "删除脚本失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<Script> detail(Long id) {
        try {
            Script script = scriptRepository.query(id);
            if (null == script) {
                log.error("Script data not exist");
                return new GeneralResult<>(false, "脚本不存在");
            }
            return new GeneralResult<>(true, script, "查询脚本详情成功");
        } catch (Exception e) {
            log.error("Query script detail fail:", e);
            return new GeneralResult<>(false, "查询脚本详情失败");
        }
    }

    @Override
    public Script get(Long id) {
        try {
            return scriptRepository.query(id);
        } catch (Exception e) {
            log.error("Query script detail fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = Script.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Script script = this.scriptRepository.query(id);
            if (null == script) {
                return new GeneralResult<>(false, "脚本不存在");
            }
            if (ApplyStatus.DEVELOPING.name().equalsIgnoreCase(status)) {
                // 检查脚本是否被使用
                checkUsed(id);
            }
            result = this.scriptRepository.updateStatus(id, status, userId);
        } catch (Exception e) {
            log.error("Update script status fail:", e);
            return new GeneralResult<>(false, "状态更新失败:" + e.getMessage());
        }
        return result ? new GeneralResult<>(true, "状态更新成功") : new GeneralResult<>(false, "状态更新失败");
    }

    @Transactional
    @Override
    public GeneralResult<Void> modifyVersion(Long id, Long userId) {
        ScriptVersion newVersion = null;
        try {
            newVersion = scriptVersionRepository.query(id);
            if (null == newVersion) {
                return new GeneralResult<>(false, "脚本版本不存在");
            }
        } catch (Exception e) {
            log.error("switch script version error:", e);
            return new GeneralResult<>(false, "版本切换失败");
        }
        String path = Script.class.getSimpleName() + "_" + newVersion.getScriptId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Script script = scriptRepository.query(newVersion.getScriptId());
            if (null == script) {
                log.error("Script data not exist");
                return new GeneralResult<>(false, "脚本不存在");
            }
            ScriptContent content = scriptContentRepository.query(newVersion.getContentId());
            if (null == content) {
                log.error("Script content data not exist");
                return new GeneralResult<>(false, "当前版本信息不存在");
            }
            script.setContent(content.getContent());
            script.setVersion(newVersion.getVersion());
            script.setVersionId(newVersion.getId());
            script.setMenderId(userId);
            script.setGmtModify(new Date());
            script.setStatus(ApplyStatus.DEVELOPING.name());
            scriptRepository.update(script);
            return new GeneralResult<>(true, "版本切换成功");
        } catch (Exception e) {
            log.error("switch script version error:", e);
            return new GeneralResult<>(false, "版本切换失败");
        }
    }

    private TaskNode taskNodeScript(int index, int num, ScriptFront scriptFront, Target target, String executeType,
        Long userId) {
        Input<ScriptFront> input = new ScriptInput();
        input.setTarget(target);
        input.setFronts(scriptFront);
        input.setExecuteType(executeType);

        TaskNode taskNode = new TaskNode();
        taskNode.setName("单脚本执行");
        taskNode.setCode("task.maintain.script");
        taskNode.setModuleType("task.maintain.script");
        taskNode.setInput(JSON.toJSONString(input));
        NodeControl control = new NodeControl(false, false, false, 3, 10, 3600);
        taskNode.setControl(JSON.toJSONString(control));
        NodeConfig config =
            new NodeConfig(UUID.randomUUID().toString(), index, 190, 180 + num * 100, "script.png", "maintain-node");
        taskNode.setConfig(JSON.toJSONString(config));
        taskNode.setNodeId(config.getId());
        return taskNode;
    }

    private TaskEdge taskEdgeScript(int index, String sourceUuid, String sourceAnchor, String targetUuid,
        String targetAnchor) {
        return new TaskEdge(UUID.randomUUID().toString(), index, "flow-edge", sourceUuid, sourceAnchor, targetUuid,
            targetAnchor, false, false);
    }

    @Override
    public TaskInstance fastExecute(FastTask task, RequestContext context) {
        // 创建任务实例
        TaskGraph graph = new TaskGraph();
        graph.setNodes(new ArrayList<>());
        graph.setEdges(new ArrayList<>());
        graph.setGroups(new ArrayList<>());
        Map<Item, TaskNode> itemMap = new HashMap<>();
        int index = 0;
        int num = 0;
        String sourceUuid = null;
        for (FastTaskItemScript itemScript : task.getItemScripts()) {
            TaskNode node = this.taskNodeScript(index, num, itemScript.getScriptFront(), task.getTarget(),
                task.getExecuteType(), context.getTarget());
            graph.getNodes().add(node);
            index++;
            num++;
            if (null != sourceUuid) {
                graph.getEdges().add(taskEdgeScript(index, sourceUuid, "bottom", node.getNodeId(), "top"));
                index++;
            }
            itemMap.put(itemScript.getItem(), node);
            sourceUuid = node.getNodeId();
        }
        TaskInstance instance = buildTaskInstance(task, graph, context);
        if (null == instance) {
            return null;
        }
        Map<String, Long> nodeMap = new HashMap<>();
        for (TaskNode node : instance.getNodes()) {
            nodeMap.put(node.getNodeId(), node.getId());
        }
        Map<Long, Long> relation = new HashMap<>();
        for (Map.Entry<Item, TaskNode> entry : itemMap.entrySet()) {
            relation.put(entry.getKey().getId(), nodeMap.get(entry.getValue().getNodeId()));
        }
        instance.setExtension(JSON.toJSONString(relation));
        return instance;
    }

    private TaskInstance buildTaskInstance(FastTask fastTask, TaskGraph graph, RequestContext context) {
        Task task = new Task();
        task.setName(fastTask.getName());
        task.setCode(fastTask.getCode());
        task.setGraph(JSONObject.toJSONString(graph));
        if (StringUtils.isEmpty(fastTask.getSource())) {
            fastTask.setSource("inspection");
        }
        task.setType(fastTask.getSource().toLowerCase());
        return taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
    }

    private void checkUsed(Long id) {
        // 巡检指标使用校验
        InspectItem inspectItem = inspectItemRepository.findByScriptId(id);
        if (null != inspectItem) {
            throw new IllegalArgumentException("脚本被巡检指标【" + inspectItem.getName() + "】引用无法操作");
        }
        // 作业模板引用校验
        List<TaskTemplate> templates =
            taskTemplateRepository.list(new Pager(1, Integer.MAX_VALUE, Lists.newArrayList(), null));
        for (TaskTemplate template : templates) {
            TaskGraph graph = JSONObject.parseObject(template.getGraph().toString(), TaskGraph.class);
            List<TaskNode> nodes = graph.getNodes();
            for (TaskNode node : nodes) {
                if (null != node.getModuleType() && node.getModuleType().contains("maintain.script")) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject fronts = input.getJSONObject("fronts");
                    Long scriptId = fronts.getLong("scriptId");
                    if (id.equals(scriptId)) {
                        throw new IllegalArgumentException("脚本被作业模板【" + template.getName() + "】引用无法操作");
                    }
                }
            }
        }
    }

    private GeneralResult<Void> commandHandler(Long userId, String content) {
        try {
            // 脚本里面含有高危命令的ID
            List<Long> commandIds = commandService.getCommands(content);
            if (commandIds.isEmpty()) {
                return new GeneralResult<>(true, "无高危命令");
            }
            // 用户无权执行命令的ID
            List<Long> ids = Lists.newArrayList();
            ids.addAll(commandIds);
            GeneralResult<List<Role>> result = smsInternalService.listByUid(userId);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            List<Role> roleList = result.getData();
            // 用户可以执行的高危命令
            List<Command> commands = Lists.newArrayList();
            for (Role role : roleList) {
                if (role.isChecked()) {
                    commands.addAll(commandRepository.listByRid(role.getId()));
                }

            }
            // 移除用户可以执行的
            for (Command command : commands) {
                for (Long commandId : commandIds) {
                    if (command.getId().equals(commandId)) {
                        ids.remove(commandId);
                    }
                }
            }
            if (!ids.isEmpty()) {
                return new GeneralResult<>(false, "脚本中存在限制执行的命令,您没有权限执行");
            }
            return new GeneralResult<>(true, "");
        } catch (Exception e) {
            log.error("command handler fail:", e);
            return new GeneralResult<>(false, "脚本高危命令处理异常");
        }
    }

    @Override
    public GeneralResult<Void> checkCommand(Task task, Long userId) {
        TaskGraph graph = JSONObject.parseObject(task.getGraph().toString(), TaskGraph.class);
        List<TaskNode> nodes = graph.getNodes();
        // 判断是否存在高危命令
        for (TaskNode node : nodes) {
            if (node.getModuleType().contains("maintain.script")) {
                GeneralResult<Void> result = commandHandler(userId, node.getInput());
                if (result.isFailed()) {
                    return result;
                }
            }
        }
        return new GeneralResult<>(true, "不存在高危命令");
    }

}
