package com.bocloud.cop.service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskTemplate;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.intf.StatsTaskService;
import com.bocloud.cop.model.ResultTrend;
import com.bocloud.cop.model.StatResult;
import com.bocloud.cop.model.TaskDigit;
import com.bocloud.cop.model.TaskStats;
import com.bocloud.cop.model.TrendData;
import com.bocloud.cop.repository.ScriptRepository;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskRepository;
import com.bocloud.cop.repository.TaskSceneRepository;
import com.bocloud.cop.repository.TaskTemplateRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 任务统计服务接口实现类
 *
 * @author 未伟
 */
@Slf4j
@Service("statsTaskService")
public class StatsTaskServiceImpl implements StatsTaskService {

    @Autowired
    private TaskRepository taskRepository;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private ScriptRepository scriptRepository;
    @Autowired
    private TaskTemplateRepository taskTemplateRepository;
    @Autowired
    private TaskSceneRepository taskSceneRepository;
    // @Autowired
    // private SocRoleService socRoleService;

    @Override
    public GeneralResult<TaskStats> statsDefault(String dataType, RequestContext context) {
        try {
            // 权限过滤
            // BaseResult<List<Role>> baseResult =
            // socRoleService.listByUid(context.getTarget());
            // if (baseResult.isFailed()) {
            // return new GeneralResult<>(false, baseResult.getMessage());
            // }
            // List<Role> roles = baseResult.getData();
            // boolean flag = true;
            // for (Role role : roles) {
            // if (role.isChecked() && role.getId().equals(1L)) {
            // flag = false;
            // }
            // }
            TaskDigit digit = getDigit(context, false);
            ResultTrend trend = getTrend(context, false);
            List<StatResult> duration = getDuration(context, false);
            List<StatResult> result = getResult(context, false);
            ResultTrend templateTrend = getTemplateTrend(context, false);
            List<StatResult> script = getScript(context, false);
            List<StatResult> template = getTempDigit();
            List<StatResult> taskTop5 = getTaskTop5();
            TaskStats stats = new TaskStats(digit, trend, duration, result, templateTrend, script);
            stats.setTemplate(template);
            stats.setTaskTop5(taskTop5);
            return new GeneralResult<>(true, stats, "获取统计信息成功");
        } catch (Exception e) {
            log.error("Stats task error:", e);
            return new GeneralResult<>(false, "获取任务统计信息失败");
        }
    }

    @Override
    public GeneralResult<List<StatResult>> stats(String code, String dataType, RequestContext context) {
        try {
            List<StatResult> result = new ArrayList<>();
            if ("TaskTemplate".equalsIgnoreCase(code)) {
                result = getTempDigit();
            } else if ("TaskScript".equalsIgnoreCase(code)) {
                result = getScript(context, false);
            } else if ("TaskStats".equalsIgnoreCase(code)) {
                result = getStatsTask(context, false);
            } else if ("TaskTop5".equalsIgnoreCase(code)) {
                result = getTaskTop5();
            } else {
                return new GeneralResult<>(false, "参数错误");
            }
            return new GeneralResult<>(true, result, "获取脚本统计信息成功");
        } catch (Exception e) {
            e.printStackTrace();
            return new GeneralResult<>(false, "获取脚本统计信息成功");
        }
    }

    @Override
    public GeneralResult<ResultTrend> statsTrend(String dataType, RequestContext context) {
        try {
            return new GeneralResult<>(true, getTrend(context, false), "获取任务趋势图信息成功");
        } catch (Exception e) {
            log.error("Stats task error:", e);
            return new GeneralResult<>(false, "获取任务统计信息失败");
        }
    }

    @Override
    public GeneralResult<List<StatResult>> statsDuration(String dataType, RequestContext context) {
        try {
            return new GeneralResult<>(true, getDuration(context, false), "获取任务执行时长信息成功");
        } catch (Exception e) {
            log.error("Stats task error:", e);
            return new GeneralResult<>(false, "获取任务统计信息失败");
        }
    }

    @Override
    public GeneralResult<List<StatResult>> statsResult(String dataType, RequestContext context) {
        try {
            return new GeneralResult<>(true, getResult(context, false), "获取任务执行结果信息成功");
        } catch (Exception e) {
            log.error("Stats task error:", e);
            return new GeneralResult<>(false, "获取任务统计信息失败");
        }
    }

    @Override
    public GeneralResult<List<StatResult>> statsDataview(String dataType, RequestContext context) {
        try {
            return new GeneralResult<>(true, getDigit(dataType, context, false), "获取任务统计信息成功");
        } catch (Exception e) {
            log.error("Stats task error:", e);
            return new GeneralResult<>(false, "获取任务统计信息失败");
        }
    }

    private List<StatResult> getScript(RequestContext context, boolean general) throws Exception {
        List<StatResult> result = Lists.newArrayList();
        List<SimpleBean> beans = scriptRepository.list(Lists.newArrayList(), null);
        int creates = 0;
        int unapplys = 0;
        int approvings = 0;
        int approveds = 0;
        int refuseds = 0;
        int developing = 0;
        for (SimpleBean bean : beans) {
            if (ApplyStatus.CREATED.name().equalsIgnoreCase(bean.getValue())) {
                creates++;
            } else if (ApplyStatus.UNAPPLY.name().equalsIgnoreCase(bean.getValue())) {
                unapplys++;
            } else if (ApplyStatus.APPROVING.name().equalsIgnoreCase(bean.getValue())) {
                approvings++;
            } else if (ApplyStatus.APPROVED.name().equalsIgnoreCase(bean.getValue())) {
                approveds++;
            } else if (ApplyStatus.REFUSED.name().equalsIgnoreCase(bean.getValue())) {
                refuseds++;
            } else if (ApplyStatus.DEVELOPING.name().equalsIgnoreCase(bean.getValue())) {
                developing++;
            }
        }
        // result.add(new StatResult("新创建", creates));
        // result.add(new StatResult("未审批", unapplys));
        // result.add(new StatResult("审批中", approvings));
        result.add(new StatResult("已发布", approveds));
        // result.add(new StatResult("审批拒绝", refuseds));
        result.add(new StatResult("开发中", developing));
        return result;
    }

    private ResultTrend getTemplateTrend(RequestContext context, boolean general) throws Exception {
        List<String> keys = Lists.newArrayList();
        List<TrendData> values = Lists.newArrayList();
        Map<String, Integer> map = new HashMap<>();
        List<SimpleBean> beans = taskSceneRepository.list(Lists.newArrayList(), null);
        for (SimpleBean bean : beans) {
            map.put(bean.getName(), 0);
        }
        List<TaskTemplate> templates =
            taskTemplateRepository.list(new Pager(1, Integer.MAX_VALUE, Lists.newArrayList(), null));
        List<Integer> datas = Lists.newArrayList();
        for (TaskTemplate template : templates) {
            if (map.containsKey(template.getSceneName())) {
                int temps = map.get(template.getSceneName()) + 1;
                map.put(template.getSceneName(), temps);
            } else {
                map.put(template.getSceneName(), 1);
            }
        }
        for (Entry<String, Integer> entry : map.entrySet()) {
            keys.add(entry.getKey());
            datas.add(entry.getValue());
        }
        values.add(new TrendData("模板数量", datas));
        return new ResultTrend(keys, values);
    }

    private List<StatResult> getDigit(String type, RequestContext context, boolean general) throws Exception {
        List<StatResult> results = Lists.newArrayList();
        List<Param> params = Lists.newArrayList();
        if ("TaskOften".equals(type)) {
            // 常规任务总数
            params.add(new Param(MapTools.simpleMap("code", Catalog.Task.STANDARD.getValue()), Sign.LK));
            int generals = this.taskRepository.count(params);
            results.add(new StatResult("常规作业数", generals, "个"));
        } else if ("TaskMonth".equals(type)) {
            Calendar calendar = Calendar.getInstance();
            // 开始时间(当前时间前30天)
            calendar.add(Calendar.DATE, -29);
            int yearStart = calendar.get(Calendar.YEAR);
            int monthStart = calendar.get(Calendar.MONTH) + 1;
            int dayStart = calendar.get(Calendar.DAY_OF_MONTH);
            String monthsStart = monthStart < 10 ? "0" + monthStart : "" + monthStart;
            // 结束时间(当前时间)
            calendar.setTime(new Date());
            int yearEnd = calendar.get(Calendar.YEAR);
            int monthEnd = calendar.get(Calendar.MONTH) + 1;
            int dayEnd = calendar.get(Calendar.DAY_OF_MONTH);
            String monthsEnd = monthEnd < 10 ? "0" + monthEnd : "" + monthEnd;
            String startTime = yearStart + "-" + monthsStart + "-" + dayStart + " 00:00:00";
            String endTime = yearEnd + "-" + monthsEnd + "-" + dayEnd + " 23:59:59";
            // 任务当月成功数
            params.clear();
            params.add(new Param(MapTools.simpleMap("gmtCreate", startTime), Sign.GET));
            params.add(new Param(MapTools.simpleMap("gmtCreate", endTime), Sign.LET));
            params.add(new Param(MapTools.simpleMap("status", Status.Inst.SUCCESS.name()), Sign.EQ));
            if (general) {
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            int success = this.taskInstanceRepository.count(params);
            // 任务当月失败数
            params.clear();
            params.add(new Param(MapTools.simpleMap("gmtCreate", startTime), Sign.GET));
            params.add(new Param(MapTools.simpleMap("gmtCreate", endTime), Sign.LET));
            params.add(new Param(MapTools.simpleMap("status", Status.Inst.FAILED.name()), Sign.EQ));
            if (general) {
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            int fails = this.taskInstanceRepository.count(params);
            // 任务当月执行中数
            params.clear();
            params.add(new Param(MapTools.simpleMap("gmtCreate", startTime), Sign.GET));
            params.add(new Param(MapTools.simpleMap("gmtCreate", endTime), Sign.LET));
            params.add(new Param(MapTools.simpleMap("status", Status.Inst.RUNNING.name()), Sign.EQ));
            if (general) {
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            int runs = this.taskInstanceRepository.count(params);
            // 任务当月取消数
            params.clear();
            params.add(new Param(MapTools.simpleMap("gmtCreate", startTime), Sign.GET));
            params.add(new Param(MapTools.simpleMap("gmtCreate", endTime), Sign.LET));
            params.add(new Param(MapTools.simpleMap("status", Status.Inst.CANCELED.name()), Sign.EQ));
            if (general) {
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            int cancels = this.taskInstanceRepository.count(params);
            // 任务当月暂停数
            params.clear();
            params.add(new Param(MapTools.simpleMap("gmtCreate", startTime), Sign.GET));
            params.add(new Param(MapTools.simpleMap("gmtCreate", endTime), Sign.LET));
            params.add(new Param(MapTools.simpleMap("status", Status.Inst.SUSPENDED.name()), Sign.EQ));
            if (general) {
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            int suspends = this.taskInstanceRepository.count(params);
            // 任务当月总数
            int monthTasks = success + fails + runs + cancels + suspends;
            results.add(new StatResult("本月执行任务数", monthTasks, "个"));
            results.add(new StatResult("执行中", runs, "个"));
            results.add(new StatResult("成功", success, "个"));
            results.add(new StatResult("失败", fails, "个"));
            results.add(new StatResult("取消", cancels, "个"));
            results.add(new StatResult("暂停", suspends, "个"));
        } else if ("TaskScheduler".equals(type)) {
            // 定时任务统计
            params.clear();
            params.add(new Param(MapTools.simpleMap("code", Catalog.Task.SCHEDULED.getValue()), Sign.EQ));
            if (general) {
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            }
            List<SimpleBean> cronBeans = this.taskRepository.list(params, null);
            int actives = 0;// 定时任务启动数
            int pauses = 0;// 定时任务暂停数
            int stops = 0;// 定时任务结束数
            for (SimpleBean bean : cronBeans) {
                if (Status.Inst.WAITTING.name().equalsIgnoreCase(bean.getValue())
                    || Status.Inst.RUNNING.name().equalsIgnoreCase(bean.getValue())) {
                    actives++;
                } else if (Status.Inst.SUSPENDED.name().equalsIgnoreCase(bean.getValue())) {
                    pauses++;
                } else if (Status.Inst.END.name().equalsIgnoreCase(bean.getValue())) {
                    stops++;
                }
            }
            // 定时任务未激活数
            int inactives = cronBeans.size() - actives - pauses - stops;
            results.add(new StatResult("定时任务数", cronBeans.size(), "个"));
            results.add(new StatResult("已激活", actives, "个"));
            results.add(new StatResult("未激活", inactives, "个"));
            results.add(new StatResult("暂停中", pauses, "个"));
            results.add(new StatResult("已结束", stops, "个"));
        }
        return results;
    }

    private List<StatResult> getTaskTop5() throws Exception {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.set(Calendar.DAY_OF_MONTH, 1);
        String start = new SimpleDateFormat("yyyy-MM-dd").format(cal.getTime()) + " 00:00:00";
        cal.roll(Calendar.DAY_OF_MONTH, -1);
        String end = new SimpleDateFormat("yyyy-MM-dd").format(cal.getTime()) + " 23:59:59";
        List<TaskInstance> instances = taskInstanceRepository.getInstanceNums(start, end);
        List<SimpleBean> tasks = taskRepository.list(new ArrayList<>(), null);
        List<StatResult> results = new ArrayList<>();
        if (ListTool.isEmpty(instances)) {
            for (SimpleBean bean : tasks) {
                results.add(new StatResult(bean.getName(), 0, 0D));
                if (results.size() == 5) {
                    break;
                }
            }
            return results;
        }
        Map<Long, String> taskMap = new HashMap<>();
        for (SimpleBean bean : tasks) {
            taskMap.put(bean.getId(), bean.getName());
        }
        long total = 0;
        for (TaskInstance instance : instances) {
            total = total + instance.getCount();
        }
        for (TaskInstance instance : instances) {
            if (!taskMap.containsKey(instance.getTaskId())) {
                continue;
            }
            double percent = new BigDecimal(instance.getCount() * 100)
                .divide(new BigDecimal(total), 2, RoundingMode.HALF_UP).doubleValue();
            results.add(new StatResult(taskMap.get(instance.getTaskId()), instance.getCount().intValue(), percent));
            if (results.size() == 5) {
                break;
            }
        }
        return results;
    }

    private List<StatResult> getTempDigit() throws Exception {
        List<Param> params = Lists.newArrayList();
        params.add(new Param(MapTools.simpleMap("internal", true), Sign.EQ));
        int internals = this.taskTemplateRepository.count(params);
        params.clear();
        params.add(new Param(MapTools.simpleMap("internal", false), Sign.EQ));
        int customs = this.taskTemplateRepository.count(params);
        List<StatResult> results = new ArrayList<StatResult>();
        results.add(new StatResult("自定义模板", customs));
        results.add(new StatResult("内置模板", internals));
        return results;
    }

    private TaskDigit getDigit(RequestContext context, boolean general) throws Exception {
        // 常规任务总数
        List<Param> params = Lists.newArrayList();
        params.add(new Param(MapTools.simpleMap("code", Catalog.Task.STANDARD.getValue()), Sign.EQ));
        List<SimpleBean> taskBeans = this.taskRepository.list(params, null);
        int approves = 0;
        int develops = 0;
        int approvings = 0;
        for (SimpleBean bean : taskBeans) {
            if (ApplyStatus.APPROVED.name().equalsIgnoreCase(bean.getValue())) {
                approves++;
            } else if (ApplyStatus.DEVELOPING.name().equalsIgnoreCase(bean.getValue())) {
                develops++;
            } else if (ApplyStatus.APPROVING.name().equalsIgnoreCase(bean.getValue())) {
                approvings++;
            }
        }
        int generals = approves + develops + approvings;
        // 任务当月成功数
        Calendar calendar = Calendar.getInstance();
        // 开始时间(当前时间前30天)
        calendar.add(Calendar.DATE, -29);
        int yearStart = calendar.get(Calendar.YEAR);
        int monthStart = calendar.get(Calendar.MONTH) + 1;
        int dayStart = calendar.get(Calendar.DAY_OF_MONTH);
        String monthsStart = monthStart < 10 ? "0" + monthStart : "" + monthStart;
        // 结束时间(当前时间)
        calendar.setTime(new Date());
        int yearEnd = calendar.get(Calendar.YEAR);
        int monthEnd = calendar.get(Calendar.MONTH) + 1;
        int dayEnd = calendar.get(Calendar.DAY_OF_MONTH);
        String monthsEnd = monthEnd < 10 ? "0" + monthEnd : "" + monthEnd;
        String startTime = yearStart + "-" + monthsStart + "-" + dayStart + " 00:00:00";
        String endTime = yearEnd + "-" + monthsEnd + "-" + dayEnd + " 23:59:59";

        params.clear();
        params.add(new Param(MapTools.simpleMap("gmtCreate", startTime), Sign.GET));
        params.add(new Param(MapTools.simpleMap("gmtCreate", endTime), Sign.LET));
        if (general) {
            params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
        }
        List<SimpleBean> beans = this.taskInstanceRepository.list(params, null);
        int success = 0;// 任务当月成功数
        int fails = 0;// 任务当月失败数
        int runs = 0;// 任务当月执行中数
        int cancels = 0;// 任务当月取消数
        int suspends = 0;// 任务当月暂停数
        for (SimpleBean bean : beans) {
            if (Status.Inst.SUCCESS.name().equalsIgnoreCase(bean.getValue())) {
                success++;
            } else if (Status.Inst.FAILED.name().equalsIgnoreCase(bean.getValue())) {
                fails++;
            } else if (Status.Inst.RUNNING.name().equalsIgnoreCase(bean.getValue())) {
                runs++;
            } else if (Status.Inst.CANCELED.name().equalsIgnoreCase(bean.getValue())) {
                cancels++;
            } else if (Status.Inst.SUSPENDED.name().equalsIgnoreCase(bean.getValue())) {
                suspends++;
            }
        }
        // 任务当月总数
        int monthTasks = success + fails + runs + cancels + suspends;
        // 定时任务总数
        params.clear();
        params.add(new Param(MapTools.simpleMap("code", Catalog.Task.SCHEDULED.getValue()), Sign.EQ));
        if (general) {
            params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
        }
        List<SimpleBean> cronBeans = this.taskRepository.list(params, null);
        int actives = 0;// 定时任务启动数
        int pauses = 0;// 定时任务暂停数
        int stops = 0;// 定时任务结束数
        for (SimpleBean bean : cronBeans) {
            if (Status.Inst.WAITTING.name().equalsIgnoreCase(bean.getValue())
                || Status.Inst.RUNNING.name().equalsIgnoreCase(bean.getValue())) {
                actives++;
            } else if (Status.Inst.SUSPENDED.name().equalsIgnoreCase(bean.getValue())) {
                pauses++;
            } else if (Status.Inst.END.name().equalsIgnoreCase(bean.getValue())) {
                stops++;
            }
        }
        // 定时任务未激活数
        int inactives = cronBeans.size() - actives - pauses - stops;
        TaskDigit digit = new TaskDigit(generals, success, fails, runs, cancels, suspends, monthTasks, actives, pauses,
            inactives, cronBeans.size());
        digit.setApproveds(approves);
        digit.setDevelops(develops);
        digit.setApprovings(approvings);
        return digit;
    }

    private List<StatResult> getStatsTask(RequestContext context, boolean general) throws Exception {
        // 常规任务总数
        List<Param> params = Lists.newArrayList();
        params.add(new Param(MapTools.simpleMap("code", Catalog.Task.STANDARD.getValue()), Sign.EQ));
        List<SimpleBean> taskBeans = this.taskRepository.list(params, null);
        int approves = 0;
        int develops = 0;
        int approvings = 0;
        for (SimpleBean bean : taskBeans) {
            if (ApplyStatus.APPROVED.name().equalsIgnoreCase(bean.getValue())) {
                approves++;
            } else if (ApplyStatus.DEVELOPING.name().equalsIgnoreCase(bean.getValue())) {
                develops++;
            } else if (ApplyStatus.APPROVING.name().equalsIgnoreCase(bean.getValue())) {
                approvings++;
            }
        }
        // 定时任务总数
        params.clear();
        params.add(new Param(MapTools.simpleMap("code", Catalog.Task.SCHEDULED.getValue()), Sign.EQ));
        List<SimpleBean> cronBeans = this.taskRepository.list(params, null);
        int actives = 0;// 定时任务启动数
        int pauses = 0;// 定时任务暂停数
        int stops = 0;// 定时任务结束数
        for (SimpleBean bean : cronBeans) {
            if (Status.Inst.WAITTING.name().equalsIgnoreCase(bean.getValue())
                || Status.Inst.RUNNING.name().equalsIgnoreCase(bean.getValue())) {
                actives++;
            } else if (Status.Inst.SUSPENDED.name().equalsIgnoreCase(bean.getValue())) {
                pauses++;
            } else if (Status.Inst.END.name().equalsIgnoreCase(bean.getValue())) {
                stops++;
            }
        }
        // 定时任务未激活数
        int inactives = cronBeans.size() - actives - pauses - stops;
        List<StatResult> results = new ArrayList<StatResult>();
        results.add(new StatResult("已发布", approves));
        results.add(new StatResult("开发中", develops));
        results.add(new StatResult("已激活", actives));
        results.add(new StatResult("未激活", inactives));
        results.add(new StatResult("暂停中", pauses));
        // results.add(new StatResult("审批中", approvings));
        return results;
    }

    private ResultTrend getTrend(RequestContext context, boolean general) throws Exception {
        List<String> keys = Lists.newArrayList();
        List<String> dateKeys = Lists.newArrayList();
        List<TrendData> values = Lists.newArrayList();
        List<Integer> runs = Lists.newArrayList();
        List<Integer> success = Lists.newArrayList();
        List<Integer> fails = Lists.newArrayList();
        List<Integer> cancels = Lists.newArrayList();
        List<Integer> suspends = Lists.newArrayList();
        List<Param> params = Lists.newArrayList();
        if (general) {
            params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
        }
        Calendar calendar = null;
        for (int i = 29; i >= 0; i--) {
            calendar = Calendar.getInstance();
            calendar.add(Calendar.DATE, -i);
            int year = calendar.get(Calendar.YEAR);
            int month = calendar.get(Calendar.MONTH) + 1;
            int day = calendar.get(Calendar.DAY_OF_MONTH);
            String months = month < 10 ? "0" + month : "" + month;
            String days = day < 10 ? "0" + day : "" + day;
            dateKeys.add(year + "-" + months + "-" + days);
            keys.add(months + "-" + days);
        }
        params.add(new Param(MapTools.simpleMap("gmtCreate", dateKeys.get(0) + " 00:00:00"), Sign.GET));
        params.add(new Param(MapTools.simpleMap("gmtCreate", dateKeys.get(keys.size() - 1) + " 23:59:59"), Sign.LET));
        List<TaskInstance> instances =
            this.taskInstanceRepository.list(new Pager(1, Integer.MAX_VALUE, params, null), false);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        Map<String, List<TaskInstance>> map = new HashMap<>();
        for (TaskInstance instance : instances) {
            String key = format.format(instance.getGmtCreate()).substring(0, 10);
            if (map.containsKey(key)) {
                List<TaskInstance> list = map.get(key);
                list.add(instance);
                map.put(key, list);
            } else {
                List<TaskInstance> list = new ArrayList<>();
                list.add(instance);
                map.put(key, list);
            }
        }
        for (String key : dateKeys) {
            int successes = 0;
            int faileds = 0;
            int runnings = 0;
            int canceleds = 0;
            int suspendeds = 0;
            List<TaskInstance> list = map.get(key);
            if (!ListTool.isEmpty(list)) {
                for (TaskInstance instance : list) {
                    if (Status.Inst.SUCCESS.name().equalsIgnoreCase(instance.getStatus())) {
                        successes++;
                    } else if (Status.Inst.FAILED.name().equalsIgnoreCase(instance.getStatus())) {
                        faileds++;
                    } else if (Status.Inst.RUNNING.name().equalsIgnoreCase(instance.getStatus())) {
                        runnings++;
                    } else if (Status.Inst.CANCELED.name().equalsIgnoreCase(instance.getStatus())) {
                        canceleds++;
                    } else if (Status.Inst.SUSPENDED.name().equalsIgnoreCase(instance.getStatus())) {
                        suspendeds++;
                    }
                }
            }
            success.add(successes);
            fails.add(faileds);
            runs.add(runnings);
            cancels.add(canceleds);
            suspends.add(suspendeds);
        }
        values.add(new TrendData("成功", success));
        values.add(new TrendData("失败", fails));
        values.add(new TrendData("执行中", runs));
        values.add(new TrendData("取消", cancels));
        values.add(new TrendData("暂停", suspends));
        return new ResultTrend(keys, values);
    }

    private List<StatResult> getDuration(RequestContext context, boolean general) throws Exception {
        int one = 0;
        int three = 0;
        int five = 0;
        int ten = 0;
        int thirty = 0;
        int hour = 0;
        List<Param> params = Lists.newArrayList();
        params.add(new Param(MapTools.simpleMap("status", Status.Inst.RUNNING.name()), Sign.UEQ));
        if (general) {
            params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
        }
        List<TaskInstance> list = taskInstanceRepository.list(new Pager(1, Integer.MAX_VALUE, params, null), false);
        for (TaskInstance instance : list) {
            if (null == instance.getGmtFinish() || null == instance.getGmtStart()) {
                continue;
            }
            BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
            BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
            double time = end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue();
            if (time <= 1 * 60) {
                one++;
            } else if (time > 1 * 60 && time <= 3 * 60) {
                three++;
            } else if (time > 3 * 60 && time <= 5 * 60) {
                five++;
            } else if (time > 5 * 60 && time <= 10 * 60) {
                ten++;
            } else if (time > 10 * 60 && time <= 30 * 60) {
                thirty++;
            } else if (time > 30 * 60) {
                hour++;
            }
        }
        List<StatResult> duration = Lists.newArrayList();
        duration.add(new StatResult("1分钟内", one));
        duration.add(new StatResult("1-3分钟内", three));
        duration.add(new StatResult("3-5分钟内", five));
        duration.add(new StatResult("5-10分钟内", ten));
        duration.add(new StatResult("10-30分钟内", thirty));
        duration.add(new StatResult("30分钟以上", hour));
        return duration;
    }

    private List<StatResult> getResult(RequestContext context, boolean general) throws Exception {
        List<Param> params = Lists.newArrayList();
        if (general) {
            params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
        }
        List<StatResult> result = Lists.newArrayList();
        List<SimpleBean> beans = this.taskInstanceRepository.list(params, null);
        int successes = 0;
        int faileds = 0;
        int runnings = 0;
        int canceleds = 0;
        int suspendeds = 0;
        for (SimpleBean bean : beans) {
            if (Status.Inst.SUCCESS.name().equalsIgnoreCase(bean.getValue())) {
                successes++;
            } else if (Status.Inst.FAILED.name().equalsIgnoreCase(bean.getValue())) {
                faileds++;
            } else if (Status.Inst.RUNNING.name().equalsIgnoreCase(bean.getValue())) {
                runnings++;
            } else if (Status.Inst.CANCELED.name().equalsIgnoreCase(bean.getValue())) {
                canceleds++;
            } else if (Status.Inst.SUSPENDED.name().equalsIgnoreCase(bean.getValue())) {
                suspendeds++;
            }
        }
        // 成功数
        result.add(new StatResult("成功", successes));
        // 失败数
        result.add(new StatResult("失败", faileds));
        // 执行中数
        result.add(new StatResult("执行中", runnings));
        // 取消数
        result.add(new StatResult("取消", canceleds));
        // 取消数
        result.add(new StatResult("暂停", suspendeds));
        return result;
    }

}
