package com.bocloud.cop.service;

import com.bocloud.cop.entity.TaskInstanceIndex;
import com.bocloud.cop.intf.TaskInstanceIndexService;
import com.bocloud.cop.model.ResultTrend;
import com.bocloud.cop.model.TrendData;
import com.bocloud.cop.repository.TaskInstanceIndexRepository;
import com.bocloud.cop.service.utils.DateUtils;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author lyy
 * @date 2023年04月04日 10:30
 */
@Slf4j
@Service("taskInstanceIndexService")
public class TaskInstanceIndexServiceImpl implements TaskInstanceIndexService {
    @Autowired
    private TaskInstanceIndexRepository taskInstanceIndexRepository;

    /**
     * 查询库里面按分钟统计的数据
     *
     * @return
     */
    @Override
    public GeneralResult<GridBean<TaskInstanceIndex>> list(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            String startTime = null;
            String endTime = null;
            for (Param pm : pager.getParams()) {
                Map<String, Object> param = pm.getParam();
                if (param.containsKey("gmtCreate")) {
                    if (null == param.get("gmtCreate")) {
                        continue;
                    }
                    String time = param.get("gmtCreate").toString();
                    if(Sign.GET.name().equals(pm.getSign().name())){
                        startTime = time;
                        param.remove("gmtCreate");
                    }else if(Sign.LET.name().equals(pm.getSign().name())){
                        endTime = time;
                        param.remove("gmtCreate");
                    }
                }
            }
            long period = 1;
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            if (StringUtils.isNotEmpty(startTime)) {
                LocalDateTime localDateTime = LocalDateTime.parse(startTime, formatter).withSecond(0);
                startTime = localDateTime.format(formatter);
                if (StringUtils.isEmpty(endTime)) {
                    LocalDateTime dateTimeEnd = localDateTime.plusHours(period);
                    endTime = dateTimeEnd.format(formatter);
                }
            }
            if (StringUtils.isNotEmpty(endTime)) {
                LocalDateTime localDateTime = LocalDateTime.parse(endTime, formatter).withSecond(0);
                endTime = localDateTime.format(formatter);
                if (StringUtils.isEmpty(startTime)) {
                    LocalDateTime dateTimeStart = localDateTime.minusHours(period);
                    startTime = dateTimeStart.format(formatter);
                }
            }
            //默认查询当前时间往前推一个小时之间的数据
            if (StringUtils.isEmpty(startTime) && StringUtils.isEmpty(endTime)) {
                LocalDateTime localDateTime = LocalDateTime.now().withSecond(0);
                endTime = localDateTime.format(formatter);
                startTime = localDateTime.minusHours(period).format(formatter);
            }
            pager.getParams().add(new Param(MapTools.simpleMap("countStartTime", startTime), Sign.GET));
            pager.getParams().add(new Param(MapTools.simpleMap("countEndTime", endTime), Sign.LET));
            //log.info("查询平台任务数据时间周期：开始时间-{}，结束时间-{}", startTime, endTime);
            GridBean<TaskInstanceIndex> bean = null;
            List<TaskInstanceIndex> list = taskInstanceIndexRepository.list(pager);
            Map<String, TaskInstanceIndex> map = new HashMap();
            for (TaskInstanceIndex index : list) {
                TaskInstanceIndex tempIndex = map.get(index.getName());
                if (tempIndex == null) {
                    tempIndex = new TaskInstanceIndex();
                    tempIndex.setId(index.getId());
                    tempIndex.setName(index.getName());
                    tempIndex.setInstanceNum(index.getInstanceNum() == null ? 0 : index.getInstanceNum());
                    tempIndex.setTargetHostNum(index.getTargetHostNum() == null ? 0 : index.getTargetHostNum());
                    tempIndex.setCountStartTime(DateUtils.stringToDate(startTime));
                    tempIndex.setCountEndTime(DateUtils.stringToDate(endTime));
                } else {
                    Long instanceNum = index.getInstanceNum() == null ? 0 : index.getInstanceNum();
                    Long targetHostNum = index.getTargetHostNum() == null ? 0 : index.getTargetHostNum();
                    tempIndex.setInstanceNum(tempIndex.getInstanceNum() + instanceNum);
                    tempIndex.setTargetHostNum(tempIndex.getTargetHostNum() + targetHostNum);
                }
                map.put(index.getName(), tempIndex);
            }
            int total = list.stream().collect(Collectors.groupingBy(TaskInstanceIndex::getName)).size();
            List<TaskInstanceIndex> returnList = map.values().stream().sorted(((o1, o2) -> -Long.compare(o1.getId(), o2.getId()))).skip((pager.getPage() - 1) * pager.getRows()).limit(pager.getRows()).collect(Collectors.toList());
            bean = new GridBean<>(pager.getPage(), pager.getRows(), total, returnList);
            return new GeneralResult<>(true, bean, "查询成功");
        } catch (Exception e) {
            log.error("List task instance index record fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> countIndex(String startTime, String endTime, Long userId, String period) {
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            //log.info("统计平台任务指标数据时间周期：开始时间-{}，结束时间-{}", startTime, endTime);
            if (StringUtils.isEmpty(period)) {
                //计算两个时间差值-分钟
                DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                LocalDateTime start = LocalDateTime.parse(startTime, dateTimeFormatter);
                LocalDateTime end = LocalDateTime.parse(endTime, dateTimeFormatter);
                Duration duration = Duration.between(start, end);
                period = duration.toMinutes() + "";
            }
            //查询平台信息
            List<TaskInstanceIndex> list = taskInstanceIndexRepository.countInstanceNum(startTime, endTime);
            for (TaskInstanceIndex index : list) {
                index.setCreatorId(userId);
                index.setPeriod(period);
                index.setCountStartTime(df.parse(startTime));
                index.setCountEndTime(df.parse(endTime));
                index.setTargetHostNum((taskInstanceIndexRepository.countTargetHostNum(index.getName(), startTime, endTime)));
            }
            if (CollectionUtils.isNotEmpty(list)) {
                taskInstanceIndexRepository.batchSave(list, 50);
            }
            return new GeneralResult<>(true, "统计成功");
        } catch (Exception e) {
            log.error("task instance index count failed:", e);
            return new GeneralResult<>(false, "统计失败");
        }
    }

    /**
     * @param periodSymbol 1D / 1H / 1M
     * @param name
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public GeneralResult<ResultTrend> applicationCount(String periodSymbol, String name, String startTime, String endTime) {
        try {
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            if (StringUtils.isNotEmpty(startTime)) {
                LocalDateTime localDateTime = LocalDateTime.parse(startTime, dateTimeFormatter);
                LocalDateTime localDateTimeStart = null;
                if ("1D".equals(periodSymbol)) {
                    localDateTimeStart = localDateTime.withHour(0).withMinute(0).withSecond(0);
                } else if ("1H".equals(periodSymbol)) {
                    localDateTimeStart = localDateTime.withMinute(0).withSecond(0);
                } else if ("1M".equals(periodSymbol)) {
                    localDateTimeStart = localDateTime.withSecond(0);
                }
                startTime = dateTimeFormatter.format(localDateTimeStart);
                if (StringUtils.isEmpty(endTime)) {
                    LocalDateTime localDateTimeEnd = null;
                    if ("1D".equals(periodSymbol)) {
                        localDateTimeEnd = localDateTimeStart.plusDays(30);
                    } else if ("1H".equals(periodSymbol)) {
                        localDateTimeEnd = localDateTimeStart.plusHours(24);
                    } else if ("1M".equals(periodSymbol)) {
                        localDateTimeEnd = localDateTimeStart.plusMinutes(60);
                    }
                    endTime = dateTimeFormatter.format(localDateTimeEnd);
                }
            }
            if (StringUtils.isNotEmpty(endTime)) {
                LocalDateTime localDateTime = LocalDateTime.parse(endTime, dateTimeFormatter);
                LocalDateTime localDateTimeEnd = null;
                if ("1D".equals(periodSymbol)) {
                    localDateTimeEnd = localDateTime.withHour(0).withMinute(0).withSecond(0);
                } else if ("1H".equals(periodSymbol)) {
                    localDateTimeEnd = localDateTime.withMinute(0).withSecond(0);
                } else if ("1M".equals(periodSymbol)) {
                    localDateTimeEnd = localDateTime.withSecond(0);
                }
                endTime = dateTimeFormatter.format(localDateTimeEnd);
                if (StringUtils.isEmpty(startTime)) {
                    LocalDateTime localDateTimeStart = null;
                    if ("1D".equals(periodSymbol)) {
                        localDateTimeStart = localDateTimeEnd.minusDays(30);
                    } else if ("1H".equals(periodSymbol)) {
                        localDateTimeStart = localDateTimeEnd.minusHours(24);
                    } else if ("1M".equals(periodSymbol)) {
                        localDateTimeStart = localDateTimeEnd.minusMinutes(60);
                    }
                    startTime = dateTimeFormatter.format(localDateTimeStart);
                }
            }
            if (StringUtils.isEmpty(startTime) && StringUtils.isEmpty(endTime)) {
                LocalDateTime localDateTime = LocalDateTime.now();
                LocalDateTime localDateTimeEnd = null;
                LocalDateTime localDateTimeStart = null;
                if ("1D".equals(periodSymbol)) {
                    localDateTimeEnd = localDateTime.withHour(0).withMinute(0).withSecond(0);
                    endTime = dateTimeFormatter.format(localDateTimeEnd);
                    localDateTimeStart = localDateTimeEnd.minusDays(30);
                    startTime = dateTimeFormatter.format(localDateTimeStart);
                } else if ("1H".equals(periodSymbol)) {
                    localDateTimeEnd = localDateTime.withMinute(0).withSecond(0);
                    endTime = dateTimeFormatter.format(localDateTimeEnd);
                    localDateTimeStart = localDateTimeEnd.minusHours(24);
                    startTime = dateTimeFormatter.format(localDateTimeStart);
                } else if ("1M".equals(periodSymbol)) {
                    localDateTimeEnd = localDateTime.withSecond(0);
                    endTime = dateTimeFormatter.format(localDateTimeEnd);
                    localDateTimeStart = localDateTimeEnd.minusMinutes(60);
                    startTime = dateTimeFormatter.format(localDateTimeStart);
                }
            }
            //log.info("查询平台任务数据时间周期：开始时间-{}，结束时间-{}", startTime, endTime);
            List<String> names = new ArrayList<>();
            if (StringUtils.isNotEmpty(name)) {
                names = Arrays.asList(name.split(","));
            }
            List<TaskInstanceIndex> list = taskInstanceIndexRepository.listApplicationCount(periodSymbol, names, startTime, endTime);
            //根据平台名称分组
            Map<String, List<TaskInstanceIndex>> taskInstanceIndexMap = list.stream().collect(Collectors.groupingBy(TaskInstanceIndex::getName));
            List<String> keys = Lists.newArrayList();
            List<String> dateKeys = Lists.newArrayList();
            List<TrendData> values = Lists.newArrayList();
            int calendarLength = 0;
            //计算两个时间差值
            LocalDateTime startDateTime = LocalDateTime.parse(startTime, dateTimeFormatter);
            LocalDateTime endDateTime = LocalDateTime.parse(endTime, dateTimeFormatter);
            Duration duration = Duration.between(startDateTime, endDateTime);
            if ("1D".equals(periodSymbol)) {
                calendarLength = Integer.valueOf(String.valueOf(duration.toDays()));
            } else if ("1H".equals(periodSymbol)) {
                calendarLength = Integer.valueOf(String.valueOf(duration.toHours()));
            } else if ("1M".equals(periodSymbol)) {
                calendarLength = Integer.valueOf(String.valueOf(duration.toMinutes()));
            }
            LocalDateTime localDateTime = null;
            for (int i = calendarLength; i >= 0; i--) {
                if ("1D".equals(periodSymbol)) {
                    localDateTime = endDateTime.minusDays(i);
                    String year = String.valueOf(localDateTime.getYear());
                    String month = localDateTime.getMonthValue() < 10 ? "0" + localDateTime.getMonthValue() : "" + localDateTime.getMonthValue();
                    String day = localDateTime.getDayOfMonth() < 10 ? "0" + localDateTime.getDayOfMonth() : "" + localDateTime.getDayOfMonth();
                    keys.add(year + "-" + month + "-" + day);
                } else if ("1H".equals(periodSymbol)) {
                    localDateTime = endDateTime.minusHours(i);
                    String year = String.valueOf(localDateTime.getYear());
                    String month = localDateTime.getMonthValue() < 10 ? "0" + localDateTime.getMonthValue() : "" + localDateTime.getMonthValue();
                    String day = localDateTime.getDayOfMonth() < 10 ? "0" + localDateTime.getDayOfMonth() : "" + localDateTime.getDayOfMonth();
                    String hour = localDateTime.getHour() < 10 ? "0" + localDateTime.getHour() : "" + localDateTime.getHour();
                    keys.add(year + "-" + month + "-" + day + " " + hour + ":" + "00");
                } else if ("1M".equals(periodSymbol)) {
                    localDateTime = endDateTime.minusMinutes(i);
                    String year = String.valueOf(localDateTime.getYear());
                    String month = localDateTime.getMonthValue() < 10 ? "0" + localDateTime.getMonthValue() : "" + localDateTime.getMonthValue();
                    String day = localDateTime.getDayOfMonth() < 10 ? "0" + localDateTime.getDayOfMonth() : "" + localDateTime.getDayOfMonth();
                    String hour = localDateTime.getHour() < 10 ? "0" + localDateTime.getHour() : "" + localDateTime.getHour();
                    String minutes = localDateTime.getMinute() < 10 ? "0" + localDateTime.getMinute() : "" + localDateTime.getMinute();
                    keys.add(year + "-" + month + "-" + day + " " + hour + ":" + minutes);
                }
                dateKeys.add(dateTimeFormatter.format(localDateTime));
            }
            for (String platformName : taskInstanceIndexMap.keySet()) {
                List<TaskInstanceIndex> taskInstanceIndexList = taskInstanceIndexMap.get(platformName);
                List<Integer> applicationTask = Lists.newArrayList();
                List<Integer> applicationHost = Lists.newArrayList();
                for (String key : dateKeys) {
                    TaskInstanceIndex taskInstanceIndex = taskInstanceIndexList.stream().filter(item -> dateTimeFormatter.format(DateUtils.dateToLocalDateTime(item.getCountStartTime())).equals(key)).findFirst().orElse(null);
                    if (null != taskInstanceIndex) {
                        //获取任务数据
                        Long instanceNum = taskInstanceIndex.getInstanceNum();
                        if (null != instanceNum) {
                            applicationTask.add(Integer.valueOf(instanceNum.toString()));
                        }
                        //获取机器数据
                        Long targetHostNum = taskInstanceIndex.getTargetHostNum();
                        if (null != targetHostNum) {
                            applicationHost.add(Integer.valueOf(targetHostNum.toString()));
                        }
                    } else {
                        applicationTask.add(0);
                        applicationHost.add(0);
                    }
                }
                values.add(new TrendData(platformName, applicationTask, "task"));
                values.add(new TrendData(platformName, applicationHost, "host"));
            }
            ResultTrend resultTrend = new ResultTrend(keys, values);
            return new GeneralResult<>(true, resultTrend, "获取应用统计信息成功");
        } catch (Exception e) {
            log.error("获取应用统计信息失败:", e);
            return new GeneralResult<>(false, "获取应用统计信息失败");
        }
    }
}
