package com.bocloud.cop.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.*;
import com.bocloud.cop.entity.backup.TaskBackupInstance;
import com.bocloud.cop.enums.*;
import com.bocloud.cop.enums.Status.Inst;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.model.*;
import com.bocloud.cop.model.params.DeployGaiaArgs;
import com.bocloud.cop.model.params.DeployGaiaParams;
import com.bocloud.cop.model.params.GaiaConfigParams;
import com.bocloud.cop.model.result.HostResult;
import com.bocloud.cop.model.result.TaskInstanceExecResult;
import com.bocloud.cop.repository.*;
import com.bocloud.cop.repository.backup.TaskBackupInstanceRepository;
import com.bocloud.cop.service.utils.DeployGaiaTaskBuilder;
import com.bocloud.cop.service.utils.NodeHelper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.enums.OSCategory;
import com.megatron.common.model.*;
import com.megatron.common.utils.*;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 记录抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.17
 */
@Slf4j
@Service("taskInstanceService")
public class TaskInstanceServiceImpl implements TaskInstanceService {

    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private TaskNodeLogRepository taskNodeLogRepository;
    @Autowired
    private CommandInstanceRepository commandInstanceRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private TaskInstanceJobRepository taskInstanceJobRepository;
    @Autowired
    private TaskInstanceHostRepository taskInstanceHostRepository;
    @Autowired
    private TaskFailRecordRepository taskFailRecordRepository;
    @Autowired
    private NodeRelationRepository nodeRelationRepository;
    @Autowired
    private IdentificationRepository identificationRepository;
    @Autowired
    private TaskBackupInstanceRepository taskBackupInstanceRepository;

    @Override
    public GeneralResult<GridBean<TaskInstance>> links(Pager pager, RequestContext context) {
        try {
            long startTime = System.currentTimeMillis();
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            String executeWorker = null;
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("executeWorker")) {
                    executeWorker = paramMap.get("executeWorker").toString();
                    paramMap.remove("executeWorker");
                    break;
                }
            }
            log.info("去除前端传参所用时间：" + (System.currentTimeMillis() - startTime) + "ms");
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<TaskInstance> list = taskInstanceRepository.linkQuery(pager, executeWorker);
            if (CollectionUtils.isEmpty(list)) {
                GridBean<TaskInstance> bean = new GridBean<>(pager.getPage(), pager.getRows(), 0, null);
                return new GeneralResult<>(true, bean, "查询实例执行链路成功");
            }
            List<Long> ids = list.stream().map(TaskInstance::getId).collect(Collectors.toList());
            List<TaskInstanceHost> taskInstanceHostList = taskInstanceHostRepository.listByInstances(ids);
            Map<Long, List<TaskInstanceHost>> instanceHostsMap = taskInstanceHostList.stream().collect(Collectors.groupingBy(TaskInstanceHost::getInstanceId));
            List<TaskNode> taskNodes = taskNodeRepository.listByInstances(ids);
            Map<Long, TaskNode> taskNodeMap = taskNodes.stream().collect(Collectors.toMap(TaskNode::getId, Function.identity()));
            log.info("查询到数据所用时间：" + (System.currentTimeMillis() - startTime) + "ms");
            for (TaskInstance instance : list) {
                if (null == instance.getGmtFinish()) {
                    instance.setCost(0.00);
                } else {
                    BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                    BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                    instance.setCost(
                            end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
                }
                List<TaskInstanceHost> taskInstanceHosts = instanceHostsMap.get(instance.getId());
                if (CollectionUtils.isEmpty(taskInstanceHosts)) {
                    continue;
                }
                for (TaskInstanceHost taskInstanceHost : taskInstanceHosts) {
                    TaskNode taskNode = taskNodeMap.get(taskInstanceHost.getNodeId());
                    if (null == taskNode) {
                        continue;
                    }
                    //处理连接用户和执行用户
                    dealUser(taskNode, taskInstanceHost);
                    if (!Status.Node.SUCCESS.name().equalsIgnoreCase(taskNode.getStatus())
                            && !Status.Node.FAILED.name().equalsIgnoreCase(taskNode.getStatus())) {
                        taskInstanceHost.setStatus(taskNode.getStatus());
                        continue;
                    }
                    if (StringUtils.isEmpty(taskNode.getOutput())) {
                        continue;
                    }
                    GeneralResult<?> result = JSONObject.parseObject(taskNode.getOutput(), GeneralResult.class);
                    List<HostResult> hostResults =
                            JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                    if (CollectionUtils.isEmpty(hostResults)) {
                        taskInstanceHost.setStatus(taskNode.getStatus());
                        continue;
                    }
                    HostResult hostResult = hostResults.stream()
                            .filter(item -> item.getHost().equals(taskInstanceHost.getHost())).findFirst().orElse(null);
                    if (null != hostResult) {
                        if (hostResult.isSuccess()) {
                            taskInstanceHost.setStatus(Status.Node.SUCCESS.name());
                        } else {
                            taskInstanceHost.setStatus(Status.Node.FAILED.name());
                        }
                    } else {
                        taskInstanceHost.setStatus(taskNode.getStatus());
                    }
                }
                instance.setTaskInstanceHosts(taskInstanceHosts);
            }
            log.info("组装数据所用时间：" + (System.currentTimeMillis() - startTime) + "ms");
            int total = taskInstanceRepository.linkCount(pager.getParams(), executeWorker);
            GridBean<TaskInstance> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            log.info("查询结束所用时间：" + (System.currentTimeMillis() - startTime) + "ms");
            return new GeneralResult<>(true, bean, "查询实例执行链路成功");
        } catch (Exception e) {
            log.error("List task links record fail:", e);
            return new GeneralResult<>(false, "查询实例执行链路失败");
        }
    }

    private void dealUser(TaskNode taskNode,TaskInstanceHost taskInstanceHost) {
        try {
            JSONObject input = JSONObject.parseObject(taskNode.getInput());
            JSONObject target = input.getJSONObject("target");
            if (null != target) {
                taskInstanceHost.setBecomeUser(target.getString("becomeUser"));
                List<JSONObject> hosts = JSONObject.parseArray(target.getString("data"), JSONObject.class);
                if (ListTool.isEmpty(hosts)) {
                    return;
                }
                for (JSONObject host : hosts) {
                    if (taskInstanceHost.getHost().equals(host.getString("host"))) {
                        taskInstanceHost.setUser(host.getString("user"));
                        if (host.containsKey("becomeUser")) {
                            taskInstanceHost.setBecomeUser(host.getString("becomeUser"));
                        }
                        return;
                    }
                }
            }
        } catch (Exception e) {
            log.error("处理主机连接用户和执行用户异常", e);
        }
    }

    @Override
    public GeneralResult<GridBean<TaskInstance>> list(Pager pager, RequestContext context) {
        List<TaskInstance> list = null;
        int total = 0;
        GridBean<TaskInstance> gridBean = null;
        try {
            String host = null;
            boolean await = false;
            Long backupId = null;
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            for (Param param : pager.getParams()) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("await")) {
                    await = Boolean.valueOf(paramMap.get("await").toString());
                    paramMap.remove("await");
                }
                // 配置对比->备份管理->备份历史
                if (paramMap.containsKey("backupId")) {
                    backupId = Long.valueOf(paramMap.get("backupId").toString());
                    paramMap.remove("backupId");
                }
                if (paramMap.containsKey("host")) {
                    host = paramMap.get("host").toString();
                    paramMap.remove("host");
                }
            }
            // 对于备份任务需要单独处理
            if (null != backupId) {
                List<TaskBackupInstance> taskBackupInstances = taskBackupInstanceRepository.listByBackupId(backupId);
                if (!ListTool.isEmpty(taskBackupInstances)) {
                    List<Long> ids = taskBackupInstances.stream().map(TaskBackupInstance::getTaskInstanceId)
                        .collect(Collectors.toList());
                    pager.getParams().add(new Param(MapTools.simpleMap("id", StringUtils.join(ids, ",")), Sign.IN));
                }
            }
            if (com.megatron.common.model.RequestContext.Catalog.Tenant == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            } else if (com.megatron.common.model.RequestContext.Catalog.User == context.getCatalog()) {
                pager.getParams().add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
                pager.getParams().add(new Param(MapTools.simpleMap("catalog", context.getCatalog()), Sign.EQ));
            }
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            if (await) {
                return listAwaits(pager, context);
            }
            list = taskInstanceRepository.list(pager, host);
            for (TaskInstance instance : list) {
                if (null == instance.getGmtFinish()) {
                    instance.setCost(0.00);
                } else {
                    BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                    BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                    instance.setCost(
                        end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
                }
                if (com.megatron.common.model.RequestContext.Catalog.Tenant == instance.getCatalog()) {
                    instance.setOperator(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getCreatorId()));
                } else {
                    instance.setOperator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId()));
                }
                if (null != instance.getTenantId() && instance.getTenantId() != 0) {
                    instance.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getTenantId()));
                }
                //执行方式
                if(StringUtils.isNotBlank(instance.getGraph())){
                    TaskGraph graph = JSONObject.parseObject(instance.getGraph(),TaskGraph.class);
                    List<TaskNode> nodes = graph.getNodes();
                    for (TaskNode node : nodes) {
                       if(StringUtils.isBlank(node.getConfig())){
                            continue;
                       }
                        NodeConfig config = JSONObject.parseObject(node.getConfig(),NodeConfig.class);
                       if("flow-circle".equals(config.getNodeType())){
                            continue;
                       }
                       JSONObject input = JSONObject.parseObject(node.getInput());
                       if(StringUtils.isNotBlank(node.getInput()) && null != input.getJSONObject("target")) {
                           instance.setExecuteType(input.getJSONObject("target").getString("executeType"));
                           break;
                       }
                    }
                }
            }
            total = taskInstanceRepository.count(pager.getParams(), host);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        } catch (Exception e) {
            log.error("List task record fail:", e);
            return new GeneralResult<>(false, "查询任务记录失败");
        }
        return new GeneralResult<>(true, gridBean, "查询任务记录成功");
    }

    @Override
    public GeneralResult<TaskInstance> detail(Long id) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            InstanceGraph graph = JSONObject.parseObject(instance.getGraph(), InstanceGraph.class);
            List<InstanceNode> graphNodes = graph.getNodes();
            Map<String, InstanceNode> nodeMap = Maps.newHashMap();
            for (InstanceNode taskNode : graphNodes) {
                NodeConfig config = JSONObject.parseObject(taskNode.getConfig(), NodeConfig.class);
                nodeMap.put(config.getId(), taskNode);
            }
            List<TaskNode> nodes = this.taskNodeRepository.listByInstance(instance.getId());
            List<JSONObject> results = Lists.newArrayList();
            for (TaskNode node : nodes) {
                if (nodeMap.containsKey(node.getNodeId())) {
                    InstanceNode taskNode = nodeMap.get(node.getNodeId());
                    taskNode.setId(node.getId());
                    taskNode.setStatus(node.getStatus());
                }
                if (null == node.getGmtFinish() || null == node.getGmtStart()) {
                    node.setCost(0.00);
                } else {
                    BigDecimal start = new BigDecimal(node.getGmtStart().getTime());
                    BigDecimal end = new BigDecimal(node.getGmtFinish().getTime());
                    node.setCost(
                        end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
                }
                if (node.getModuleType().contains("maintain.backup") && null != node.getOutput()) {
                    GeneralResult<?> result = JSONObject.parseObject(node.getOutput(), GeneralResult.class);
                    List<HostResult> hostResults =
                        JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                    if (!ListTool.isEmpty(hostResults)) {
                        for (HostResult hostResult : hostResults) {
                            if (hostResult.isFailed()) {
                                continue;
                            }
                            JSONObject object = new JSONObject();
                            object.put("host", hostResult.getHost());
                            JSONObject message = JSONTools.isJSONObj(hostResult.getMessage());
                            if (null == message) {
                                object.put("path", hostResult.getMessage());
                            } else {
                                object.put("path", message.getString("dest"));
                            }
                            results.add(object);
                        }
                    }
                }
            }
            if (null == instance.getGmtFinish()) {
                instance.setCost(0.00);
            } else {
                BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                instance
                    .setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
            }
            JSONObject object = JSONObject.parseObject(instance.getGraph());
            JSONArray array = new JSONArray();
            for (InstanceNode instanceNode : graphNodes) {
                JSONObject node = JSONObject.parseObject(JSONObject.toJSONString(instanceNode));
                JSONObject input =  JSONObject.parseObject(instanceNode.getInput());
                node.put("input", input);
                if(null != input && null != input.getJSONObject("target") && StringUtils.isNotBlank(instance.getExecuteType())){
                    instance.setExecuteType(input.getJSONObject("target").getString("executeType"));
                }
                node.put("config", JSONObject.parseObject(instanceNode.getConfig()));
                node.put("control", JSONObject.parseObject(instanceNode.getControl()));
                array.add(node);
            }
            object.put("nodes", array);
            instance.setGraph(JSONObject.toJSONString(object));
            instance.setResults(results);
            if (com.megatron.common.model.RequestContext.Catalog.Tenant == instance.getCatalog()) {
                instance.setOperator(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getCreatorId()));
            } else {
                instance.setOperator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId()));
            }
            if (null != instance.getTenantId() && instance.getTenantId() != 0) {
                instance.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getTenantId()));
            }
            return new GeneralResult<>(true, instance, "查询任务记录详细成功");
        } catch (Exception e) {
            log.error("Query task record detail fail:", e);
            return new GeneralResult<>(false, "查询任务记录详细失败");
        }
    }

    @Override
    public GeneralResult<TaskInstance> log(Long id) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            List<TaskNode> nodes = this.taskNodeRepository.listByInstance(instance.getId());
            instance.setNodes(nodes);
            // cost
            if (null == instance.getGmtFinish()) {
                instance.setCost(0.00);
            } else {
                BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                instance
                    .setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
            }
            return new GeneralResult<>(true, instance, "查询任务日志成功");
        } catch (Exception e) {
            log.error("Query task record detail fail:", e);
            return new GeneralResult<>(false, "查询任务日志失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(TaskInstance instance) {
        try {
            Boolean result = taskInstanceRepository.update(instance);
            return result ? new GeneralResult<>(true, "修改任务记录成功") : new GeneralResult<>(false, "修改任务记录失败");
        } catch (Exception e) {
            log.error("Modify task record fail:", e);
            return new GeneralResult<>(false, "修改任务记录失败");
        }
    }

    @Override
    public TaskInstance get(Long id) {
        try {
            TaskInstance instance = taskInstanceRepository.query(id);
            if (com.megatron.common.model.RequestContext.Catalog.Tenant == instance.getCatalog()) {
                instance.setOperator(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getCreatorId()));
            } else {
                instance.setOperator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId()));
            }
            if (null != instance.getTenantId() && instance.getTenantId() != 0) {
                instance.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getTenantId()));
            }
            return instance;
        } catch (Exception e) {
            log.error("Get task record fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            if (Status.Inst.APPROVING.name().equals(instance.getStatus())) {
                return new GeneralResult<>(false, "审批中无法删除");
            }
            this.taskNodeLogRepository.deleteByInstanceId(id);
            this.taskNodeRepository.removeByInstance(id, userId);
            this.taskInstanceRepository.remove(id, userId);
            this.commandInstanceRepository.deleteByInstance(id);
        } catch (Exception e) {
            log.error("Remove record fail:", e);
            return new GeneralResult<>(false, "删除记录失败");
        }
        return new GeneralResult<>(true, "删除记录成功");
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = TaskInstance.class.getSimpleName() + "_" + id;
        boolean result = false;
        TaskInstance record = null;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            record = this.taskInstanceRepository.query(id);
            Assert.notNull(record, "数据不存在！");
            result = this.taskInstanceRepository.updateStatus(id, status, userId);
        } catch (Exception e) {
            log.error("Update task instance status fail:", e);
            return new GeneralResult<>(false, "状态更新失败");
        }
        return result ? new GeneralResult<>(true, "状态更新成功") : new GeneralResult<>(false, "状态更新失败");
    }

    @Override
    public Task recover(Long id, RequestContext context) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            Assert.isTrue(Status.Inst.SUCCESS.name().equalsIgnoreCase(instance.getStatus()), "任务不是成功状态，不能执行恢复操作！");
            // 任务
            Task task = new Task();
            task.setName(instance.getName() + "恢复");
            task.setCode(Catalog.Task.STANDARD.getValue());
            if ("compare".equals(instance.getType()) || "backup".equals(instance.getType())) {
                task.setType("recover");
            } else {
                task.setType(instance.getType());
            }
            InstanceGraph graph = JSONObject.parseObject(instance.getGraph(), InstanceGraph.class);
            List<TaskNode> nodes = taskNodeRepository.listByInstance(id);
            List<InstanceNode> taskNodes = Lists.newArrayList();
            for (TaskNode node : nodes) {
                InstanceNode taskNode = new InstanceNode(node.getName() + "恢复", "task.maintain.recovery",
                    node.getNodeId(), null, node.getConfig(), node.getControl(), "task.maintain.recovery");
                if (node.getModuleType().contains("backup")) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject inputTarget = input.getJSONObject("target");
                    JSONArray hosts = inputTarget.getJSONArray("data");
                    // 后面sources顺序必须和主机顺序一致
                    List<String> ips = new ArrayList<>();
                    if (!ListTool.isEmpty(hosts)) {
                        for (int i = 0; i < hosts.size(); i++) {
                            JSONObject host = hosts.getJSONObject(i);
                            ips.add(host.getString("host"));
                        }
                    }
                    JSONObject fileParam = input.getJSONObject("fronts");
                    String target = fileParam.getString("source");
                    if (target.contains("\\")) {
                        if (!target.endsWith("\\")) {
                            target = target.substring(0, target.lastIndexOf("\\") + 1);
                        }
                    } else {
                        if (!target.endsWith("/")) {
                            target = target.substring(0, target.lastIndexOf("/") + 1);
                        }
                    }
                    GeneralResult<?> nodeResult = JSONObject.parseObject(node.getOutput(), GeneralResult.class);
                    List<String> sources = Lists.newArrayList();
                    List<JSONObject> results =
                        JSONArray.parseArray(JSONObject.toJSONString(nodeResult.getData()), JSONObject.class);
                    for (String ip : ips) {
                        String source = "";
                        for (JSONObject result : results) {
                            if (result.getBoolean("success") && ip.equals(result.getString("host"))) {
                                JSONObject message = JSONObject.parseObject(result.getString("message"));
                                source = message.getString("dest");
                                break;
                            }

                        }
                        sources.add(source);
                    }
                    JSONObject fronts = new JSONObject();
                    fronts.put("source", JSONObject.toJSONString(sources));
                    fronts.put("target", target);
                    input.put("fronts", fronts);
                    taskNode.setInput(input.toJSONString());
                }
                taskNodes.add(taskNode);
            }
            graph.setNodes(taskNodes);
            task.setGraph(JSONObject.toJSONString(graph));
            task.setStatus(Status.Inst.CREATED.name());
            return task;
        } catch (Exception e) {
            log.error("Excute task recovery fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<List<TaskInstance>> listByExecHost(Inst status, String executeHost) {
        try {
            List<TaskInstance> list = taskInstanceRepository.listByHost(status, executeHost, null);
            return new GeneralResult<>(true, list, "任务实例查询成功");
        } catch (Exception e) {
            log.error("list task instance is error :", e);
            return new GeneralResult<>(false, new ArrayList<>(), "任务实例查询失败");
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * com.bocloud.task.intf.TaskInstanceService#listExceptionInstance(java.lang.
     * String, java.lang.String)
     */
    @Override
    public GeneralResult<List<TaskInstance>> listExceptionInstance(String executeHost, String excludeHost) {
        try {
            List<TaskInstance> list = taskInstanceRepository.listExpInstance(executeHost, excludeHost);
            return new GeneralResult<>(true, list, "任务实例查询成功");
        } catch (Exception e) {
            log.error("list exception task instance is error :", e);
            return new GeneralResult<>(false, new ArrayList<>(), "异常任务实例查询失败");
        }
    }

    @Override
    public GeneralResult<TaskInstance> handlerCheck(Long id, TaskAction action) {
        try {
            TaskInstance instance = taskInstanceRepository.query(id);
            if (null == instance) {
                return new GeneralResult<>(false, "任务不存在");
            }
            if (action.equals(TaskAction.SUSPEND)) {
                if (!Status.Inst.RUNNING.name().equals(instance.getStatus())) {
                    return new GeneralResult<>(false, "任务不在执行，无法暂停");
                }
            }
            List<TaskNode> nodes = taskNodeRepository.listByInstance(id);
            if (nodes.isEmpty()) {
                return new GeneralResult<>(false, "任务不存在任务节点");
            }
            int size = 0;
            int ready = 0;
            for (TaskNode node : nodes) {
                if (node.getModuleType().contains("crtl.start") || node.getModuleType().contains("crtl.stop")) {
                    continue;
                }
                if (Status.Node.READY.name().equals(node.getStatus())) {
                    ready++;
                }
                size++;
            }
            if (size == 1) {
                return new GeneralResult<>(false, "任务只有一个工作节点");
            }
            if (ready == 0) {
                return new GeneralResult<>(false, "任务不存在没有执行的任务节点");
            }
            return new GeneralResult<>(true, instance, "success");
        } catch (Exception e) {
            log.error("Check handler error :", e);
            return new GeneralResult<>(false, "数据库操作有误");
        }
    }

    @Override
    public GeneralResult<GridBean<TaskInstance>> listAwaits(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            pager.getParams().add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<TaskInstance> list = taskInstanceRepository.listAwaits(pager);
            int total = taskInstanceRepository.countAwaits(pager.getParams());
            GridBean<TaskInstance> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询任务记录成功");
        } catch (Exception e) {
            log.error("List task record fail:", e);
            return new GeneralResult<>(false, "查询任务记录失败");
        }
    }

    @Override
    public GeneralResult<List<TaskInstanceExecResult>> result(Long id) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            Assert.notNull(instance, "数据不存在！");
            List<TaskNode> nodes = this.taskNodeRepository.listByInstance(instance.getId());
            Map<String, List<Result>> targetMap = Maps.newHashMap();
            for (TaskNode node : nodes) {
                if (node.getModuleType().equalsIgnoreCase(NodeCode.Ctrl.Start.getValue())
                    || node.getModuleType().equalsIgnoreCase(NodeCode.Ctrl.Stop.getValue())) {
                    continue;
                }
                JSONObject input = JSONObject.parseObject(node.getInput());
                JSONObject target = input.getJSONObject("target");
                if (null != target) {
                    List<WorkerHost> targets = JSONObject.parseArray(target.getString("data"), WorkerHost.class);
                    for (WorkerHost workerHost : targets) {
                        targetMap.put(workerHost.getHost(), Lists.newArrayList());
                    }
                }
            }
            for (TaskNode node : nodes) {
                if (node.getModuleType().equalsIgnoreCase(NodeCode.Ctrl.Start.getValue())
                    || node.getModuleType().equalsIgnoreCase(NodeCode.Ctrl.Stop.getValue())) {
                    continue;
                }
                if (null != node.getOutput()) {
                    GeneralResult<?> result = JSONObject.parseObject(node.getOutput(), GeneralResult.class);
                    List<HostResult> hostResults =
                        JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                    for (HostResult hostResult : hostResults) {
                        String host = hostResult.getHost();
                        if (node.getModuleType().contains("maintain.database")) {
                            host = host.substring(host.indexOf("//") + 2);
                        }
                        List<Result> results = targetMap.get(host);
                        results.add(new Result(hostResult.isSuccess(), hostResult.getMessage()));
                        targetMap.put(host, results);
                    }
                }
            }
            List<TaskInstanceExecResult> list = Lists.newArrayList();
            for (String key : targetMap.keySet()) {
                TaskInstanceExecResult object = new TaskInstanceExecResult();
                object.setHost(key);
                object.setResults(targetMap.get(key));
                list.add(object);
            }
            return new GeneralResult<>(true, list, "查询任务执行详细成功");
        } catch (Exception e) {
            log.error("Query task record detail fail:", e);
            return new GeneralResult<>(false, "查询任务执行详细失败");
        }
    }

    @Override
    public List<TaskInstance> list(Long taskId) {
        try {
            return taskInstanceRepository.listByTask(taskId);
        } catch (Exception e) {
            log.error("list task instance is error :", e);
            return null;
        }
    }

    @Override
    public GeneralResult<GridBean<TaskInstance>> list(Pager pager, Long commandId) {
        List<TaskInstance> list = null;
        int total = 0;
        GridBean<TaskInstance> gridBean = null;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            list = taskInstanceRepository.list(pager, commandId);
            for (TaskInstance instance : list) {
                if (null == instance.getGmtFinish()) {
                    instance.setCost(0.00);
                } else {
                    BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                    BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                    instance.setCost(
                        end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
                }
                if (com.megatron.common.model.RequestContext.Catalog.Tenant == instance.getCatalog()) {
                    instance.setOperator(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getCreatorId()));
                } else {
                    instance.setOperator(redisTemplate.opsForValue().get("User_Name_" + instance.getCreatorId()));
                }
                if (null != instance.getTenantId() && instance.getTenantId() != 0) {
                    instance.setTenantName(redisTemplate.opsForValue().get("Tenant_Name_" + instance.getTenantId()));
                }
            }
            total = taskInstanceRepository.count(pager.getParams(), commandId);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        } catch (Exception e) {
            log.error("List task record fail:", e);
            return new GeneralResult<>(false, "查询任务记录失败");
        }
        return new GeneralResult<>(true, gridBean, "查询任务记录成功");
    }

    @Override
    public GeneralResult<Void> cancel(Long id) {
        String path = TaskInstance.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskInstance record = this.taskInstanceRepository.query(id);
            Assert.notNull(record, "数据不存在！");
            if (!Status.Inst.CREATED.name().equalsIgnoreCase(record.getStatus())) {
                return new GeneralResult<>(false, "任务取消失败：状态不是未执行");
            }
            this.taskInstanceJobRepository.delete(record.getUuid());
            record.setStatus(Status.Inst.CANCELED.name());
            record.setGmtStart(new Date());
            record.setGmtFinish(new Date());
            this.taskInstanceRepository.update(record);
            return new GeneralResult<>(true, "任务取消成功");
        } catch (Exception e) {
            log.error("cancel task instance fail:", e);
            return new GeneralResult<>(false, "任务取消失败");
        }
    }

    @Override
    public GeneralResult<Void> cancel(List<Long> ids) {
        for (Long id : ids) {
            cancel(id);
        }
        return new GeneralResult<>(true, "任务批量取消成功");
    }

    @Override
    public GeneralResult<?> putJob(Job job) {
        try {
            TaskInstanceJob object =
                new TaskInstanceJob(job.getTaskId(), JSONObject.toJSONString(job), job.getPriority());
            taskInstanceJobRepository.save(object);
            return new GeneralResult<>(true, "添加任务记录Job成功");
        } catch (Exception e) {
            log.error("Create task instance job fail:", e);
            return new GeneralResult<>(false, "添加任务记录Job失败");
        }
    }

    @Override
    public List<Job> getJobs() {
        String path = TaskInstanceJob.class.getSimpleName();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "请求超时");
            int count = taskInstanceJobRepository.count();
            if (count == 0) {
                return new ArrayList<>();
            }
            int rows = 0;
            if (count <= 30) {
                rows = 3;
            } else if (count <= 50) {
                rows = 5;
            } else if (count <= 100) {
                rows = 10;
            } else if (count <= 200) {
                rows = 20;
            } else if (count <= 300) {
                rows = 30;
            } else if (count <= 500) {
                rows = 50;
            } else {
                rows = 100;
            }
            List<TaskInstanceJob> jobs = taskInstanceJobRepository.list(rows);
            List<Job> list = new ArrayList<>();
            List<Long> ids = new ArrayList<>();
            for (TaskInstanceJob job : jobs) {
                list.add(JSONObject.parseObject(job.getJob(), Job.class));
                ids.add(job.getId());
            }
            if (!ids.isEmpty()) {
                taskInstanceJobRepository.delete(ids);
            }
            return list;
        } catch (Exception e) {
            log.error("get instance job fail:", e);
            return new ArrayList<>();
        }
    }

    @Override
    public List<Job> getAllJobs() {
        try {
            List<TaskInstanceJob> jobs = taskInstanceJobRepository.list();
            List<Job> list = new ArrayList<>();
            for (TaskInstanceJob job : jobs) {
                list.add(JSONObject.parseObject(job.getJob(), Job.class));
            }
            return list;
        } catch (Exception e) {
            log.error("list task instance job fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<GridBean<TaskFailRecord>> listFailRecord(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<TaskFailRecord> list = taskFailRecordRepository.list(pager);
            Set<String> keys = redisTemplate.keys("Treatment_*");
            if (!CollectionUtils.isEmpty(keys)) {
                for (TaskFailRecord record : list) {
                    String message = record.getMessage();
                    if (StringUtils.isEmpty(message)) {
                        continue;
                    }
                    List<String> values = new ArrayList<>();
                    for (String key : keys) {
                        if (!message.toLowerCase().contains(key.replaceFirst("Treatment_", "").toLowerCase())) {
                            continue;
                        }
                        String value = redisTemplate.opsForValue().get(key);
                        if (StringUtils.isNotEmpty(value)) {
                            values.add(value);
                        }
                    }
                    record.setTreatment(values);
                }
            }

            int total = taskFailRecordRepository.count(pager.getParams());
            GridBean<TaskFailRecord> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询任务记录成功");
        } catch (Exception e) {
            log.error("List task record fail:", e);
            return new GeneralResult<>(false, "查询任务记录失败");
        }
    }

    @Override
    public GeneralResult<?> updateLimited(Long id, Long userId, String dealMethod, Long platformAlertId) {
        try {
            if (PlatformAlertConstant.DealMethod.DISCARD.name().equalsIgnoreCase(dealMethod)) {
                taskInstanceRepository.updateLimitedDiscard(id, userId, platformAlertId);
            } else {
                taskInstanceRepository.updateLimitedQueue(id, userId, platformAlertId);
            }
            return new GeneralResult<>(true, "更新成功");
        } catch (Exception e) {
            log.error("handle host inspect record fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public TaskInstance initInstance(Task task, String mode, RequestContext context) {
        try {
            // 创建任务实例
            Long userId = context.getTarget();
            TaskInstance instance = new TaskInstance(userId, userId, userId, task.getName(),
                task.isUntreate() ? Status.Inst.UNTREATED.name() : Status.Inst.CREATED.name(), task.getRemark(),
                task.getCode(), task.getGraph().toString(), task.getCron(), context.getTenant(), context.getCatalog());
            if (StringUtils.isEmpty(task.getType())) {
                instance.setType(Catalog.Task.TEMPORARY.getValue());
            } else {
                instance.setType(task.getType());
            }
            instance.setUuid(IDFactory.instance().uuid());
            instance.setName(instance.getName() + "_" + instance.getUuid().substring(0, 8));
            instance.setTaskId(task.getId());
            if (StringUtils.isEmpty(mode)) {
                mode = ExecuteMode.WEB.name();
            }
            instance.setExecuteMode(mode);
            instance.setTenantId(context.getTenant());
            instance.setIsBackup(false);
            instance.setProps(JSONObject.toJSONString(context));
            if (!StringUtils.isEmpty(task.getPlatform())) {
                instance.setPlatform(task.getPlatform());
            }
            instance.setPriority(task.getPriority());
            instance.setEmailEnabled(null == task.getEmailEnabled() ? false : task.getEmailEnabled());
            this.taskInstanceRepository.save(instance);
            // 解析任务节点
            TaskGraph graph = JSONObject.parseObject(instance.getGraph(), TaskGraph.class);
            List<TaskNode> nodes = graph.getNodes();
            Map<String, Long> nodeMap = Maps.newHashMap();
            boolean isBackup = true;

            List<TaskInstanceHost> instanceHosts = new ArrayList<>();
            for (TaskNode node : nodes) {
                NodeConfig config = JSONObject.parseObject(node.getConfig(), NodeConfig.class);
                if (null == config || config.isIgnore()) {
                    continue;
                }
                String code = node.getCode();
                if (!code.contains("start") && !code.contains("stop") && !code.contains("backup")) {
                    isBackup = false;
                }
                if (code.startsWith("task")) {
                    node.setCatalog(Catalog.Node.TASK.name());
                    if (code.contains("maintain")) {
                        node.setCategory(Category.Node.MAINTAIN.name());
                    } else if (code.contains("software")) {
                        node.setCategory(Category.Node.SOFTWARE.name());
                    }
                } else {
                    node.setCatalog(Catalog.Node.CTRL.name());
                    if (code.contains("start")) {
                        node.setCategory(Category.Node.START.name());
                    } else if (code.contains("stop")) {
                        node.setCategory(Category.Node.STOP.name());
                    } else if (code.contains("judge")) {
                        node.setCategory(Category.Node.JUDGE.name());
                    }
                }
                List<TaskInstanceHost> taskInstanceHosts = new ArrayList<>();
                if (!code.contains("ctrl")) {
                    JSONObject input = JSONObject.parseObject(node.getInput());
                    JSONObject target = input.getJSONObject("target");
                    if (null != target) {
                        boolean sudo = target.getBooleanValue("sudo");
                        List<JSONObject> hosts = JSONObject.parseArray(target.getString("data"), JSONObject.class);
                        if (ListTool.isEmpty(hosts)) {
                            hosts = new ArrayList<>();
                        }
                        for (JSONObject host : hosts) {
                            // 记录任务实例和主机关系
                            TaskInstanceHost taskInstanceHost = new TaskInstanceHost();
                            taskInstanceHost.setHost(host.getString("host"));
                            taskInstanceHost.setInstanceId(instance.getId());
                            taskInstanceHosts.add(taskInstanceHost);
                            if (null != host.getInteger("port")) {
                                continue;
                            }
                            String osName = host.getString("category");
                            if (!org.springframework.util.StringUtils.hasText(osName)) {
                                osName = host.getString("osName");
                            }
                            if (org.springframework.util.StringUtils.hasText(osName)) {
                                host.put("port", osName.toLowerCase().contains("window") ? 5985 : 22);
                                host.put("category", osName.toLowerCase().contains("window") ? OSCategory.WINDOWS.name()
                                    : OSCategory.LINUX.name());
                            } else {
                                host.put("port", 22);
                                host.put("category", OSCategory.LINUX.name());
                            }
                            host.put("sudo", sudo);
                        }
                        if (AuthenMode.BATCH.name().equalsIgnoreCase(target.getString("authenMode"))) {
                            Identification ident = identificationRepository.query(target.getLong("identId"));
                            if (null == ident) {
                                this.taskInstanceRepository.remove(instance.getId(), userId);
                                log.warn("认证账户为空，任务实例化失败");
                                return null;
                            }
                            for (JSONObject host : hosts) {
                                host.put("user", ident.getUsername());
                                host.put("pasd", ident.getPassword());
                            }
                        } else if (AuthenMode.ACCOUNT.name().equalsIgnoreCase(target.getString("authenMode"))) {
                            for (JSONObject host : hosts) {
                                Long identId = host.getLong("identId");
                                if (null == identId) {
                                    log.warn("认证账户为空，任务实例化失败");
                                    this.taskInstanceRepository.remove(instance.getId(), userId);
                                    return null;
                                }
                                Identification ident = identificationRepository.query(identId);
                                if (null == ident) {
                                    log.warn("认证账户为空，任务实例化失败");
                                    this.taskInstanceRepository.remove(instance.getId(), userId);
                                    return null;
                                }
                                host.put("user", ident.getUsername());
                                host.put("pasd", ident.getPassword());
                            }
                        }
                        target.put("data", hosts);
                        input.put("target", target);
                        node.setInput(input.toString());
                    }
                }
                node.setUuid(IDFactory.instance().uuid());
                node.preSave(userId, userId, instance.getId(), Status.Node.READY.name());
                node.setNodeId(config.getId());
                this.taskNodeRepository.save(node);
                if (!taskInstanceHosts.isEmpty()) {
                    taskInstanceHosts.forEach(taskInstanceHost -> {
                        taskInstanceHost.setNodeId(node.getId());
                        taskInstanceHost.setNodeName(node.getName());
                    });
                    instanceHosts.addAll(taskInstanceHosts);
                }
                nodeMap.put(node.getNodeId(), node.getId());
            }
            if (isBackup) {
                instance.setIsBackup(isBackup);
                taskInstanceRepository.update(instance);
            }
            // 解析任务节点关系
            List<TaskEdge> edges = graph.getEdges();
            Map<String, String> ingoreNodes = NodeHelper.getIngoreNodes(nodes);
            if (!ListTool.isEmpty(edges)) {
                List<NodeRelation> relations = new ArrayList<>();
                for (TaskEdge edge : edges) {
                    if (edge.isIgnore()) {
                        continue;
                    }
                    String source = edge.getSource();
                    source = ingoreNodes.containsKey(source) ? ingoreNodes.get(source) : source;
                    String target = edge.getTarget();
                    target = ingoreNodes.containsKey(target) ? ingoreNodes.get(target) : target;
                    NodeRelation relation = new NodeRelation(nodeMap.get(source), nodeMap.get(target), instance.getId(),
                        Status.Node.READY.name(), edge.isDicision());
                    relations.add(relation);
                }
                if (!relations.isEmpty()) {
                    this.nodeRelationRepository.batchSave(relations, 100);
                }
            }
            if (!instanceHosts.isEmpty()) {
                Set<String> hostNumSet =
                    instanceHosts.stream().map(TaskInstanceHost::getHost).collect(Collectors.toSet());
                instance.setTargetHostNum(hostNumSet.size());
                taskInstanceHostRepository.batchSave(instanceHosts, 100);
            }
            taskInstanceRepository.updateTime(instance.getId());
            instance.setContext(context);
            instance.setNodes(nodes);
            return instance;
        } catch (Exception e) {
            log.error("initInstance [{}] exception !", task.getName(), e);
            return null;
        }
    }

    @Override
    public GeneralResult<TaskInstance> initFailedStatus(Long id, RequestContext context) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            if (null == instance) {
                return new GeneralResult<>(false, "任务实例数据不存在");
            }
            // 作业实例状态更新
            if (!Status.Inst.FAILED.name().equals(instance.getStatus())) {
                return new GeneralResult<>(false, "作业状态不是失败，不能重试");
            }
            List<TaskNode> nodes = taskNodeRepository.listByStatus(id, Status.Node.FAILED.name());
            for (TaskNode node : nodes) {
                // task_node 初始状态： Status.Node.READY.name()
                taskNodeRepository.updateFailedStatus(node.getId(), Status.Node.READY.name(), context.getTarget());
                // task_node_relation 初始状态：Status.Node.READY.name()
                nodeRelationRepository.updateFailedStatus(id, node.getId(), Status.Node.READY.name());
                // task_node_log 清除失败节点日志
                taskNodeLogRepository.deleteByNodeId(node.getId());
            }
            // task_instance 初始状态：CREATED APPROVED
            taskInstanceRepository.initStatus(id, Status.Inst.CREATED.name(), context.getTarget());
            instance.setNodes(taskNodeRepository.listByInstance(id));
            return new GeneralResult<>(true, instance, "初始化任务实例状态成功");
        } catch (Exception e) {
            log.error("update task instance status fail:", e);
            return new GeneralResult<>(false, "初始化任务实例状态失败");
        }
    }

    @Override
    public GeneralResult<TaskInstance> executeCheck(Long id, RequestContext context) {
        try {
            TaskInstance instance = this.taskInstanceRepository.query(id);
            if (!Status.Inst.FAILED.name().equals(instance.getStatus())) {
                return new GeneralResult<>(false, "作业状态不是失败状态，无法人工介入执行");
            }
            List<TaskNode> taskNodes = taskNodeRepository.listByInstance(id);
            TaskGraph graph = JSONObject.parseObject(instance.getGraph(), TaskGraph.class);
            Map<String, String> ingoreMap = NodeHelper.getIngoreNodes(graph.getNodes());
            Map<String, List<TaskNode>> nextMap = NodeHelper.getNextNodes(graph.getEdges(), ingoreMap, taskNodes);
            boolean flag = false;
            for (TaskNode taskNode : taskNodes) {
                if (!Status.Node.FAILED.name().equalsIgnoreCase(taskNode.getStatus())) {
                    continue;
                }
                NodeControl control = JSONObject.parseObject(taskNode.getControl(), NodeControl.class);
                if (null != control && control.isSkippable()) {
                    continue;
                }
                if (!nextMap.containsKey(taskNode.getNodeId())) {
                    flag = true;
                    break;
                }
                List<TaskNode> nextNodes = nextMap.get(taskNode.getNodeId());
                for (TaskNode nextNode : nextNodes) {
                    if (!nextNode.getModuleType().contains("ctrl.judge")) {
                        flag = true;
                        break;
                    }
                }
            }
            if (flag) {
                return new GeneralResult<>(false, "人工介入执行失败：任务实例存在失败节点，且节点没有设置失败跳过");
            }
            taskInstanceRepository.initStatus(id, Status.Inst.CREATED.name(), context.getTarget());
            instance.setNodes(taskNodes);
            return new GeneralResult<>(true, instance, "获取任务实例数据成功");
        } catch (Exception e) {
            log.error("execute check task instance fail:", e);
            return new GeneralResult<>(false, "获取任务实例数据失败");
        }
    }

    @Override
    public GeneralResult<Task> taskDeployGaia(DeployGaiaParams params) {
        try {
            Target target = params.getTarget();
            if(null == target){
                return new GeneralResult<>(false, "目标主机不存在");
            }
            if (CollectionUtils.isEmpty(target.getData())) {
                return new GeneralResult<>(false, "目标主机不存在");
            }
            if (!paramCheck(params)) {
                return new GeneralResult<>(false, "参数异常");
            }
            // 依据playbook模板构建任务
            Task task = DeployGaiaTaskBuilder.getTemplateTask(params);
            if (null == task) {
                return new GeneralResult<>(false, "作业构建失败");
            }
            return new GeneralResult<>(true, task, "作业构建成功");
        } catch (Exception e) {
            log.error("作业构建异常", e);
            return new GeneralResult<>(false, "作业构建失败");
        }
    }
    private boolean paramCheck(DeployGaiaParams params) {
        //参数验证
        DeployGaiaArgs args = params.getArgs();
        if (null == args) {
            log.error("deploy gaia args is null");
            return false;
        }
        if ("add".equalsIgnoreCase(args.getType())) {
            //gaia部署包路径，默认路径：/home/cmp/packages
            /*if (StringUtils.isEmpty(args.getGaia_package())) {
                log.error("deploy gaia args.gaia_package is null");
                return false;
            }*/
            if (null != args.getConfig_share_path() && args.getConfig_share_path()) {
                JSONObject gfsServer = args.getGfs_server();
                if (null == gfsServer) {
                    log.error("deploy gaia args.gfs_server is not object");
                    return false;
                }
                if (!gfsServer.containsKey("gfs_server1_host") || !gfsServer.containsKey("gfs_server2_host")) {
                    log.error("deploy gaia args.gfs_server gfs_server1_host or gfs_server2_host is null");
                    return false;
                }
            }
            GaiaConfigParams gaiaConfig = args.getGaia_config();
            if (null == gaiaConfig) {
                log.error("deploy gaia args.gaia_config is null");
                return false;
            }
            if (StringUtils.isEmpty(gaiaConfig.getZookeeper())) {
                log.error("deploy gaia args.gaia_config.zookeeper is null");
                return false;
            }
            if (CollectionUtils.isEmpty(gaiaConfig.getRabbitmq())) {
                log.error("deploy gaia args.gaia_config.rabbitmq is empty");
                return false;
            }
        } else if ("delete".equalsIgnoreCase(args.getType())) {
            return true;
        } else {
            log.error("deploy gaia args.type is error");
            return false;
        }
        return true;
    }



}
