package com.bocloud.cop.service;

import com.bocloud.cop.entity.TaskPlatform;
import com.bocloud.cop.entity.TaskPlatformConfig;
import com.bocloud.cop.intf.TaskPlatformService;
import com.bocloud.cop.repository.TaskPlatformConfigRepository;
import com.bocloud.cop.repository.TaskPlatformRepository;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 任务实例平台接口实现
 *
 * @author majiali
 * @version 1.0
 * @since 2022.08.24
 */
@Slf4j
@Service("taskPlatformService")
public class TaskPlatformServiceImpl implements TaskPlatformService {

    @Autowired
    private TaskPlatformRepository taskPlatformRepository;

    @Autowired
    private TaskPlatformConfigRepository taskPlatformConfigRepository;

    @Override
    public GeneralResult<GridBean<TaskPlatform>> list(Pager pager, RequestContext context) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<TaskPlatform> list = taskPlatformRepository.list(pager);
            int total = taskPlatformRepository.count(pager.getParams());
            GridBean<TaskPlatform> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, bean, "查询任务平台成功");
        } catch (Exception e) {
            log.error("List task platform record fail:", e);
            return new GeneralResult<>(false, "查询任务平台失败");
        }
    }

    @Override
    public GeneralResult<?> listAll() {
        try {
            List<TaskPlatform> list = taskPlatformRepository.listAll();
            List<SimpleBean> beans = new ArrayList<>();
            SimpleBean bean;
            for (TaskPlatform platform : list) {
                bean = new SimpleBean(platform.getId(), platform.getName(), platform.getName());
                beans.add(bean);
            }
            return new GeneralResult<>(true, beans, "查询任务平台成功");
        } catch (Exception e) {
            log.error("List task platform record fail:", e);
            return new GeneralResult<>(false, "查询任务平台失败");
        }
    }

    @Override
    public GeneralResult<Void> create(TaskPlatform platform, Long userId) {
        try {
            TaskPlatform config = taskPlatformRepository.exists(platform.getName());
            if (null != config) {
                log.error("TaskPlatform name exist");
                return new GeneralResult<>(false, "任务平台名称已经存在");
            }
            platform.setDeleted(false);
            platform.setCreatorId(userId);
            platform.setMenderId(userId);
            taskPlatformRepository.save(platform);
            return new GeneralResult<>(true, "添加任务平台成功");
        } catch (Exception e) {
            log.error("Create task platform record fail:", e);
            return new GeneralResult<>(false, "添加任务平台失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(TaskPlatform platform, Long userId) {
        try {
            TaskPlatform platformTemp = taskPlatformRepository.query(platform.getId());
            if (null == platformTemp) {
                return new GeneralResult<>(false, "任务平台不存在");
            }
            if(!platformTemp.getName().equals(platform.getName())){
                return new GeneralResult<>(false, "任务平台名称不能修改");
            }
            platformTemp.setName(platform.getName());
            platformTemp.setRemark(platform.getRemark());
            platformTemp.setGmtModify(new Date());
            platformTemp.setMenderId(userId);
            taskPlatformRepository.update(platformTemp);
            return new GeneralResult<>(true, "修改任务平台成功");
        } catch (Exception e) {
            log.error("Modify task platform record fail:", e);
            return new GeneralResult<>(false, "修改任务平台失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        try {
            TaskPlatform instance = this.taskPlatformRepository.query(id);
            if (null == instance) {
                return new GeneralResult<>(false, "任务平台不存在");
            }
            List<TaskPlatformConfig> configList = taskPlatformConfigRepository.getByPlatform(instance.getName());
            if(CollectionUtils.isNotEmpty(configList)){
                return new GeneralResult<>(false, "任务平台存在告警配置信息，不可删除");
            }
            taskPlatformRepository.remove(id, userId);
            return new GeneralResult<>(true, "删除任务平台成功");
        } catch (Exception e) {
            log.error("Remove task platform record fail:", e);
            return new GeneralResult<>(false, "删除任务平台失败");
        }
    }

}
