package com.bocloud.cop.service;


import com.bocloud.cop.entity.TaskTreatment;
import com.bocloud.cop.intf.TaskTreatmentService;
import com.bocloud.cop.repository.TaskTreatmentRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author huliwei
 * @date 2023/4/23
 */
@Service("taskTreatmentService")
@Slf4j
public class TaskTreatmentServiceImpl implements TaskTreatmentService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskTreatmentRepository taskTreatmentRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<TaskTreatment>> list(Pager pager, Long userId) {
        GridBean<TaskTreatment> gridBean = null;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = taskTreatmentRepository.count(pager.getParams());
            List<TaskTreatment> list = taskTreatmentRepository.list(pager);
            for (TaskTreatment template : list) {
                template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
                if (null != template.getMenderId()) {
                    template.setMender(redisTemplate.opsForValue().get("User_Name_" + template.getMenderId()));
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        } catch (Exception e) {
            log.error("List taskTreatment fail:", e);
            return new GeneralResult<>(false, "查询任务处理方案失败");
        }
        return new GeneralResult<>(true, gridBean, "查询任务处理方案成功");
    }

    @Override
    public GeneralResult<TaskTreatment> create(TaskTreatment template,RequestContext context) {
        try {
            if(taskTreatmentRepository.exists(template.getName())){
                return new GeneralResult<>(false,  "该名称已存在，请重新输入!");
            }
            template.setGmtCreate(new Date());
            template.setGmtModify(new Date());
            template.setCreatorId(context.getTarget());
            template.setMenderId(context.getTarget());
            template.setDeleted(false);
            taskTreatmentRepository.save(template);
            redisTemplate.opsForValue().set("Treatment_" + template.getName(),template.getValue());
            return new GeneralResult<>(true,  "添加任务处理方案成功");
        } catch (Exception e) {
            log.error("query taskTreatment fail:", e);
            return new GeneralResult<>(false, "添加任务处理方案失败");
        }
    }

    @Override
    public GeneralResult<TaskTreatment> modify(TaskTreatment template, RequestContext context) {
        String path = TaskTreatment.class.getSimpleName() + "_" + template.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskTreatment taskTemplate = taskTreatmentRepository.query(template.getId());
            Assert.notNull(taskTemplate, "数据不存在！");
            if(!taskTemplate.getName().equals(template.getName())){
                return new GeneralResult<>(false, "该名称不能修改!");
            }
            taskTemplate.setValue(template.getValue());
            taskTemplate.setMenderId(context.getTarget());
            boolean update = taskTreatmentRepository.update(taskTemplate);
            if(update){
                redisTemplate.opsForValue().set("Treatment_" + template.getName(),template.getValue());
            }
            return new GeneralResult<>(true, "修改任务处理方案成功");
        } catch (Exception e) {
            log.error("Update taskTreatment fail:", e);
            return new GeneralResult<>(false, "修改任务处理方案失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = TaskTreatment.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskTreatment taskTemplate = taskTreatmentRepository.query(id);
            if(null != taskTemplate) {
                boolean remove = taskTreatmentRepository.remove(id,userId);
                if(remove){
                    redisTemplate.delete("Treatment_"+taskTemplate.getName());
                }else{
                    return new GeneralResult<>(false, "删除任务处理方案失败");
                }
            }
            return new GeneralResult<>(true, "删除任务处理方案成功");
        } catch (Exception e) {
            log.error("Remove taskTreatment fail:", e);
            return new GeneralResult<>(false, "删除任务处理方案失败");
        }
    }
}
