package com.bocloud.cop.service.builder;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.TargetGroup;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.service.utils.worker.WorkerDataUtils;
import com.megatron.common.model.GeneralResult;

/**
 * @author lyy
 * @date 2022/9/8
 */
public class BackUpTaskBuilder {
    public static Task getTelnetTaskTask(JSONObject object, Target target, Integer timeout) {
        JSONObject task = new JSONObject();
        task.put("code", "temporary");
        if (null != object.get("name")) {
            task.put("name", object.get("name").toString());
        } else {
            task.put("name", "配置对比-文件备份作业");
        }
        task.put("sceneId", object.get("sceneId"));
        JSONArray nodes = new JSONArray();
        List<TaskEdge> edges = new ArrayList<>();
        getNodes(object, target, WorkerDataUtils.getWorkerHostMap(target.getData()), timeout, nodes, edges);
        JSONObject graph = new JSONObject();
        graph.put("nodes", nodes);
        graph.put("edges", edges);
        task.put("graph", graph);
        return JSONObject.parseObject(task.toJSONString(), Task.class);
    }

    private static void getNodes(JSONObject object, Target target, Map<Long, List<WorkerHost>> hostMap,
        Integer timeout, JSONArray nodes, List<TaskEdge> edges) {
        Iterator<Long> iterator = hostMap.keySet().iterator();
        int x = 400;
        int index = 1;
        int y = 50;
        int pre = 10;
        int num = 0;
        while (iterator.hasNext()) {
            Long key = iterator.next();
            GeneralResult<TargetGroup> groupTarget = WorkerDataUtils.groupTargetByWorkerHost(hostMap.get(key));
            List<List<WorkerHost>> linuxs = WorkerDataUtils.splitTarget(groupTarget.getData().getLinuxs());
            List<List<WorkerHost>> windows = WorkerDataUtils.splitTarget(groupTarget.getData().getWindows());
            for (List<WorkerHost> linux : linuxs) {
                JSONObject node = new JSONObject();
                node.put("name", "文件备份(" + pre + "" + index + ")");
                node.put("code", "task.maintain.backup");
                node.put("moduleType", "task.maintain.backup");
                NodeConfig config = new NodeConfig(UUID.randomUUID().toString(), num, x, y,
                    "script.png", "maintain-node");
                NodeControl control = new NodeControl(false, false, false, 3, 10, timeout);
                node.put("control", control);
                node.put("config", config);
                node.put("input", getInput(object, target, linux));
                nodes.add(node);
                num++;
                y = y + 100;
                index++;
            }
            for (List<WorkerHost> window : windows) {
                JSONObject node = new JSONObject();
                node.put("name", "文件备份(" + pre + "" + index + ")");
                node.put("code", "task.maintain.backup");
                node.put("moduleType", "task.maintain.backup");
                NodeConfig config = new NodeConfig(UUID.randomUUID().toString(), num, x, y,
                    "script.png", "maintain-node");
                NodeControl control = new NodeControl(false, false, false, 3, 10, timeout);
                node.put("control", control);
                node.put("config", config);
                node.put("input", getInput(object, target, window));
                nodes.add(node);
                num++;
                y = y + 100;
                index++;
            }
            y = 50;
            x = x + 250;
            index = 1;
            pre++;
        }
    }

    private static JSONObject getInput(JSONObject object, Target targetInfo, List<WorkerHost> hosts) {
        JSONObject input = new JSONObject();
        JSONObject fronts = new JSONObject();
        fronts.put("source", object.get("source"));
        input.put("fronts", fronts);

       JSONObject target = new JSONObject();
        target.put("authenMode",targetInfo.getAuthenMode());
        target.put("sudo", targetInfo.getSudo());
        target.put("becomeUser", targetInfo.getBecomeUser());
        target.put("becomePass", targetInfo.getBecomePass());
        target.put("executeType", targetInfo.getExecuteType());
        target.put("shareHostPool", targetInfo.getShareHostPool());
        target.put("identId", targetInfo.getIdentId());
        target.put("data", hosts);
        input.put("target", target);
        return input;
    }
}
