package com.bocloud.cop.service.compare;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.bocloud.cop.model.Target;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.recipes.leader.LeaderLatchListener;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.DigestUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.backup.TaskBackup;
import com.bocloud.cop.entity.compare.StandardDocument;
import com.bocloud.cop.entity.compare.StandardDocumentVersion;
import com.bocloud.cop.entity.compare.TaskDocumentComparison;
import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparison;
import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparisonItem;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.Comparison;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.enums.Option;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.enums.TaskParam;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.compare.TaskDocumentComparisonService;
import com.bocloud.cop.model.Job;
import com.bocloud.cop.model.NotifyMessage;
import com.bocloud.cop.repository.TaskHostRepository;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskNodeRepository;
import com.bocloud.cop.repository.compare.StandardDocumentRepository;
import com.bocloud.cop.repository.compare.StandardDocumentVersionRepository;
import com.bocloud.cop.repository.compare.TaskDocumentComparisonRepository;
import com.bocloud.cop.repository.compare.TaskInstanceDocumentComparisonItemRepository;
import com.bocloud.cop.repository.compare.TaskInstanceDocumentComparisonRepository;
import com.bocloud.cop.service.builder.BackUpTaskBuilder;
import com.bocloud.cop.service.internal.CopInternalService;
import com.bocloud.cop.service.utils.CompressUtil;
import com.bocloud.cop.service.utils.JobScheduler;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 16:04:48
 * @Description:
 */
@Slf4j
@Service("documentComparisonService")
public class TaskDocumentComparisonServiceImpl implements TaskDocumentComparisonService {

    private static final String FIXED = "fixed";
    private static final String SKIPED = "skiped";
    private static final Map<Long, ScheduledFuture<?>> SCHEDULED_FUTURES = new ConcurrentHashMap<>();
    private ThreadPoolTaskScheduler threadPoolTaskScheduler = new ThreadPoolTaskScheduler();

    @Value("${comparison.upload.path:/home/cmp/standard-file}")
    private String standardFilepath;

    @Value("${spring.upload.path:/home/cmp}")
    private String basePath;

    @Value("${comparison.decompression.path:/home/cmp/decompression}")
    private String decompressionPath;

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StandardDocumentRepository standardDocumentRepository;
    @Autowired
    private JobScheduler taskScheduler;
    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private TaskInstanceService taskInstanceService;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private CopInternalService copInternalService;
    @Autowired
    private StandardDocumentVersionRepository documentVersionRepository;
    @Autowired
    private TaskDocumentComparisonRepository taskComparisonRepository;
    @Autowired
    private TaskInstanceDocumentComparisonItemRepository instanceComparisonItemRepository;
    @Autowired
    private TaskInstanceDocumentComparisonRepository instanceComparisonRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private AmqpTemplate amqpTemplate;

    @Override
    public GeneralResult<GridBean<TaskDocumentComparison>> list(Pager pager) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.taskComparisonRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.taskComparisonRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<TaskDocumentComparison> list = this.taskComparisonRepository.list(pager.getPage(), pager.getRows(),
                    pager.getParams(), pager.getSorter());
                for (TaskDocumentComparison comparison : list) {
                    if (null != comparison.getHostIds()) {
                        List<Long> ids = JSONObject.parseArray(comparison.getHostIds(), Long.class);
                        List<TaskHost> targets = taskHostRepository.queryByIds(ids);
                        String hostIps = targets.stream().map(TaskHost::getIp).collect(Collectors.joining(","));
                        comparison.setHostIps(hostIps);
                    }
                    comparison.setCreator(redisTemplate.opsForValue().get("User_Name_" + comparison.getCreatorId()));
                    if (null != comparison.getMenderId()) {
                        comparison.setMender(redisTemplate.opsForValue().get("User_Name_" + comparison.getMenderId()));
                    }
                    if (Comparison.category.standardVersionDocument.name().equals(comparison.getCategory())) {
                        List<JSONObject> objects =
                            JSONObject.parseArray(comparison.getComparisonObject(), JSONObject.class);
                        List<Long> versionIds =
                            objects.stream().map(item -> item.getLong("versionId")).collect(Collectors.toList());
                        List<StandardDocumentVersion> versions = documentVersionRepository.queryByIds(versionIds);
                        if (CollectionUtils.isNotEmpty(versions)) {
                            String versionName = String.join(",",
                                versions.stream().map(StandardDocumentVersion::getName).collect(Collectors.toList()));
                            comparison.setVersionName(versionName);
                        }
                    } else if (Comparison.category.standardDocument.name().equals(comparison.getCategory())) {
                        StandardDocumentVersion version = documentVersionRepository.query(comparison.getVersionId());
                        if (null != version) {
                            comparison.setVersionName(version.getName());
                        }
                    }
                }
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<TaskDocumentComparison>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List Query Is Fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 创建任务
     * 
     * @param param
     * @param context
     * @return
     */
    @Override
    public GeneralResult<Void> create(TaskDocumentComparison param, RequestContext context) {
        try {
            if (Comparison.category.standardVersionDocument.name().equals(param.getCategory())) {
                param.setName(Comparison.category.standardVersionDocument.getValue());
                List<JSONObject> objects = JSONObject.parseArray(param.getComparisonObject(), JSONObject.class);
                if (2 != objects.size()) {
                    return new GeneralResult<>(false, "有且只能选择两个版本！");
                }
                if (Option.CUSTOMIZE.name().equals(param.getOptions())) {
                    JSONObject objectOne = objects.get(0);
                    JSONObject objectTwo = objects.get(1);
                    Long versionIdOne = objectOne.getLong("versionId");
                    Long versionIdTwo = objectTwo.getLong("versionId");
                    if (versionIdOne.equals(versionIdTwo)) {
                        return new GeneralResult<>(false, "比较版本不得相同！");
                    }
                    List<Long> fileIdsOne = objectOne.getJSONArray("fileIds").toJavaList(Long.class);
                    List<Long> fileIdsTwo = objectTwo.getJSONArray("fileIds").toJavaList(Long.class);
                    if (fileIdsOne.size() == 0 || fileIdsTwo.size() == 0 || fileIdsOne.size() != fileIdsTwo.size()) {
                        return new GeneralResult<>(false, "版本之间需要有比对文件！");
                    }
                    // 重复判断
                    Map<Long, Long> map = new HashMap<>();
                    for (int i = 0; i < fileIdsOne.size(); i++) {
                        if (map.containsKey(fileIdsOne.get(i))) {
                            if (map.get(fileIdsOne.get(i)).equals(fileIdsTwo.get(i))) {
                                return new GeneralResult<>(false, "对比文件不可以重复！");
                            }
                        } else {
                            map.put(fileIdsOne.get(i), fileIdsTwo.get(i));
                        }
                    }
                }
            } else if (Comparison.category.standardDocument.name().equals(param.getCategory())) {
                // 校验版本文件路径不能相同
                List<JSONObject> objects = JSONObject.parseArray(param.getComparisonObject(), JSONObject.class);
                Map<Long, Set<String>> comparisonMap = new HashMap<>();
                for (JSONObject comparisonObject : objects) {
                    Long fileId = comparisonObject.getLong("fileId");
                    if (!comparisonMap.containsKey(fileId)) {
                        Set<String> sourceSet = new HashSet<>();
                        sourceSet.add(comparisonObject.getString("source"));
                        comparisonMap.put(fileId, sourceSet);
                        continue;
                    }
                    Set<String> sourceSet = comparisonMap.get(fileId);
                    boolean source = sourceSet.add(comparisonObject.getString("source"));
                    if (!source) {
                        return new GeneralResult<>(false, "版本文件路径不能相同！");
                    }
                }
            } else if (Comparison.category.historyDocument.name().equals(param.getCategory())
                || Comparison.category.crossMachineDocument.name().equals(param.getCategory())) {
                List<JSONObject> objects = JSONObject.parseArray(param.getComparisonObject(), JSONObject.class);
                Set<String> set = new HashSet<>();
                for (JSONObject comparisonObject : objects) {
                    String source = comparisonObject.getString("source");
                    if (!set.contains(source)) {
                        set.add(source);
                        continue;
                    }
                    return new GeneralResult<>(false, "对比文件路径不能相同！");
                }
            }
            param.setId(null);
            param.setCode(IDFactory.instance().uuid());
            param.setCreatorId(context.getTarget());
            param.setMenderId(context.getTarget());
            param.setGmtCreate(new Date());
            param.setStatus(Status.Task.CREATED.name());
            param.setName(param.getName() + "_" + param.getCode().substring(0, 8));
            taskComparisonRepository.save(param);
            if (Status.TaskType.IMMEDIATE.name().equals(param.getTaskType())) {
                // 下发任务
                this.start(param, context);
                taskComparisonRepository.updateStatus(param.getId(), Status.Task.EXECUTED.name(), context.getTarget());
                return new GeneralResult<>(true, "任务已下发成功!");
            }
            return new GeneralResult<>(true, "定时对比任务已创建");
        } catch (Exception e) {
            log.error("Creation failed :::", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = TaskDocumentComparisonServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskDocumentComparison comparison = taskComparisonRepository.query(id);
            if (null == comparison) {
                return new GeneralResult<>(false, "数据不存在");
            }
            taskComparisonRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Delete failed:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    /**
     * 定时任务操作-启动/暂停
     * 
     * @param action
     * @param id
     * @param context
     * @return
     */
    @Override
    public GeneralResult<Void> operate(String action, Long id, RequestContext context) {
        String path = TaskBackup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskDocumentComparison comparison = taskComparisonRepository.query(id);
            if (null == comparison) {
                return new GeneralResult<>(false, "任务不存在");
            }
            if ("run".equals(action)) {
                // 先修改一遍保留当前操作人员信息
                taskComparisonRepository.updateStatus(id, comparison.getStatus(), context.getTarget());
                return this.run(id, context.getTarget());
            } else if ("stop".equals(action)) {
                return this.stop(id, context.getTarget());
            }
            return new GeneralResult<>(false, "操作失败!");
        } catch (Exception e) {
            log.error("operate error :::" + e);
            return GeneralResult.FAILED("操作失败!");
        }
    }

    /**
     * 定时对比添加任务(主服务有效)
     * 
     * @param id
     * @param context
     * @return
     */
    @Override
    public GeneralResult<String> addScheduleIfLeader(Long id, RequestContext context) {
        if (!currentService.isLeader()) {
            log.info("收到定时任务[id={}]添加定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            TaskDocumentComparison timing = taskComparisonRepository.query(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.addSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("AddScheduleIfLeader fail :::", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    /**
     * 定时对比删除任务(主服务有效)
     * 
     * @param id
     * @return
     */
    @Override
    public GeneralResult<String> removeScheduleIfLeader(Long id) {
        if (!currentService.isLeader()) {
            log.info("收到定时任务[id={}]删除定时任务请求，当前服务非主服务，已经忽略", id);
            return new GeneralResult<>(true, SKIPED, "success");
        }
        try {
            TaskDocumentComparison timing = taskComparisonRepository.query(id);
            Assert.notNull(timing, "定时任务数据不存在");
            this.removeSchedule(timing);
            return new GeneralResult<>(true, FIXED, "success");
        } catch (Exception e) {
            log.error("RemoveScheduleIfLeader fail :::", e);
            return new GeneralResult<>(false, "fail");
        }
    }

    @Override
    public void onApplicationEvent() {
        threadPoolTaskScheduler.setPoolSize(Runtime.getRuntime().availableProcessors());
        threadPoolTaskScheduler.setThreadNamePrefix("TimeInspect-");
        threadPoolTaskScheduler.initialize();
        final TaskDocumentComparisonServiceImpl service = this;
        LeaderLatchListener listener = new LeaderLatchListener() {
            @Override
            public void isLeader() {
                List<TaskDocumentComparison> list = taskComparisonRepository.findAllByStatus(Status.Task.ACTIVE.name());
                list.forEach(service::addSchedule);
            }

            @Override
            public void notLeader() {
                List<TaskDocumentComparison> list =
                    taskComparisonRepository.findAllByStatus(Status.Task.SUSPEND.name());
                list.forEach(service::removeSchedule);
            }
        };
        try {
            currentService.addListener(listener);
        } catch (Exception e) {
            log.error("定时任务初始化失败 :::{}", e.getMessage());
        }
        if (currentService.isLeader()) {
            listener.isLeader();
        }
    }

    @Override
    public GeneralResult<GridBean<TaskInstanceDocumentComparisonItem>> items(Long id, Pager pager) {
        GridBean gridBean = null;
        try {
            TaskInstanceDocumentComparison instance = instanceComparisonRepository.queryByDocumentComparisonId(id);
            if (null == instance) {
                return new GeneralResult<GridBean<TaskInstanceDocumentComparisonItem>>(true, gridBean, "查询成功");
            }
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getParams()
                .add(new Param(MapTools.simpleMap("documentComparisonInstanceId", instance.getId()), Sign.EQ));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.instanceComparisonItemRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = instanceComparisonItemRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<TaskInstanceDocumentComparisonItem> list = instanceComparisonItemRepository.list(pager.getPage(),
                    pager.getRows(), pager.getParams(), pager.getSorter());
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<TaskInstanceDocumentComparisonItem>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List Query Is Fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private GeneralResult<Void> start(TaskDocumentComparison param, RequestContext context) {
        // 生成任务实例数据
        TaskInstanceDocumentComparison instance = new TaskInstanceDocumentComparison();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        String time = dateFormat.format(new Date());
        instance.setName(param.getName() + "_" + time);
        instance.setDocumentComparisonId(param.getId());
        instance.setUuid(IDFactory.instance().uuid());
        instance.setGmtStart(new Date());
        instance.setCreatorId(context.getTarget());
        instance.setGmtCreate(new Date());
        instance.setStatus(Status.Inst.RUNNING.name());
        instance.setCategory(param.getCategory());
        instanceComparisonRepository.save(instance);
        // 对比任务放入线程池
        taskExecutor.execute(() -> {
            // 任务对比
            GeneralResult<Void> result = comparison(param, instance, context);
            if (result.isFailed()) {
                instance.setGmtFinish(new Date());
                instance.setStatus(Status.Inst.FAILED.name());
                instance.setGmtModify(new Date());
                instance.setMenderId(context.getTarget());
                instanceComparisonRepository.update(instance);
            }
            // 发送消息通知前端
            NotifyMessage notifyMessage =
                new NotifyMessage(context, "文件对比任务", "任务执行结果:[" + result.getMessage() + "]", true, false);
            amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue",
                notifyMessage);
        });
        return new GeneralResult<>(true, "任务下发成功");
    }

    private void addSchedule(TaskDocumentComparison task) {
        if (SCHEDULED_FUTURES.containsKey(task.getId())) {
            this.removeSchedule(task);
        }
        Trigger trigger = new CronTrigger(task.getCron());
        ScheduledFuture<?> future = threadPoolTaskScheduler.schedule(() -> {
            RequestContext context = new RequestContext();
            context.setTarget(task.getMenderId());
            log.info("-----定时备份任务-------");
            this.start(task, context);
        }, trigger);
        SCHEDULED_FUTURES.put(task.getId(), future);
        log.info("定时任务[{}]已经添加到定时任务，调度策略：{}", task.getName(), task.getCron());
        log.info("定时任务：" + JSONObject.toJSONString(SCHEDULED_FUTURES.keySet()));
    }

    private void removeSchedule(TaskDocumentComparison task) {
        ScheduledFuture<?> future = SCHEDULED_FUTURES.remove(task.getId());
        if (future != null) {
            future.cancel(true);
        }
        log.info("定时任务[{}]已经从定时任务中移除", task.getName());
    }

    private GeneralResult<Void> run(Long id, Long userId) {
        try {
            boolean success = copInternalService.scheduleInspectCompare(id, HttpMethod.GET);
            if (success) {
                taskComparisonRepository.updateStatus(id, Status.Task.ACTIVE.name(), userId);
            }
            return new GeneralResult<>(success, success ? "定时任务激活成功" : "启动定时任务激活失败");
        } catch (Exception e) {
            log.error("run fail :::", e);
        }
        return new GeneralResult<>(false, "执行失败");
    }

    private GeneralResult<Void> stop(Long id, Long userId) {
        try {
            boolean success = copInternalService.scheduleInspectCompare(id, HttpMethod.DELETE);
            if (success) {
                taskComparisonRepository.updateStatus(id, Status.Task.SUSPEND.name(), userId);
            }
            return new GeneralResult<>(success, success ? "定时任务挂起成功" : "定时任务挂起失败");
        } catch (Exception e) {
            log.error("stop fail :::", e);
        }
        return new GeneralResult<>(false, "执行失败");
    }

    private GeneralResult<Void> comparison(TaskDocumentComparison param, TaskInstanceDocumentComparison instance,
        RequestContext context) {
        List<FileInputStream> filetreams = new ArrayList<>();
        // 解压路径
        String filePath = null;
        try {
            List<TaskInstanceDocumentComparisonItem> instanceItems = Lists.newArrayList();
            instance.setStatus(Status.Inst.SUCCESS.name());
            if (Comparison.category.standardVersionDocument.name().equals(param.getCategory())) {
                Map<String, List<StandardDocument>> documentMap = new HashMap<>();
                List<JSONObject> comparisons = JSONObject.parseArray(param.getComparisonObject(), JSONObject.class);
                if (Option.ALL.name().equals(param.getOptions())) {
                    // 分全量对比 - 对比版本下所有可以匹配的文件
                    List<Long> ids =
                        comparisons.stream().map(item -> item.getLong("versionId")).collect(Collectors.toList());
                    List<StandardDocument> documents = standardDocumentRepository.queryByVersionIds(ids);
                    documentMap = documents.stream().collect(Collectors.groupingBy(StandardDocument::getName));
                } else {
                    // 自定义对比 - 对比版本下指定的文件
                    JSONObject comparisonOne = comparisons.get(0);
                    JSONObject comparisonTwo = comparisons.get(1);
                    List<Long> fileIdsOne = comparisonOne.getJSONArray("fileIds").toJavaList(Long.class);
                    List<Long> fileIdsTwo = comparisonTwo.getJSONArray("fileIds").toJavaList(Long.class);
                    for (int i = 0; i < fileIdsOne.size(); i++) {
                        List<StandardDocument> documents = new ArrayList<>();
                        StandardDocument sourceDocument = standardDocumentRepository.query(fileIdsOne.get(i));
                        StandardDocument targetDocument = standardDocumentRepository.query(fileIdsTwo.get(i));
                        documents.add(sourceDocument);
                        documents.add(targetDocument);
                        documentMap.put(String.valueOf(i), documents);
                    }
                }
                for (String key : documentMap.keySet()) {
                    // 根据名称匹配版本内的文件，过滤掉匹配失败的数据
                    List<StandardDocument> documents = documentMap.get(key);
                    if (2 == documents.size()) {
                        StandardDocument sourceDocument = documents.get(0);
                        StandardDocument targetDocument = documents.get(1);
                        FileInputStream sourceStream = new FileInputStream(standardFilepath + File.separatorChar
                            + sourceDocument.getVersionCode() + File.separatorChar + sourceDocument.getName());
                        FileInputStream targetStream = new FileInputStream(standardFilepath + File.separatorChar
                            + targetDocument.getVersionCode() + File.separatorChar + targetDocument.getName());
                        String sourceMd5 = DigestUtils.md5DigestAsHex(sourceStream);
                        String targetMd5 = DigestUtils.md5DigestAsHex(targetStream);
                        filetreams.add(sourceStream);
                        filetreams.add(targetStream);
                        TaskInstanceDocumentComparisonItem item = new TaskInstanceDocumentComparisonItem();
                        item.setDocumentComparisonInstanceId(instance.getId());
                        item.setContrast(!sourceMd5.equals(targetMd5));
                        item.setSourceFile(sourceDocument.getVersionName() + "-" + sourceDocument.getName());
                        item.setSourceFileId(sourceDocument.getId());
                        item.setTargetFile(targetDocument.getVersionName() + "-" + targetDocument.getName());
                        item.setTargetFileId(targetDocument.getId());
                        item.setSourceMd5(sourceMd5);
                        item.setTargetMd5(targetMd5);
                        item.setCreatorId(context.getTarget());
                        item.setGmtCreate(new Date());
                        item.setStatus(Status.Node.SUCCESS.name());
                        instanceItems.add(item);
                    }
                }
            } else {
                Target target = JSONObject.parseObject(param.getTarget(),Target.class);
                List<JSONObject> comparisonObjects = JSONArray.parseArray(param.getComparisonObject()).toJavaList(JSONObject.class);
                List<Future<GeneralResult<Map<String, Object>>>> futures = new ArrayList<>();
                // 根据对比路径进行划分任务，原因是现有worker只能实现一条路径的备份
                for (int i = 0; i < comparisonObjects.size(); i++) {
                    JSONObject comparison = comparisonObjects.get(i);
                    log.info("任务名称:" + param.getName());
                    comparison.put("name", param.getName());
                    Task task = BackUpTaskBuilder.getTelnetTaskTask(comparison, target, 30 * 60);
                    // 执行备份任务
                    futures.add(taskExecutor.submit(() -> executeTask(comparison, task, context)));
                }
                filePath = decompressionPath + File.separatorChar + instance.getId();
                // 获取线程池返回结果
                List<String> instanceIds = Lists.newArrayList();
                for (int j = 0; j < futures.size(); j++) {
                    GeneralResult<Map<String, Object>> result = futures.get(j).get();
                    if (result.isFailed()) {
                        log.info("对比备份任务失败:::::" + result.getMessage());
                        instance.setStatus(Status.Node.FAILED.name());
                        continue;
                    }
                    Map<String, Object> resultData = result.getData();
                    // 将备份任务关联文件对比实例
                    String instanceId = String.valueOf(resultData.get("instanceId"));
                    instanceIds.add(instanceId);
                    List<JSONObject> nodes =
                        JSONArray.parseArray(String.valueOf(resultData.get("output"))).toJavaList(JSONObject.class);
                    List<TaskInstanceDocumentComparisonItem> comparisonItems = new ArrayList<>();
                    for (JSONObject node : nodes) {
                        log.info("node:::" + node.toJSONString());
                        JSONObject input = JSONObject.parseObject(String.valueOf(resultData.get("input")));
                        TaskInstanceDocumentComparisonItem item = new TaskInstanceDocumentComparisonItem();
                        item.setDocumentComparisonInstanceId(instance.getId());
                        item.setCreatorId(context.getTarget());
                        item.setGmtCreate(new Date());
                        item.setHostFilePath(input.getString("source"));
                        item.setTargetHost(node.getString("targetHost"));
                        String sourceMd5 = null;
                        String targetMd5 = null;
                        if (node.getBooleanValue("success")) {
                            // 要解压文件的绝对路径/home/cmp/10.20.12.65/test1.txt_20230427162723.tar.gz
                            String compressFilePath = basePath + node.getString("dest");
                            // 解压路径规则：comparison.decompression.path+当前实例id+主机ip+id =>
                            // /home/cmp/decompression/52/10.20.12.65/255
                            String decompressionFile =
                                decompressionPath + File.separatorChar + instance.getId() + File.separatorChar
                                    + node.getString("targetHost") + File.separatorChar + node.getInteger("id");
                            // 获取对比文件的绝对路径=> /home/cmp/decompression/52/10.20.12.65/255/test1.txt
                            String decompressionFilePath = decompressionFile + File.separatorChar
                                + StringUtils.substringAfterLast(input.getString("source"), "/");
                            targetMd5 = this.getTargetMd5(compressFilePath, decompressionFile, decompressionFilePath);
                            // node.getString("dest") => /10.20.12.65/test1.txt_20230427162723.tar.gz
                            item.setTargetFile(node.getString("dest"));
                        }
                        if (Comparison.category.standardDocument.name().equals(param.getCategory())) {
                            Long sourceFileId = input.getLong("fileId");
                            StandardDocument standardDocument = standardDocumentRepository.query(sourceFileId);
                            FileInputStream sourceStream = new FileInputStream(standardFilepath + File.separatorChar
                                + standardDocument.getVersionCode() + File.separatorChar + standardDocument.getName());
                            sourceMd5 = DigestUtils.md5DigestAsHex(sourceStream);
                            filetreams.add(sourceStream);
                            item.setSourceFile(standardDocument.getVersionName() + "-" + standardDocument.getName());
                            item.setSourceFileId(standardDocument.getId());
                        } else if (Comparison.category.historyDocument.name().equals(param.getCategory())) {
                            // 获取上次执行的targetMd5;第一次执行是没有历史数据的此时默认是相等的。
                            TaskInstanceDocumentComparison comparison =
                                instanceComparisonRepository.queryByDocumentComparisonId(param.getId());
                            if (null == comparison) {
                                sourceMd5 = targetMd5;
                                item.setSourceFile(item.getTargetFile());
                            } else {
                                TaskInstanceDocumentComparisonItem comparisonItem =
                                    instanceComparisonItemRepository.queryByInstanceAndIp(comparison.getId(),
                                        node.getString("targetHost"), input.getString("source"));
                                if (null != comparisonItem) {
                                    sourceMd5 = comparisonItem.getTargetMd5();
                                    item.setSourceFile(comparisonItem.getTargetFile());
                                }
                            }
                        }
                        log.info("sourceMd5::::" + sourceMd5 + ":::::targetMd5::::" + targetMd5);
                        item.setSourceMd5(sourceMd5);
                        item.setTargetMd5(targetMd5);
                        if (null == targetMd5 || null == sourceMd5) {
                            item.setContrast(true);
                        } else {
                            item.setContrast(!targetMd5.equals(sourceMd5));
                        }
                        item.setStatus(Status.Node.SUCCESS.name());
                        if (!Comparison.category.crossMachineDocument.name().equals(param.getCategory())) {
                            instanceItems.add(item);
                        } else {
                            comparisonItems.add(item);
                        }
                    }
                    // 单独处理跨机对比信息
                    if (Comparison.category.crossMachineDocument.name().equals(param.getCategory())) {
                        TaskInstanceDocumentComparisonItem comparisonItem = new TaskInstanceDocumentComparisonItem();
                        BeanUtils.copyProperties(comparisonItem, comparisonItems.get(1));
                        comparisonItem.setSourceHost(comparisonItems.get(0).getTargetHost());
                        comparisonItem.setSourceFile(comparisonItems.get(0).getTargetFile());
                        comparisonItem.setSourceMd5(comparisonItems.get(0).getTargetMd5());
                        if (null == comparisonItem.getSourceMd5() || null == comparisonItem.getTargetMd5()) {
                            comparisonItem.setContrast(true);
                        } else {
                            comparisonItem
                                .setContrast(!comparisonItem.getSourceMd5().equals(comparisonItem.getTargetMd5()));
                        }
                        instanceItems.add(comparisonItem);
                    }
                }
                // 备份任务更新到对比实例记录中
                instance.setTaskInstanceIds(String.valueOf(instanceIds));
            }
            if (!instanceItems.isEmpty()) {
                instanceComparisonItemRepository.batchSave(instanceItems, 100);
            }
            instance.setGmtFinish(new Date());
            instance.setGmtModify(new Date());
            instance.setMenderId(context.getTarget());
            instanceComparisonRepository.update(instance);
            if (null != filePath) {
                // 删除解压后的的所有文件，防止占用空间;
                log.info("删除解压路径下的文件：" + filePath);
                FileUtils.deleteDirectory(new File(filePath));
            }
            return new GeneralResult<>(true, "文件对比任务执行成功");
        } catch (Exception e) {
            log.error("Failed to execute the file comparison task :::" + e);
            return new GeneralResult<>(false, "文件对比任务执行失败");
        } finally {
            for (FileInputStream stream : filetreams) {
                try {
                    if (null != stream) {
                        stream.close();
                    }
                } catch (IOException e) {
                    log.error("sourceFileInputStream or targetFileInputStream close error :::" + e);
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 执行备份任务
     *
     * @param comparison
     * @param task
     * @param context
     * @return
     * @throws Exception
     */
    private GeneralResult<Map<String, Object>> executeTask(JSONObject comparison, Task task, RequestContext context)
        throws Exception {
        task.setType("compare");
        TaskInstance instance = taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
        if (null == instance) {
            return new GeneralResult<>(false, "执行失败：原因可能是作业节点分类参数格式错误");
        }
        boolean flag = taskScheduler.insertTask(buildJob(instance));
        if (!flag) {
            return new GeneralResult<>(false, "执行失败：文件备份任务下发失败");
        }
        long start = System.currentTimeMillis();
        Map<String, Object> map = new HashMap<>();
        Long timeout = 1800 * 1000L;
        while (true) {
            TaskInstance taskInstance = taskInstanceRepository.query(instance.getId());
            if ("SUCCESS".equalsIgnoreCase(taskInstance.getStatus())
                || "FAILED".equalsIgnoreCase(taskInstance.getStatus())) {
                map.putAll(getResult(taskInstance.getId(), comparison));
                break;
            }
            if (System.currentTimeMillis() - start > timeout) {
                map.putAll(getResult(taskInstance.getId(), comparison));
                log.info("Host ChangePswds Task get result time out");
                break;
            }
            try {
                Thread.sleep(5 * 1000L);
            } catch (InterruptedException e) {
                log.error("executeTask error :::" + e);
            }
        }
        return new GeneralResult<>(true, map, "任务执行完成");
    }

    private Map<String, Object> getResult(Long instanceId, JSONObject comparison) throws Exception {
        Map<String, Object> map = new HashMap<>();
        map.put("input", comparison);
        map.put("instanceId", instanceId);
        List<JSONObject> outData = Lists.newArrayList();
        List<TaskNode> nodes = taskNodeRepository.listByInstance(instanceId);
        log.info("jsonjsonjsonjson------" + JSONArray.toJSONString(nodes));
        // tasknode中的数据是按照目标主机存储的;存储每个目标主机完成任务的情况
        for (TaskNode node : nodes) {
            // 处理输入
            JSONObject output = JSONObject.parseObject(node.getOutput());
            log.info("outputoutputoutputoutput------" + JSONArray.toJSONString(output));
            if (null == output) {
                JSONObject value = new JSONObject();
                value.put("success", false);
                value.put("message", "执行异常");
                outData.add(value);
                continue;
            }
            List<JSONObject> datas = JSONObject.parseArray(output.getString("data"), JSONObject.class);
            if (ListTool.isEmpty(datas)) {
                JSONObject value = new JSONObject();
                value.put("success", false);
                value.put("message", "执行异常");
                outData.add(value);
                continue;
            }
            for (JSONObject data : datas) {
                log.info("datadatadatadata------" + JSONArray.toJSONString(data));
                JSONObject value = new JSONObject();
                value.put("id", data.getInteger("id"));
                value.put("success", data.getBooleanValue("success"));
                value.put("message", data.getString("message"));
                value.put("targetHost", data.getString("host"));
                if (data.getBooleanValue("success")) {
                    value.put("dest", data.getJSONObject("message").getString("dest"));
                    value.put("md5sum", data.getJSONObject("message").getString("md5sum"));
                }
                outData.add(value);
            }
        }
        map.put("output", outData);
        return map;
    }

    private Job buildJob(TaskInstance instance) {
        Job job = new Job();
        job.setParam(TaskParam.TASKDATA.name(), JSONObject.toJSONString(instance));
        job.setParam(TaskParam.TASKTYPE.name(), Catalog.Task.STANDARD.getValue());
        job.setTaskId(instance.getUuid());
        job.setParam(Common.NAME, instance.getName());
        job.setParam(TaskParam.TASKCODE.name(), instance.getCode());
        return job;
    }

    private String getTargetMd5(String compressFile, String decompressionFile, String decompressionFilePath) {
        String targetMd5 = null;
        FileInputStream in = null;
        try {
            log.info("解压路径：" + decompressionFile);
            CompressUtil.decompressTarGz(new File(compressFile), decompressionFile);
            in = new FileInputStream(decompressionFilePath);
            targetMd5 = DigestUtils.md5DigestAsHex(in);
        } catch (IOException e) {
            log.error("get md5 error :::" + e);
        } finally {
            try {
                if (null != in) {
                    in.close();
                }
            } catch (IOException e) {
                log.error("in close error :::" + e);
                e.printStackTrace();
            }
        }
        return targetMd5;
    }

}