package com.bocloud.cop.service.compare;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.compare.StandardDocument;
import com.bocloud.cop.entity.compare.StandardDocumentVersion;
import com.bocloud.cop.entity.compare.TaskDocumentComparison;
import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparison;
import com.bocloud.cop.entity.compare.TaskInstanceDocumentComparisonItem;
import com.bocloud.cop.enums.Comparison;
import com.bocloud.cop.intf.compare.TaskInstanceDocumentComparisonService;
import com.bocloud.cop.model.compare.ContrastFileModel;
import com.bocloud.cop.repository.TaskHostRepository;
import com.bocloud.cop.repository.compare.StandardDocumentRepository;
import com.bocloud.cop.repository.compare.StandardDocumentVersionRepository;
import com.bocloud.cop.repository.compare.TaskDocumentComparisonRepository;
import com.bocloud.cop.repository.compare.TaskInstanceDocumentComparisonItemRepository;
import com.bocloud.cop.repository.compare.TaskInstanceDocumentComparisonRepository;
import com.bocloud.cop.service.utils.CompressUtil;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.IDFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 17:36:49
 * @Description:
 */
@Slf4j
@Service("taskInstanceDocumentComparisonService")
public class TaskInstanceDocumentComparisonServiceImpl implements TaskInstanceDocumentComparisonService {

    @Autowired
    private TaskInstanceDocumentComparisonRepository comparisonRepository;
    @Autowired
    private TaskInstanceDocumentComparisonItemRepository comparisonItemRepository;
    @Autowired
    private StandardDocumentRepository documentRepository;
    @Autowired
    private StandardDocumentVersionRepository documentVersionRepository;
    @Autowired
    private TaskDocumentComparisonRepository documentComparisonRepository;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Value("${comparison.upload.path:/home/cmp/standard-file}")
    private String standardFilepath;
    @Value("${spring.upload.path:/home/cmp}")
    private String basePath;
    @Value("${comparison.result.path:/home/cmp/comparison/result}")
    private String comparisonResultPath;

    @Override
    public GeneralResult<GridBean<TaskInstanceDocumentComparison>> list(Pager pager) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = comparisonRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = comparisonRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<TaskInstanceDocumentComparison> list =
                    comparisonRepository.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter());
                if (CollectionUtils.isEmpty(list)) {
                    gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
                    return new GeneralResult<>(true, gridBean, "查询成功");
                }
                List<Long> comparisonIds = list.stream().map(TaskInstanceDocumentComparison::getDocumentComparisonId)
                    .collect(Collectors.toList());
                List<TaskDocumentComparison> comparisons = documentComparisonRepository.queryByIds(comparisonIds);
                for (TaskInstanceDocumentComparison instance : list) {
                    TaskDocumentComparison comparison =
                        comparisons.stream().filter(item -> item.getId().equals(instance.getDocumentComparisonId()))
                            .findFirst().orElse(null);
                    if (null != comparison) {
                        List<Long> ids = JSONObject.parseArray(comparison.getHostIds(), Long.class);
                        List<TaskHost> targets = taskHostRepository.queryByIds(ids);
                        String hostIps = targets.stream().map(TaskHost::getIp).collect(Collectors.joining(","));
                        instance.setCron(comparison.getCron());
                        instance.setHostIps(hostIps);
                        instance.setCreator(redisTemplate.opsForValue().get("User_Name_" + comparison.getCreatorId()));
                        if (null == instance.getGmtFinish() || null == instance.getGmtStart()) {
                            instance.setCost(Double.valueOf(0));
                        } else {
                            BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
                            BigDecimal end = new BigDecimal(instance.getGmtFinish().getTime());
                            instance.setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP)
                                .doubleValue());
                        }
                    }
                }
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List Query Is Fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<ContrastFileModel>> contrast(Long id, Long itemId) {
        String decompressionFile = null;
        try {
            TaskInstanceDocumentComparison instance = comparisonRepository.query(id);
            if (null == instance) {
                return new GeneralResult<>(false, "任务实例数据不存在");
            }
            TaskInstanceDocumentComparisonItem instanceItem = comparisonItemRepository.query(itemId);
            if (null == instanceItem) {
                return new GeneralResult<>(false, "对比详情不存在");
            }
            TaskDocumentComparison comparison = documentComparisonRepository.query(instance.getDocumentComparisonId());
            if (null == comparison) {
                return new GeneralResult<>(false, "任务不存在");
            }
            String category = comparison.getCategory();
            List<ContrastFileModel> fileModels = new ArrayList<>();
            if (Comparison.category.standardVersionDocument.name().equals(category)) {
                List<Long> ids = Lists.newArrayList();
                ids.add(instanceItem.getSourceFileId());
                ids.add(instanceItem.getTargetFileId());
                List<StandardDocument> documents = documentRepository.queryByIds(ids);
                for (StandardDocument document : documents) {
                    StandardDocumentVersion version = documentVersionRepository.query(document.getVersionId());
                    fileModels.add(new ContrastFileModel(standardFilepath + File.separatorChar + version.getCode()
                        + File.separatorChar + document.getName()));
                }
            } else if (Comparison.category.standardDocument.name().equals(category)) {
                // 源文件
                StandardDocument document = documentRepository.query(instanceItem.getSourceFileId());
                StandardDocumentVersion version = documentVersionRepository.query(document.getVersionId());
                fileModels.add(new ContrastFileModel(standardFilepath + File.separatorChar + version.getCode()
                    + File.separatorChar + document.getName()));
                // 目标文件
                String targetFile = instanceItem.getTargetFile();
                if (StringUtils.isNotEmpty(targetFile)) {
                    // 解压路径规则：file.decompression.path+当前实例id+主机ip+id
                    decompressionFile =
                        comparisonResultPath + File.separatorChar + instanceItem.getDocumentComparisonInstanceId()
                            + File.separatorChar + instanceItem.getId() + IDFactory.instance().uuid();
                    String targetFilePath = this.getPecompressionFilePath(instanceItem, targetFile, decompressionFile);
                    if (StringUtils.isEmpty(targetFilePath)) {
                        return new GeneralResult<>(false, "获取解压文件失败");
                    }
                    fileModels.add(new ContrastFileModel(targetFilePath));
                }
            } else {
                // 源文件
                String sourceFile = instanceItem.getSourceFile();
                if (StringUtils.isNotEmpty(sourceFile)) {
                    // 解压路径规则：file.decompression.path+当前实例id+主机ip+id
                    decompressionFile =
                        comparisonResultPath + File.separatorChar + instanceItem.getDocumentComparisonInstanceId()
                            + File.separatorChar + instanceItem.getId() + IDFactory.instance().uuid();
                    String sourceFilePath = this.getPecompressionFilePath(instanceItem, sourceFile, decompressionFile);
                    if (StringUtils.isEmpty(sourceFilePath)) {
                        return new GeneralResult<>(false, "获取解压文件失败");
                    }
                    fileModels.add(new ContrastFileModel(sourceFilePath));
                }
                String targetFile = instanceItem.getTargetFile();
                if (StringUtils.isNotEmpty(targetFile)) {
                    // 目标文件
                    // 解压路径规则：file.decompression.path+当前实例id+主机ip+id
                    decompressionFile =
                        comparisonResultPath + File.separatorChar + instanceItem.getDocumentComparisonInstanceId()
                            + File.separatorChar + instanceItem.getId() + IDFactory.instance().uuid();
                    String targetFilePath = this.getPecompressionFilePath(instanceItem, targetFile, decompressionFile);
                    if (StringUtils.isEmpty(targetFilePath)) {
                        return new GeneralResult<>(false, "获取解压文件失败");
                    }
                    fileModels.add(new ContrastFileModel(targetFilePath));
                }
            }
            for (ContrastFileModel fileModel : fileModels) {
                File file = new File(fileModel.getFilePath());
                try (FileReader reader = new FileReader(file)) {
                    int readerLength;
                    StringBuilder builder = new StringBuilder();
                    char[] buffer = new char[5];
                    while ((readerLength = reader.read(buffer)) != -1) {
                        builder.append(buffer, 0, readerLength);
                    }
                    fileModel.setContent(builder.toString());
                } catch (FileNotFoundException e) {
                    log.error("文件读取失败", e);
                    return new GeneralResult<>(false, "文件[" + fileModel.getFilePath() + "]读取失败");
                }
            }
            if (null != decompressionFile) {
                // 删除解压过的文件
                FileUtils.deleteDirectory(new File(
                    comparisonResultPath + File.separatorChar + instanceItem.getDocumentComparisonInstanceId()));
            }
            return new GeneralResult<>(true, fileModels, "查询成功");
        } catch (Exception e) {
            log.error("Details query failed:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private String getPecompressionFilePath(TaskInstanceDocumentComparisonItem comparisonItem, String filePath,
        String decompressionFile) {
        String decompressionFilePath = null;
        try {
            // 要解压文件的绝对路径
            String compressFilePath = basePath + File.separatorChar + filePath;
            CompressUtil.decompressTarGz(new File(compressFilePath), decompressionFile);
            // 获取对比文件的绝对路径
            decompressionFilePath = decompressionFile + File.separatorChar
                + StringUtils.substringAfterLast(comparisonItem.getHostFilePath(), "/");
        } catch (Exception e) {
            log.error("getPecompressionFilePath error:", e);
        }
        return decompressionFilePath;
    }

    @Override
    public GeneralResult<List<TaskHost>> instanceHostList(Long id, RequestContext context) {
        try {
            TaskInstanceDocumentComparison comparison = comparisonRepository.query(id);
            String instanceIds = comparison.getTaskInstanceIds();
            if (null == instanceIds) {
                return new GeneralResult<>(true, "查询成功");
            }
            List<TaskInstanceDocumentComparisonItem> items =
                comparisonItemRepository.queryByInstanceId(comparison.getId());
            if (CollectionUtils.isEmpty(items)) {
                return new GeneralResult<>(true, "查询成功");
            }
            List<String> hostIpList =
                items.stream().map(TaskInstanceDocumentComparisonItem::getTargetHost).collect(Collectors.toList());
            // 去重
            List<String> hostIps = hostIpList.stream().distinct().collect(Collectors.toList());
            List<TaskHost> taskHosts = taskHostRepository.queryByIps(hostIps);
            taskHosts = taskHosts.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(TaskHost::getIp))), ArrayList::new));
            return new GeneralResult<>(true, taskHosts, "查询成功");
        } catch (Exception e) {
            log.error("query taskInstanceHostList error", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<StandardDocument>> instanceStandardFiles(Long id, RequestContext context) {
        try {
            List<StandardDocument> documents = new ArrayList<>();
            TaskInstanceDocumentComparison comparison = comparisonRepository.query(id);
            if (null == comparison) {
                return new GeneralResult<>(false, "没有对应实例信息");
            }
            List<TaskInstanceDocumentComparisonItem> items = comparisonItemRepository.queryByInstanceId(id);
            if (CollectionUtils.isEmpty(items)) {
                return new GeneralResult<>(true, "查询成功");
            }
            // 去重
            List<TaskInstanceDocumentComparisonItem> comparisonItems = new ArrayList<>();
            if (Comparison.category.standardDocument.name().equals(comparison.getCategory())) {
                comparisonItems = items.stream()
                    .collect(Collectors.collectingAndThen(Collectors.toCollection(
                        () -> new TreeSet<>(Comparator.comparing(TaskInstanceDocumentComparisonItem::getSourceFile))),
                        ArrayList::new));
            } else {
                comparisonItems = items.stream()
                    .collect(Collectors.collectingAndThen(Collectors.toCollection(
                        () -> new TreeSet<>(Comparator.comparing(TaskInstanceDocumentComparisonItem::getHostFilePath))),
                        ArrayList::new));
            }
            comparisonItems.forEach(item -> {
                StandardDocument document = new StandardDocument();
                document.setId(item.getSourceFileId());
                if (Comparison.category.standardDocument.name().equals(comparison.getCategory())) {
                    document.setSourceFile(item.getSourceFile());
                } else {
                    document.setHostFilePath(item.getHostFilePath());
                }
                documents.add(document);
            });
            return new GeneralResult<>(true, documents, "查询成功");
        } catch (Exception e) {
            log.error("query taskInstanceHostList error", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }
}