package com.bocloud.cop.service.inspection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.bocloud.cop.entity.inspection.InspectItem;
import com.megatron.common.utils.Common;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.intf.inspection.ExecuteTaskService;
import com.bocloud.cop.intf.inspection.InspectRecordService;
import com.bocloud.cop.intf.inspection.InspectRoutineService;
import com.bocloud.cop.intf.inspection.InspectService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class InspectServiceImpl implements InspectService {

    @Autowired
    private InspectRoutineService inspectRoutineService;
    @Autowired
    private InspectRecordService inspectRecordService;

    @Override
    public GeneralResult<Object> firstPageData(Integer limitDay, RequestContext context, String type, String code) {
        try {
            List<Param> params = new ArrayList<>();
            if (StringUtils.isBlank(type)) {
                type = InspectItem.Type.INSPECTION.name();
            }
            params.add(new Param(MapTools.simpleMap(Common.TYPE, type), Sign.EQ));
            Map<String, Object> result = new HashMap<>();
            if ("statsInfo".equalsIgnoreCase(code)) {
                Long allTodayTime = inspectRecordService.count(0, null, null, null, null, 1L, type);
                Long youTodayTime = inspectRecordService.count(0, null, null, null, context.getTarget(), 1L, type);
                Long warningTodayTime = inspectRecordService.count(0, null, null, true, null, 1L, type);
                Long errorTodayTime =
                    inspectRecordService.count(0, null, InspectRecord.Status.ERROR.name(), null, null, 1L, type);
                result.put("allTodayTime", allTodayTime);
                result.put("youTodayTime", youTodayTime);
                result.put("warningTodayTime", warningTodayTime);
                result.put("errorTodayTime", errorTodayTime);
            } else if ("limitRecord10".equalsIgnoreCase(code)) {
                GeneralResult<?> rt = inspectRecordService.limit(new Pager(0, 10, params, null), 1L);
                return new GeneralResult<>(true, rt.getData(), "查询成功");
            } else if ("statsHistory".equalsIgnoreCase(code)) {
                result = inspectRecordService.getChartData(limitDay, null, 1L, type);
            } else if ("statsHistory10".equalsIgnoreCase(code)) {
                result = inspectRecordService.getPeiChartData(limitDay, null, 1L, type);
            } else if ("category".equalsIgnoreCase(code)) {
                result = inspectRecordService.countGroupByCategory(0, type);
            }
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("summary fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> run(ExecuteTaskService.RunData data, RequestContext context) {
        data.setType(InspectRecord.Type.FAST.ordinal());
        data.setCode(Catalog.Task.STANDARD.getValue());
        return inspectRoutineService.execute(data, context.getTarget(), "F-" + UUID.randomUUID());
    }

    @Override
    public GeneralResult<Map<String, Object>> firstPageDataOverview(RequestContext context, String type) {
        try {
            Long allTodayTime = inspectRecordService.count(0, null, null, null, null, 1L, type);
            Long youTodayTime = inspectRecordService.count(0, null, null, null, context.getTarget(), 1L, type);
            Long warningTodayTime = inspectRecordService.count(0, null, null, true, null, 1L, type);
            Long errorTodayTime =
                inspectRecordService.count(0, null, InspectRecord.Status.ERROR.name(), null, null, 1L, type);
            Map<String, Object> category = inspectRecordService.countGroupByCategory(0, type);

            JSONObject json = new JSONObject();
            json.put("allTodayTime", allTodayTime);
            json.put("youTodayTime", youTodayTime);
            json.put("warningTodayTime", warningTodayTime);
            json.put("errorTodayTime", errorTodayTime);
            json.put("category", category);
            return new GeneralResult<>(true, json, "查询成功");
        } catch (Exception e) {
            log.error("summary current fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Map<String, Object>> firstPageDataHistoryInfo(Integer limitDay, String category,
        RequestContext context, String type) {
        try {
            // 查询数据
            Map<String, Object> chartData = inspectRecordService.getChartData(limitDay, category, 1L, type);
            Map<String, Object> peiChartData = inspectRecordService.getPeiChartData(limitDay, category, 1L, type);
            // 查询列表
            List<Param> params = new ArrayList<>();
            if (!StringUtils.isEmpty(category)) {
                params.add(new Param(MapTools.simpleMap("category", category), Sign.EQ));
            }
            GeneralResult<?> result = inspectRecordService.list(new Pager(0, 10, params, null), 1L);
            JSONObject json = new JSONObject();
            json.put("chartData", chartData);
            json.put("peiChartData", peiChartData);
            json.put("tableData", result.getData());
            return new GeneralResult<>(true, json, "查询成功");
        } catch (Exception e) {
            log.error("summary history fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
