package com.bocloud.cop.service.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cop.model.Role;
import com.bocloud.cop.model.UserGroupRelationBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

@Component
public class SmsInternalService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsInternalService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private static final Service SERVICE = Service.create(BoCloudService.SMS.getName());
    private final Map<String, Object> headers = new HashMap<>(4);

    public GeneralResult<List<Role>> listByUid(Long userId) {
        String url = "/v1/users/" + userId + "/roles";
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseArray(data, Role.class), "success");
    }

    public GeneralResult<JSONObject> getUser(Long userId) {
        String url = "/v1/users/" + userId;
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data), "success");
    }

    /**
     * 根据用户id查询用户组
     * 
     * @param userId
     *            用户id
     * @return
     */
    public GeneralResult<List<UserGroupRelationBean>> listUserGroupRelation(Long userId) {
        String url = "/v1/users/groups/relation/users/" + userId;
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseArray(data, UserGroupRelationBean.class), "success");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
