package com.bocloud.cop.service.patch;

import com.bocloud.cop.entity.patch.PatchDeployHistory;
import com.bocloud.cop.entity.patch.PatchDeployHistoryDetail;
import com.bocloud.cop.intf.patch.PatchDeployHistoryService;
import com.bocloud.cop.model.ResultTrend;
import com.bocloud.cop.model.StatResult;
import com.bocloud.cop.model.TrendData;
import com.bocloud.cop.repository.patch.PatchDeployHistoryDetailRepository;
import com.bocloud.cop.repository.patch.PatchDeployHistoryRepository;
import com.google.common.collect.Lists;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.ContentDisposition;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Repository;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.*;

@Slf4j
@Repository("patchDeployHistoryService")
public class PatchDeployHistoryServiceImpl implements PatchDeployHistoryService {

    @Autowired
    private PatchDeployHistoryRepository patchDeployHistoryRepository;
    @Autowired
    private PatchDeployHistoryDetailRepository patchDeployHistoryDetailRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    private static final short BOLD_FONT = 20;
    private static final short NORMAL_FONT = 12;

    @Override
    public GeneralResult<GridBean<PatchDeployHistory>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int count = patchDeployHistoryRepository.count(pager.getParams());
            List<PatchDeployHistory> list = patchDeployHistoryRepository.list(pager);
            for (PatchDeployHistory history : list) {
                history.setCreator(redisTemplate.opsForValue().get("User_Name_" + history.getCreatorId()));
            }
            GridBean<PatchDeployHistory> bean = new GridBean<>(pager.getPage(), pager.getRows(), count, list);
            return new GeneralResult<>(true, bean, "获取数据成功！");
        } catch (Exception e) {
            log.error("List deploy patch history fail:", e);
            return new GeneralResult<>(false, "获取数据失败！");
        }
    }

    @Override
    public GeneralResult<GridBean<PatchDeployHistoryDetail>> listDetails(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int count = patchDeployHistoryDetailRepository.count(pager.getParams());
            List<PatchDeployHistoryDetail> list = patchDeployHistoryDetailRepository.list(pager);
            for (PatchDeployHistoryDetail detail : list) {
                detail.setCreator(redisTemplate.opsForValue().get("User_Name_" + detail.getCreatorId()));
            }
            GridBean<PatchDeployHistoryDetail> bean = new GridBean<>(pager.getPage(), pager.getRows(), count, list);
            return new GeneralResult<>(true, bean, "获取数据成功！");
        } catch (Exception e) {
            log.error("List deploy patch detail fail:", e);
            return new GeneralResult<>(false, "获取数据失败！");
        }
    }

    @Override
    public void excel(Long id, HttpServletResponse response) {
        SimpleDateFormat sdfTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String, Object> map = new HashMap<>();
        HSSFWorkbook workbook = new HSSFWorkbook();
        int j = 13;
        try {
            PatchDeployHistory history = patchDeployHistoryRepository.query(id);

            HSSFSheet sheet = workbook.createSheet();
            sheet.setColumnWidth(0, 30 * 256);
            sheet.setColumnWidth(1, 20 * 256);
            sheet.setColumnWidth(2, 30 * 256);
            sheet.setColumnWidth(3, 40 * 256);
            sheet.setColumnWidth(4, 40 * 256);
            sheet.setColumnWidth(5, 60 * 256);
            sheet.setDefaultRowHeight((short) 400);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, 3));
            HSSFRow row0 = sheet.createRow(0);
            HSSFCell cell = row0.createCell(0);
            cell.setCellStyle(getStyle(workbook, BOLD_FONT, true));
            cell.setCellValue(history.getName());

            HSSFRow row1 = sheet.createRow(2);
            cell = row1.createCell(0);
            cell.setCellValue("任务名称");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row1.createCell(1);
            cell.setCellValue(history.getName());
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(2, 2, 1, 3));

            HSSFRow row2 = sheet.createRow(3);
            cell = row2.createCell(0);
            cell.setCellValue("任务类型");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row2.createCell(1);
            cell.setCellValue(history.getType());
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(3, 3, 1, 3));

            HSSFRow row3 = sheet.createRow(4);
            cell = row3.createCell(0);
            cell.setCellValue("执行人员");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row3.createCell(1);
            cell.setCellValue(redisTemplate.opsForValue().get("User_Name_" + history.getCreatorId()));
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(4, 4, 1, 3));

            HSSFRow row4 = sheet.createRow(5);
            cell = row4.createCell(0);
            cell.setCellValue("完成时间");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row4.createCell(1);
            cell.setCellValue(sdfTime.format(history.getGmtModify()));
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(5, 5, 1, 3));

            HSSFRow row5 = sheet.createRow(6);
            cell = row5.createCell(0);
            cell.setCellValue("服务器数量");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row5.createCell(1);
            cell.setCellValue(history.getServeNum());
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(6, 6, 1, 3));

            HSSFRow row6 = sheet.createRow(7);
            cell = row6.createCell(0);
            cell.setCellValue("补丁数");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row6.createCell(1);
            cell.setCellValue(history.getPatchNum());
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(7, 7, 1, 3));

            String[] failSuccessTotals = history.getFailSuccessTotal().split("[/]");

            HSSFRow row8 = sheet.createRow(8);
            cell = row8.createCell(0);
            cell.setCellValue("安装补丁失败数");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row8.createCell(1);
            cell.setCellValue(Integer.valueOf(failSuccessTotals[0].trim()));
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(8, 8, 1, 3));

            HSSFRow row9 = sheet.createRow(9);
            cell = row9.createCell(0);
            cell.setCellValue("安装补丁成功数");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row9.createCell(1);
            cell.setCellValue(Integer.valueOf(failSuccessTotals[1].trim()));
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(9, 9, 1, 3));

            HSSFRow row10 = sheet.createRow(10);
            cell = row10.createCell(0);
            cell.setCellValue("需安装补丁总数");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row10.createCell(1);
            cell.setCellValue(Integer.valueOf(failSuccessTotals[2].trim()));
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
            sheet.addMergedRegion(new CellRangeAddress(10, 10, 1, 3));

            HSSFRow row = sheet.createRow(j);
            cell = row.createCell(0);
            cell.setCellValue("主机名称");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row.createCell(1);
            cell.setCellValue("主机IP");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row.createCell(2);
            cell.setCellValue("补丁名称");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row.createCell(3);
            cell.setCellValue("状态");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row.createCell(4);
            cell.setCellValue("更新时间");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));
            cell = row.createCell(5);
            cell.setCellValue("报错信息");
            cell.setCellStyle(getStyle(workbook, NORMAL_FONT, true));

            List<PatchDeployHistoryDetail> details = patchDeployHistoryDetailRepository.listByHistoryId(id);
            j = j + 1;
            for (PatchDeployHistoryDetail detail : details) {
                row = sheet.createRow(j);
                cell = row.createCell(0);
                cell.setCellValue(detail.getName());
                cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                cell = row.createCell(1);
                cell.setCellValue(detail.getHostIp());
                cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                cell = row.createCell(2);
                cell.setCellValue(detail.getPatchName());
                cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                cell = row.createCell(3);
                cell.setCellValue("success".equalsIgnoreCase(detail.getStatus()) ? "成功" : "失败");
                cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                cell = row.createCell(4);
                cell.setCellValue(sdfTime.format(detail.getGmtModify()));
                cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                cell = row.createCell(5);
                cell.setCellValue(detail.getErrorInfo());
                cell.setCellStyle(getStyle(workbook, NORMAL_FONT, false));
                j++;
            }
            String name = history.getName() + ".xls";
            map.put("name", name);
            ContentDisposition disposition =
                ContentDisposition.attachment().filename(name, StandardCharsets.UTF_8).build();
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, disposition.toString());
            response.setStatus(HttpStatus.OK.value());
            workbook.write(response.getOutputStream());
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (Exception e) {
            log.error("Export excel error: {}", e.getMessage(), e);
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                log.error("workbook close error:", e);
            }
        }
    }

    @Override
    public void pdf(Long id, HttpServletResponse response) {
        PatchDeployHistory history = patchDeployHistoryRepository.query(id);
        String name = history.getName() + ".pdf";
        ContentDisposition disposition = ContentDisposition.attachment().filename(name, StandardCharsets.UTF_8).build();
        response.addHeader(HttpHeaders.CONTENT_DISPOSITION, disposition.toString());
        response.setStatus(HttpStatus.OK.value());
        try {
            this.covertPdf(history, response.getOutputStream());
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (Exception e) {
            log.error("将补丁历史导出为PDF失败：{}", e);
        }
    }

    private void covertPdf(PatchDeployHistory history, OutputStream outputStream) throws Exception {
        SimpleDateFormat sdfTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 第一步，实例化一个document对象
        Document document = new Document();
        // 第三步,设置字符,宋体
        BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        Font titleFont = new Font(bfChinese, 16.0F, Font.NORMAL);
        Font contentFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        Font tableFont = new Font(bfChinese, 8.0F, Font.NORMAL);
        // 第四步，将pdf文件输出到磁盘
        PdfWriter.getInstance(document, outputStream);
        // 第五步，打开生成的pdf文件
        document.open();

        Font greenFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        greenFont.setColor(BaseColor.GREEN);
        Font redFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        redFont.setColor(BaseColor.RED);
        Font orangeFont = new Font(bfChinese, 10.0F, Font.NORMAL);
        orangeFont.setColor(BaseColor.ORANGE);

        String title = history.getName();
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Element.ALIGN_CENTER);
        paragraph.add(new Chunk(title, titleFont).setLocalDestination(title));
        document.add(paragraph);
        document.add(new Paragraph("\n"));

        PdfPTable table = new PdfPTable(2);
        table.setHorizontalAlignment(Element.ALIGN_CENTER);
        float[] widths = {44f, 100f};
        table.setWidths(widths);

        PdfPCell cell = new PdfPCell(new Paragraph("任务名称", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(history.getName(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("任务类型", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(history.getType(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("执行人员", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(
            new Paragraph(redisTemplate.opsForValue().get("User_Name_" + history.getCreatorId()), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("更新时间", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(sdfTime.format(history.getGmtModify()), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("服务器数量", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(history.getServeNum() + "", contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("补丁数", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(history.getPatchNum() + "", contentFont));
        setAndAddCellStyle(table, cell);

        String[] failSuccessTotals = history.getFailSuccessTotal().split("[/]");

        cell = new PdfPCell(new Paragraph("安装补丁失败数", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(failSuccessTotals[0].trim(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("安装补丁成功数", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(failSuccessTotals[1].trim(), contentFont));
        setAndAddCellStyle(table, cell);

        cell = new PdfPCell(new Paragraph("需安装补丁总数", contentFont));
        setAndAddCellStyle(table, cell);
        cell = new PdfPCell(new Paragraph(failSuccessTotals[2].trim(), contentFont));
        setAndAddCellStyle(table, cell);

        PdfPTable table1 = new PdfPTable(6);
        table1.setHorizontalAlignment(Element.ALIGN_CENTER);

        float[] floats = {16f, 16f, 16f, 16f, 16f, 16f};
        table1.setWidths(floats);
        cell = new PdfPCell(getDetailInfo(history.getId(), table1, tableFont));
        cell.setColspan(2);
        setAndAddCellStyle(table, cell);
        document.add(table);
        // 关闭document
        document.close();
    }

    private void setAndAddCellStyle(PdfPTable table, PdfPCell cell) {
        cell.setMinimumHeight(30.0F);
        cell.setPaddingTop(10f);
        cell.setPaddingBottom(10f);
        setCellCenter(cell);
        cell.setBorder(Rectangle.NO_BORDER);
        table.addCell(cell);
    }

    private void setCellCenter(PdfPCell cell) {
        cell.setMinimumHeight(40);
        cell.setHorizontalAlignment(Element.ALIGN_CENTER);
        cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
    }

    private PdfPTable getDetailInfo(Long id, PdfPTable table, Font tableFont) {
        PdfPCell cell = new PdfPCell(new Paragraph("主机名称", tableFont));
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("主机ip", tableFont));
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("补丁名称", tableFont));
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("状态", tableFont));
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("更新时间", tableFont));
        table.addCell(cell);

        cell = new PdfPCell(new Paragraph("报错信息", tableFont));
        table.addCell(cell);

        SimpleDateFormat sdfTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<PatchDeployHistoryDetail> details = patchDeployHistoryDetailRepository.listByHistoryId(id);
        for (PatchDeployHistoryDetail detail : details) {
            cell = new PdfPCell(new Paragraph(detail.getName(), tableFont));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph(detail.getHostIp()));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph(detail.getPatchName(), tableFont));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph("success".equalsIgnoreCase(detail.getStatus()) ? "成功" : "失败", tableFont));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph(sdfTime.format(detail.getGmtModify())));
            setCellCenter(cell);
            table.addCell(cell);

            cell = new PdfPCell(new Paragraph(detail.getErrorInfo(), tableFont));
            setCellCenter(cell);
            table.addCell(cell);
        }
        return table;
    }

    @Override
    public GeneralResult<List<StatResult>> statsStatus(RequestContext context) {
        Pager pager = new Pager();
        pager.setRows(Integer.MAX_VALUE);
        List<PatchDeployHistoryDetail> list = patchDeployHistoryDetailRepository.list(pager);
        int success = 0;
        int error = 0;
        for (PatchDeployHistoryDetail detail : list) {
            if ("fail".equalsIgnoreCase(detail.getStatus())) {
                error++;
                continue;
            } else if ("success".equalsIgnoreCase(detail.getStatus())) {
                success++;
                continue;
            }
        }
        List<StatResult> results = new ArrayList<>();
        results.add(new StatResult("成功", success));
        results.add(new StatResult("失败", error));
        return new GeneralResult<>(true, results, "获取补丁安装状态统计成功");
    }

    @Override
    public GeneralResult<ResultTrend> statsTrend(RequestContext context) {
        List<String> keys = Lists.newArrayList();
        List<String> dateKeys = Lists.newArrayList();
        List<TrendData> values = Lists.newArrayList();
        List<Integer> success = Lists.newArrayList();
        List<Integer> fails = Lists.newArrayList();
        List<Param> params = Lists.newArrayList();

        Calendar calendar = null;
        for (int i = 29; i >= 0; i--) {
            calendar = Calendar.getInstance();
            calendar.add(Calendar.DATE, -i);
            int year = calendar.get(Calendar.YEAR);
            int month = calendar.get(Calendar.MONTH) + 1;
            int day = calendar.get(Calendar.DAY_OF_MONTH);
            String months = month < 10 ? "0" + month : "" + month;
            String days = day < 10 ? "0" + day : "" + day;
            dateKeys.add(year + "-" + months + "-" + days);
            keys.add(months + "-" + days);
        }
        params.add(new Param(MapTools.simpleMap("gmtCreate", dateKeys.get(0) + " 00:00:00"), Sign.GET));
        params.add(new Param(MapTools.simpleMap("gmtCreate", dateKeys.get(keys.size() - 1) + " 23:59:59"), Sign.LET));

        List<PatchDeployHistoryDetail> records =
            patchDeployHistoryDetailRepository.list(new Pager(1, Integer.MAX_VALUE, params, null));
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        Map<String, List<PatchDeployHistoryDetail>> map = new HashMap<>();
        for (PatchDeployHistoryDetail record : records) {
            String key = format.format(record.getGmtCreate()).substring(0, 10);
            if (map.containsKey(key)) {
                List<PatchDeployHistoryDetail> list = map.get(key);
                list.add(record);
                map.put(key, list);
            } else {
                List<PatchDeployHistoryDetail> list = new ArrayList<>();
                list.add(record);
                map.put(key, list);
            }
        }
        for (String key : dateKeys) {
            int successes = 0;
            int faileds = 0;
            List<PatchDeployHistoryDetail> list = map.get(key);
            if (!ListTool.isEmpty(list)) {
                for (PatchDeployHistoryDetail record : list) {
                    if ("fail".equalsIgnoreCase(record.getStatus())) {
                        faileds++;
                        continue;
                    } else if ("success".equalsIgnoreCase(record.getStatus())) {
                        successes++;
                        continue;
                    }
                }
            }
            success.add(successes);
            fails.add(faileds);
        }
        values.add(new TrendData("成功", success));
        values.add(new TrendData("失败", fails));
        return new GeneralResult<>(true, new ResultTrend(keys, values), "获取补丁执行趋势数据成功");
    }

    @Override
    public List<PatchDeployHistoryDetail> queryByPatchIdAndStatus(Long patchId, String status) {
        return patchDeployHistoryDetailRepository.listByPatchIdAndStatus(patchId, status);
    }

    @Override
    public List<PatchDeployHistoryDetail> queryByDeployIdAndStatus(Long deployId, String status) {
        return patchDeployHistoryDetailRepository.listByDeployIdAndStatus(deployId, status);
    }



    /**
     * 标题或者第一列数据信息单元格样式
     *
     * @param fontHeight
     *            标题20， 第一列12，第二列12
     */
    private  HSSFCellStyle getStyle(HSSFWorkbook workbook, short fontHeight, boolean isBold) {

        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((fontHeight));
        font.setBold(isBold);
        font.setFontName("宋体");

        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        return style;
    }
}
