package com.bocloud.cop.service.patch;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.QueueBuilder;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.patch.Patch;
import com.bocloud.cop.entity.patch.PatchDeploy;
import com.bocloud.cop.entity.patch.PatchDeployHistory;
import com.bocloud.cop.entity.patch.PatchDeployHistoryDetail;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.Catalog;
import com.bocloud.cop.enums.ExecuteMode;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.patch.PatchDeployHistoryService;
import com.bocloud.cop.intf.patch.PatchDeployService;
import com.bocloud.cop.model.FileFront;
import com.bocloud.cop.model.NodeConfig;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.ScriptFront;
import com.bocloud.cop.model.ScriptInput;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.TaskEdge;
import com.bocloud.cop.model.TaskGraph;
import com.bocloud.cop.model.UploadInput;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.repository.TaskHostRepository;
import com.bocloud.cop.repository.patch.PatchDeployHistoryDetailRepository;
import com.bocloud.cop.repository.patch.PatchDeployHistoryRepository;
import com.bocloud.cop.repository.patch.PatchDeployRepository;
import com.bocloud.cop.repository.patch.PatchRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("patchDeployService")
public class PatchDeployServiceImpl implements PatchDeployService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RabbitAdmin rabbitAdmin;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private PatchDeployRepository patchDeployRepository;
    @Autowired
    private PatchDeployHistoryDetailRepository patchDeployHistoryDetailRepository;
    @Autowired
    private PatchDeployHistoryRepository patchDeployHistoryRepository;
    @Autowired
    private TaskHostRepository taskHostRepository;
    @Autowired
    private PatchRepository patchRepository;
    @Autowired
    private PatchDeployHistoryService patchDeployHistoryService;
    @Autowired
    private TaskInstanceService taskInstanceService;

    @Override
    public GeneralResult<GridBean<PatchDeploy>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int count = patchDeployRepository.count(pager.getParams());
            List<PatchDeploy> list = patchDeployRepository.list(pager);
            for (PatchDeploy patch : list) {
                patch.setCreator(redisTemplate.opsForValue().get("User_Name_" + patch.getCreatorId()));
                // 已安装主机数量
                patch.setInstallCount(
                    patchDeployHistoryService.queryByDeployIdAndStatus(patch.getId(), "success").size());
            }
            GridBean<PatchDeploy> bean = new GridBean<>(pager.getPage(), pager.getRows(), count, list);
            return new GeneralResult<>(true, bean, "获取数据成功！");
        } catch (Exception e) {
            log.error("List deployPatch fail", e);
            return new GeneralResult<>(false, "获取数据失败！");
        }
    }

    @Override
    public GeneralResult<PatchDeploy> create(PatchDeploy patchDeploy, boolean isRun, Long userId) {
        try {
            if (StringUtils.isEmpty(patchDeploy.getName())) {
                return new GeneralResult<>(false, "任务名称不能为空");
            }
            buildPatchDeploy(patchDeploy, userId);
            boolean exist = patchDeployRepository.exist(patchDeploy.getName());
            if (exist) {
                return new GeneralResult<>(false, "名称已经存在！");
            }
            if (null != patchDeploy.getPatchTarget()) {
                List<PatchDeploy.Patch> patchs =
                    JSON.parseArray(patchDeploy.getPatchTarget().toString(), PatchDeploy.Patch.class);
                if (patchs.size() > 0) {
                    patchDeploy.setPatchId(patchs.get(0).getId());
                }
            }
            patchDeployRepository.save(patchDeploy);

            if (isRun) {
                patchDeployRepository.updateStatus(patchDeploy.getId(), ApplyStatus.APPROVED.name(), userId);
            }
            return new GeneralResult<>(true, patchDeploy, "创建成功！");
        } catch (Exception e) {
            log.error("补丁部署创建失败:", e);
            return new GeneralResult<>(false, "创建失败！");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = PatchDeploy.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PatchDeploy patchDeploy = patchDeployRepository.get(PatchDeploy.class, id);
            if (ApplyStatus.APPROVED.name().equals(patchDeploy.getStatus())) {
                return new GeneralResult<>(false, "选择的补丁部署已发布，请下架后再操作！");
            }
            boolean result = patchDeployRepository.remove(id, userId);
            return result ? new GeneralResult<>(true, "删除任务成功") : new GeneralResult<>(false, "删除任务失败");
        } catch (Exception e) {
            log.error("remove PatchDeploy error:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<PatchDeploy> modify(PatchDeploy patchDeploy, Long userId) {
        String path = PatchDeploy.class.getSimpleName() + "_" + patchDeploy.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PatchDeploy bean = patchDeployRepository.get(PatchDeploy.class, patchDeploy.getId());
            if (ApplyStatus.APPROVED.name().equals(bean.getStatus())) {
                return new GeneralResult<>(false, "选择的补丁部署已发布，请下架后再操作！");
            }
            if (!bean.getName().equals(patchDeploy.getName())) {
                boolean exist = patchDeployRepository.exist(patchDeploy.getName());
                if (exist) {
                    return new GeneralResult<>(false, "名称已存在，请重新输入");
                }
            }
            patchDeploy.setGmtModify(new Date());
            patchDeploy.setMenderId(userId);
            patchDeploy.setStatus(ApplyStatus.DEVELOPING.name());
            patchDeploy.setParam(JSONArray.toJSONString(patchDeploy.getParam()));
            patchDeploy.setTarget(JSONArray.toJSONString(patchDeploy.getTarget()));
            patchDeploy.setChoosePatchPath(JSONArray.toJSONString(patchDeploy.getChoosePatchPath()));
            patchDeploy.setPatchTarget(JSONArray.toJSONString(patchDeploy.getPatchTarget()));
            boolean update = patchDeployRepository.update(patchDeploy);
            if (update) {
                return new GeneralResult<>(true, patchDeploy, "修改成功");
            }
            return new GeneralResult<>(false, "修改失败");
        } catch (Exception e) {
            log.error("modify PatchDeploy error:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<PatchDeploy> detail(Long id) {
        try {
            PatchDeploy patchDeploy = patchDeployRepository.detail(id);
            if (patchDeploy == null) {
                return new GeneralResult<>(false, "未查询到相应补丁部署详情");
            }
            return new GeneralResult<>(true, patchDeploy, "获取详情成功");
        } catch (Exception e) {
            log.error("Query patchDeploy detail error:", e);
            return new GeneralResult<>(false, "获取详情失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public GeneralResult<TaskInstance> run(Long id, RequestContext context) {
        String path = PatchDeploy.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PatchDeploy deploy = patchDeployRepository.selectDeployById(id);
            if (!ApplyStatus.APPROVED.name().equals(deploy.getStatus())) {
                return new GeneralResult<>(false, "选择的补丁部署未发布，请先发布后再执行！");
            }
            // target
            Target target = JSON.parseObject(deploy.getTarget().toString(), Target.class);
            // 构建任务拓扑
            TaskGraph graph = buildTaskGraph(deploy);
            // 处理节点参数
            preNode(graph, deploy, target, context);
            // 构建保存任务实例
            TaskInstance instance = buildTaskInstance(graph, deploy, context);
            if (null == instance) {
                return new GeneralResult<>(false, "操作失败");
            }
            // 更新补丁部署绑定实例 ,如果可以多次执行实际意义不大
            patchDeployRepository.updateInstanceId(instance.getId(), deploy.getId());
            // 构建保存补丁部署历史记录
            buildPatchDeployHistory(deploy, target.getData(), context, instance);
            return new GeneralResult<>(true, instance, "操作成功");
        } catch (Exception e) {
            log.error("run PatchDeploy error:", e);
            return new GeneralResult<>(false, "操作失败");
        }
    }

    @Override
    public GeneralResult<Void> timing(JSONObject obj, Long userId, Long id) {
        String time = obj.getString("timing");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date;
        try {
            date = simpleDateFormat.parse(time);
        } catch (Exception e) {
            log.error("参数timing格式错误：{}", e.getMessage(), e);
            return new GeneralResult<>(false, "参数timing格式错误");
        }
        if (date.getTime() < System.currentTimeMillis()) {
            return new GeneralResult<>(false, "定时时间不能小于当前时间");
        }

        String path = PatchDeploy.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            final String redisKey = "task.job.patch.delay." + id;
            final String queueName = "task.job.patch.delay." + IDFactory.instance().uuid();
            final String exchangeName = "bocloud.direct.exchange";
            long timeout = date.getTime() - System.currentTimeMillis();
            Map<String, Object> args = new HashMap<>(4);
            args.put("x-dead-letter-exchange", exchangeName);
            args.put("x-dead-letter-routing-key", "task.job.patch.delay");
            args.put("x-message-ttl", timeout);
            Queue queue = QueueBuilder.durable(queueName).withArguments(args).build();
            rabbitAdmin.declareQueue(queue);
            Binding binding =
                new Binding(queueName, Binding.DestinationType.QUEUE, exchangeName, queueName, Collections.emptyMap());
            rabbitAdmin.declareBinding(binding);

            Map<String, Object> message = new HashMap<>();
            message.put("id", id);
            message.put("queue", queueName);
            amqpTemplate.convertAndSend(exchangeName, queueName, message);
            redisTemplate.boundValueOps(redisKey).set(queueName, timeout, TimeUnit.MILLISECONDS);
            patchDeployRepository.timing(date, userId, id);
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("timing patchDeploy error:", e);
            return new GeneralResult<>(false, "操作失败");
        }
    }

    @Override
    public GeneralResult<Void> cancel(Long id, Long userId) {
        String path = PatchDeploy.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            final String redisKey = "task.job.patch.delay." + id;
            String queueName = redisTemplate.boundValueOps(redisKey).get();
            if (StringUtils.isNotEmpty(queueName)) {
                rabbitAdmin.purgeQueue(queueName);
                rabbitAdmin.deleteQueue(queueName);
            }
            patchDeployRepository.cancel(id, userId);
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("cancel PatchDeploy error:", e);
            return new GeneralResult<>(false, "操作失败");
        }
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = PatchDeploy.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            patchDeployRepository.updateStatus(id, status, userId);
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("updateStatus deployPatch fail:", e);
            return new GeneralResult<>(false, "操作失败");
        }
    }

    @Override
    public GeneralResult<GridBean<TaskHost>> listInstallHost(Pager pager, Long patchId, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<PatchDeployHistoryDetail> details =
                patchDeployHistoryService.queryByPatchIdAndStatus(patchId, "success");
            List<Long> hostIds = details.stream().map(t -> t.getHostId()).collect(Collectors.toList());
            GridBean<TaskHost> bean = null;
            if (!ListTool.isEmpty(hostIds)) {
                int count = taskHostRepository.countFormView(pager.getParams(), hostIds);
                List<TaskHost> hostList = taskHostRepository.listFormView(pager, hostIds);
                bean = new GridBean<>(pager.getPage(), pager.getRows(), count, hostList);
            }
            return new GeneralResult<>(true, bean, "获取数据成功！");
        } catch (Exception e) {
            log.error("listInstallHost fail", e);
            return new GeneralResult<>(false, "获取数据失败！");
        }
    }

    private TaskGraph buildTaskGraph(PatchDeploy patchDeploy) {
        TaskGraph graph = new TaskGraph();
        List<TaskNode> nodes = new ArrayList<>();

        List<PatchDeploy.Patch> patchList =
            JSON.parseArray(patchDeploy.getPatchTarget().toString(), PatchDeploy.Patch.class);
        PatchDeploy.Patch patch = patchList.get(0);
        Patch patchBean = patchRepository.get(Patch.class, patch.getId());
        // 获取文件路径
        List<Patch.File> fileList = JSONObject.parseArray(patchBean.getFiles().toString(), Patch.File.class);
        String path = fileList.stream().map(t -> t.getPath()).collect(Collectors.joining(","));
        patchBean.setPath("null".equalsIgnoreCase(path) ? null : path);

        boolean hasFileNode = StringUtils.isBlank(patchBean.getPath()) ? false : true;
        // 脚本执行节点
        TaskNode scriptNode = buildScriptNode(patchBean);
        nodes.add(scriptNode);
        // 文件分发节点
        if (hasFileNode) {
            TaskNode fileNode = buildFileNode(patchBean, patchDeploy);
            nodes.add(fileNode);
            // 连接线
            TaskEdge edge = buildEdge(fileNode, scriptNode);
            graph.setEdges(Arrays.asList(edge));
        }
        graph.setNodes(nodes);
        return graph;
    }

    private TaskNode buildScriptNode(Patch patch) {
        Integer zIndex = 2;
        if (null == patch.getPath()) {
            zIndex = 1;
        }
        TaskNode node = new TaskNode();
        node.setModuleType("task.maintain.script");
        node.setCode("task.maintain.script");
        node.setName("单脚本执行");
        // front
        ScriptFront front = new ScriptFront(Arrays.asList(patch.getScriptCategoryId()), patch.getScriptId(), null,
            patch.getScriptContent(), patch.getScriptType());
        Map<String, Object> input = new HashMap<>();
        input.put("fronts", front);
        node.setInput(JSONObject.toJSONString(input));
        // control
        NodeControl control = new NodeControl(false, false, false, 3, 10, 3000);
        node.setControl(JSONObject.toJSONString(control));
        // config
        NodeConfig config =
            new NodeConfig(UUID.randomUUID().toString(), zIndex, 550, 140, "script.png", "maintain-node");
        config.setComponent("flow-rect-component");
        node.setConfig(JSONObject.toJSONString(config));
        return node;
    }

    private TaskNode buildFileNode(Patch patch, PatchDeploy deploy) {
        TaskNode node = new TaskNode();
        node.setName("文件分发");
        node.setCode("task.maintain.upload");
        node.setModuleType("task.maintain.upload");
        // front
        FileFront front = new FileFront(patch.getPath(), deploy.getDeployPath());
        Map<String, Object> input = new HashMap<>();
        input.put("fronts", front);
        node.setInput(JSONObject.toJSONString(input));
        // control
        NodeControl control = new NodeControl(false, false, false, 3, 10, 3000);
        node.setControl(JSONObject.toJSONString(control));
        // config
        NodeConfig config = new NodeConfig(UUID.randomUUID().toString(), 1, 250, 140, "file.png", "maintain-node");
        config.setComponent("flow-rect-component");
        node.setConfig(JSONObject.toJSONString(config));
        return node;
    }

    private TaskEdge buildEdge(TaskNode source, TaskNode target) {
        NodeConfig sconf = JSON.parseObject(source.getConfig(), NodeConfig.class);
        NodeConfig tconf = JSON.parseObject(target.getConfig(), NodeConfig.class);
        TaskEdge edge = new TaskEdge(UUID.randomUUID().toString(), 1, "flow-edge", sconf.getId(), "right",
            tconf.getId(), "left", false, false);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("name", "manhattan");
        edge.setRouter(jsonObject);
        edge.setConnector("normal");
        return edge;
    }

    private void preNode(TaskGraph graph, PatchDeploy deploy, Target target, RequestContext context) {
        List<PatchDeploy.Patch> patchs = JSON.parseArray(deploy.getPatchTarget().toString(), PatchDeploy.Patch.class);
        patchs.sort(Comparator.comparing(PatchDeploy.Patch::getOrder));
        for (TaskNode node : graph.getNodes()) {
            if ("task.maintain.script".equals(node.getModuleType())) {
                ScriptInput input = JSON.parseObject(node.getInput(), ScriptInput.class);
                input.setExecuteType(target.getExecuteType());
                input.setTarget(target);
                // 将准备好的input放回去
                node.setInput(JSON.toJSONString(input));
            } else if ("task.maintain.upload".equals(node.getModuleType())) {
                UploadInput input = JSON.parseObject(node.getInput(), UploadInput.class);
                input.setExecuteType(target.getExecuteType());
                input.setTarget(target);
                // 设置文件上传的目标路径
                input.getFronts().setTarget(deploy.getDeployPath());
                // 将要上传的文件源路径拼接起来
                PatchDeploy.Patch patch = patchs.get(0);
                Patch patchBean = patchRepository.get(Patch.class, patch.getId());
                // 获取文件路径
                List<Patch.File> files = JSONObject.parseArray(patchBean.getFiles().toString(), Patch.File.class);
                String sourceFiles = files.stream().map(t -> t.getPath()).collect(Collectors.joining(","));
                input.getFronts().setSource(sourceFiles);
                // 将准备好的input放回去
                node.setInput(JSON.toJSONString(input));
            } else {
                throw new IllegalArgumentException("所选择的执行模板存在除脚本执行和文件分发之外的作业节点类型");
            }
        }
    }

    private TaskInstance buildTaskInstance(TaskGraph graph, PatchDeploy patchDeploy, RequestContext context) {
        Task task = new Task();
        task.setName(patchDeploy.getName());
        task.setCode(Catalog.Task.STANDARD.getValue());
        task.setGraph(JSONObject.toJSONString(graph));
        task.setType("patch");
        return taskInstanceService.initInstance(task, ExecuteMode.WEB.name(), context);
    }

    private void buildPatchDeployHistory(PatchDeploy deploy, List<WorkerHost> hosts, RequestContext context,
        TaskInstance instance) {
        List<PatchDeploy.Patch> patches = JSON.parseArray(deploy.getPatchTarget().toString(), PatchDeploy.Patch.class);
        patches.sort(Comparator.comparing(PatchDeploy.Patch::getOrder));

        PatchDeployHistory history = new PatchDeployHistory();
        history.setName(deploy.getName());
        history.setStatus("RUNNING");
        history.setPatchId(deploy.getPatchId());
        history.setType(deploy.getType());
        history.setPatchNum(patches.size());
        history.setServeNum(hosts.size());
        history.setFailSuccessTotal(String.format("0 / 0 / %d", hosts.size()));
        history.setCreatorId(context.getTarget());
        history.setMenderId(context.getTarget());
        history.setOwnerId(context.getTarget());
        history.setDeployId(deploy.getId());
        history.setTaskInstanceId(instance.getId());
        patchDeployHistoryRepository.save(history);

        for (WorkerHost host : hosts) {
            PatchDeployHistoryDetail detail = new PatchDeployHistoryDetail();
            detail.setHistoryId(history.getId());
            detail.setHostIp(host.getHost());
            detail.setHostId(host.getId());
            detail.setType(deploy.getType());
            detail.setCreatorId(context.getTarget());
            detail.setMenderId(context.getTarget());
            detail.setOwnerId(context.getTarget());
            for (PatchDeploy.Patch patch : patches) {
                Patch patchBean = patchRepository.get(Patch.class, patch.getId());
                detail.setPatchName(patchBean.getName());
                detail.setName(host.getRawName());
                detail.setStatus("RUNNING");
                detail.setId(null);
                detail.setPatchId(patchBean.getId());
                detail.setDeployId(deploy.getId());
                patchDeployHistoryDetailRepository.save(detail);
            }
        }
    }

    private void buildPatchDeploy(PatchDeploy deploy, Long userId) {
        deploy.setIsTiming(false);
        deploy.setGmtModify(new Date());
        deploy.setGmtCreate(new Date());
        deploy.setCreatorId(userId);
        deploy.setMenderId(userId);
        deploy.setDeleted(false);
        deploy.setStatus(ApplyStatus.DEVELOPING.name());
        deploy.setParam(JSONArray.toJSONString(deploy.getParam()));
        deploy.setTarget(JSONArray.toJSONString(deploy.getTarget()));
        deploy.setChoosePatchPath(JSONArray.toJSONString(deploy.getChoosePatchPath()));
        deploy.setPatchTarget(JSONArray.toJSONString(deploy.getPatchTarget()));
    }

}
