package com.bocloud.cop.service.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;

/**
 * 参数帮助类
 * 
 * @author dmw
 */
public class ParamHelper {

    public static List<Param> getParams(Map<String, Object> map, Sign sign) {
        List<Param> list = new ArrayList<>();
        Param param = new Param(map, sign);
        list.add(param);
        return list;
    }

    /**
     * 验证字符串参数不为空，如果存在为空的参数值则抛出异常
     * 
     * @author sxp
     * @since Dec 28, 2017
     * @param paramNames
     *            参数名称的数组
     * @param paramValues
     *            字符串参数值的数组
     */
    public static void validateNotEmpty(String[] paramNames, String[] paramValues) {
        if (paramNames == null || paramNames.length == 0) {
            return;
        }
        if (paramValues == null || paramValues.length == 0) {
            return;
        }
        if (paramNames.length != paramValues.length) {
            throw new RuntimeException("参数名称、参数值的数组长度不一致，请检查。");
        }

        StringBuilder sb = new StringBuilder();
        String name;
        String value;

        int size = paramNames.length;

        for (int i = 0; i < size; i++) {
            name = paramNames[i];
            value = paramValues[i];
            if (StringUtils.isEmpty(name)) {
                continue;
            }
            if (StringUtils.isEmpty(value)) {
                sb.append("参数[").append(name).append("]的值不能为空");
                if (i == size - 1) {
                    sb.append("。");
                } else {
                    sb.append("，");
                }
            }
        }

        if (sb.length() > 0) {
            throw new RuntimeException(sb.toString());
        }
    }

    private ParamHelper() {
        throw new IllegalStateException("Utility class");
    }

}
