package com.bocloud.cop.service.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.enums.TaskHostType;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class TaskHostDataUtil {

    /**
     * 已知状态列表
     */
    private static List<String> knowStatus = new ArrayList<>();

    private final static Pattern patten =
        Pattern.compile("\\s([1-9]\\d*\\.?\\d*)\\s|\\s(0\\.\\d*[1-9])\\s|\\s(\\d*\\/\\d*)\\s");

    /**
     * 根据平台类型与privateIps字符串获取所有的ip
     *
     * @param vendorType
     * @param privateIps
     * @return
     */
    public static String getIpsFromPrivateIps(String vendorType, String privateIps) {
        JSONArray ips = new JSONArray();
        if (StringUtils.isBlank(privateIps) || "[]".equals(privateIps.trim()) || StringUtils.isBlank(vendorType)) {
            return JSON.toJSONString(ips);
        }
        try {
            // ALIYUN,TENCENT不需要修改
            if ("ALIYUN".equalsIgnoreCase(vendorType) || "TENCENT".equalsIgnoreCase(vendorType)
                    || "AWS".equalsIgnoreCase(vendorType) || "AZURE".equalsIgnoreCase(vendorType)
                    || "TIANYI".equalsIgnoreCase(vendorType) || "FUSIONSPHERE".equalsIgnoreCase(vendorType)
                    || "KVM".equalsIgnoreCase(vendorType) || "CTSTACK".equalsIgnoreCase(vendorType)) {
                return privateIps;
            } else if ("OPENSTACK".equalsIgnoreCase(vendorType) || "FUSIONCLOUD".equalsIgnoreCase(vendorType)
                    || "EASYSTACK".equalsIgnoreCase(vendorType)) {// OPENSTACK
                List<JSONObject> privateIpList = JSONArray.parseArray(privateIps, JSONObject.class);
                for (JSONObject privateIpJson : privateIpList) {
                    if (null == privateIpJson) {
                        continue;
                    }
                    String addressStr = privateIpJson.getString("addresses");
                    if (StringUtils.isBlank(addressStr)) {
                        continue;
                    }
                    List<JSONObject> addressList = JSONArray.parseArray(addressStr, JSONObject.class);
                    for (JSONObject addressJson : addressList) {
                        if (null == addressJson) {
                            continue;
                        }
                        String ip = addressJson.getString("address");
                        if (StringUtils.isBlank(ip)) {
                            continue;
                        }
                        ips.add(ip);
                    }
                }
                return JSON.toJSONString(ips);
            } else if ("QCLOUD".equalsIgnoreCase(vendorType)) {
                List<JSONObject> privateIpList = JSONObject.parseArray(privateIps, JSONObject.class);
                if (!ListTool.isEmpty(privateIpList)) {
                    JSONObject jsonObject = privateIpList.get(0);
                    if (null != jsonObject) {
                        String ip = jsonObject.getString("address");
                        ips.add(ip);
                    }
                }
                return JSON.toJSONString(ips);
            }
            //
            String key = "";
            if ("VMWARE".equalsIgnoreCase(vendorType) || "HCSO".equalsIgnoreCase(vendorType)
                    || "HUAWEI".equalsIgnoreCase(vendorType) || "MANAGEONE".equalsIgnoreCase(vendorType)) {
                key = "address";
            } else if ("VOLCENGINE".equalsIgnoreCase(vendorType)) {
                key = "primaryIpAddress";
            } else if ("SMARTX".equalsIgnoreCase(vendorType)) {
                key = "ip";
            } else {
                return JSON.toJSONString(ips);
            }

            List<JSONObject> privateIpList = JSONArray.parseArray(privateIps, JSONObject.class);
            for (JSONObject ipJson : privateIpList) {
                if (null == ipJson) {
                    continue;
                }
                String ip = ipJson.getString(key);
                if (StringUtils.isEmpty(ip)) {
                    continue;
                }
                ips.add(ip);
            }

        } catch (Exception e) {
            log.error("ip转换失败:", e);
        }
        return JSON.toJSONString(ips);
    }

    /**
     * 根据manageIp字符串获取所有的ip
     *
     * @param manageIp
     * @return
     */
    public static String getIpsFromManageIp(String manageIp) {
        if (StringUtils.isNotBlank(manageIp)) {
            return "'[\"" + manageIp + "\"]'";
        } else {
            return "'[]'";
        }
    }

    public static String getIpsFromManageIp2(String manageIp) {
        if (StringUtils.isNotBlank(manageIp)) {
            return "[\"" + manageIp + "\"]";
        } else {
            return "[]";
        }
    }

    /**
     * 将宿主机接口返回的数据转换为TaskHost
     *
     * @param data
     * @return
     */
    public static List<TaskHost> convertPmToTaskHost(JSONObject data, RequestContext context) {
        List<TaskHost> list = new ArrayList<>();
        JSONArray rows = data.getJSONArray("rows");
        Iterator<Object> iterator = rows.iterator();
        while (iterator.hasNext()) {
            Object obj = iterator.next();
            JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(obj));
            String instanceId = jsonObject.getString("uuid");
            if (StringUtils.isBlank(instanceId)) {
                instanceId = jsonObject.getString("guid");
            }
            if (StringUtils.isBlank(instanceId)) {
                continue;
            }
            String status = jsonObject.getString("status");
            if ("BUILDING".equalsIgnoreCase(status)) {
                continue;
            } else if ("DANGER".equalsIgnoreCase(status) || "WARN".equalsIgnoreCase(status) || "ONLINE".equalsIgnoreCase(status)) {// 告警状态改为运行中
                status = "RUNNING";
            }
            String manageIp = jsonObject.getString("manageIp");
            TaskHost taskHost = new TaskHost();
            taskHost.setSourceType(TaskHostType.SourceType.IMS.name());
            taskHost.setInstanceId(instanceId);
            taskHost.setVendorId(jsonObject.getLong("vendorId"));
            taskHost.setCategory(TaskHostType.Category.PHYSICAL.name());
            taskHost.setCatalog("Computer");
            taskHost.setKnown(null);
            String osCategory = jsonObject.getString("osCategory");
            taskHost.setOsCategory(null != osCategory ? osCategory : null);
            taskHost.setDeleted(false);
            taskHost.setVendorType(jsonObject.getString("vendorType"));
            taskHost.setName(jsonObject.getString("name"));
            taskHost.setStatus(status);
            taskHost.setVendorName(jsonObject.getString("vendorName"));
            taskHost.setUpdateMode(TaskHostType.UpdateModel.AUTO.name());
            if(StringUtils.isNotBlank(taskHost.getName()) && taskHost.getName().contains("'")) {
                taskHost.setIps(getIpsFromManageIp2(manageIp));
            }else {
                taskHost.setIps(getIpsFromManageIp(manageIp));
            }
            taskHost.setCreatorId(jsonObject.getLong("creatorId"));
            taskHost.setMenderId(jsonObject.getLong("menderId"));
            Long tenantId = jsonObject.getLong("tenantId");
            taskHost.setTenantId(null == tenantId ? 0 : tenantId);
            taskHost.setOwnerId(jsonObject.getLong("ownerId"));
            taskHost.setUserCatalog("Manager");
            // 当前创建人为空时，赋值1
            if (null == taskHost.getCreatorId()) {
                taskHost.setCreatorId(1L);
            }
            list.add(taskHost);
        }
        return list;
    }

    /**
     * 将云主机接口返回的数据转换成TaskHost
     *
     * @param data
     * @return
     */
    public static List<TaskHost> convertVmToTaskHost(JSONObject data, RequestContext context,
        Map<String, String> osNameMap) {
        List<TaskHost> list = new ArrayList<>();
        JSONArray rows = data.getJSONArray("rows");
        Iterator<Object> iterator = rows.iterator();
        while (iterator.hasNext()) {
            Object obj = iterator.next();
            JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(obj));
            Boolean isTemplate = jsonObject.getBoolean("isTemplate");
            String instanceId = jsonObject.getString("instanceId");
            if (null == isTemplate || isTemplate || StringUtils.isBlank(instanceId)) {
                continue;
            }
            String status = jsonObject.getString("status");
            if ("BUILDING".equalsIgnoreCase(status)) {
                continue;
            }
            String vendorType = jsonObject.getString("vendorType");
            TaskHost taskHost = new TaskHost();
            taskHost.setSourceType(TaskHostType.SourceType.IMS.name());
            taskHost.setInstanceId(instanceId);
            taskHost.setVendorId(jsonObject.getLong("vendorId"));
            taskHost.setCategory(TaskHostType.Category.LOGICAL.name());
            taskHost.setCatalog("Computer");
            taskHost.setKnown(null);
            String osCategory = jsonObject.getString("osCategory");
            taskHost.setOsCategory(null != osCategory ? osCategory : null);
            taskHost.setOsName(jsonObject.getString("osName"));
            handleOsName(osNameMap, taskHost);
            if (StringUtils.isBlank(taskHost.getOsVersion())) {
                taskHost.setOsVersion(jsonObject.getString("osVersion"));
            }
            taskHost.setDeleted(false);
            taskHost.setVendorType(vendorType);
            taskHost.setName(jsonObject.getString("name"));
            String privateIpsStr = jsonObject.getString("privateIps");
            taskHost.setVendorName(jsonObject.getString("vendorName"));
            taskHost.setUpdateMode(TaskHostType.UpdateModel.AUTO.name());
            taskHost.setStatus(status);
            taskHost.setCreatorId(jsonObject.getLong("creatorId"));
            taskHost.setMenderId(jsonObject.getLong("menderId"));
            Long tenantId = jsonObject.getLong("tenantId");
            taskHost.setTenantId(null == tenantId ? 0 : tenantId);
            taskHost.setOwnerId(jsonObject.getLong("ownerId"));
            String catalog = jsonObject.getString("catalog");
            taskHost.setUserCatalog(StringUtils.isNotBlank(catalog) ? catalog : "Manager");
            // 租户端创建的如果没有连接协议，默认为remote
            if (null != taskHost.getTenantId() && !taskHost.getTenantId().equals(0L)) {
                List<String> connProtocols = new ArrayList<>();
                connProtocols.add("Remote");
                taskHost.setConnProtocol(JSONArray.toJSONString(connProtocols).replaceAll("\\\\",""));
            }
            // 根据平台类型获取云主机的全量ip
            if(StringUtils.isNotBlank(taskHost.getName()) && taskHost.getName().contains("'")) {
                taskHost.setIps(getIpsFromPrivateIps(vendorType, privateIpsStr));
            }else {
                taskHost.setIps(handleIpsFromPrivateIps(vendorType, privateIpsStr));
            }
            // 当前创建人为空时，赋值1
            if (null == taskHost.getCreatorId()) {
                taskHost.setCreatorId(1L);
            }
            list.add(taskHost);
        }
        return list;
    }

    /**
     * 对jsonArry数组进行处理
     * 
     * @param vendorType
     *            平台
     * @param privateIps
     *            privateIps
     * @return 处理后的ips
     */
    private static String handleIpsFromPrivateIps(String vendorType, String privateIps) {
        String ips = getIpsFromPrivateIps(vendorType, privateIps);
        return handleArrStr(ips);
    }

    /**
     * 对jsonArry数组进行处理
     * 
     * @param str
     *            字符串数组
     * @return 处理后的ips
     */
    public static String handleArrStr(String str) {
        if (StringUtils.isEmpty(str)) {
            return str;
        }
        return "'" + str + "'";
    }

    public static TaskHost updateTaskHost(TaskHost source, TaskHost target) {
        target.setSourceType(TaskHostType.SourceType.IMS.name());
        target.setInstanceId(source.getInstanceId());
        target.setVendorId(source.getVendorId());
        target.setCategory(source.getCategory());
        target.setCatalog("Computer");
        target.setOsCategory(source.getOsCategory());
        // 如果同步到操作系统版本就进行修改，没有就不修改
        if (StringUtils.isNotBlank(source.getOsName())) {
            target.setOsName(source.getOsName());
        }
        if (StringUtils.isNotBlank(source.getOsVersion())) {
            target.setOsVersion(source.getOsVersion());
        }
        target.setDeleted(source.getDeleted());
        target.setName(source.getName());
        target.setUpdateMode(source.getUpdateMode());
        // 根据平台类型获取云主机的全量ip
        target.setIps(source.getIps());
        target.setStatus(source.getStatus());
        target.setGmtModify(new Date());
        target.setIp(source.getIp());
        target.setSegmentId(source.getSegmentId());
        target.setTenantId(source.getTenantId());
        if (null != source.getCreatorId()) {
            target.setCreatorId(source.getCreatorId());
        }
        if (null != source.getMenderId()) {
            target.setMenderId(source.getMenderId());
        }
        if (null != source.getOwnerId()) {
            target.setOwnerId(source.getOwnerId());
        }
        // 如果vendorId没有变化，又没有获取到新的平台名称与平台类型，则平台名称与类型保持不变
        if (null != target.getVendorId() && compare(target.getVendorId(), source.getVendorId())) {
            if (StringUtils.isBlank(source.getVendorName())) {
                source.setVendorName(target.getVendorName());
            }
            if (StringUtils.isBlank(source.getVendorName())) {
                source.setVendorType(target.getVendorType());
            }
        }

        target.setVendorType(source.getVendorType());
        target.setVendorName(source.getVendorName());

        if (StringUtils.isNotBlank(source.getUsername())) {// 如果有值就进行修改
            target.setUsername(source.getUsername());
            target.setPassword(source.getPassword());
        }
        // 用户类型
        target.setUserCatalog(source.getUserCatalog());
        // 如果主机资源没有链接协议，而同步过来的有，则覆盖
        if (null == target.getConnProtocol() && null != source.getConnProtocol()) {
            target.setConnProtocol(source.getConnProtocol().replaceAll("\\\\",""));
        }else if(StringUtils.isNotBlank(target.getConnProtocol())){
            target.setConnProtocol(target.getConnProtocol().replaceAll("\\\\",""));
        }
        return target;
    }

    /**
     * 将云主机接口返回的数据转换成TaskHost
     *
     * @param data
     * @return
     */
    public static TaskHost convertMessageToTaskHost(JSONObject data, RequestContext context,
        Map<String, String> osNameMap) {

        TaskHost taskHost = JSONObject.parseObject(data.toJSONString(), TaskHost.class);
        if (TaskHostType.Category.PHYSICAL.name().equalsIgnoreCase(taskHost.getCategory())) {
            taskHost.setInstanceId(data.getString("uuid"));
            taskHost.setIps(data.getString("manageIp"));
            taskHost.setUserCatalog("Manager");
        } else if (TaskHostType.Category.LOGICAL.name().equalsIgnoreCase(taskHost.getCategory())) {
            taskHost.setIps(getIpsFromPrivateIps(taskHost.getVendorType(), data.getString("privateIps")));
            handleOsName(osNameMap, taskHost);
            taskHost.setUserCatalog(StringUtils.isNotBlank(taskHost.getCatalog()) ? taskHost.getCatalog() : "Manager");
            // 租户端的如果没有连接协议，默认为remote
            if (null != taskHost.getTenantId() && !taskHost.getTenantId().equals(0L)) {
                List<String> connProtocols = new ArrayList<>();
                connProtocols.add("Remote");
                taskHost.setConnProtocol(JSONArray.toJSONString(connProtocols));
            }
        }
        // 没有instacneId过滤掉
        if (StringUtils.isBlank(taskHost.getInstanceId())) {
            return null;
        }
        // 没有vendorId的也剔除
        if (null == taskHost.getVendorId()) {
            return null;
        }
        if (null != taskHost.getIsTemplate() && taskHost.getIsTemplate()) {// 模板默认是删除(可能是从云主机转换的，变更为删除)
            taskHost.setDeleted(true);
        }
        // id需要设置为null,因为接收的信息里有id
        taskHost.setId(null);
        // 变更时不变更密码
        taskHost.setUsername(null);
        taskHost.setPassword(null);
        taskHost.setCatalog("Computer");
        taskHost.setUpdateMode(TaskHostType.UpdateModel.AUTO.name());
        taskHost.setSourceType(TaskHostType.SourceType.IMS.name());
        if (null == taskHost.getTenantId()) {
            taskHost.setTenantId(0L);
        }
        // 当前创建人为空时，赋值1
        if (null == taskHost.getCreatorId()) {
            taskHost.setCreatorId(1L);
        }
        return taskHost;
    }

    /**
     * 获取列表中所有的ip
     *
     * @param taskHosts
     * @return
     */
    public static Set<String> getAllIp(List<TaskHost> taskHosts) {
        Set<String> ipSet = new HashSet<>();
        for (TaskHost taskHost : taskHosts) {
            String allIps = taskHost.getIps();
            if (null == allIps) {
                continue;
            }
            String value = allIps.replace("[", "").replace("]", "").replace("\"", "");
            if (StringUtils.isEmpty(value)) {
                continue;
            }
            String[] ips = value.split(",");
            if (null == ips || ips.length == 0) {
                continue;
            }
            for (String ip : ips) {
                ipSet.add(ip);
            }
        }
        return ipSet;
    }

    /**
     * 获取列表中所有的ip
     *
     * @param taskHosts
     * @param excludeId
     *            不能为null
     * @return
     */
    public static Set<String> getAllIp(List<TaskHost> taskHosts, Long excludeId) {
        Set<String> ipSet = new HashSet<>();
        for (TaskHost taskHost : taskHosts) {
            if (excludeId.equals(taskHost.getId())) {
                continue;
            }
            String allIps = taskHost.getIps();
            if (null == allIps) {
                continue;
            }
            String value = allIps.replace("[", "").replace("]", "").replace("\"", "");
            if (StringUtils.isEmpty(value)) {
                continue;
            }
            String[] ips = value.split(",");
            if (null == ips || ips.length == 0) {
                continue;
            }
            for (String ip : ips) {
                ipSet.add(ip);
            }
        }
        return ipSet;
    }

    /**
     * 检查ip格式是否正确
     *
     * @param allIps
     * @return
     */
    public static String checkIp(String allIps) {
        if (StringUtils.isBlank(allIps)) {
            return null;
        }
        String value = allIps.replace("[", "").replace("]", "").replace("\"", "");
        if (StringUtils.isBlank(value)) {
            return null;
        }
        String[] ips = value.split(",");
        if (null == ips || ips.length == 0) {
            return null;
        }
        // 判断ip是否存在
        for (String ip : ips) {
            if (!IpUtil.isIP4(ip)) {
                return ip;
            }
        }
        return null;
    }

    /**
     * 检查ip是否可用
     *
     * @param allIps
     * @param ipSet
     * @return
     */
    public static String checkIp(String allIps, Set<String> ipSet) {
        if (StringUtils.isBlank(allIps)) {
            return null;
        }
        String value = allIps.replace("[", "").replace("]", "").replace("\"", "");
        if (StringUtils.isBlank(value)) {
            return null;
        }
        String[] ips = value.split(",");
        if (null == ips || ips.length == 0) {
            return null;
        }
        // 判断ip是否存在
        for (String ip : ips) {
            if (ipSet.contains(ip)) {
                return ip;
            }
        }
        return null;
    }

    /**
     * 根据ips字符串获取ip
     * @param allIps ips字符串(["192.168.13.1","192.168.13.2"])
     * @return ip列表
     */
    public static List<String> getIps(String allIps) {
        if (StringUtils.isBlank(allIps)) {
            return new ArrayList<>();
        }
        String value = allIps.replace("[", "").replace("]", "").replace("\"", "");
        if (StringUtils.isBlank(value)) {
            return new ArrayList<>();
        }
        String[] ips = value.split(",");
        if (ips.length == 0) {
            return new ArrayList<>();
        }
        return Arrays.asList(ips);
    }

    public static void setIp(TaskHost host, List<NetworkSegment> segments) {
        host.setIp(null);
        host.setSegmentId(null);
        host.setUpdateMode(TaskHostType.UpdateModel.MANUAL.name());
        if (null == host.getIps()) {
            return;
        }
        String value = host.getIps().replace("[", "").replace("]", "").replace("\"", "").replace("\'", "");
        if (StringUtils.isEmpty(value)) {
            return;
        }
        String[] ips = value.split(",");
        if (null == ips || ips.length == 0) {
            return;
        }
        // 计算AllIp里面可以匹配的纳管网段
        for (NetworkSegment segment : segments) {
            boolean inSegment = false;
            for (String ip : ips) {
                if (IpUtil.isIP(ip) && IpUtil.isInRange(ip, segment.getName())) {
                    host.setSegmentId(segment.getId());
                    host.setIp(ip);
                    host.setUpdateMode(TaskHostType.UpdateModel.AUTO.name());
                    inSegment = true;
                    break;
                }
            }
            if (inSegment) {
                break;
            }
        }
    }

    /**
     * 将逗号分割的字符串转换为数组
     * 
     * @param str
     *            逗号分割字符串
     * @return 数组字符串
     */
    public static String handleArr(String str) {
        JSONArray arr = new JSONArray();
        if (StringUtils.isBlank(str)) {
            return arr.toJSONString();
        }
        String value = str.replace("[", "").replace("]", "").replace("\"", "");
        if (StringUtils.isBlank(value)) {
            return arr.toJSONString();
        }
        String[] ips = value.split(",");
        if (null == ips || ips.length == 0) {
            return arr.toJSONString();
        }
        // 计算AllIp里面可以匹配的纳管网段
        for (String ip : ips) {
            arr.add(ip);
        }
        return arr.toJSONString();
    }

    /**
     * 将逗号分割的字符串转换为数组
     *
     * @param str
     *            逗号分割字符串
     * @return 数组字符串
     */
    public static List<String> getConnProtocolList(String str) {
        List<String> list = new ArrayList<>();
        if (StringUtils.isBlank(str)) {
            return list;
        }
        String value = str.replace("[", "").replace("]", "").replace("\"", "");
        if (StringUtils.isBlank(value)) {
            return list;
        }
        String[] protocols = value.split(",");
        if (null == protocols || protocols.length == 0) {
            return list;
        }
        // 计算AllIp里面可以匹配的纳管网段
        for (String protocol : protocols) {
            if(protocol.equalsIgnoreCase("Remote")){
                protocol = "Remote";
            }else if(protocol.equalsIgnoreCase("SaltStack")){
                protocol = "SaltStack";
            }
            list.add(protocol);
        }
        return list;
    }

    /**
     * 更加taskHost获取平台id
     *
     * @param list
     * @return
     */
    public static List<Long> getVendorIds(List<TaskHost> list) {
        List<Long> vendorIds = new ArrayList<>();
        for (TaskHost taskHost : list) {
            if (null != taskHost.getVendorId()) {
                vendorIds.add(taskHost.getVendorId());
            }
        }
        return vendorIds;
    }

    /**
     * 比较字符串是否相等
     *
     * @param str1
     * @param str2
     * @return
     */
    public static boolean compare(String str1, String str2) {
        if (null == str1) {
            if (null == str2) {
                return true;
            } else {
                return false;
            }
        }
        if (str1.equals(str2)) {
            return true;
        }
        return false;
    }

    /**
     * 比较Long是否相等
     *
     * @param lo1
     * @param lo2
     * @return
     */
    public static boolean compare(Long lo1, Long lo2) {
        if (null == lo1) {
            if (null == lo2) {
                return true;
            } else {
                return false;
            }
        }
        if (lo1.equals(lo2)) {
            return true;
        }
        return false;
    }

    /**
     * aes解密
     *
     * @param pwd
     * @return
     */
    public static String aesDecryptPwd(String pwd) {
        if (StringUtils.isBlank(pwd)) {
            return pwd;
        }
        Encryptor encryptor = new AESEncryptor();
        String password = encryptor.decrypt(pwd.trim(), null);
        return password;
    }

    /**
     * des加密
     *
     * @param pwd
     * @return
     */
    public static String desEncryptPwd(String pwd) {
        if (StringUtils.isBlank(pwd)) {
            return pwd;
        }
        Encryptor encryptor = new DESEncryptor();
        String password = encryptor.encrypt(pwd, "BocloudCMP");
        return password;
    }

    /**
     * des解密
     *
     * @param pwd
     * @return
     */
    public static String desDecryptPwd(String pwd) {
        if (StringUtils.isBlank(pwd)) {
            return pwd;
        }
        Encryptor encryptor = new DESEncryptor();
        String password = encryptor.decrypt(pwd.trim(), "BocloudCMP");
        return password;
    }

    /**
     * aes加密
     *
     * @param pwd
     * @return
     */
    public static String aesEncryptPwd(String pwd) {
        if (StringUtils.isBlank(pwd)) {
            return pwd;
        }
        Encryptor encryptor = new AESEncryptor();
        String password = encryptor.encrypt(pwd, null);
        return password;
    }

    public static NetworkSegment getSegment(String host, List<NetworkSegment> segments) {
        if (null == segments || segments.isEmpty()) {
            return null;
        }
        NetworkSegment object = null;
        for (NetworkSegment segment : segments) {
            boolean inSegment = false;
            if (IpUtil.isInRange(host, segment.getName())) {
                object = segment;
                inSegment = true;
                break;
            }
            if (inSegment) {
                break;
            }
        }
        return object;
    }

    /**
     * 获取已知状态
     *
     * @return
     */
    public static List<String> getKnowStatus() {
        if (knowStatus.isEmpty()) {
            knowStatus.add("RUNNING");
            knowStatus.add("STOPPED");
            knowStatus.add("STOPPING");
            knowStatus.add("STARTING");
            knowStatus.add("SUSPENDED");
            knowStatus.add("SUSPENDING");
            knowStatus.add("ACTIVING");
            knowStatus.add("PAUSED");
            knowStatus.add("PAUSING");
            knowStatus.add("RECOVERING");
            knowStatus.add("RESTARTING");
            knowStatus.add("EXCEPTION");
            knowStatus.add("SYNSEXCEPTION");
            knowStatus.add("UNKNOWN");
            knowStatus.add("UNKNOWNON");
            knowStatus.add("UNKNOWNOFF");
            knowStatus.add("UNKNOWNSUSPEND");
        }
        return knowStatus;
    }

    /**
     * 获取与字典表对应的操作系统名
     * 
     * @param taskHost
     *            主机信息
     * @param osNameMap
     *            操作系统名称表
     * @return 操作系统名称
     */
    private static void handleOsName(Map<String, String> osNameMap, TaskHost taskHost) {
        String osName = taskHost.getOsName();
        if (StringUtils.isBlank(osName)) {
            return;
        }
        String osNameStr = osName.replaceAll("\\s+", "").toLowerCase();
        for (String key : osNameMap.keySet()) {
            if (osNameStr.contains(key)) {
                // Windows Server 需要特殊处理，直接匹配无法区分Windows与Windows Server
                if (osName.contains("server") && osName.contains("windows") && !key.contains("server")) {
                    continue;
                } else {
                    taskHost.setOsName(osNameMap.get(key));
                    taskHost.setOsVersion(getOsVersion(osName));
                    break;
                }
            }
        }

    }

    /**
     * 获取操作系统版本
     * 
     * @param osName
     *            操作系统名称
     * @return 操作系统版本
     */
    private static String getOsVersion(String osName) {
        // 通过正则匹配
        Matcher m = patten.matcher(osName);
        if (m.find()) {
            return m.group(0);
        }
        return null;
    }

    public static void setIp(TaskHost host, List<String> ips, List<NetworkSegment> segments) {
        host.setSegmentId(null);
        host.setIp(null);
        host.setUpdateMode(TaskHostType.UpdateModel.AUTO.name());
        if (CollectionUtils.isEmpty(ips) || CollectionUtils.isEmpty(segments)) {
            return;
        }
        // 计算AllIp里面可以匹配的纳管网段
        for (NetworkSegment segment : segments) {
            boolean inSegment = false;
            for (String ip : ips) {
                if (IpUtil.isIP(ip) && IpUtil.isInRange(ip, segment.getName())) {
                    host.setSegmentId(segment.getId());
                    host.setIp(ip);
                    host.setUpdateMode(TaskHostType.UpdateModel.AUTO.name());
                    inSegment = true;
                    break;
                }
            }
            if (inSegment) {
                break;
            }
        }
    }
}
