package com.bocloud.cop.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.enums.TaskHostType;
import com.bocloud.cop.model.NotifyMessage;
import com.bocloud.cop.repository.NetworkSegmentRepository;
import com.bocloud.cop.repository.TaskHostRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.springframework.amqp.core.AmqpTemplate;

import java.util.*;

@Slf4j
public class TaskHostExcelHelper {
    private TaskHostRepository taskHostRepository;
    private NetworkSegmentRepository networkSegmentRepository;
    private static final String subject = "主机资源";
    private AmqpTemplate amqpTemplate;
    private ResourceEventPublisher resourceEventPublisher;
    private static final String operate = "task.host.upload";
    private static final List<String> PROTOCOL_LIST = new ArrayList<String>(){{
        add("Remote");
        add("SaltStack");
    }};

    public TaskHostExcelHelper() {
        super();
    }

    public TaskHostExcelHelper(TaskHostRepository taskHostRepository, NetworkSegmentRepository networkSegmentRepository,
                               AmqpTemplate amqpTemplate, ResourceEventPublisher resourceEventPublisher) {
        super();
        this.taskHostRepository = taskHostRepository;
        this.networkSegmentRepository = networkSegmentRepository;
        this.amqpTemplate = amqpTemplate;
        this.resourceEventPublisher = resourceEventPublisher;
    }

    /**
     * 生成模板
     *
     * @param workbook
     * @throws Exception
     */
    public void generateSheet(Workbook workbook) throws Exception {
        Sheet sheet = workbook.createSheet("主机资源");
        Row row0 = sheet.createRow(0);
        row0.setHeight((short) 400);
        Row row1 = sheet.createRow(1);
        row1.setHeight((short) 400);
        // 定义颜色
        CellStyle cellStyle = workbook.createCellStyle();
        // cellStyle.setFillForegroundColor(HSSFColor.HSSFColorPredefined.ROYAL_BLUE.getIndex());
        // 填充模式
        // cellStyle.setFillPattern(FillPatternType.DIAMONDS);
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle.setBorderRight(BorderStyle.THIN);
        Font font = workbook.createFont();
        font.setFontName("仿宋_GB2312");
        font.setBold(true);// 粗体显示
        font.setFontHeightInPoints((short) 12);
        cellStyle.setFont(font);

        CellStyle bodyCellStyle = workbook.createCellStyle();
        // 居中
        bodyCellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        bodyCellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        bodyCellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        bodyCellStyle.setBorderRight(BorderStyle.THIN);
        Font bodyFont = workbook.createFont();
        bodyFont.setFontName("仿宋_GB2312");
        bodyFont.setFontHeightInPoints((short) 11);
        bodyCellStyle.setFont(bodyFont);

        // 生成表头
        generateHeader(sheet, cellStyle, row0);
        // 生成文本
        generateBody(sheet, bodyCellStyle, row1);
        // 设置文本格式以及调整列宽,使之能完整显示字段名称
        ExcelHelper.setTextformatAndColumnwidth(workbook);
    }

    /**
     * 生成表头
     *
     * @param sheet
     * @param cellStyle
     * @param row0
     * @throws Exception
     */
    private void generateHeader(Sheet sheet, CellStyle cellStyle, Row row0) throws Exception {
        Cell cell0 = null;
        // 设置主机excel表头属性信息
        List<String> properties = fileHeaderProperties();
        int index = 0;
        for (String property : properties) {
            cell0 = row0.createCell(index);
            cell0.setCellValue(new ExcelFactory(row0).newRichTextString(property));
            cell0.setCellStyle(cellStyle);
            if (index == 1 || index == 3 || index == 7) {
                sheet.setColumnWidth(index, 60 * 256);
            } else {
                sheet.setColumnWidth(index, 20 * 256);
            }
            index++;
        }

    }

    /**
     * 生成表内容
     *
     * @param sheet
     * @param cellStyle
     * @param row1
     * @throws Exception
     */
    private void generateBody(Sheet sheet, CellStyle cellStyle, Row row1) throws Exception {
        // 设置主机excel默认信息
        Cell cell0 = row1.createCell(0);
        cell0.setCellValue("名称");
        cell0.setCellStyle(cellStyle);
        sheet.setColumnWidth(0, 20 * 256);
        Cell cell1 = row1.createCell(1);
        cell1.setCellValue("100.100.100.100,101.101.101.101");
        cell1.setCellStyle(cellStyle);
        sheet.setColumnWidth(1, 60 * 256);
        Cell cell2 = row1.createCell(2);
        cell2.setCellValue("LINUX");
        cell2.setCellStyle(cellStyle);
        sheet.setColumnWidth(2, 20 * 256);
        Cell cell3 = row1.createCell(3);
        cell3.setCellValue("PHYSICAL/LOGICAL(LOGICAL云主机,PHYSICAL宿主机,括号中的内容为注释，导入时需要删除)");
        cell3.setCellStyle(cellStyle);
        sheet.setColumnWidth(3, 60 * 256);
        Cell cell4 = row1.createCell(4);
        cell4.setCellValue("平台类型");
        cell4.setCellStyle(cellStyle);
        sheet.setColumnWidth(4, 20 * 256);
        Cell cell5 = row1.createCell(5);
        cell5.setCellValue("");
        cell5.setCellStyle(cellStyle);
        sheet.setColumnWidth(5, 20 * 256);
        Cell cell6 = row1.createCell(6);
        cell6.setCellValue("");
        cell6.setCellStyle(cellStyle);
        cell6.setCellType(CellType.STRING);
        sheet.setColumnWidth(6, 20 * 256);
        Cell cell7 = row1.createCell(7);
        cell7.setCellValue("[Remote,SaltStack]");
        cell7.setCellStyle(cellStyle);
        sheet.setColumnWidth(7, 20 * 256);
        Cell cell8 = row1.createCell(8);
        cell8.setCellValue("RUNNING(运行中)/STOPPED(关机)/SUSPENDED(挂起)/PAUSED(停止)");
        cell8.setCellStyle(cellStyle);
        sheet.setColumnWidth(8, 60 * 256);
    }

    public static List<String> fileHeaderProperties() {
        List<String> properties = new ArrayList<>();
        properties.add("主机名称*");
        properties.add("所有ip*");
        properties.add("系统类别*");
        properties.add("主机类型*");
        properties.add("平台类型");
        properties.add("系统名称*");
        properties.add("系统版本*");
        properties.add("连接协议*");
        properties.add("状态*");
        return properties;
    }

    private void validateHeader(Map<String, Integer> header, List<String> properties) throws Exception {
        //判断标题字段的未知是否相同
        for (String name : header.keySet()) {
            if(properties.contains(name) && !header.get(name).equals(properties.indexOf(name))){
                throw new Exception("表头中字段位置不匹配:"+name);
            }
        }
        Set<String> propSetImport = new HashSet<>(header.keySet());
        Set<String> propSetNew = new HashSet<>(properties);
        Set<String> propSetNewTemp = new HashSet<>(propSetNew);
        propSetNewTemp.removeAll(propSetImport);
        if (!propSetNewTemp.isEmpty()) {
            throw new Exception("表头中缺少以下属性：" + propSetNewTemp.toString());
        }
    }

    private void validataWb(Workbook wb) throws Exception {
        List<String> properties = fileHeaderProperties();
        Sheet sheet = wb.getSheetAt(0);
        Map<String, Integer> header = readHeader4NameKey(sheet.getRow(0));
        validateHeader(header, properties);
    }

    public Map<String, Integer> readHeader4NameKey(Row row) {
        if (row == null) {
            throw new RuntimeException("1、请检查是否是最新版本;\n" + "2、不要更改模板格式;\n" + "3、请重新下载模板;");
        }
        Map<String, Integer> header = new HashMap<String, Integer>(30);
        try {
            for (int i = 0; i < getLastCellNum(row); i++) {
                Cell cell = row.getCell(i);
                // 有错误信息栏目
                if (cell != null && !"".equals(ExcelHelper.getCellValue(cell))) {
                    String value = ExcelHelper.getCellValue(cell);
                    header.put(value, new Integer(i));
                }
            }
        } catch (Exception e) {
            log.error("检查表头报错", e);
            throw new RuntimeException("1、请检查是否是最新版本;\n" + "2、不要更改模板格式;\n" + "3、请重新下载模板;\n");
        }

        return header;
    }

    private int getLastCellNum(Row row) {
        // 最后一个有效单元格的位置
        short lastNum = 0;
        if (row.getLastCellNum() < BoStringUtils.parseInt(ExcelHelper.EnumHelper.ExcelColumnMax.getValue())) {
            return row.getLastCellNum();
        }
        for (int i = 0; i < BoStringUtils.parseInt(ExcelHelper.EnumHelper.ExcelColumnMax.getValue()); i++) {
            // 如果当前单元格不是null 并且有值
            if (!"".equals(ExcelHelper.getCellValue(row.getCell(i)))) {
                lastNum = (short) i;
            }
        }
        return lastNum;
    }

    private GeneralResult checkData(Workbook wb, List<NetworkSegment> segments, Set<String> allIpSet) {
        log.info("开始主机资源Excel数据");
        try {
            boolean checkResult = true;
            String message = "";
            Sheet sheet = wb.getSheetAt(0);
            // 表格中所有ip
            Set<String> ips = new HashSet<>();
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                Row row = sheet.getRow(i);
                if (row == null) {
                    continue;
                }
                Cell nameCell = row.getCell(0);
                String name = null;
                if (null != nameCell) {
                    nameCell.setCellType(CellType.STRING);
                    name = String.valueOf(nameCell);
                }
                Cell allIpsCell = row.getCell(1);
                String allIps = null;
                if (null != allIpsCell) {
                    allIpsCell.setCellType(CellType.STRING);
                    allIps = String.valueOf(allIpsCell);
                }
                Cell osCategoryCell = row.getCell(2);
                String osCategory = null;
                if (null != osCategoryCell) {
                    osCategoryCell.setCellType(CellType.STRING);
                    osCategory = String.valueOf(osCategoryCell);
                }
                Cell categoryCell = row.getCell(3);
                String category = null;
                if (null != categoryCell) {
                    categoryCell.setCellType(CellType.STRING);
                    category = String.valueOf(categoryCell);
                }
                Cell osNameCell = row.getCell(5);
                String osName = null;
                if (null != osNameCell) {
                    osNameCell.setCellType(CellType.STRING);
                    osName = String.valueOf(osNameCell);
                }
                Cell osVersionCell = row.getCell(6);
                String osVersion = null;
                if (null != osVersionCell) {
                    osVersionCell.setCellType(CellType.STRING);
                    osVersion = String.valueOf(osVersionCell);
                }
                Cell connProtocolCell = row.getCell(7);
                String connProtocol = null;
                if (null != connProtocolCell) {
                    connProtocolCell.setCellType(CellType.STRING);
                    connProtocol = String.valueOf(connProtocolCell);
                }
                Cell statusCell = row.getCell(8);
                String status = null;
                if (null != statusCell) {
                    statusCell.setCellType(CellType.STRING);
                    status = String.valueOf(statusCell);
                }

                if (StringUtils.isEmpty(name) || StringUtils.isEmpty(category) || StringUtils.isEmpty(allIps)
                        || StringUtils.isEmpty(osCategory) || StringUtils.isEmpty(status) || StringUtils.isEmpty(osName)
                        || StringUtils.isEmpty(connProtocol) || StringUtils.isEmpty(osVersion)) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行数据不全";
                    log.warn(message);
                    break;
                }
                // 验证ip是否是ip
                String invalidIp = TaskHostDataUtil.checkIp(allIps);
                if (null != invalidIp) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行数据ip无效:" + invalidIp;
                    log.warn(message);
                    break;
                }
                // 验证ip是否在表格内重复
                String ip = TaskHostDataUtil.checkIp(allIps, ips);
                if (null != ip) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行数据ip表格内重复:" + ip;
                    log.warn(message);
                    break;
                } else {
                    ips.addAll(TaskHostDataUtil.getIps(allIps));
                }
                // 验证ip是否在数据库存在
                String existIp = TaskHostDataUtil.checkIp(allIps, allIpSet);
                if (null != existIp) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行数据ip已经存在:" + existIp;
                    log.warn(message);
                    break;
                }
                if (!"LOGICAL".equalsIgnoreCase(category) && !"PHYSICAL".equalsIgnoreCase(category)) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行类型不支持";
                    log.warn(message);
                    break;
                }
                //connProtocol 进行校验
                List<String> protocols = TaskHostDataUtil.getConnProtocolList(connProtocol);
                if (!PROTOCOL_LIST.containsAll(protocols)) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行连接协议类型不支持";
                    log.warn(message);
                    break;
                }
            }
            return new GeneralResult(checkResult, message);
        } catch (Exception e) {
            log.error("主机资源Excel数据检查失败:" + e);
            return new GeneralResult(false, "主机资源Excel数据检查失败");
        }

    }

    private void saveData(Workbook wb, RequestContext context, List<NetworkSegment> segments) {
        log.info("开始保存主机资源");
        NotifyMessage notifyMessage;
        String message = "";
        try {
            Sheet sheet = wb.getSheetAt(0);
            List<TaskHost> addList = new ArrayList<>();
            List<TaskHost> singleAddHosts = new ArrayList<>();
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                Row row = sheet.getRow(i);
                if(null == row){
                    continue;
                }
                Cell nameCell = row.getCell(0);
                nameCell.setCellType(CellType.STRING);
                String name = String.valueOf(nameCell);
                Cell allIpsCell = row.getCell(1);
                allIpsCell.setCellType(CellType.STRING);
                String allIps = String.valueOf(allIpsCell);
                Cell osCategoryCell = row.getCell(2);
                osCategoryCell.setCellType(CellType.STRING);
                String osCategory = String.valueOf(osCategoryCell);
                Cell categoryCell = row.getCell(3);
                categoryCell.setCellType(CellType.STRING);
                String category = String.valueOf(categoryCell);
                Cell vendorTypeCell = row.getCell(4);
                String vendorType = null;
                if (null != vendorTypeCell) {
                    vendorTypeCell.setCellType(CellType.STRING);
                    vendorType = String.valueOf(vendorTypeCell);
                }
                Cell osNameCell = row.getCell(5);
                osNameCell.setCellType(CellType.STRING);
                String osName = String.valueOf(osNameCell);
                Cell osVersionCell = row.getCell(6);
                osVersionCell.setCellType(CellType.STRING);
                String osVersion = String.valueOf(osVersionCell);
                Cell connProtocolCell = row.getCell(7);
                connProtocolCell.setCellType(CellType.STRING);
                String connProtocol = String.valueOf(connProtocolCell);
                Cell statusCell = row.getCell(8);
                statusCell.setCellType(CellType.STRING);
                String status = String.valueOf(statusCell);

                TaskHost taskHost = new TaskHost();
                taskHost.setSourceType(TaskHostType.SourceType.IMPORT.name());
                taskHost.setName(name);
                // object类型会直接拼接到sql，需要加单引号
                taskHost.setIps(TaskHostDataUtil.handleArrStr(TaskHostDataUtil.handleArr(allIps)));
                taskHost.setCategory(StringUtils.isNotBlank(category) ? category.toUpperCase() : null);
                taskHost.setOsCategory(StringUtils.isNotBlank(osCategory) ? osCategory.toUpperCase() : null);
                taskHost.setCatalog("Computer");
                taskHost.setVendorType(StringUtils.isNotBlank(vendorType) ? vendorType.toUpperCase() : null);
                taskHost.setStatus(StringUtils.isNoneBlank(status) ? status.toUpperCase() : null);
                taskHost.setOsName(osName);
                taskHost.setOsVersion(osVersion);
                taskHost.setConnProtocol(JSONObject.toJSONString(TaskHostDataUtil.getConnProtocolList(connProtocol)));
                taskHost.setCreatorId(null == context ? 1L : context.getTarget());
                taskHost.setOwnerId(null == context ? 1L : context.getTarget());
                taskHost.setTenantId(null == context ? 0L : context.getTenant());
                taskHost.setUserCatalog(null == context ? null : context.getCatalog().name());
                TaskHostDataUtil.setIp(taskHost, segments);
                taskHost.setDeleted(false);
                if (StringUtils.isNotBlank(taskHost.getName()) && taskHost.getName().contains("'")) {
                    singleAddHosts.add(taskHost);
                } else {
                    addList.add(taskHost);
                }
                log.info("主机资源【" + taskHost.getName() + "】保存成功");
            }
            if (addList.size() > 0) {
                taskHostRepository.batchSave(addList, 1000);
            }
            singleAddHosts.forEach(host -> taskHostRepository.save(host));
            log.info("主机资源批量导入结束，具体细节请查看站内信通知");
            message = "主机资源批量导入成功";
            // 操作通知
            resourceEventPublisher.send(
                    new OperateResult(true, message, operate, null, OperateResult.OperateCategory.ResourceEvent, context));
            // 发送站内信
            notifyMessage = new NotifyMessage(context, subject, message, true, false);
            amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
        } catch (Exception e) {
            message = "主机资源批量导入失败:" + e;
            log.error(message);
            // 操作通知
            resourceEventPublisher.send(
                    new OperateResult(false, message, operate, null, OperateResult.OperateCategory.ResourceEvent, context));
            // 发送站内信
            notifyMessage = new NotifyMessage(context, subject, message, true, false);
            amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
        }
    }

    public void readExcel(Workbook wb, RequestContext context) {
        boolean validateSuccess = true;
        try {
            validataWb(wb);
        } catch (Exception e) {
            validateSuccess = false;
        }
        String message = "";
        NotifyMessage notifyMessage;
        if (validateSuccess) {
            try {
                List<NetworkSegment> segments = networkSegmentRepository.list();
                List<TaskHost> taskHosts = taskHostRepository.listAllNoVendor();
                // 遍历出所有的ip
                Set<String> allipSet = TaskHostDataUtil.getAllIp(taskHosts);
                GeneralResult checkResult = checkData(wb, segments, allipSet);
                if (checkResult.isFailed()) {
                    message = "导入错误:" + checkResult.getMessage();
                    log.error(message);
                    // 操作通知
                    resourceEventPublisher.send(new OperateResult(false, message, operate, null,
                            OperateResult.OperateCategory.ResourceEvent, context));
                    // 发送站内信
                    notifyMessage = new NotifyMessage(context, subject, message, true, false);
                    amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                            CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
                } else {
                    saveData(wb, context, segments);
                }
            } catch (Exception e) {
                message = "导入错误:" + e;
                log.error(message);
                // 操作通知
                resourceEventPublisher.send(new OperateResult(false, message, operate, null,
                        OperateResult.OperateCategory.ResourceEvent, context));
                // 发送站内信
                notifyMessage = new NotifyMessage(context, subject, message, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                        CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
            }
        } else {
            message = "导入错误:EXCEL校验失败";
            log.error(message);
            // 操作通知
            resourceEventPublisher.send(
                    new OperateResult(false, message, operate, null, OperateResult.OperateCategory.ResourceEvent, context));
            // 发送站内信
            notifyMessage = new NotifyMessage(context, subject, message, true, false);
            amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, notifyMessage);
        }
    }
}
