package com.bocloud.cop.service.utils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;

/**
 * 任务执行结果导出
 *
 * @author hqy
 */
public class TaskResulExcelHelper {

    public TaskResulExcelHelper() {
        super();
    }

    /**
     * 生成模板
     *
     * @param workbook
     *            workbook
     * @throws Exception
     *             异常
     */
    public void generateSheet(Workbook workbook, Map<String, Object> exportData) throws Exception {
        Sheet sheet = workbook.createSheet("执行历史");
        Row row0 = sheet.createRow(0);
        row0.setHeight((short)400);
        // 生成文本
        generateBody(workbook, sheet, exportData);
        // 设置文本格式以及调整列宽,使之能完整显示字段名称
        ExcelHelper.setTextformatAndColumnwidth(workbook);
    }

    /**
     * 生成表内容
     *
     * @param sheet
     *            表
     * @throws Exception
     *             异常
     */
    private void generateBody(Workbook workbook, Sheet sheet, Map<String, Object> exportData) throws Exception {

        // 获取样式
        CellStyle whiteRowCellStyle = getWhiteRowCellStyle(workbook);

        int rowIndex = 0;
        Object taskInstance = exportData.get("taskInstance");
        if (taskInstance instanceof TaskInstance) {
            TaskInstance instance = (TaskInstance)taskInstance;
            rowIndex = generateHeaderInfo(workbook, sheet, rowIndex, instance);
            generateWhiteRow(sheet, rowIndex++, whiteRowCellStyle);
            rowIndex = generateInstanceInfo(workbook, sheet, rowIndex, instance);
            generateWhiteRow(sheet, rowIndex++, whiteRowCellStyle);
        }
        Object taskNodes = exportData.get("taskNodes");
        Object nodeIdToTarget = exportData.get("nodeToTarget");
        Map<String, JSONArray> targetMap = new HashMap<>();
        if (null != nodeIdToTarget) {
            targetMap = (Map<String, JSONArray>)nodeIdToTarget;
        }
        int taskNodeIndex = 0;
        if (null != taskNodes) {
            List<TaskNode> nodes = (List<TaskNode>)taskNodes;
            for (TaskNode node : nodes) {
                rowIndex = generateNodeInfo(workbook, sheet, rowIndex, node, taskNodeIndex, targetMap);
                taskNodeIndex++;
                if (nodes.size() != taskNodeIndex) {
                    generateWhiteRow(sheet, rowIndex++, whiteRowCellStyle);
                }
            }
        }
    }

    /**
     * 构建EXCEL头部
     *
     * @param workbook
     *            workbook
     * @param sheet
     *            sheet
     * @param rowIndex
     *            行索引
     * @param instance
     *            任务实例
     * @return 行索引
     */
    private int generateHeaderInfo(Workbook workbook, Sheet sheet, int rowIndex, TaskInstance instance) {
        // 获取样式
        CellStyle headCellStyle = getHeaderCellStyle(workbook);
        CellStyle contentCellStyle = getContentCellStyle(workbook);
        // 第一行
        Row row0 = sheet.createRow(rowIndex++);
        createCell(row0, 0, headCellStyle).setCellValue(instance.getName() + "作业执行报告");
        createCell(row0, 1, 7, headCellStyle);
        // 单元格合并 起始行 结束行 起始列 结束列
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));

        // 第二行
        Row row1 = sheet.createRow(rowIndex++);
        createCell(row1, 0, 5, contentCellStyle);
        createCell(row1, 6, contentCellStyle).setCellValue(DateUtils.dateToString(new Date(), "yyyy-MM-dd HH:mm:ss"));
        createCell(row1, 7, contentCellStyle);
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 5));
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 6, 7));
        return rowIndex;
    }

    /**
     * 构建任务实例相关信息
     *
     * @param workbook
     *            workbook
     * @param sheet
     *            sheet
     * @param rowIndex
     *            行索引
     * @param instance
     *            任务实例
     * @return 行索引
     */
    private int generateInstanceInfo(Workbook workbook, Sheet sheet, int rowIndex, TaskInstance instance) {
        // 获取样式
        CellStyle contentCellStyle = getContentCellStyle(workbook);
        CellStyle contentCellStyleAddForegroundColor = getContentCellStyleAddForegroundColor(workbook);

        Row row1 = sheet.createRow(rowIndex++);
        createCell(row1, 0, contentCellStyleAddForegroundColor).setCellValue("基本信息");
        createCell(row1, 1, 7, contentCellStyleAddForegroundColor);
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));

        // 作业名称
        Row row2 = sheet.createRow(rowIndex++);
        createCell(row2, 0, contentCellStyle).setCellValue("作业名称");
        createCell(row2, 1, contentCellStyle).setCellValue(instance.getName());
        createCell(row2, 2, 3, contentCellStyle);
        // 执行结果
        createCell(row2, 4, contentCellStyle).setCellValue("执行结果");
        createCell(row2, 5, contentCellStyle).setCellValue(BoStringUtils.getStatusName(instance.getStatus()));
        // 启动人
        createCell(row2, 6, contentCellStyle).setCellValue("启动人");
        createCell(row2, 7, contentCellStyle).setCellValue(instance.getOperator());
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 1, 3));

        // 开始时间
        Row row3 = sheet.createRow(rowIndex++);
        createCell(row3, 0, contentCellStyle).setCellValue("开始时间");
        createCell(row3, 1, contentCellStyle)
            .setCellValue(DateUtils.dateToString(instance.getGmtStart(), "yyyy-MM-dd HH:mm:ss"));
        createCell(row3, 2, contentCellStyle);
        // 结束时间
        createCell(row3, 3, contentCellStyle).setCellValue("结束时间");
        createCell(row3, 4, contentCellStyle)
            .setCellValue(DateUtils.dateToString(instance.getGmtFinish(), "yyyy-MM-dd HH:mm:ss"));
        createCell(row3, 5, contentCellStyle);
        // 总耗时(s)
        createCell(row3, 6, contentCellStyle).setCellValue("总耗时(s)");
        createCell(row3, 7, contentCellStyle)
            .setCellValue(BoStringUtils.getConsumeTime(instance.getGmtStart(), instance.getGmtFinish()));
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 1, 2));
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 4, 5));
        return rowIndex;
    }

    /**
     * 构建节点相关信息
     *
     * @param workbook
     *            workbook
     * @param sheet
     *            workbook
     * @param rowIndex
     *            行索引
     * @param node
     *            任务节点信息
     * @param nodeIndex
     *            任务节点索引
     * @param targetMap
     *            任务目标机器信息
     * @return 行索引
     */
    private int generateNodeInfo(Workbook workbook, Sheet sheet, int rowIndex, TaskNode node, int nodeIndex,
        Map<String, JSONArray> targetMap) {
        // 获取样式
        CellStyle contentCellStyle = getContentCellStyle(workbook);
        CellStyle contentCellStyleAddForegroundColor = getContentCellStyleAddForegroundColor(workbook);

        if (0 == nodeIndex) {
            // 任务详情
            Row row1 = sheet.createRow(rowIndex++);
            createCell(row1, 0, contentCellStyleAddForegroundColor).setCellValue("任务详情");
            createCell(row1, 1, 7, contentCellStyleAddForegroundColor);
            sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));
        }
        // 任务节点
        Row row2 = sheet.createRow(rowIndex++);
        createCell(row2, 0, contentCellStyle).setCellValue("任务节点" + (nodeIndex + 1));
        createCell(row2, 1, 7, contentCellStyle);
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));

        // 节点名称
        Row row3 = sheet.createRow(rowIndex++);
        createCell(row3, 0, contentCellStyle).setCellValue("节点名称" + (nodeIndex + 1));
        createCell(row3, 1, contentCellStyle).setCellValue(node.getName());
        createCell(row3, 2, 3, contentCellStyle);
        // 节点类型
        createCell(row3, 4, contentCellStyle).setCellValue("节点类型");
        createCell(row3, 5, contentCellStyle).setCellValue(BoStringUtils.getApplyName(node.getCategory()));
        // 节点状态
        createCell(row3, 6, contentCellStyle).setCellValue("节点状态");
        createCell(row3, 7, contentCellStyle).setCellValue(BoStringUtils.getStatusName(node.getStatus()));
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 1, 3));

        // 开始时间
        Row row4 = sheet.createRow(rowIndex++);
        createCell(row4, 0, contentCellStyle).setCellValue("开始时间");
        createCell(row4, 1, contentCellStyle)
            .setCellValue(DateUtils.dateToString(node.getGmtStart(), "yyyy-MM-dd HH:mm:ss"));
        createCell(row4, 2, contentCellStyle);
        // 结束时间
        createCell(row4, 3, contentCellStyle).setCellValue("结束时间");
        createCell(row4, 4, contentCellStyle)
            .setCellValue(DateUtils.dateToString(node.getGmtFinish(), "yyyy-MM-dd HH:mm:ss"));
        createCell(row4, 5, contentCellStyle);
        // 总时间
        createCell(row4, 6, contentCellStyle).setCellValue("总时间(s)");
        createCell(row4, 7, contentCellStyle)
            .setCellValue(null != node.getCost() ? String.valueOf(node.getCost().intValue()) : "");
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 1, 2));
        sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 4, 5));

        JSONArray targetArr = targetMap.get(String.valueOf(node.getId()));
        if (null != targetArr && targetArr.size() > 0) {
            // 执行结果
            Row row5 = sheet.createRow(rowIndex++);
            createCell(row5, 0, contentCellStyleAddForegroundColor).setCellValue("执行结果");
            createCell(row5, 1, 7, contentCellStyleAddForegroundColor);
            sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));
            for (Object target : targetArr) {
                JSONObject targetJson = JSONObject.parseObject(JSONObject.toJSONString(target));
                // 目标主机
                if (null != targetJson.getString("host")) {
                    Row row6 = sheet.createRow(rowIndex++);
                    createCell(row6, 0, contentCellStyle).setCellValue("目标主机");
                    createCell(row6, 1, contentCellStyle);
                    createCell(row6, 2, contentCellStyle).setCellValue(targetJson.getString("host"));
                    createCell(row6, 3, contentCellStyle);
                    createCell(row6, 4, contentCellStyle).setCellValue("执行结果");
                    createCell(row6, 5, contentCellStyle);
                    createCell(row6, 6, contentCellStyle).setCellValue(targetJson.getBoolean("success") ? "成功" : "失败");
                    createCell(row6, 7, contentCellStyle);
                    sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 1));
                    sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 2, 3));
                    sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 4, 5));
                    sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 6, 7));
                }
                // 执行输出
                Row row7 = sheet.createRow(rowIndex++);
                createCell(row7, 0, contentCellStyle).setCellValue("执行输出");
                createCell(row7, 1, contentCellStyle);
                // 如果需要标记message内容不为公式，需要在内容前加个单引号
                createCell(row7, 2, contentCellStyle).setCellValue(targetJson.getString("message"));
                createCell(row7, 3, 7, contentCellStyle);
                sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 1));
                sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 2, 7));
            }
        } else {
            // 执行结果
            Row row5 = sheet.createRow(rowIndex++);
            createCell(row5, 0, contentCellStyleAddForegroundColor).setCellValue("执行结果");
            createCell(row5, 1, 7, contentCellStyleAddForegroundColor);
            sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));

            Row row6 = sheet.createRow(rowIndex++);
            createCell(row6, 0, contentCellStyle).setCellValue("无执行结果");
            sheet.addMergedRegion(new CellRangeAddress(rowIndex - 1, rowIndex - 1, 0, 7));
        }
        return rowIndex;
    }

    /**
     * 构建一个空行
     *
     * @param sheet
     *            页
     * @param rowIndex
     *            行的索引
     */
    private void generateWhiteRow(Sheet sheet, int rowIndex, CellStyle cellStyle) {
        Row row = sheet.createRow(rowIndex);
        createCell(row, 0, 7, cellStyle);
    }

    /**
     * 创建单元格
     *
     * @param row
     *            行
     * @param start
     *            开始的列索引
     * @param end
     *            结束的列索引
     * @param cellStyle
     *            单元格样式
     */
    private void createCell(Row row, int start, int end, CellStyle cellStyle) {
        for (int i = start; i <= end; i++) {
            Cell cell = row.createCell(i);
            cell.setCellStyle(cellStyle);
        }
    }

    /**
     * 创建单元格
     *
     * @param row
     *            行
     * @param index
     *            列索引
     * @param cellStyle
     *            单元格样式
     * @return 单元格
     */
    private Cell createCell(Row row, int index, CellStyle cellStyle) {
        Cell cell = row.createCell(index);
        cell.setCellStyle(cellStyle);
        return cell;
    }

    /**
     * 标题样式
     *
     * @return 样式
     */
    private CellStyle getHeaderCellStyle(Workbook workbook) {
        CellStyle cellStyle = workbook.createCellStyle();
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle.setBorderRight(BorderStyle.THIN);
        Font font = workbook.createFont();
        font.setFontName("Arial");
        font.setFontHeightInPoints((short)20);
        cellStyle.setFont(font);
        return cellStyle;
    }

    /**
     * 内容样式
     *
     * @param workbook
     *            excel
     * @return 样式
     */
    private CellStyle getContentCellStyle(Workbook workbook) {
        CellStyle cellStyle = workbook.createCellStyle();
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.LEFT);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle.setBorderRight(BorderStyle.THIN);
        Font bodyFont = workbook.createFont();
        bodyFont.setFontName("Arial");
        bodyFont.setFontHeightInPoints((short)10);
        cellStyle.setFont(bodyFont);
        return cellStyle;
    }

    /**
     * 内容样式（加了背景）
     *
     * @param workbook
     *            excel
     * @return 样式
     */
    private CellStyle getContentCellStyleAddForegroundColor(Workbook workbook) {
        CellStyle cellStyle = getContentCellStyle(workbook);
        // 填充
        cellStyle.setFillForegroundColor(HSSFColor.GREY_25_PERCENT.index);
        cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        return cellStyle;
    }

    /**
     * 获取空行单元格样式
     *
     * @param workbook
     *            excel
     * @return 样式
     */
    private CellStyle getWhiteRowCellStyle(Workbook workbook) {
        CellStyle cellStyle = workbook.createCellStyle();
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.LEFT);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        Font bodyFont = workbook.createFont();
        bodyFont.setFontName("Arial");
        bodyFont.setFontHeightInPoints((short)10);
        cellStyle.setFont(bodyFont);
        return cellStyle;
    }
}
