package com.bocloud.cop.service.utils;

import java.io.ByteArrayOutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

/**
 * 任务执行结果导出
 *
 * @author hqy
 */
public class TaskResulPdfHelper {

    public TaskResulPdfHelper() {
        super();
    }

    /**
     * 生成pdf
     *
     * @param exportData
     *            导出数据
     * @return pdf内容byte数组
     * @throws Exception
     *             异常
     */
    public byte[] generatePdf(Map<String, Object> exportData) throws Exception {
        // 字体
        BaseFont chinese = BaseFont.createFont("STSongStd-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
        // 创建文档
        Document document = new Document(PageSize.A4, 50, 50, 50, 50);
        // 创建输出流
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PdfWriter tWriter = PdfWriter.getInstance(document, baos);

        document.open();
        document.newPage();
        Object taskInstance = exportData.get("taskInstance");
        if (taskInstance instanceof TaskInstance) {
            TaskInstance instance = (TaskInstance)taskInstance;
            generateInstanceInfo(document, instance, chinese);
        }
        Object taskNodes = exportData.get("taskNodes");
        Object nodeIdToTarget = exportData.get("nodeToTarget");
        Map<String, JSONArray> targetMap = new HashMap<>();
        if (null != nodeIdToTarget) {
            targetMap = (Map<String, JSONArray>)nodeIdToTarget;
        }
        int taskNodeIndex = 0;
        if (null != taskNodes) {
            java.util.List<TaskNode> nodes = (List<TaskNode>)taskNodes;
            for (TaskNode node : nodes) {
                generateNodeInfo(document, node, chinese, taskNodeIndex, targetMap);
                taskNodeIndex++;
            }
        }
        document.close();
        tWriter.close();
        return baos.toByteArray();

    }

    /**
     * 生成任务实例信息
     *
     * @param document
     *            pdf文档
     * @param instance
     *            任务实例
     * @param font
     *            字体
     * @throws DocumentException
     *             异常
     */
    private void generateInstanceInfo(Document document, TaskInstance instance, BaseFont font)
        throws DocumentException {
        // 字体样式
        Font headerFont = new Font(font, 24, Font.BOLD);
        Font timeFont = new Font(font, 12, Font.BOLD);
        Font contentFont = new Font(font, 12, Font.NORMAL);
        // 生成标题
        Paragraph header = new Paragraph(instance.getName() + "作业执行报告", headerFont);
        header.setAlignment(1);
        // 生成时间
        Paragraph time = new Paragraph(DateUtils.dateToString(new Date(), "yyyy-MM-dd HH:mm:ss"), timeFont);
        time.setAlignment(2);

        // 创建表格
        PdfPTable table = new PdfPTable(8);
        table.setTotalWidth(540);
        table.setLockedWidth(true);
        table.setSpacingBefore(20);
        table.addCell(addCell("基本属性", contentFont, BaseColor.GRAY, 8, 15));
        table.addCell(addCell("作业名称", contentFont, null, 0, 0));
        table.addCell(addCell(instance.getName(), contentFont, null, 3, 0));
        table.addCell(addCell("执行结果", contentFont, null, 0, 0));
        table.addCell(addCell(BoStringUtils.getStatusName(instance.getStatus()), contentFont, null, 0, 0));
        table.addCell(addCell("启动人", contentFont, null, 0, 0));
        table.addCell(addCell(instance.getOperator(), contentFont, null, 0, 0));
        table.addCell(addCell("开始时间", contentFont, null, 0, 0));
        table.addCell(
            addCell(DateUtils.dateToString(instance.getGmtStart(), "yyyy-MM-dd HH:mm:ss"), contentFont, null, 2, 0));
        table.addCell(addCell("结束时间", contentFont, null, 0, 0));
        table.addCell(
            addCell(DateUtils.dateToString(instance.getGmtFinish(), "yyyy-MM-dd HH:mm:ss"), contentFont, null, 2, 0));
        table.addCell(addCell("总耗时(s)", contentFont, null, 0, 0));
        table.addCell(addCell(BoStringUtils.getConsumeTime(instance.getGmtStart(), instance.getGmtFinish()),
            contentFont, null, 0, 0));

        document.add(header);
        document.add(time);
        document.add(table);
    }

    /**
     * 生成节点信息
     *
     * @param document
     *            pdf文档
     * @param node
     *            任务节点
     * @param font
     *            字体
     * @param nodeIndex
     *            节点索引
     * @param targetMap
     *            节点目标主机
     * @throws DocumentException
     *             异常
     */
    private void generateNodeInfo(Document document, TaskNode node, BaseFont font, int nodeIndex,
        Map<String, JSONArray> targetMap) throws DocumentException {
        // 字体样式
        Font contentFont = new Font(font, 12, Font.NORMAL);
        // 创建表格
        PdfPTable nodeTable = new PdfPTable(8);
        nodeTable.setTotalWidth(540);
        nodeTable.setLockedWidth(true);
        nodeTable.setSpacingBefore(20);
        if (0 == nodeIndex) {
            nodeTable.addCell(addCell("任务详情", contentFont, BaseColor.GRAY, 8, 15));
        }
        nodeTable.addCell(addCell("任务节点" + (nodeIndex + 1), contentFont, BaseColor.LIGHT_GRAY, 8, 15));
        nodeTable.addCell(addCell("节点名称", contentFont, null, 0, 0));
        nodeTable.addCell(addCell(node.getName(), contentFont, null, 3, 0));
        nodeTable.addCell(addCell("节点类型", contentFont, null, 0, 0));
        nodeTable.addCell(addCell(BoStringUtils.getApplyName(node.getCategory()), contentFont, null, 0, 0));
        nodeTable.addCell(addCell("节点状态", contentFont, null, 0, 0));
        nodeTable.addCell(addCell(BoStringUtils.getStatusName(node.getStatus()), contentFont, null, 0, 0));
        nodeTable.addCell(addCell("开始时间", contentFont, null, 0, 0));
        nodeTable.addCell(
            addCell(DateUtils.dateToString(node.getGmtStart(), "yyyy-MM-dd HH:mm:ss"), contentFont, null, 2, 0));
        nodeTable.addCell(addCell("结束时间", contentFont, null, 0, 0));
        nodeTable.addCell(
            addCell(DateUtils.dateToString(node.getGmtStart(), "yyyy-MM-dd HH:mm:ss"), contentFont, null, 2, 0));
        nodeTable.addCell(addCell("总时间(s)", contentFont, null, 0, 0));
        nodeTable.addCell(
            addCell(null != node.getCost() ? String.valueOf(node.getCost().intValue()) : "", contentFont, null, 0, 0));

        document.add(nodeTable);

        JSONArray targetArr = targetMap.get(String.valueOf(node.getId()));
        if (null != targetArr && targetArr.size() > 0) {
            for (Object target : targetArr) {
                JSONObject targetJson = JSONObject.parseObject(JSONObject.toJSONString(target));
                PdfPTable targetTable = new PdfPTable(8);
                targetTable.setTotalWidth(540);
                targetTable.setLockedWidth(true);
                targetTable.setSpacingBefore(0);
                targetTable.addCell(addCell("执行结果", contentFont, BaseColor.LIGHT_GRAY, 8, 15));
                // 目标主机
                if (null != targetJson.getString("host")) {
                    targetTable.addCell(addCell("目标主机", contentFont, null, 2, 0));
                    targetTable.addCell(addCell(targetJson.getString("host"), contentFont, null, 3, 0));
                    targetTable.addCell(addCell("执行结果", contentFont, null, 2, 0));
                    targetTable
                        .addCell(addCell(targetJson.getBoolean("success") ? "成功" : "失败", contentFont, null, 0, 0));
                }
                targetTable.addCell(addCell("执行输出", contentFont, null, 2, 0));
                targetTable.addCell(addCell(targetJson.getString("message"), contentFont, null, 6, 0));
                document.add(targetTable);
            }
        } else {
            // 执行结果
            PdfPTable targetTable = new PdfPTable(1);
            targetTable.setTotalWidth(540);
            targetTable.setLockedWidth(true);
            targetTable.setSpacingBefore(0);
            targetTable.addCell(addCell("无执行结果", contentFont, BaseColor.LIGHT_GRAY, 0, 15));
            document.add(targetTable);
        }
    }

    /**
     * 添加单元格
     *
     * @param val
     *            单元格内容
     * @param font
     *            单元格字体
     * @param backgroudColor
     *            单元格背景颜色
     * @param colSpan
     *            单元格跨列列数
     * @param disableBorderSize
     *            单元格边框隐藏设置
     */

    private static PdfPCell addCell(String val, Font font, BaseColor backgroudColor, int colSpan,
        int disableBorderSize) {
        String val1 = null == val ? " " : val;
        // 新建单元格
        PdfPCell cell = new PdfPCell(new Phrase(val1, font));
        // 水平居中
        cell.setHorizontalAlignment(Element.ALIGN_LEFT);
        if (0 != disableBorderSize) {
            cell.disableBorderSide(disableBorderSize);
        }
        // 设置宽度

        // 单元格高度
        cell.setMinimumHeight(20);
        if (backgroudColor != null) {
            // 设置背景颜色
            cell.setBackgroundColor(backgroudColor);
        }
        if (colSpan != 0) {
            // 设置跨列列数
            cell.setColspan(colSpan);
        }
        return cell;
    }
}
