package com.bocloud.cop.service.utils;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.xwpf.usermodel.ParagraphAlignment;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTBorder;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTShd;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTblPr;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTblWidth;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTc;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTcBorders;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTcPr;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STBorder;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STMerge;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STShd;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STTblWidth;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;

/**
 * 任务执行结果导出
 *
 * @author hqy
 */
public class TaskResulWordHelper {

    public TaskResulWordHelper() {
        super();
    }

    /**
     * 生成docx文档
     *
     * @param exportData
     *            导出数据
     * @return 文档
     * @throws IOException
     *             异常
     */
    public XWPFDocument generateWord2007(Map<String, Object> exportData) throws IOException {
        // 创建文本对象
        XWPFDocument docxDocument = new XWPFDocument();
        // 创建样式信息
        docxDocument.createStyles();
        // 设置纸张大小
        Object taskInstance = exportData.get("taskInstance");
        if (taskInstance instanceof TaskInstance) {
            TaskInstance instance = (TaskInstance)taskInstance;
            generateInstanceInfo(docxDocument, instance);
        }
        Object taskNodes = exportData.get("taskNodes");
        Object nodeIdToTarget = exportData.get("nodeToTarget");
        Map<String, JSONArray> targetMap = new HashMap<>();
        if (null != nodeIdToTarget) {
            targetMap = (Map<String, JSONArray>)nodeIdToTarget;
        }
        int taskNodeIndex = 0;
        if (null != taskNodes) {
            List<TaskNode> nodes = (List<TaskNode>)taskNodes;
            for (TaskNode node : nodes) {
                generateNodeInfo(docxDocument, node, ++taskNodeIndex, targetMap);
                if (nodes.size() != taskNodeIndex) {
                    generateWhiteRow(docxDocument);
                }
            }
        }
        return docxDocument;
    }

    /**
     * 任务实例信息
     *
     * @param document
     *            文档
     * @param instance
     *            实例
     */
    private void generateInstanceInfo(XWPFDocument document, TaskInstance instance) {
        // 创建第一段落
        XWPFParagraph paragraph1 = document.createParagraph();
        paragraph1.setAlignment(ParagraphAlignment.CENTER);
        XWPFRun runTitle = paragraph1.createRun();
        runTitle.setText(instance.getName() + "作业执行报告");
        runTitle.setBold(true);
        runTitle.setFontSize(24);
        runTitle.setFontFamily("宋体(正文)");
        runTitle.addCarriageReturn();// 回车键
        runTitle.setKerning(30);
        // 创建第二段落
        XWPFParagraph paragraph2 = document.createParagraph();
        paragraph2.setAlignment(ParagraphAlignment.RIGHT);
        XWPFRun run = paragraph2.createRun();
        run.setText(DateUtils.dateToString(new Date(), "yyyy-MM-dd HH:mm:ss"));
        run.setBold(true);
        run.setFontSize(12);
        run.setFontFamily("宋体");
        generateWhiteRow(document);
        createTableTitle(document, "基本信息", 4, "666666");
        generateInstanceContent(document, instance);
        generateWhiteRow(document);
    }

    /**
     * 生成节点信息
     *
     * @param document
     *            文档
     * @param node
     *            节点
     * @param nodeIndex
     *            节点索引
     * @param targetMap
     *            节点目标主机
     */
    private void generateNodeInfo(XWPFDocument document, TaskNode node, int nodeIndex,
        Map<String, JSONArray> targetMap) {
        if (1 == nodeIndex) {
            createTableTitle(document, "任务详情", 12, "666666");
        }
        createTableTitle(document, "任务节点" + nodeIndex, 12, "D3D3D3");
        createNodeContent(document, node, targetMap);
    }

    /**
     * 生成空白段落
     *
     * @param document
     */
    private void generateWhiteRow(XWPFDocument document) {
        XWPFParagraph paragraph = document.createParagraph();
    }

    /**
     * 设置单元格内容
     *
     * @param cell
     *            单元格
     * @param content
     *            内容
     */
    private void setCellContent(XWPFTableCell cell, String content) {
        cell.setText(content);
        cell.setVerticalAlignment(XWPFTableCell.XWPFVertAlign.CENTER);
    }

    /**
     * 创建标题行
     *
     * @param document
     *            文档
     * @param content
     *            内容
     * @param size
     *            表格大小
     * @param color
     *            背景色
     */
    private void createTableTitle(XWPFDocument document, String content, int size, String color) {
        XWPFTable table = document.createTable(1, size);

        // 设置表格宽度
        table.setWidth(5000);
        CTTblPr ctTblPr = table.getCTTbl().getTblPr();
        ctTblPr.getTblW().setType(STTblWidth.PCT);
        ctTblPr.getTblBorders().getRight().setVal(STBorder.NIL);
        XWPFTableRow row0 = table.getRow(0);
        // 合并单元格
        getCellCTTcPr(row0.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
        setCellWith(row0.getCell(0), 200);
        for (int i = 1; i < size; i++) {
            setCellWith(row0.getCell(0), 100 * i);
            getCellCTTcPr(row0.getCell(i)).addNewHMerge().setVal(STMerge.CONTINUE);
        }

        CTTcPr cellCTTcPr = getCellCTTcPr(row0.getCell(0));
        CTShd cellCtshd = null == cellCTTcPr.getShd() ? cellCTTcPr.addNewShd() : cellCTTcPr.getShd();

        CTTcBorders cellBorders =
            null == cellCTTcPr.getTcBorders() ? cellCTTcPr.addNewTcBorders() : cellCTTcPr.getTcBorders();
        CTBorder leftBorder = null == cellBorders.getLeft() ? cellBorders.addNewLeft() : cellBorders.getLeft();
        leftBorder.setVal(STBorder.NIL);
        CTBorder rightBorder = null == cellBorders.getRight() ? cellBorders.addNewRight() : cellBorders.getRight();
        rightBorder.setVal(STBorder.NIL);
        CTBorder topBorder = null == cellBorders.getTop() ? cellBorders.addNewTop() : cellBorders.getTop();
        topBorder.setVal(STBorder.NIL);
        CTBorder bottomBorder = null == cellBorders.getBottom() ? cellBorders.addNewBottom() : cellBorders.getBottom();
        bottomBorder.setVal(STBorder.NIL);

        cellCtshd.setVal(STShd.CLEAR);
        cellCtshd.setFill(color);
        // 表格内容

        XWPFTableCell r0c1 = row0.getCell(0);
        r0c1.setText(content);
    }

    /**
     * @param document
     * @param instance
     */
    private void generateInstanceContent(XWPFDocument document, TaskInstance instance) {
        XWPFTable table = document.createTable(3, 4);
        // 设置表格宽度
        table.setWidth(6000);
        CTTblPr tblPr = table.getCTTbl().getTblPr();
        tblPr.getTblW().setType(STTblWidth.PCT);
        // 设置单元格宽
        XWPFTableRow row0 = table.getRow(0);
        CTTblWidth width0 = getCellCTTcPr(row0.getCell(0)).addNewTcW();
        width0.setType(STTblWidth.PCT);
        width0.setW(new BigInteger("800"));
        CTTblWidth width1 = getCellCTTcPr(row0.getCell(1)).addNewTcW();
        width1.setType(STTblWidth.PCT);
        width1.setW(new BigInteger("1900"));
        CTTblWidth width2 = getCellCTTcPr(row0.getCell(2)).addNewTcW();
        width2.setType(STTblWidth.PCT);
        width2.setW(new BigInteger("800"));
        CTTblWidth width3 = getCellCTTcPr(row0.getCell(3)).addNewTcW();
        width3.setType(STTblWidth.PCT);
        width3.setW(new BigInteger("1500"));
        // 填充内容
        setCellContent(table.getRow(0).getCell(0), "作业名称");
        setCellContent(table.getRow(0).getCell(1), instance.getName());
        setCellContent(table.getRow(0).getCell(2), "执行结果");
        setCellContent(table.getRow(0).getCell(3), BoStringUtils.getStatusName(instance.getStatus()));
        setCellContent(table.getRow(1).getCell(0), "启动人");
        setCellContent(table.getRow(1).getCell(1), instance.getOperator());
        setCellContent(table.getRow(1).getCell(2), "总耗时(s)");
        setCellContent(table.getRow(1).getCell(3),
            BoStringUtils.getConsumeTime(instance.getGmtStart(), instance.getGmtFinish()));
        setCellContent(table.getRow(2).getCell(0), "开始时间");
        setCellContent(table.getRow(2).getCell(1),
            DateUtils.dateToString(instance.getGmtStart(), "yyyy-MM-dd HH:mm:ss"));
        setCellContent(table.getRow(2).getCell(2), "结束时间");
        setCellContent(table.getRow(2).getCell(3),
            DateUtils.dateToString(instance.getGmtFinish(), "yyyy-MM-dd HH:mm:ss"));
    }

    private void createNodeContent(XWPFDocument document, TaskNode node, Map<String, JSONArray> targetMap) {
        XWPFTable table = document.createTable(2, 12);
        // 设置表格宽度
        table.setWidth(5000);
        CTTblPr tblPr = table.getCTTbl().getTblPr();
        tblPr.getTblW().setType(STTblWidth.PCT);
        // 合并
        XWPFTableRow row0 = table.getRow(0);
        getCellCTTcPr(row0.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(1)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(2)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(3)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(4)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(5)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(6)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(7)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(8)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(9)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(10)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(11)).addNewHMerge().setVal(STMerge.CONTINUE);
        XWPFTableRow row1 = table.getRow(1);
        getCellCTTcPr(row1.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row1.getCell(1)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row1.getCell(2)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row1.getCell(3)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row1.getCell(4)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row1.getCell(5)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row1.getCell(6)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row1.getCell(7)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row1.getCell(8)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row1.getCell(9)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row1.getCell(10)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row1.getCell(11)).addNewHMerge().setVal(STMerge.CONTINUE);

        // 填充内容
        setCellContent(table.getRow(0).getCell(0), "节点名称");
        setCellContent(table.getRow(0).getCell(2), node.getName());
        setCellContent(table.getRow(0).getCell(4), "节点类型");
        setCellContent(table.getRow(0).getCell(6), BoStringUtils.getApplyName(node.getCategory()));
        setCellContent(table.getRow(0).getCell(8), "节点状态");
        setCellContent(table.getRow(0).getCell(10), BoStringUtils.getStatusName(node.getStatus()));
        setCellContent(table.getRow(1).getCell(0), "开始时间");
        setCellContent(table.getRow(1).getCell(2), DateUtils.dateToString(node.getGmtStart(), "yyyy-MM-dd HH:mm:ss"));
        setCellContent(table.getRow(1).getCell(4), "结束时间");
        setCellContent(table.getRow(1).getCell(6), DateUtils.dateToString(node.getGmtStart(), "yyyy-MM-dd HH:mm:ss"));
        setCellContent(table.getRow(1).getCell(8), "总时间(s)");
        setCellContent(table.getRow(1).getCell(10),
            null != node.getCost() ? String.valueOf(node.getCost().intValue()) : "");

        JSONArray targetArr = targetMap.get(String.valueOf(node.getId()));
        if (null != targetArr && targetArr.size() > 0) {
            createTableTitle(document, "执行结果", 12, "D3D3D3");
            for (Object target : targetArr) {
                JSONObject targetJson = JSONObject.parseObject(JSONObject.toJSONString(target));
                createTargetContent(document, targetJson);
            }
        } else {
            createNoTargetContent(document);
        }
    }

    private void createTargetContent(XWPFDocument document, JSONObject json) {

        if (null != json.getString("host")) {
            XWPFTable table = document.createTable(2, 12);
            // 设置表格宽度
            table.setWidth(5000);
            CTTblPr tblPr = table.getCTTbl().getTblPr();
            tblPr.getTblW().setType(STTblWidth.PCT);
            // 设置单元格宽
            XWPFTableRow row0 = table.getRow(0);
            CTTblWidth width0 = getCellCTTcPr(row0.getCell(0)).addNewTcW();
            width0.setType(STTblWidth.PCT);
            width0.setW(new BigInteger("1100"));
            CTTblWidth width1 = getCellCTTcPr(row0.getCell(1)).addNewTcW();
            width1.setType(STTblWidth.PCT);
            width1.setW(new BigInteger("3900"));
            // 合并单元格
            getCellCTTcPr(row0.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row0.getCell(1)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(2)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(3)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row0.getCell(4)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(5)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(6)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row0.getCell(7)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(8)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(9)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row0.getCell(10)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(11)).addNewHMerge().setVal(STMerge.CONTINUE);
            XWPFTableRow row1 = table.getRow(1);
            getCellCTTcPr(row1.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row1.getCell(1)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(2)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(3)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row1.getCell(4)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(5)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(6)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(7)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(8)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(9)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(10)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row1.getCell(11)).addNewHMerge().setVal(STMerge.CONTINUE);

            // 填充内容
            setCellContent(table.getRow(0).getCell(0), "目标主机");
            setCellContent(table.getRow(0).getCell(3), json.getString("host"));
            setCellContent(table.getRow(0).getCell(6), "执行结果");
            setCellContent(table.getRow(0).getCell(9), json.getBoolean("success") ? "成功" : "失败");
            setCellContent(table.getRow(1).getCell(0), "执行输出");
            setCellContent(table.getRow(1).getCell(3), json.getString("message"));
            // setCellContent(table.getRow(0).getCell(3), "");
        } else {
            XWPFTable table = document.createTable(1, 12);
            // 设置表格宽度
            table.setWidth(5000);
            CTTblPr tblPr = table.getCTTbl().getTblPr();
            tblPr.getTblW().setType(STTblWidth.PCT);
            XWPFTableRow row0 = table.getRow(0);
            getCellCTTcPr(row0.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row0.getCell(1)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(2)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(3)).addNewHMerge().setVal(STMerge.RESTART);
            getCellCTTcPr(row0.getCell(4)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(5)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(6)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(7)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(8)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(9)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(10)).addNewHMerge().setVal(STMerge.CONTINUE);
            getCellCTTcPr(row0.getCell(11)).addNewHMerge().setVal(STMerge.CONTINUE);
            // 填充内容
            setCellContent(table.getRow(0).getCell(0), "执行输出");
            setCellContent(table.getRow(0).getCell(3), json.getString("message"));
        }
    }

    private void createNoTargetContent(XWPFDocument document) {
        XWPFTable table = document.createTable(1, 12);
        // 设置表格宽度
        table.setWidth(5000);
        CTTblPr tblPr = table.getCTTbl().getTblPr();
        tblPr.getTblW().setType(STTblWidth.PCT);
        // 合并单元格
        XWPFTableRow row0 = table.getRow(0);
        getCellCTTcPr(row0.getCell(0)).addNewHMerge().setVal(STMerge.RESTART);
        getCellCTTcPr(row0.getCell(1)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(2)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(3)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(4)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(5)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(6)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(7)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(8)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(9)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(10)).addNewHMerge().setVal(STMerge.CONTINUE);
        getCellCTTcPr(row0.getCell(11)).addNewHMerge().setVal(STMerge.CONTINUE);
        // 填充内容
        setCellContent(table.getRow(0).getCell(0), "无执行结果");
    }

    public CTTcPr getCellCTTcPr(XWPFTableCell cell) {
        CTTc cttc = cell.getCTTc();
        CTTcPr tcPr = cttc.isSetTcPr() ? cttc.getTcPr() : cttc.addNewTcPr();
        return tcPr;
    }

    private void setCellWith(XWPFTableCell cell, int with) {
        CTTblWidth ctTblWidth = getCellCTTcPr(cell).addNewTcW();
        ctTblWidth.setType(STTblWidth.DXA);
        ctTblWidth.setW(new BigInteger(String.valueOf(with)));
    }
}
