package com.bocloud.cop.service.utils.worker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.data.redis.core.StringRedisTemplate;

import com.megatron.common.enums.OSCategory;
import com.megatron.common.model.GeneralResult;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.TaskHost;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.model.TargetGroup;
import com.bocloud.cop.model.WorkerHost;
import com.google.common.collect.Lists;

public class WorkerDataUtils {

    /**
     * 根据网段id分组
     * @param hosts
     * @return
     */
    public static Map<Long, List<TaskHost>> getHostMap(List<TaskHost> hosts) {
        Map<Long, List<TaskHost>> hostMap = new HashMap<>();
        for (TaskHost host : hosts) {
            if (hostMap.containsKey(host.getSegmentId())) {
                hostMap.get(host.getSegmentId()).add(host);
            } else {
                List<TaskHost> list = new ArrayList<>();
                list.add(host);
                hostMap.put(host.getSegmentId(), list);
            }
        }
        return hostMap;
    }

    /**
     * 根据网段id分组
     * @param hosts
     * @return
     */
    public static Map<Long, List<WorkerHost>> getWorkerHostMap(List<WorkerHost> hosts) {
        Map<Long, List<WorkerHost>> hostMap = new HashMap<>();
        for (WorkerHost host : hosts) {
            if (hostMap.containsKey(host.getSegmentId())) {
                hostMap.get(host.getSegmentId()).add(host);
            } else {
                List<WorkerHost> list = new ArrayList<>();
                list.add(host);
                hostMap.put(host.getSegmentId(), list);
            }
        }
        return hostMap;
    }

    public static Map<Long, List<TaskWorker>> getWorkerMap(Map<Long, List<TaskHost>> hostMap, List<TaskWorker> workers,
        List<NetworkSegment> segments) {
        // 分组和worker关系
        Map<Long, List<TaskWorker>> workerMap = new HashMap<>();
        for (TaskWorker worker : workers) {
            if (null == worker.getGroupId()) {
                continue;
            }
            if (workerMap.containsKey(worker.getGroupId())) {
                workerMap.get(worker.getGroupId()).add(worker);
            } else {
                List<TaskWorker> list = new ArrayList<>();
                list.add(worker);
                workerMap.put(worker.getGroupId(), list);
            }
        }
        // 网段和worker关系
        Map<Long, List<TaskWorker>> segWorkerMap = new HashMap<>();
        for (NetworkSegment segment : segments) {
            if (!hostMap.containsKey(segment.getId())) {
                continue;
            }
            if (null == segment.getGroupId()) {
                continue;
            }
            if (workerMap.containsKey(segment.getGroupId())) {
                segWorkerMap.put(segment.getId(), workerMap.get(segment.getGroupId()));
            }
        }
        return segWorkerMap;
    }

    /**
     * 进行步长分组
     *
     * @param targets
     * @return
     */
    public static List<List<WorkerHost>> splitTarget(List<WorkerHost> targets) {
        List<List<WorkerHost>> list = new ArrayList<>();
        int step = 50;
        for (int i = 0; i < targets.size(); i = i + step) {
            List<WorkerHost> target = new ArrayList<>();
            for (int j = i; j < i + step && j < targets.size(); j++) {
                target.add(targets.get(j));
            }
            list.add(target);
        }
        return list;
    }

    /**
     * 根据操作类型系统分组
     *
     * @param hosts
     * @param template
     * @return
     * @throws
     */
    public static GeneralResult<TargetGroup> groupTarget(List<TaskHost> hosts, StringRedisTemplate template)
        throws Exception {
        List<WorkerHost> linuxs = Lists.newArrayList();
        List<WorkerHost> windows = Lists.newArrayList();
        for (TaskHost host : hosts) {
            if (null != host.getOsCategory() && host.getOsCategory().toLowerCase().contains("window")) {
                WorkerHost target =
                    new WorkerHost(host.getId(), host.getIp(), host.getUsername(), null, 5985, OSCategory.WINDOWS);
                windows.add(target);
            } else {
                WorkerHost target =
                    new WorkerHost(host.getId(), host.getIp(), host.getUsername(), null, 22, OSCategory.LINUX);
                linuxs.add(target);
            }
        }
        return new GeneralResult<TargetGroup>(true, new TargetGroup(windows, linuxs), "success");
    }

    /**
     * 根据操作系统类型封装机器信息
     * @param hosts
     * @return
     */
    public static GeneralResult<TargetGroup> groupTarget(List<TaskHost> hosts) {
        List<WorkerHost> linuxs = Lists.newArrayList();
        List<WorkerHost> windows = Lists.newArrayList();
        for (TaskHost host : hosts) {
            if (null != host.getOsName() && host.getOsName().toLowerCase().contains("window")) {
                WorkerHost target = new WorkerHost(host.getId(), host.getIp(), host.getUsername(), host.getPassword(), 5985, OSCategory.WINDOWS);
                target.setSegmentId(host.getSegmentId());
                windows.add(target);
            } else {
                WorkerHost target = new WorkerHost(host.getId(), host.getIp(), host.getUsername(), host.getPassword(), 22, OSCategory.LINUX);
                target.setSegmentId(host.getSegmentId());
                linuxs.add(target);
            }
        }
        return new GeneralResult<TargetGroup>(true, new TargetGroup(windows, linuxs), "success");
    }

    /**
     * 根据操作系统类型封装机器信息
     * @param hosts
     * @return
     */
    public static GeneralResult<TargetGroup> groupTargetByWorkerHost(List<WorkerHost> hosts) {
        List<WorkerHost> linuxs = Lists.newArrayList();
        List<WorkerHost> windows = Lists.newArrayList();
        for (WorkerHost host : hosts) {
            if (null != host.getCategory() && host.getCategory().toLowerCase().contains("window")) {
                windows.add(host);
            } else {
                linuxs.add(host);
            }
        }
        return new GeneralResult<TargetGroup>(true, new TargetGroup(windows, linuxs), "success");
    }

}
