package com.bocloud.cop.service.utils.worker;

import java.util.Map;


import com.megatron.framework.http.model.ServiceParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import com.megatron.framework.http.core.HttpMethod;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.megatron.framework.core.RegistryService;
import com.megatron.framework.core.Service;

@Slf4j
@Repository("workerServiceFactory")
public class WorkerServiceFactory {

    @Autowired
    private RegistryService registryService;

    public WorkerRemoteService build(BoCloudService service, String url, HttpMethod method,
        Map<String, Object> headers, Map<String, Object> params) {
        return build(Service.create(String.valueOf(service)), url, method, headers, params);
    }

    private WorkerRemoteService build(Service service, String url, HttpMethod method, Map<String, Object> headers,
        Map<String, Object> params) {
        String value = this.registryService.getService(service);
        WorkerRemoteService remoteService = null;
        if (!StringUtils.isEmpty(value)) {
            ServiceParam param = new ServiceParam(params, headers);
            StringBuilder builder = new StringBuilder();
            builder.append("http://").append(value).append(url);
            remoteService = new WorkerRemoteService(service, param, true, builder.toString(), method);
        } else {
            log.warn("Remote Service [{}] does not exist!", service.getName());
            remoteService = new WorkerRemoteService(service, false);
        }

        return remoteService;
    }

    public WorkerRemoteService build(BoCloudService service, String instance, String uri, HttpMethod method,
        Map<String, Object> headers, Map<String, Object> params) {
        String url = "http://" + instance + uri;
        ServiceParam param = new ServiceParam(params, headers);
        return new WorkerRemoteService(Service.create(String.valueOf(service)), param, true, url, method);
    }
}
